#ifndef _AXIS_H_
#define _AXIS_H_

#include <vector>
#include <stdlib.h>
#include <string>
#include <algorithm>
#include <iostream>
#include <sys/time.h>

#include <unistd.h>


#include "Motor.h"
#include "Encoder.h"
#include "Controller.h"
#include "MTypes.h"

namespace Axis_ns {

#define RAWPOS_WAITTMO_TIMES 10

typedef enum {
	FSM_ERROR,
	FSM_NOTINITED,
	FSM_INITIALIZING,
	FSM_WAIT,
	FSM_STANDSTILL,
	FSM_MOVING,
	FSM_START_POSITIONING,
	FSM_POSITIONING,
	FSM_STOP_POSITIONING,
	FSM_POSITIONING_END,
	FSM_FORWARD,
	FSM_BACKWARD,
	FSM_FINDHOME,
	FSM_FINDRLS,
	FSM_FINDFLS,
	FSM_FINDINDEX,
	FSM_FINDDONE,
	FSM_FINDENCSIGN_LS,
	FSM_FINDENCSIGN,
	FSM_RAWPOS_WAITTMO
} AxisFSM_t;

const char * const AxisFSMName[] = {
	"FSM_ERROR",
	"FSM_NOTINITED",
	"FSM_INITIALIZING",
	"FSM_WAIT",
	"FSM_STANDSTILL",
	"FSM_MOVING",
	"FSM_START_POSITIONING",
	"FSM_POSITIONING",
	"FSM_STOP_POSITIONING",
	"FSM_POSITIONING_END",
	"FSM_FORWARD",
	"FSM_BACKWARD",
	"FSM_FINDHOME",
	"FSM_FINDRLS",
	"FSM_FINDFLS",
	"FSM_FINDINDEX",
	"FSM_FINDDONE",
	"FSM_FINDENCSIGN_LS",
	"FSM_FINDENCSIGN",
	"FSM_RAWPOS_WAITTMO"
};

typedef enum {
	AXIS_E000,
	AXIS_E001,
	AXIS_E002,
	AXIS_E003,
	AXIS_E004,
	AXIS_E005,
	AXIS_E006,
	AXIS_E007,
	AXIS_E008,
	AXIS_E009,
	AXIS_E010,
	AXIS_E011,
	AXIS_E012
} AxisError_t;

const char * const AxisErrorName[] = {
	"AXIS_E000 : no error",
	"AXIS_E001 : controller/database motor type mismatch",
	"AXIS_E002 : max number of restoration reached",
	"AXIS_E003 : restoration error on defposition()",
	"AXIS_E004 : restoration impossible due to data not valid!",
	"AXIS_E005 : error on findlimit",
	"AXIS_E006 : no motion started!",
	"AXIS_E007 : Not allowed: previous positioning in progress",
	"AXIS_E008 : Not allowed: unready state",
	"AXIS_E009 : no motion due to motor OFF",
	"AXIS_E010 : error in positioning()",
	"AXIS_E011 : positioning inhibited/stopped by limit switch",
	"AXIS_E012 : cmd rejected by driver"
};

typedef enum {
	FSM_E000,
	FSM_E001,
	FSM_E002,
	FSM_E003,
	FSM_E004,
	FSM_E005,
	FSM_E006,
	FSM_E007,
	FSM_E008,
	FSM_E009,
	FSM_E101,
	FSM_E102,
	FSM_E103,
	FSM_E104,
	FSM_E105,
	FSM_E106
} AxisFSMError_t;

const char * const AxisFSMErrorName[] = {
	"FSM_E000 : no error",
	"FSM_E001 : positioning inhibited by limit switch or failed to start the hw motion",
	"FSM_E002 : positioning ended with error",
	"FSM_E003 : program not running",
	"FSM_E004 : error on init() on define position",
	"FSM_E005 : error on find limit",
	"FSM_E006 : error on configured home init",
	"FSM_E007 : positioning ended due to limit switch",
	"FSM_E008 : error on init()",
	"FSM_E009 : error on get encoder signature",
	"FSM_E101 : init() phase, error on stop()",
	"FSM_E102 : init() phase, error on motor_on()",
	"FSM_E103 : init() phase, error on move",
	"FSM_E104 : init() phase, error on stop reason",
	"FSM_E105 : init() phase, error on stop_no_brake()",
	"FSM_E106 : Cia402 init() phase, error"
};

typedef enum {
	FSMT_NOP,
	FSMT_INIT,
	FSMT_ABORT,
	FSMT_POSITION,
	FSMT_DEF_POSITION,
	FSMT_FINDRLS,
	FSMT_FINDFLS,
	FSMT_FINDHOME,
	FSMT_FINDENCSIGN,
	FSMT_RAWPOS_WAITTMO
} AxisFSMTransitions_t;

const char * const AxisFSMTransitions[] = {
	"FSMT_NOP: no operation",
	"FSMT_INIT: initialize motor",
	"FSMT_ABORT: abort FSM job",
	"FSMT_POSITION: axis position",
	"FSMT_DEF_POSITION: axis define/force position",
	"FSMT_FINDRLS: axis find rls",
	"FSMT_FINDFLS: axis find fls",
	"FSMT_FINDHOME: axis find home",
	"FSMT_FINDENCSIGN: axis find encoder signature",
	"FSMT_RAWPOS_WAITTMO: wait raw positioning end"
};

typedef enum {
	LIMIT_NC,
	LIMIT_NO
} AxisLimitType_t;

const char * const AxisLimitType[] = {
	"LIMIT_NC: normally closed",
	"LIMIT_NO: normally opened"
};

typedef enum {
	LIMIT_FLS,
	LIMIT_RLS,
	LIMIT_HOME
} AxisLimit_t;

typedef enum {
	HOME_HALF_TRAVEL,	// e' attivo per meta' della corsa totale (Galil nativo)
	HOME_POINT		// e' attivo solo in uno punto
} AxisHomeType_t;

const char * const AxisHomeTypeName[] = {
	"HOME_HALF_TRAVEL",
	"HOME_POINT"
};

typedef enum {
	HOME_POL_POS, // era 1
	HOME_POL_NEG // era -1
} AxisHomePolarity_t;

const char * const AxisHomePolarityName[] = {
	"HOME_POL_POS",
	"HOME_POL_NEG"
};

typedef enum {
	FIND_HOME_HALFTRAVEL,
	FIND_HOME_PLUS, // era 1
	FIND_HOME_MINUS, // era -1
	FIND_HOME_UNKNOWN
} AxisFindHomeVersus_t;

typedef enum {
	FIND_POS = 1,
	FIND_NEG = -1
} AxisFindPolarity_t;

const char * const AxisFindPolarityName[] = {
	"FIND_POS",
	"FIND_NEG"
};

typedef struct {
	bool 		have_limit;
	AxisLimit_t	limit;
	AxisLimitType_t	type;
	union {
		struct {
			AxisHomeType_t 		htype;
			AxisHomePolarity_t 	pol;
		} home;
		struct {
			int n;
		} ls;
		struct {
			int n;
		} idx;
	} extra;
} AxisLimitConf_t;

const char * const AxisFindHomeVersusName[] = {
	"FIND_HOME_HALFTRAVEL",
	"FIND_HOME_PLUS",
	"FIND_HOME_MINUS"
};

typedef enum {
	FINDLIMIT_UNKNOWN,
	FINDLIMIT_DONE,
	FINDLIMIT_RUN,
	FINDLIMIT_ERROR,
	FINDLIMIT_H1
} AxisFindLimit_t;

const char * const AxisFindLimitName[] = {
	"FINDLIMIT_UNKNOWN",
	"FINDLIMIT_DONE",
	"FINDLIMIT_RUN",
	"FINDLIMIT_ERROR",
	"FINDLIMIT_H1"
};



typedef enum {
	INIT_ON_DEFPOS = 0,
	INIT_ON_HOME = 1,
	INIT_ON_RLS = 2,
	INIT_NO_3,
	INIT_ON_FLS = 4,
	INIT_ON_CUSTOM = 5,
	INIT_NO_6,
	INIT_ON_RLSIDX = 7,
	INIT_ON_FLSIDX = 8,
	INIT_ON_EDGEPOSDEF = 9,
	INIT_ON_EDGENEGDEF = 10,
	INIT_ON_ABSOLUTE,
	INIT_ON_SENCPOS,
	INIT_ON_SENCNEG,
	INIT_ON_ROTIDXPOS,
	INIT_ON_ROTIDXNEG,
	CIA402_INIT_ON_RLS,
	CIA402_INIT_ON_FLS,
	CIA402_INIT_ON_RLSIDX,
	CIA402_INIT_ON_FLSIDX,
	CIA402_INIT_ON_ROTIDXPOS,
	CIA402_INIT_ON_ROTIDXNEG,
	INIT_VOID
} AxisInitType_t;

const char * const AxisInitTypeName[] = {
	"INIT_ON_DEFPOS",
	"INIT_ON_HOME",
	"INIT_ON_RLS",
	"",
	"INIT_ON_FLS",
	"INIT_ON_CUSTOM",
	"",
	"INIT_ON_RLSIDX",
	"INIT_ON_FLSIDX",
	"INIT_ON_EDGEPOSDEF",
	"INIT_ON_EDGENEGDEF",
	"INIT_ON_ABSOLUTE",
	"INIT_ON_SENCPOS",
	"INIT_ON_SENCNEG",
	"INIT_ON_ROTIDXPOS",
	"INIT_ON_ROTIDXNEG",
	"CIA402_INIT_ON_RLS",
	"CIA402_INIT_ON_FLS",
	"CIA402_INIT_ON_RLSIDX",
	"CIA402_INIT_ON_FLSIDX",
	"CIA402_INIT_ON_ROTIDXPOS",
	"CIA402_INIT_ON_ROTIDXNEG",
	"INIT_VOID",
	"INIT_VOID1"
};

typedef enum {
	IFSM_IDLE,
	IFSM_STOP,
	IFSM_START,
	IFSM_CHECK_SWITCH1,
	IFSM_WAIT_FAST_MOVE,
	IFSM_FASTTOSWRUN,
	IFSM_FASTTOSWDONE,
	IFSM_WAIT_FASTLEAVE_MOVE,
	IFSM_FASTLEAVESWRUN,
	IFSM_FASTLEAVESWDONE,
	IFSM_WAIT_SLOW_MOVE,
	IFSM_SLOWTOSWRUN,
	IFSM_SLOWTOSWDONE,
	IFSM_STEP4RUN,
	IFSM_STEP4DONE,
	IFSM_DONE,
	IFSM_ERROR,
	IFSM_LEAVEHOME,
	IFSM_START_FE,
	IFSM_WAIT_FE,
	IFSM_WAIT_FESTART,
	IFSM_WAIT_FEONSTART,
	IFSM_WAIT_FEON,
	IFSM_WAIT_FEOFFSTART,
	IFSM_WAIT_FEOFF,
	IFSM_WAIT_FEOFFSTOP,
	IFSM_FASTTOSWRUNSTART,
	IFSM_START_FI,
	IFSM_WAIT_FI,
	IFSM_WAIT_FISTART,
	IFSM_WAIT_FIDONE,
	IFSM_WAIT,
	IFSM_FASTTOSWRUNWAIT,
	IFSM_FIDONE,
	IFSM_LEAVEFI,
	IFSM_WAIT_LEAVEFI,
	IFSM_LEAVEFIDONE,

	IFSM_LEAVESWSTART,
	IFSM_LEAVESWDONE,
	IFSM_WAIT_LEAVESW
} AxisInitFSM_t;

const char * const AxisIFSMName[] = {
	"IFSM_IDLE",
	"IFSM_STOP",
	"IFSM_START",
	"IFSM_CHECK_SWITCH1",
	"IFSM_WAIT_FAST_MOVE",
	"IFSM_FASTTOSWRUN",
	"IFSM_FASTTOSWDONE",
	"IFSM_WAIT_FASTLEAVE_MOVE",
	"IFSM_FASTLEAVESWRUN",
	"IFSM_FASTLEAVESWDONE",
	"IFSM_WAIT_SLOW_MOVE",
	"IFSM_SLOWTOSWRUN",
	"IFSM_SLOWTOSWDONE",
	"IFSM_STEP4RUN",
	"IFSM_STEP4DONE",
	"IFSM_DONE",
	"IFSM_ERROR",
	"IFSM_LEAVEHOME",
	"IFSM_START_FE",
	"IFSM_WAIT_FE",
	"IFSM_WAIT_FESTART",
	"IFSM_WAIT_FEONSTART",
	"IFSM_WAIT_FEON",
	"IFSM_WAIT_FEOFFSTART",
	"IFSM_WAIT_FEOFF",
	"IFSM_WAIT_FEOFFSTOP",
	"IFSM_FASTTOSWRUNSTART",
	"IFSM_START_FI",
	"IFSM_WAIT_FI",
	"IFSM_WAIT_FISTART",
	"IFSM_WAIT_FIDONE",
	"IFSM_WAIT",
	"IFSM_FASTTOSWRUNWAIT",
	"IFSM_FIDONE",
	"IFSM_LEAVEFI",
	"IFSM_WAIT_LEAVEFI",
	"IFSM_LEAVEFIDONE",

	"IFSM_LEAVESWSTART",
	"IFSM_LEAVESWDONE",
	"IFSM_WAIT_LEAVESW"
};

typedef enum {
	CHK_ENC_UNKNOWN,
	CHK_ENC_OK,
	CHK_ENC_ERROR,
	CHK_ENC_ERROR_ONSTAY,
	CHK_ENC_ERROR_ONRUN
} chk_enc_t;

const char * const CheckEncName[] = {
	"CHK_ENC_UNKNOWN",
	"CHK_ENC_OK",
	"CHK_ENC_ERROR",
	"CHK_ENC_ERROR_ONSTAY",
	"CHK_ENC_ERROR_ONRUN"
};
const char * const CheckEncMsg[] = {
	"Encoder/Motor xcheck: unknown",
	"Encoder/Motor xcheck: ok",
	"Encoder/Motor xcheck: error",
	"Encoder/Motor xcheck: error while motor not moving",
	"Encoder/Motor xcheck: error while motor moving"
};


// meccanismo valido per posizionamenti tramite FSM/POSITION, che si basa sul microcodice galil
// che "garantisce" il posizionamento entro Dbw
typedef struct {
	bool       data_valid;
	double     last_position;
	int	   nb_of_restoration;
	int 	   max_nb_of_restoration;
} restore_last_position_t;


class Axis : public Motor_ns::Motor, public Encoder_ns::Encoder {

private:
	string _name;
	string _channel;
	Controller_ns::Controller *_controller;

	bool _prtLog;

	double _position;  	// essenzialmente steps i.e. steps
	double _target;  	// dove devo andare i.e. steps
	string _units;		// ad es. steps

	bool _fls;
	bool _rls;
	bool _home;

	int _verbose;

	double _emr;	// encoder/motor ratio i.e, angular coeff. e = emr * m + emo
	double _emo;

	double _per;	// position = per * enc + peo
	double _peo;

	Controller_ns::mconf_t _mc;
	Controller_ns::econf_t _ec;


	double _mpos_on_init;
	double _pos_on_init;
	
	#if 0
		bool _have_fls;
		bool _have_rls;
		bool _have_home;
		bool _have_index;

		AxisHomeType_t 		_home_type;
		AxisHomePolarity_t 	_home_polarity;
	#endif

	AxisLimitConf_t   _fls_conf;
	AxisLimitConf_t   _rls_conf;
	AxisLimitConf_t  _home_conf;
	AxisLimitConf_t _index_conf;

	AxisFindHomeVersus_t	_home_versus; //versus of home procedure

	AxisFSM_t _FSM;
	AxisFSMError_t _FSMerror;

	AxisInitType_t _mInitType;
	AxisInitFSM_t  _mInitFSM;

	AxisFindLimit_t _findlimit_st;


	double _speed_save;


	vector<double> _find_speeds;

	AxisFSMTransitions_t _oldtr;

        struct timeval _last_update; // succesfull update
        struct timeval _last_refresh;

	bool _chk_enc_enable;
	double _chk_enc_last_value;
	double _chk_enc_last_diff;
	bool	_chk_enc_last_ok;
	int _chk_enc_wait;
	double _chk_mot_last_value;
	double _chk_mot_last_diff;
	bool	_chk_mot_last_ok;
	int _chk_enc_wait_on_run;

	double _chk_diff_on_run;

	vector <double> __save_mpos;
	vector <double> __save_epos;
	double _relative_mpos_preinit;
	double _relative_epos_preinit;

	AxisError_t _AXISError;

	restore_last_position_t _rest_last_pos;

	// Axis_sencInit()
	double	_epos0, _epos1;
        double  __current_versus;
	int 	_howmany_success_fi;
        int 	_howmany_fi;
        int 	_howmany_switch;

	double _raw_pos_tmo;
	double _raw_pos_tmo_sec;

	Controller_ns::opmode_t _opmode;


public:
	Axis(string name, string channel, Controller_ns::Controller *controller, bool prtLog);
	~Axis() {};

	void		prtLog(bool v) { _prtLog = v; };
	bool		prtLog() {return _prtLog; };

        int            verbose()               	{ return _verbose;};
	void           verbose(int b);

        void    	lnk_verbose(int b);
        int     	lnk_verbose();

        void    	drv_verbose(int b);
        int     	drv_verbose();


	void		name(string n)		{ _name = n; };
	string		name()			{ return _name; };

        string          channel()               { return _channel; };
        void            channel(string v)       { _channel = v; };

	string 		controller_typeName()	{ return _controller->controller_typeName(); };

	Controller_ns::opmode_t		opmode()				{ return _controller->opmode(_channel); };
	bool 				opmode(Controller_ns::opmode_t op)	{ return _controller->opmode(_channel, op); };
	string				opmode2str(Controller_ns::opmode_t op)  { return _controller->OpMode2str(op); };

	// MOTION
	//	new values after update()

        bool            forward(double v);
        bool            backward(double v);

	bool		findedge(double v);
	bool 		findindex(double v);


        double          position()              { return _position; };
	bool		position(double p);
	bool		rawposition(double p, bool rel); // rel=TRUE => relative positioning

	double		speed()			{ return motor2speed(mspeed()); };
	bool		speed(double v)		{ bool b = mspeed(speed2motor(v)); refresh(_controller->max_tmo()); return b;};

	double		acceleration()		{ return motor2acceleration(macceleration()); };
	bool		acceleration(double a)	{ return macceleration(acceleration2motor(a)); };

	double		deceleration()		{ return motor2deceleration(mdeceleration()); };
	bool		deceleration(double a)	{ return mdeceleration(deceleration2motor(a)); };

	double		velocity()		{ return enc2speed(evelocity()); };

	// CONFIGURATION
	//
	double		per()			{ return _per; };
	bool 		per(double v)		{ _per = v; return true; };
	double		peo()			{ return _peo; };
	bool 		peo(double v)		{ _peo = v; return true; };

	double 		pmr()			{ return _per * _emr; };

	double		emr()			{ return _emr; };
	bool 		emr(double v)		{ if (_etype == Encoder_ns::ENC_NO) _emr = 1; else _emr = v; return true; };
	double		emo()			{ return _emo; };
	bool 		emo(double v)		{ if (_etype == Encoder_ns::ENC_NO) _emo = 1; _emo = v; return true; };

	double		position2motor(double p);
	double		motor2position(double m);
	double		position2enc(double p);
	double		enc2position(double e);


	double		motor2enc(double m);
	double		enc2motor(double e);

	double		speed2motor(double v);
	double		motor2speed(double vm);
	double		speed2enc(double v);
	double		enc2speed(double ve);

	double		acceleration2motor(double a);
	double		motor2acceleration(double am);
	double 		enc2acceleration(double ae);
	double 		acceleration2enc(double a);

	double		deceleration2motor(double a);
	double		motor2deceleration(double am);
	double 		enc2deceleration(double ae);
	double 		deceleration2enc(double a);


        bool            	mconf(Controller_ns::mconf_t mc);
        Controller_ns::mconf_t	mconf()				{ return _mc; };
	Controller_ns::mconf_t  get_mconf();

        bool            	econf(Controller_ns::econf_t ec);
        Controller_ns::econf_t	econf()				{ return _ec; };
	Controller_ns::econf_t  get_econf();

	//
	//

	bool 		fls()		{ if (pmr() > 0) return _fls; else return _rls; };
	bool 		rls()		{ if (pmr() > 0) return _rls; else return _fls; };
	bool		home();

        double  	get_ls_tolerance();

	bool 		hwfls()		{ return _fls; };
	bool 		hwrls()		{ return _rls; };
	bool		hwhome()	{ return _home; };

	AxisLimit_t	axis2motorLimit(AxisLimit_t l);

	double 		target() 		{ return _target; };

	bool 		mInit();
	bool 		rflsInit(AxisLimit_t l);

	bool 		homeInit(AxisFindHomeVersus_t v);

	bool 		indexInit(AxisInitType_t t);

	bool		absoluteInit();
	bool		sencInit(AxisFindPolarity_t p);

	bool 		rotIdxInit(AxisInitType_t t);

	/*************************************************
	*
	*   Cia402 Init
	*/
	bool 		Cia402_rflsInit(AxisLimit_t l);
	bool 		Cia402Init(AxisInitType_t l);


	AxisFindHomeVersus_t findedge_versus()	{ // WHEN HOME NOT ACTIVE
						  switch (_home_conf.type){
							case LIMIT_NC:
								switch (_home_conf.extra.home.pol){
									case HOME_POL_POS:
										// motor: FIND_HOME_PLUS;
										if ( pmr() > 0 ) return FIND_HOME_PLUS;
										else return FIND_HOME_MINUS;
										break;
									case HOME_POL_NEG:
										break;
								}
								break;
							case LIMIT_NO:
								return FIND_HOME_PLUS;
								break;
						  }
						  return FIND_HOME_UNKNOWN;
						};


	vector<double>	find_speeds()		{ return _find_speeds; };
	bool 		find_speeds(vector<double> v){ _find_speeds = v; return true; };

	AxisFindLimit_t	findlimit(AxisLimit_t l, AxisFindLimit_t oldstate);
	bool		findhome() 		{
						  if ( fsm(Axis_ns::FSMT_FINDHOME) == FSM_ERROR ) {
							_AXISError = AXIS_E005;
							return false;
						  } else return true;
						  return false;
						};
	bool		findfls() 		{ return false; };
	bool		findrls() 		{ return false; };

	bool		update();		// update all relevant 'current' values
	bool		update(double tmo);	// update all relevant 'current' values
						// onexit:
						//	true  = effective new values
						//	false = time out expired! no new values
	bool		refresh();		// update all 'primary' values

	bool		refresh(double tmo);	// refresh all 'secondary' values:
						// onexit:
						//	true  = effective new values
						//	false = time out expired! no new values

		

	AxisFSM_t  	fsm(AxisFSMTransitions_t tr);		// proceed with FSM
	AxisFSM_t	fsm_state() 		{ return _FSM; };
	string		fsm_status() 		{ return AxisFSMName[_FSM]; };

	AxisFSMError_t	fsm_error_state() 	{ return _FSMerror; };
	string		fsm_error_status() 	{ return AxisFSMErrorName[_FSMerror]; };
	bool 		fsm_error_clear()	{ _FSMerror = FSM_E000; return true;};

	AxisInitFSM_t	ifsm_state() 		{ return _mInitFSM; };
	string		ifsm_status()		{ return AxisIFSMName[_mInitFSM]; };

	void		config_InitType(AxisInitType_t t) 	{ _mInitType = t; };
	AxisInitType_t	config_InitType() 			{ return _mInitType; };
	string		config_InitType2str() 			{ return AxisInitTypeName[_mInitType]; };
	

	void		config_rls(AxisLimitConf_t v)	{ _rls_conf.have_limit 	= v.have_limit;
							  _rls_conf.limit	= v.limit;  
							  _rls_conf.type	= v.type;
							  _rls_conf.extra.ls.n	= v.extra.ls.n;
							};
	void		config_fls(AxisLimitConf_t v)	{ _fls_conf.have_limit 	= v.have_limit;
							  _fls_conf.limit	= v.limit;  
							  _fls_conf.type	= v.type;
							  _fls_conf.extra.ls.n	= v.extra.ls.n;
							};
	void		config_home(AxisLimitConf_t v)	{ _home_conf.have_limit 	= v.have_limit;
							  _home_conf.limit		= v.limit;  
							  _home_conf.type		= v.type;
							  _home_conf.extra.home.htype	= v.extra.home.htype;
							  _home_conf.extra.home.pol	= v.extra.home.pol;
							};
	void		config_index(AxisLimitConf_t v)	{ _index_conf.have_limit 	= v.have_limit;
							  _index_conf.limit		= v.limit;  
							  _index_conf.type		= v.type;
							  _index_conf.extra.idx.n	= v.extra.idx.n;
							};


	AxisLimitConf_t	config_rls()	{ return _rls_conf; };
	AxisLimitConf_t	config_fls()	{ return _fls_conf; };
	AxisLimitConf_t	config_home()	{ return _home_conf; };
	AxisLimitConf_t	config_index()	{ return _index_conf; };


	bool	have_rls()	{ return _rls_conf.have_limit; };
	bool	have_fls()	{ return _fls_conf.have_limit; };
	bool	have_home()	{ return _home_conf.have_limit; };
	bool	have_index()	{ return _index_conf.have_limit; };



	AxisHomeType_t	home_type()	{ return _home_conf.extra.home.htype; };
	//bool		home_type(AxisHomeType_t t) { _home_type = t; return true; };
	AxisHomePolarity_t home_polarity()	{ return _home_conf.extra.home.pol; };

	AxisFindHomeVersus_t home_versus()	{ return _home_versus; };
	bool 		home_versus(AxisFindHomeVersus_t v) { _home_versus = v; return true; };

	bool		check_mconf_mt();

	bool            get_SencSignature();

	string 		units()           	{ return _units; };
	bool   		units(string v)   	{ _units = v; return true; };

	bool		isinit(); // is motor inited?
	bool		isinit(bool do_update); // is motor inited?

	bool 		stop()			{ fsm(FSMT_ABORT); _FSMerror = FSM_E000; bool b1 = Motor::stop();
						//N5N5
						  bool b2 = false;
                        			  switch (_controller->controller()){
                                			  default:
								  b2 = true;
                                        			  break;
                                			  case Controller_ns::N5:
								  if ( opmode() != _opmode )
                                        			  	b2 = opmode(Controller_ns::PROFILE_POSITION);
								  else
									b2 = true;
                                        			  break;
                        			  }
						  return (b1 && b2);
						};


	bool		mpos_on_init(double p){ _mpos_on_init = p; return true; };
	double		mpos_on_init()	{ return _mpos_on_init; };

	bool		pos_on_init(double p){ _pos_on_init = p; return true; };
	double		pos_on_init()	{ return _pos_on_init; };
	bool		defposition(double p);

	vector<double>	saved_mpos()	{ return __save_mpos; };
	vector<double>	saved_epos()	{ return __save_epos; };
	double		relative_mpos_preinit() { return _relative_mpos_preinit; };
	double		relative_epos_preinit() { return _relative_epos_preinit; };

	chk_enc_t	check_encoder();
	bool		check_encoder_restart()	{ _chk_enc_last_ok = false; _chk_mot_last_ok = false; _chk_enc_wait_on_run = -1; _chk_enc_wait=1; return true; };
	double		chk_enc_last_value()	{ return _chk_enc_last_value; };
	double		chk_mot_last_value()	{ return _chk_mot_last_value; };
	double 		chk_diff_on_run()	{ return _chk_diff_on_run; };
	void 		clr_chk_diff_on_run()	{ _chk_mot_last_diff = 0; _chk_enc_last_diff = 0; _chk_diff_on_run = 0;};
	bool		chk_enc_enable()	{ return _chk_enc_enable; };
	void		chk_enc_enable(bool e)	{ _chk_enc_enable = e; };

	bool 		force_init(double p); // force init state with position p

	string		axis_error_str()	{ switch(_AXISError){
								case AXIS_E000:
        							case AXIS_E001:
        							case AXIS_E002:
        							case AXIS_E003:
        							case AXIS_E004:
        							case AXIS_E005:
        							case AXIS_E006:
        							case AXIS_E007:
        							case AXIS_E008:
        							case AXIS_E009:
        							case AXIS_E010:
        							case AXIS_E011:
        							case AXIS_E012:
									return AxisErrorName[_AXISError];
									break;
								default: {
										std::stringstream ss;
										ss << "-?- [AXISError= " << _AXISError << " ]";
						  				return ss.str();
									}
									break;
						  }
						  return "-?-";
						};

	string		check_enc_name_str(chk_enc_t ce){ switch(ce){
								case CHK_ENC_UNKNOWN:
								case CHK_ENC_OK:
								case CHK_ENC_ERROR:
								case CHK_ENC_ERROR_ONSTAY:
								case CHK_ENC_ERROR_ONRUN:
									return CheckEncName[ce];
									break;
								default:
							  		return "-!-";
									break;
							  }
							  return "-!-";
							}
	restore_last_position_t get_restore_last_position() { return _rest_last_pos; };
	bool	restore_last_position();

private:
	bool 		set_init(bool b);
	bool 		set_init(double tmo, bool b);


	bool		__position(double p);
	bool		__rawposition(double p, bool rel);

	bool 		check_motion_progress(double tmo);

}; /* class Axis */

} /* namespace Axis_ns */

#endif /* _AXIS_H_ */
