#ifndef _CONTROLLER_H_
#define _CONTROLLER_H_

#include "Link.h"
#include "Drv.h"
#include "verbose.h"

#ifdef _WITH_BCSDRV_
#include "BcsDrv.h"
#endif

#ifdef _WITH_SIMULDRV_
#include "SimulDrv.h"
#endif

#ifdef _WITH_N5DRV_
#include "N5Drv.h"
#endif

#if defined(_WITH_GALILDMC2XXXDRV_) || defined(_WITH_YAMSDRV_)
#include "GalilDMC2xxxDrv.h"
#endif

#ifdef _WITH_YAMSDRV_
#include "YamsDrv.h"
#endif

#ifdef _WITH_DMC2XXXPIEZODRV_
#include "DMC2xxxPiezoDrv.h"
#endif

#if defined(_WITH_GALILDMC2XXXDRV_) || defined(_WITH_YAMSDRV_) || defined(_WITH_DMC2XXXPIEZODRV_)
#include "GalilTangoLink.h"
#endif




namespace Controller_ns {

typedef enum {
	GALILDMC2xxx,
	Yams,
	DMC2xxxPiezo,
	N5,
	SIMUL,
	BCS
} ControllerType_t;

const char * const ControllerTypeName[] = {
        "GALILDMC2xxx",
        "Yams",
	"DMC2xxxPiezo",
	"N5",
	"SIMUL",
	"BCS"
};

typedef enum {
	CTRL_UNKNOWN,
	CTRL_CLOSE,
	CTRL_UP,
	CTRL_DOWN
} ControllerState_t;

const char * const ControllerStateName[] = {
	"CTRL_UNKNOWN",
	"CTRL_CLOSE",
	"CTRL_UP",
	"CTRL_DOWN"
};

class Controller {

private:
	Drv *_drv;
	ControllerType_t _controller;
	string _target;
	string _channel;
	string _name;
	int _verbose;
	double _max_tmo;
	bool _prtLog;


	void ControllerConstruct(ControllerType_t controller, string target, string extra, bool prtLog);

public:
	Controller(ControllerType_t controller, string target, string extra){
		ControllerConstruct(controller, target, extra, true);};
	Controller(ControllerType_t controller, string target, string extra, bool prtLog){
		ControllerConstruct(controller, target, extra, prtLog);};

	~Controller();

	bool prtLog();
	void prtLog(bool v);


	ControllerType_t 	controller();
	string			controller_typeName();
	string			target();	

        string 			raw_io(string quest);

	opmode_t        opmode(string channel);
        bool            opmode(string channel, opmode_t op);

        std::string     OpMode2str(opmode_t op);

        Link_ns::uvalues_t 	update(string channel);
        Link_ns::rvalues_t 	refresh(string channel);

	bool 			stop(string channel);
	bool 			stop_no_brake(string channel);

	bool 			speed(string channel, double v);
	double 			speed(string channel) ;

	bool 			acceleration(string channel, double a);
	double 			acceleration(string channel);
	bool 			deceleration(string channel, double a);
	double 			deceleration(string channel);

	bool 			forward(string channel, double v);
	bool 			backward(string channel, double v);

	bool 			findedge(string channel, double v);
	bool 			findindex(string channel, double v);

	double 			mpos(string channel);
	bool 			mpos(string channel, double p);  // mpos : motor position

	bool 			xpos(string channel, double p);  // xpos : motor position in encoder units

	bool 			rpos(string channel, double p);  // rpos : motor relative position

	bool            	defmpos(string channel, double p);
	bool            	defepos(string channel, double p);

	bool            	brake(string channel, bool b);
	bool			motor_on(string channel);
	bool			motor_off(string channel);

	int 			hw_brake_conf(string channel);

        int 			verbose();
        void 			verbose(int b);

        int 			drv_verbose();
        void 			drv_verbose(int b);

        int 			lnk_verbose();
        void 			lnk_verbose(int b);

        int 			lnk_flag();
        void 			lnk_flag(int b);

	string  		stop_code2str(unsigned long stop_code);
	string 			CC_code2str(unsigned long CC_code);

	Drv*			drv();

	bool 			hw_write_enable();
	void 			hw_write_enable(bool b);

	double 			max_tmo();
	bool 			max_tmo(double v) { _max_tmo = v; return true; };

	double 			updateDelay();

	Link_ns::LinkState_t 	linkstate();
	string 			linkstatus();

	ControllerState_t 	state();
	string			status();

	bool 			mconf(string channel, mconf_t mc);
	mconf_t			mconf(string channel);

	bool 			econf(string channel, econf_t ec);
	econf_t			econf(string channel);

	bool 			check_motion_end(unsigned long CC_code);
	bool 			check_motion_progress(unsigned long CC_code);
	bool 			check_motion_cmd(unsigned long CC_code);

}; /* class Controller */

} /* namespace Controller_ns */

#endif /* _CONTROLLER_H_ */
