#ifndef _DRV_H_
#define _DRV_H_

#include "Link.h"
#include "verbose.h"
#include <vector>
#include "MTypes.h"

#if defined(_WITH_GALILDMC2XXXDRV_) || defined(_WITH_YAMSDRV_) || defined(_WITH_DMC2XXXPIEZODRV_)
#ifdef _PRT_EXCEPT
#define RAW_IO(l1, l2, obj, n, ans, ret)                \
        prtverbose(l2, obj, "cmd="+n);                  \
        try {                                           \
                ans = raw_io(n);                        \
                prtverbose(l2, obj, "ans="+ans);        \
        } catch (Controller_ns::DrvExcpt &e) {          \
		ss.str("");				\
                ss << e.what() << " : ";                \
                ss << e.report() << " : ";              \
                ss << e.ans() << " : ";                 \
                ss << e.errmsg();                       \
                prtverbose(l1, obj, "Excpt! "+ss.str());\
                prtverbose(0, obj, "Excpt! "+ss.str()); \
                return ret;                             \
        } catch (...) {                                 \
                prtverbose(l1, obj, "Other excpt!");    \
                return ret;                             \
        }                                               
#else /* _PRT_EXCEPT */
#define RAW_IO(l1, l2, obj, n, ans, ret)                \
        prtverbose(l2, obj, "cmd="+n);                  \
        try {                                           \
                ans = raw_io(n);                        \
                prtverbose(l2, obj, "ans="+ans);        \
        } catch (Controller_ns::DrvExcpt &e) {          \
		ss.str("");				\
                ss << e.what() << " : ";                \
                ss << e.report() << " : ";              \
                ss << e.ans() << " : ";                 \
                ss << e.errmsg();                       \
                prtverbose(l1, obj, "Excpt! "+ss.str());\
                return ret;                             \
        } catch (...) {                                 \
                prtverbose(l1, obj, "Other excpt!");    \
                return ret;                             \
        }                                               
#endif /* _PRT_EXCEPT */
#endif

#define CHECK_BIT(var,pos) ( ((var) & (1<<(pos)))==0?false:true )
#define BIT_YGO_POS_CMD   0        //if ( CC_code & 0x1     ) cc.append("[POS]: Positioning Cmd. ");
#define BIT_YGO_PROGRESS  8        //if ( CC_code & 0x100   ) cc.append("[InPrgrss]: Movement in progress. ");
#define BIT_YGO_END_OK    9        //if ( CC_code & 0x200   ) cc.append("[OK]: Positioning ended without error. ");
#define BIT_YGO_END_NOK  10        //if ( CC_code & 0x400   ) cc.append("[NOK]: Positioning ended with error. ");



namespace Controller_ns {

//=========================================================================================
//	Cia402

typedef enum {
         AUTO_SETUP            = -2
        ,CLOCK_DIRECTION       = -1
        ,NO_MODE               = 0
        ,PROFILE_POSITION      = 1
        ,VELOCITY              = 2
        ,PROFILE_VELOCITY      = 3
        ,PROFILE_TORQUE        = 4
        ,RESERVED              = 5
        ,HOMING                = 6
        ,INTERPOLATED_POSITION = 7
        ,CYCLIC_SYNC_POSITION  = 8
        ,CYCLIC_SYNC_VELOCITY  = 9
        ,CYCLIC_SYNC_TORQUE    =  10
} opmode_t;

/*
StatusWord in HOMING MODE:
Bit 13 0 0 0 0 1 1
Bit 12 0 0 1 1 0 0
Bit 10 0 1 0 1 0 1
       | | | | | >Error during homing, motor at standstill
       | | | | >Error during homing, motor still turning
       | | | >Homing completed
       | | >Homing confirmed, but target not yet reached
       | >Homing is interrupted or not started
       >Homing is performed (ma forse meglio sarebbe dire: Homing richiesto o da fare, BOH!)
*/

typedef enum {
	 CIA402_HOME_PERFORMED
	,CIA402_HOME_INTERRUPTED
	,CIA402_HOME_PROGRESS
	,CIA402_HOME_COMPLETED
	,CIA402_HOME_ERROR_MOVING
	,CIA402_HOME_ERROR_STANDSTILL
} Cia402HomingState_t;

typedef struct {
	double home_offset;
	double home_speed_switch;
	double home_speed_reference;
	double home_acceleration;
	double home_switch_tolerance_band;
	double home_min_current_block_detection;
	double home_period_of_blocking;
} Cia402HomingParams_t;

//=========================================================================================

typedef enum {
         UNCONFIGURED
        ,VOLT_5_UNIPOL
        ,VOLT_5_BIPOL
        ,VOLT_10_UNIPOL
        ,VOLT_10_BIPOL
        ,AMP_020MA
        ,AMP_420MA
} AnalogType_t;

typedef struct {
        AnalogType_t type;
        double value;
} Analog_t;

//=========================================================================================


typedef enum {
	RAWIO
} DrvExcptNo_t;

class DrvExcpt {
  private:
        int _ret;
        string _message;
        string _ans;
        string _errmsg;

  public:
        DrvExcpt(int r, string m, string ans, string errmsg){ _ret = r; _message = m; _ans = ans; _errmsg = errmsg;};

        string what()   { return "Drv Layer Exception"; };
        string report() { return _message;};
        string ans() 	{ return _ans;};
        string errmsg()	{ return _errmsg;};
        int    number() { return _ret;};
};

class Drv {

protected:
	Link_ns::Link *_link;
	string _target;
	string _extra;
	int _verbose;
	double _last_refresh_ms;

	bool __hw_write_enable;
	bool _with_microcode;

	bool _prtLog;

	string _name;

public:
	Drv(string target, string extra){
		_target = target;
		_extra = extra;
		_verbose = 0;
		_name = "Drv";
		_prtLog = true;
	};
	virtual ~Drv() {};

	virtual string 			raw_io(string quest);
	virtual Link_ns::uvalues_t 	update(string channel);
	virtual Link_ns::rvalues_t 	refresh(string channel)	= 0;

	Link_ns::uvalues_t		uv();

        virtual int8_t  opmode(string channel)		        = 0;
        virtual bool    opmode(string channel, int8_t op)       = 0;

	virtual bool 	stop(string channel) 			= 0;
	virtual bool 	stop_no_brake(string channel) 		= 0;

	virtual bool 	speed(string channel, double v)		= 0;
	virtual double 	speed(string channel)			= 0;

	virtual bool 	acceleration(string channel, double a)	= 0;
	virtual double 	acceleration(string channel)		= 0;
	virtual bool 	deceleration(string channel, double a)	= 0;
	virtual double 	deceleration(string channel)		= 0;

	virtual bool    forward(string channel, double v)	= 0;
	virtual bool    backward(string channel, double v)	= 0;

	virtual bool    findedge(string channel, double v)	= 0;
	virtual bool    findindex(string channel, double v)	= 0;

        virtual double  mpos(string channel)                    = 0;
        virtual bool    mpos(string channel, double p)          = 0;       // mpos : motor position

        virtual bool    xpos(string channel, double p)          = 0;       // xpos : motor position in encoder units

        virtual bool    defmpos(string channel, double p)       = 0;       // define motor position
        virtual bool    defepos(string channel, double p)       = 0;       // define encoder position

        virtual bool    rpos(string channel, double p)          = 0;       // rpos : motor relative position

	virtual bool 	brake(string channel, bool b)		= 0;
	virtual int     hw_brake_conf(string channel)		= 0;

	virtual bool 	motor_off(string channel)		= 0;
	virtual bool 	motor_on(string channel)		= 0;
	
	virtual	string  stop_code2str(unsigned long stop_code)	= 0;

	virtual	bool program_running();
	bool    with_microcode() { return _with_microcode;};

	bool 	hw_write_enable();
	void    hw_write_enable(bool b);

	void    verbose(int b);
	int     verbose();

	void 	lnk_verbose(int b);
	int     lnk_verbose();

	double refresh_etime();
	double update_etime();

        int 	lnk_flag();
        bool 	lnk_flag(int v);

        bool    lnk_prtLog();
        void    lnk_prtLog(bool v);

	virtual string CC_code2str(VNOTU unsigned long CC_code);

	double          updateDelay();
	string		linkstatus();
	Link_ns::LinkState_t 	linkstate();

	virtual bool    mconf(string channel, mconf_t mc) = 0;
	virtual mconf_t mconf(string channel) 	          = 0;

	virtual bool    econf(string channel, econf_t ec) = 0;
	virtual econf_t econf(string channel) 	          = 0;

        virtual long  get_ls_tolerance(string channel)	  = 0; // <0: not config, >0 config


	/*****************************************
	*
	*   Cia402
	*/
        virtual bool                    Cia402HomingStart(int mode) = 0;
        virtual bool                    Cia402HomingEndRestore(int mode) = 0;
        virtual Cia402HomingState_t     Cia402HomingState()  = 0;
	virtual Cia402HomingParams_t	Cia402HomingParams() = 0;
	virtual bool			Cia402HomingParams(Cia402HomingParams_t p)= 0;


}; /* class Drv */


} /* namespace Controller_ns */

#endif /* _DRV_H_ */
