#ifndef _GALILDMC2XXXDRV_H_
#define _GALILDMC2XXXDRV_H_

#include "Drv.h"
#include <stdio.h>
#include <math.h>
#include <algorithm>

namespace Controller_ns {

#define MIN_SPx_VALUE 2
#define MIN_DMC_FLOAT 0.0001

typedef enum{
	MOTOR_RUN_INDEP          = 0,	// case   0: return "Motors are running, independent mode"; break;
	MOTOR_DECEL_STP          = 1,	// case   1: return "Motors decelerating or stopped at commanded independent position"; break;
	MOTOR_DECEL_STP_LIMITPOS = 2,	// case   2: return "Decelerating or stopped by FWD limit switch or soft limit FL"; break;
	MOTOR_DECEL_STP_LIMITNEG = 3,	// case   3: return "Decelerating or stopped by REV limit switch or soft limit BL"; break;
	MOTOR_DECEL_STP_STOP     = 4,	// case   4: return "Decelerating or stopped by Stop Command (ST)"; break;
	MOTOR_STP_ABORT_INP      = 6,	// case   6: return "Stopped by Abort input"; break;
	MOTOR_STP_ABORT_ABORT    = 7, 	// case   7: return "Stopped by Abort Command (AB)"; break;
	MOTOR_DECEL_STP_EOI      = 8,	// case   8: return "Decelerating or stopped by On-off-Error (EOI)"; break;
	MOTOR_STP_FE             = 9,	// case   9: return "Stopped after Finding Edge (FE)"; break;
	MOTOR_STP_HM_FI          = 10,	// case  10: return "Stopped after Homing/Find Index (HM/FI)"; break;
	MOTOR_STP_ABORT_INP_SEL  = 11,	// case  11: return "Stopped by Selective Abort input"; break;
	MOTOR_CONTR_RUN          = 50,	// case  50: return "Contour running"; break;
	MOTOR_CONTR_STOP         = 51,	// case  51: return "Contour Stop"; break;
	MOTOR_MC_TMO             = 99,	// case  99: return "MC Timeout"; break;
	MOTOR_VECT_RUN           = 100,	// case 100: return "Motors are running, vector sequence"; break;
	MOTOR_VECT_STOP          = 101  // case 101: return "Motors stopped at commanded vector"; break;
} stop_code_t;

class GalilDMC2xxxDrv : public Drv {

public:
	GalilDMC2xxxDrv(string target, string extra);
	~GalilDMC2xxxDrv();

        string  raw_io(string quest);
	int 	cnt_io(string quest);
	int 	chk_io(string ans);


	Link_ns::rvalues_t      refresh(string channel);

	bool 	stop(string channel);
	bool 	stop_no_brake(string channel) { return GalilDMC2xxxDrv::stop(channel); };

	bool 	speed(string channel, double v);
	double 	speed(string channel);

	bool 	acceleration(string channel, double a);
	double 	acceleration(string channel);
	bool 	deceleration(string channel, double a);
	double 	deceleration(string channel);

	bool 	forward(string channel, double v);
	bool 	backward(string channel, double v);

	bool    findedge(string channel, double v);
	bool    findindex(string channel, double v);

        double  mpos(string channel);
	bool    mpos(string channel, double p);       // mpos : motor position

	bool    xpos(VNOTU string channel, VNOTU double p)	{ return false; };       // xpos : motor position in encoder units

	bool    defmpos(string channel, double p);       // define motor position
	bool    defepos(string channel, double p);       // define hw encoder position

        bool    rpos(string channel, double p);       // rpos : motor relative position

	bool    brake(VNOTU string channel, VNOTU bool b) { return true;}
        int     hw_brake_conf(VNOTU string channel)		{ return 0; };

     	bool    motor_off(string channel);
        bool    motor_on(string channel);


	string 	stop_code2str(unsigned long stop_code);

	bool    mconf(VNOTU string channel, VNOTU mconf_t mc) { return true; };
	mconf_t mconf(VNOTU string channel) { mconf_t mc; return mc; };

	bool    econf(VNOTU string channel, VNOTU econf_t ec) { return true; };
	econf_t econf(VNOTU string channel) { econf_t ec; return ec; };

	bool	program_running();

        long  get_ls_tolerance(string channel)  { return (long)(-1); };

	//====
	int8_t  opmode(string channel) {return 0;};
	bool    opmode(string channel, int8_t op) {return false;};
	bool    Cia402HomingStart(int mode) {return false;};
	bool    Cia402HomingEndRestore(int mode) {return false;};
	Cia402HomingState_t Cia402HomingState() {return CIA402_HOME_INTERRUPTED;};
	Cia402HomingParams_t Cia402HomingParams() {Cia402HomingParams_t hp; return hp;};
	bool   Cia402HomingParams(Cia402HomingParams_t p) {return false;};


}; /* class GalilDMC2xxxDrv */


} /* namespace Controller_ns */

#endif /* _GALILDMC2XXXDRV_H_ */
