#ifndef _LINK_H_
#define _LINK_H_


#include <math.h>

#include <string>
#include "verbose.h"

#include "BriefLog.h"

using namespace std;

namespace Link_ns {

enum LinkType { NULL_CHAN, TCP_CHAN, TANGO_CHAN, SERIAL_CHAN, WEBREST_CHAN, SIMUL_CHAN};

#define NULL_CHAN_TYPE   'N'
#define TCP_CHAN_TYPE    'C'
#define TANGO_CHAN_TYPE  'A'
#define SERIAL_CHAN_TYPE 'S'
#define WEBREST_CHAN_TYPE 'R'
#define SIMUL_CHAN_TYPE 'X'

#define _LINKFLAG_NRBITS     8
#define _LINKFLAG_TYPEBITS   8
#define _LINKFLAG_NRMASK     ((1 << _LINKFLAG_NRBITS)-1)
#define _LINKFLAG_TYPEMASK   ((1 << _LINKFLAG_TYPEBITS)-1)
#define _LINKFLAG_NRSHIFT    0
#define _LINKFLAG_TYPESHIFT  (_LINKFLAG_NRSHIFT+_LINKFLAG_NRBITS)

#define _LINKFLAG(type,nr) \
           (((type) << _LINKFLAG_TYPESHIFT) | \
           ((nr)   << _LINKFLAG_NRSHIFT))

class LinkExcpt {
  private:
        int _ret;
        string _message;

  public:
        LinkExcpt(int r, string m): _ret(-1), _message("Unspecified") { _ret = r; _message = m; };

        string what() 	{ return "Communication Link Exception"; };
        string report()	{ return _message;};
        int    number()	{ return _ret;};
};


typedef enum {
	LINK_CLOSE,
	LINK_CONNECTED,
	LINK_REACTIVE,
	LINK_TIMEOUTED,
	LINK_UNREACTIVE,
	LINK_OTHEREXCEPT,
	LINK_GENERALEXCEPT
} LinkState_t;

const char * const LinkStateNames[] = {
	"LINK_CLOSE",
	"LINK_CONNECTED",
	"LINK_REACTIVE",
	"LINK_TIMEOUTED",
	"LINK_UNREACTIVE",
	"LINK_OTHEREXCEPT",
	"LINK_GENERALEXCEPT"
};

typedef enum {
	STANDSTILL,
	MOVING,
	ERROR,
	OFF,
	SERVO,
	SWCTL
} status_t;

const char * const StatusName[] = {
	"STANDSTILL",
	"MOVING",
	"ERROR",
	"OFF",
	"SERVO",
	"SWCTL"
};

// values for update (fast)
typedef struct {
	double motor;
	double encoder;
	double velocity;
	double analog;
	bool fls;
	bool rls;
	bool home;
	unsigned long stop_code;
	unsigned long CC_code;
	status_t status;
} uvalues_t;

// values for refresh (slow)
typedef struct {
	double speed;
	double acceleration;
	double deceleration;
} rvalues_t;

class Link
{

public:
        Link(LinkType t, std::string name);
        virtual ~Link() 	{if (bf) delete bf;};

        virtual string 		raw_io(string quest, bool setstate) 	= 0;
        string 			raw_io(string quest);

	virtual uvalues_t 	update(string channel) 	= 0;
	uvalues_t		uv();

	double   		updateDelay();

	virtual bool 		refresh() 		= 0;
	
        void verbose(int b);

        int verbose();

        double update_etime();

        int flag();
        bool flag(int v);

        bool prtLog();
        void prtLog(bool v);

	LinkState_t linkstate();
	string 		linkstatus();

protected:
	string _name;
	string _target;
        int _verbose;
	double _last_update_ms;
	int _flag;
	bool _prtLog;

	double _updateDelay;

	LinkState_t _linkstate;
	uvalues_t _uv;

        BriefLog_ns::BriefLog *bf;

//private:
        LinkType _channel_type;

}; /* class Link */


} /* namespace Link_ns */

#endif /* _LINK_H_*/
