#ifndef _MTYPES_H_
#define _MTYPES_H_

#ifdef __GNUC__
#define VARIABLE_IS_NOT_USED __attribute__ ((unused))
#else
#define VARIABLE_IS_NOT_USED
#endif

#define VNOTU VARIABLE_IS_NOT_USED


namespace Controller_ns {

typedef struct {
	bool	have_brake;
	bool	automatic;
	int	close_time;		//[ms] Close Brake Idle Time
	int 	shutdown_time;		//[ms] Shutdown Power Idle Time
	int	open_time;		//[ms] Open Brake Delay Time
	int 	start_time;		//[ms] Start Operation Delay Time
	int	pwm_freq;		//[Hz] PWM Frequency
	int	pwm_dutycycle;		//[%]  PWM percent
} brake_conf_t;

typedef struct {
	double mt;		// Motor type
				/*
					Galil:
					n =    1 Specifies Servo motor
					n =   -1 Specifies Servo motor with reversed polarity
					n =   -2 Specifies Step motor with active high step pulses
					n =    2 Specifies Step motor with active low step pulses
					n = -2.5 Specifies Step motor with reversed direction and active high step pulses
					n =  2.5 Specifies Step motor with reversed direction and active low step pulses
				
					N5-1:
					servo or stepper, possiamo codificare
					n =    1 Specifies Servo motor
					n =   -1 Specifies Servo motor with reversed polarity
					n =   -2 Specifies Step motor
					n = -2.5 Specifies Step motor with reversed direction 
					
				*/
        long dbw;               // Dead Band Width = GalilMotionAccuracy positioning accuracy. Written by the client in encoder units
        double scale;           // EncoderMotorRatio = encoder impulse / motor step. Written by the client
        double pc;              // Percent of the movement to be performed from 0.0 to 1.0
				/*
					N5-1:
					bck e' un int (essendo in encoder units!)
				*/
        double bck;             // Backlash. Encoder units
        long dir;               // Direction of the last positioning movement in backlash compensation
        long cp;                // Type/presence of encoder: 0 = no encoder
        long nbrtr;             // Number of retries before giving up/ending with movement
	int home_polarity;
	int hw_brake_conf;	// deriva dal valore memorizzato in Brake[.] nel microcodice
        double brake_ONOFF_extra_TMO; // ritardo (microsec) aggiuntivo (bloccante, ybrake()) dopo azione su brake
	double raw_brakeON_TMO; // ritardo (no. cicli FSM) dopo il quale, nella modalita' RAW, si attiva il freno.
} yconf_t;

typedef struct {
	double mt;		// Motor type
	bool mo;		// Motor off true/false
} piezoconf_t;

typedef union {
        yconf_t yc;
	piezoconf_t pc;
} mconf_t;

typedef struct {
        bool enabled;
        double m;
        double q;
} mOffOn_t;

typedef enum {
	ECONF_NONE,
	ECONF_CE,
	ECONF_MOFFON,
	ECONF_ALL
} econf_which_t;

typedef struct {
	econf_which_t which;
        int ce;
	mOffOn_t mOffOn;
} econf_t;

} /* Controller_ns */

#endif /* _MTYPES_H_ */
