#ifndef _N5DRV_H_
#define _N5DRV_H_

#include "Drv.h"
#include <stdio.h>
#include <math.h>
#include <algorithm>

#include <bitset> // per stampare in binario

#include "N5WebLink.h"

#undef __CIA402_INIT_STATE__

namespace Controller_ns {

// values for refresh
typedef struct {
        long target;            // Target position provided by the client in encoder units.
        long nbrtr;             // Number of retries before giving up/ending with movement
        double err;             // Positioning error. Encoder units
        double olderr;          // Previous positioning error. Encoder units
        unsigned long CC_code;
        yconf_t yc;
} n5rvalues_t;


typedef enum {
         STATE_UNKNOWN
        ,NOT_READY_TO_SWITCHON
        ,SWITCHON_DISABLED
        ,READY_TO_SWITCHON
        ,SWITCHON
        ,OPERATION_ENABLED
        ,QUICK_STOP
        ,FAULT_REACTION
        ,FAULT
} state_t;

const char * const statusNames[] = {
         "State unknown"
        ,"Not ready to switch on"
        ,"Switch on disabled"
        ,"Ready to switch on"
        ,"Switched on"
        ,"Operation enabled"
        ,"Quick stop active"
        ,"Fault reaction active"
        ,"Fault"
};

typedef enum {
         SHUTDOWN_CMD
        ,SWITCH_ON_CMD
        ,ENABLE_OPERATION_CMD
        ,DISABLE_OPERATION_CMD
        ,QUICK_STOP_CMD
        ,REMOVE_QUICK_STOP_CMD
        ,ENABLE_VOLTAGE_CMD
        ,DISABLE_VOLTAGE_CMD
        ,FAULT_RESET_CMD
        ,HALT_CMD
        ,CLEAR_HALT_CMD
} stateCmd_t;

const char * const stateCmdNames[] = {
         "Shutdown"
        ,"Switch on"
        ,"Enable operation"
        ,"Disable operation"
        ,"Quick STOP"
        ,"Remove quick stop"
        ,"Enable voltage"
        ,"Disable voltage"
        ,"Fault RESET"
        ,"Halt"
        ,"Clear halt"
};

typedef enum {
        n5cmd_go   = 5,
        n5cmd_stop = 6
} n5cmd_t;

typedef enum {
	 N5_ENC_NOTYPE = -1
	,N5_ENC_DIFFERENTIAL
	,N5_ENC_SINGLE
} n5enc_type_t;

typedef enum {
	 N5_ENC_NOV = -1
	,N5_ENC_5V
	,N5_ENC_24V
} n5enc_supply_t;

typedef struct {
	n5enc_type_t   type;
	n5enc_supply_t supply;
} n5enc_conf_t;

//-----------------------------------------------------------------------
//	N5 VERSION 2
typedef enum {
	 N5_ENC_NONE
	,N5_ENC_ID_SENSORLESS = 1
	,N5_ENC_ID_HALL
	,N5_ENC_ID_ENC1
} n5v2enc_id_t;

const char * const n5v2encidNames[] = {
	 "N5_ENC_NONE"
	,"N5_ENC_ID_SENSORLESS"
	,"N5_ENC_ID_HALL"
	,"N5_ENC_ID_ENC1"
};

typedef struct {
	n5v2enc_id_t commutation_sensor;
	n5v2enc_id_t position_sensor;
	n5v2enc_id_t speed_sensor;
} n5v2_feedback_map_t;

//-----------------------------------------------------------------------
class N5Drv : public Drv {

private:
	Link_ns::N5WebLink *_n5link;

        int8_t          _opmode;
        uint16_t        _statusword;
        uint16_t        _controlword;
        state_t         _state;

	Analog_t	_analog1, _analog2;

        uint16_t        statusWord();
        std::string     statusWord2STR(uint16_t sw);
        std::string     statusWordSTR();
	void		prt_statusExt();

        bool            SetPointAckd();

        uint16_t        controlWord();
        bool            controlWord(uint16_t cw);

        state_t         CiA402StateCmd(stateCmd_t cmd);
	bool 		CiA402FaultResetOnLimitSwitch();

	bool 		Cia402HomeMode35();

	Cia402HomingParams_t _Cia402HomingParams;
	bool 		Cia402_opEnable();
	bool 		Cia402_opDisable();


        state_t         state();

        std::string     status();

        double          powerVoltage();
        double          logicVoltage();
        double          pcbTemperature();

	int32_t		get_sw_limit_min();
	int32_t		get_sw_limit_max();
	uint32_t	get_ls_tolerance();

	int8_t		get_opmode();
	bool		set_opmode(int8_t op);

	bool		_isinit;

	bool    ce_econf(string channel, econf_t ec);
        econf_t ce_econf(string channel);

	AnalogType_t	get_analogtype(int ch);
	Analog_t	analog(int ch);

	long 		_nbrtr;

	int 		_hw_brake_conf;
	brake_conf_t	_brake_conf;

	string		hardware_version()	{
						  if (_n5link) return _n5link->hardware_version();
						  else return "";
						};
	string		software_version()	{
						  if (_n5link) return _n5link->software_version();
						  else return "";
						};
	unsigned long	_sw_vers;

	n5v2_feedback_map_t _n5v2_feedback_map;

public:
        long  get_ls_tolerance(string channel)	{ return (long)get_ls_tolerance(); };


	std::string     CiA402() { return status(); };
        state_t         CiA402(stateCmd_t cmd) { return CiA402StateCmd(cmd); };

	bool			Cia402HomingStart(int mode);
        bool                    Cia402HomingEndRestore(int mode);

	Cia402HomingState_t 	Cia402HomingState();
        Cia402HomingParams_t    Cia402HomingParams();
        bool                    Cia402HomingParams(Cia402HomingParams_t p);

	vector<string>		getErrors();
	string			getLastError();

#if 0
	void		describe();
#else
	std::vector<std::string>	describe();
#endif
	std::vector<std::string>	dumpConfig();

	std::string     OpMode2str(int8_t op);

	int8_t  opmode(string channel);    
        bool    opmode(string channel, int8_t op);

        bool    homing_method(int8_t h);
        int8_t  homing_method();

public:
        N5Drv(string target, string extra);
        ~N5Drv();

        string  raw_io(string quest);
        int     cnt_io(string quest);
        int     chk_io(string ans);


        Link_ns::rvalues_t      refresh(string channel);

        bool    stop(string channel);
        bool    stop_no_brake(string channel);

	bool 		get_rls();
	bool 		get_fls();
	bool 		get_home();

	bool		have_brake();
	brake_conf_t	brake_conf();

	uint32_t	pp_speed();
	bool		pp_speed(uint32_t v);
        bool    	speed(string channel, double v);
        double  	speed(string channel);

	
	uint32_t	pp_acceleration(); 
	bool		pp_acceleration(uint32_t v);
        bool    	acceleration(string channel, double a);
        double  	acceleration(string channel);

	uint32_t	pp_deceleration(); 
	bool		pp_deceleration(uint32_t v);
        bool    	deceleration(string channel, double a);
        double  	deceleration(string channel);

        bool    forward(string channel, double v);
        bool    backward(string channel, double v);

        bool    findedge(string channel, double v);
        bool    findindex(string channel, double v);

        double  mpos(string channel);
        bool    mpos(string channel, double p);       // mpos : motor position

        bool    xpos(VNOTU string channel, VNOTU double p);       // xpos : motor position in encoder units

        bool    defmpos(string channel, double p);       // define motor position
        bool    defepos(string channel, double p);       // define hw encoder position

        bool    rpos(string channel, double p);       // rpos : motor relative position

        bool    brake(VNOTU string channel, VNOTU bool b);
        int     hw_brake_conf(VNOTU string channel);

        bool    motor_off(string channel);
        bool    motor_on(string channel);


        string  stop_code2str(unsigned long stop_code);
	
	bool 	is_bldc();
	bool 	is_bldc(bool bldc);

	n5enc_conf_t enc_conf();

	bool	pp_polarity();
	bool 	pp_polarity(bool pos);

        bool    mconf(VNOTU string channel, VNOTU mconf_t mc);
        mconf_t mconf(VNOTU string channel);

        bool    econf(VNOTU string channel, VNOTU econf_t ec);
        econf_t econf(VNOTU string channel);

        bool    program_running();

	bool	get_cinit();
	bool 	cinit(string channel);         // true = INIT / false = NO INIT
        bool 	cinit(string channel, bool b); // true = INIT / false = DEINIT

	Analog_t	analog1();
	Analog_t	analog2();

	//-----------------------------------------------------------------------
	//ifdef _WITH_N5MICROCODE_
	#if 1

        n5rvalues_t n5refresh(VNOTU string channel);

private:
        long n5target(VNOTU string channel);           	// depends on Cp, Scale....            i.e. in encoder units!!!!
        bool n5target(VNOTU string channel, long p); 		// p: position, nbrtr: number of retries

        bool n5nbrtr(VNOTU string channel, int nbrtr);

        unsigned long   n5CC_code(VNOTU string channel);
        unsigned long   n5CC_code(VNOTU string channel, bool do_update);
        bool            n5CC_code(VNOTU string channel, unsigned long CC_code);

        bool n5CC_cmd(VNOTU string channel, n5cmd_t cmd);

        bool n5go(VNOTU string channel, long p, int nbrtr); 	// p: position, nbrtr: number of retries

        bool n5conf(VNOTU string channel, yconf_t yc);

	bool n5stop(VNOTU string channel);


	#endif /* _WITH_N5MICROCODE_ */
	//-----------------------------------------------------------------------
	//
	//	N5: V E R S I O N 	2
	

	n5v2_feedback_map_t n5v2_feedback_map();
	//-----------------------------------------------------------------------


}; /* class N5Drv */


} /* namespace Controller_ns */


#endif /* _N5DRV_H_ */
