#ifndef _N5OBJECTS_H_
#define _N5OBJECTS_H_

#include <unistd.h>

namespace nanotec {

typedef enum {
 BOOLEAN = 1,
 INTEGER8,
 INTEGER16,
 INTEGER32,
 UNSIGNED8,
 UNSIGNED16,
 UNSIGNED32,
 REAL32,
 VISIBLE_STRING,
 OCTECT_STRING,
 UNICODE_STRING
} objtype_t;

//==========================================================
//
//READ OBJECTS

#define POWER_VOLTAGE 		"4014/01" /* INTEGER32 */
#define LOGIC_VOLTAGE 		"4014/02" /* INTEGER32 */
#define PCB_TEMPERATURE 	"4014/03" /* INTEGER32 */

#define HARDWARE_VERSION	"1009/00" /* STRING */
#define SOFTWARE_VERSION	"100A/00" /* STRING */

//230Fh Uptime Seconds
#define UPTIME			"230F/00" /* UNSIGNED32 */ /* seconds max: 136 anni... */

#define STATUSWORD 		"6041/00" /* UNSIGNED16 */

#define OPMODE_RD 		"6061/00" /* INTEGER8 */

#define MOTOR_CURRENT_I_D	"2039/01" /* INTEGER32 */
#define MOTOR_CURRENT_I_Q	"2039/02" /* INTEGER32 */
#define MOTOR_CURRENT_I_A	"2039/03" /* INTEGER32 */
#define MOTOR_CURRENT_I_B	"2039/04" /* INTEGER32 */


#define ENCODER_ACT_INTERNAL	"6063/00" /* INTEGER32 */ /* encoder */
#define POSITION_DEMANDED	"6062/00" /* INTEGER32 */ /*  motor  */

// 6044h Vl Velocity Actual Value
#define VL_VELOCITY_ACTUAL	"6044/00" /* INTEGER16 */

//60FDh Digital Inputs
#define DIGITAL_INPUT		"60FD/00" /* UNSIGNED32 */ /** b0 = NLS/RLS | b1 = PLS/FLS | b2 = HS/HOME **/

//3220h Analog Inputs
#define ANALOG1_INPUT		"3220/01" /* INTEGER16 */
#define ANALOG2_INPUT		"3220/02" /* INTEGER16 */

//3320h Read Analogue Input
#define ANALOG_INPUT_1		"3320/01" /* INTEGER32 */
#define ANALOG_INPUT_2		"3320/02" /* INTEGER32 */

//2301h NanoJ Status
#define NANOJ_STATUS		"2301/00" /* UNSIGNED32 */

//603Fh Error Code
#define LAST_ERROR_CODE		"603F/00" /* UNSIGNED16 */

//==========================================================
//
// READ-WRITE OBJECTS

#define CONTROLWORD 		"6040/00" /* UNSIGNED16 */

//3202h Motor Drive Submode Select
#define MOTOR_DRIVE_SUBMODE	"3202/00" /* UNSIGNED32 */

//607Eh Polarity
#define MOTOR_POLARITY		"607E/00" /* UNSIGNED8 */

#define POLE_PAIR_COUNT		"2030/00" /* UNSIGNED32 */

//
//2031h STEPPER:: Rated current (bipolar) in mA
//2031h BLDC::    Maximum Current (mA, RMS)
#define MOTOR_CURRENT		"2031/00" /* UNSIGNED32 */

//203Bh I2t Parameters
#define MOTOR_NOMINAL_CURRENT	"203B/01" /* UNSIGNED32 */
#define MOTOR_PEAK_CURRENT_TIME	"203B/02" /* UNSIGNED32 */ //Maximum Duration (ms) Of Peak Current
#define MOTOR_CURRENT_THRESHOLD	"203B/03" /* UNSIGNED32 */ //mA
#define MOTOR_CURRENT_CALCVALUE	"203B/04" /* UNSIGNED32 */
#define MOTOR_LIMITED_CURRENT	"203B/05" /* UNSIGNED32 */ //momentary current as root mean square set by I2t
#define MOTOR_I2T_STATUS	"203B/06" /* UNSIGNED32 */ //"0", I2t is deactivated; if the value is "1", I2t is activated

//0x20B0/01 motor resistance, /02 Inductance
#define MOTOR_RESISTANCE 	"20B0/01" /* UNSIGNED32 */	/* ieee-754 32 bit float */
#define MOTOR_INDUCTANCE 	"20B0/02" /* UNSIGNED32 */	/* ieee-754 32 bit float */
#define MOTOR_FLUX 		"20B0/03" /* UNSIGNED32 */	/* ieee-754 32 bit float */

//2059h Encoder Configuration
// bit0: "0" = the supply voltage for the encoder is set to 5 V , "1" = 24V
// bit1: "0" = differential encoder; "1" = single-ended encoder
#define ENCODER_CONFIGURATION	"2059/00" /* UNSIGNED32 */

// 3209:01  Motor drive sensor source closed loop - commutation
#define CLOOP_SOURCE_SENSOR	"3209/01" /* INTEGER32 */ // -1 (?) sensorless
							  // 1 encoder (+index)
							  // 2 hall
							  // 3 hell + encoder

//2050h Encoder Alignment
#define ENCODER_ALIGNMENT	"2050/00" /* INTEGER32 */

//C010    UNSIGNED32              Motion Debug Entries
#define MOTION_DEBUG_ENTRIES	"C010/01" /* UNSIGNED32 */

//607Dh Software Position Limit
#define SW_LIMIT_MIN		"607D/01" /* INTEGER32 */
#define SW_LIMIT_MAX		"607D/02" /* INTEGER32 */

//3240h Digital Inputs Control
//01 Special Function Enable UNSIGNED32		0=DISABLED 1=ENABLED
						// Bit 0: Negative limit switch
						// Bit 1: Positive limit switch
						// Bit 2: Home switch
#define INPUT_SPECIAL_FUNC_ENABLE "3240/01" /* UNSIGNED32 */

//02 Function Inverted UNSIGNED32 		0=DO_NOT_INVERT	1=INVERT
						// Bit 0: Input 1 / -Limit
						// Bit 1: Input 2 / +Limit
						// Bit 2: Input 3 / Home
						// Bit N: Input N+1
						// Bit 1: Input 2
#define INPUT_INVERT		  "3240/02" /* UNSIGNED32 */
					
//03 Force Enable UNSIGNED32			0=DO_NOT_FORCE/GET_REAL 1=FORCE
#define INPUT_FORCE_ENABLE	  "3240/03" /* UNSIGNED32 */

//04 Force Value UNSIGNED32
#define INPUT_FORCE_VALUE	  "3240/04" /* UNSIGNED32 */

//05 Raw Value UNSIGNED32		UNMODIFIED/REAL input VALUE
#define INPUT_RAW_VALUE		  "3240/05" /* UNSIGNED32 */

//06 Input Range Select UNSIGNED32		0=5V 1=24V
#define INPUT_RANGE		  "3240/06" /* UNSIGNED32 */

//07 Differential Select UNSIGNED32	??????????????????????
#define INPUT_DIFFERENTIAL	  "3240/07" /* UNSIGNED32 */

//08 Routing Enable UNSIGNED32		1=ENABLED
#define INPUT_ROUTING_ENABLE	  "3240/08" /* UNSIGNED32 */



//2056h Limit Switch Tolerance Band
#define LS_TOLERANCE		"2056/00" /* UNSIGNED32 */


//608Fh Position Encoder Resolution
#define VIRT_ENC_INCREMENTS	"608F/01" /* UNSIGNED32 */ 	/* increments in VIRT_MOT_REVOLUTIONS */
#define VIRT_MOT_REVOLUTIONS	"608F/02" /* UNSIGNED32 */ 

//60FEh Digital Outputs
#define DIGITAL_OUTPUTS		"60FE/01" /* UNSIGNED32 */

//3221h Analogue Inputs Control
#define ANALOG_INPUT_CONTROL	"3221/00" /* INTEGER32 */ 	// b0 = 0 ana1=volt 
								// b0 = 1 ana1=mA

//6060h Modes Of Operation
#define OP_MODE			"6060/00" /* INTEGER8 */

//6098h Homing Method
#define HOMING_METHOD		"6098/00" /* INTEGER8 */

//607Ch Home Offset
#define HOME_OFFSET		"607C/00" /* INTEGER32 */

//6099h Homing Speed
#define HOME_SPEED_SWITCH	"6099/01" /* UNSIGNED32 */ // search speed for switch
#define HOME_SPEED_REFERENCE	"6099/02" /* UNSIGNED32 */ // search speed for reference

//609Ah Homing Acceleration
#define HOME_ACCELERATION	"609A/00" /* UNSIGNED32 */ 

//203Ah Homing On Block Configuration
#define HOME_BLOCK_MIN_CURR	"203A/01" /* INTEGER32 */ // Minimum Current For Block Detection
#define HOME_BLOCK_DECT_TIME	"203A/02" /* INTEGER32 */ // Block Detection Time

//2052h Encoder Resolution
#define ENCODER_RESOLUTION	"2052/00" /* INTEGER32 */ // A negative value means that the encoder 
							  // is driven in the opposite direction of the motor

//3221h Analogue Inputs Control
#define ANALOG_IN_CONTROL	"3221/00" /* INTEGER32 */ // bit0 ch1: 0=volt 1=mA
							  // bit1 ch2: 0=volt 1=mA

//2038h Brake Controller Timing
//
#define BRAKE_CLOSE_TIME	"2038/01" /* UNSIGNED32 */	// Close Brake Idle Time
#define BRAKE_SHUTDOWN_TIME	"2038/02" /* UNSIGNED32 */	// Shutdown Power Idle Time
#define BRAKE_OPEN_TIME		"2038/03" /* UNSIGNED32 */	// Open Brake Delay Time
#define BRAKE_START_TIME	"2038/04" /* UNSIGNED32 */	// Start Operation Delay Time
#define BRAKE_PWM_FREQ		"2038/05" /* UNSIGNED32 */	// PWM Frequency
#define BRAKE_PWM_DUTYCYCLE	"2038/06" /* UNSIGNED32 */	// PWM Duty Cycle

//
//3210h Motor Drive Parameter Set
//
// CLOSED LOOP
//
//Position Loop, Proportional Gain (closed Loop)
#define CLOOP_POSITION_P	"3210/01" /* UNSIGNED32 */
//Position Loop, Integral Gain (closed Loop)
#define CLOOP_POSITION_I	"3210/02" /* UNSIGNED32 */
//Velocity Loop, Proportional Gain (closed Loop)
#define CLOOP_VELOCITY_P	"3210/03" /* UNSIGNED32 */
//Velocity Loop, Integral Gain (closed Loop)
#define CLOOP_VELOCITY_I	"3210/04" /* UNSIGNED32 */
//Flux Current Loop, Proportional Gain (closed Loop)
#define CLOOP_FLUX_CURRENT_P	"3210/05" /* UNSIGNED32 */
//Flux Current Loop, Integral Gain (closed Loop)
#define CLOOP_FLUX_CURRENT_I	"3210/06" /* UNSIGNED32 */
//Torque Current Loop, Proportional Gain (closed Loop)
#define CLOOP_TORQUE_CURRENT_P	"3210/07" /* UNSIGNED32 */
//Torque Current Loop, Integral Gain (closed Loop)
#define CLOOP_TORQUE_CURRENT_I	"3210/08" /* UNSIGNED32 */
//
// OPEN LOOP
//
//Torque Current Loop, Proportional Gain (open Loop)
#define OLOOP_TORQUE_CURRENT_P	"3210/09" /* UNSIGNED32 */
//Torque Current Loop, Integral Gain (open Loop)
#define OLOOP_TORQUE_CURRENT_I	"3210/0A" /* UNSIGNED32 */

//320Ah Motor Drive Sensor Display Open Loop
#define OLOOP_DISPLAY_VELOCITY	"320A/03" /* INTEGER32 */ //| X =  6044
#define OLOOP_DISPLAY_POSITION	"320A/04" /* INTEGER32 */ //| X =  6064
							  /*|
							    | Value = "-1": The internally calculated set value is entered in object X
							    | Value = "0" : The value is kept at 0
							    | Value = "1" : The encoder value is entered in object X
							  */

//320Bh Motor Drive Sensor Display Closed Loop
#define CLOOP_DISPLAY_VELOCITY	"320B/03" /* INTEGER32 */ //| X =  6044
#define CLOOP_DISPLAY_POSITION	"320B/04" /* INTEGER32 */ //| X =  6064

//++++++++++++++ USER UNITS ++++++++++++++++++++
//   SPEED
//				n_speed = USER_VELOCITY_NUMERATOR/USER_VELOCITY_DENOMINATOR
//				Internal_value[rps] = n_speed x input_value
//2061h Velocity Numerator
//2062h Velocity Denominator
#define USER_VELOCITY_NUMERATOR   "2061/00" /* UNSIGNED32 */
#define USER_VELOCITY_DENOMINATOR "2062/00" /* UNSIGNED32 */
//
// ACCEL/DECEL
//				n_accel = USER_ACCEL_NUMERATOR/USER_ACCEL_DENOMINATOR
//				Internal_value[rps2] = n_accel x input_value	rps2 = rev/s^2
//2063h Acceleration Numerator
//2064h Acceleration Denominator
#define USER_ACCEL_NUMERATOR   "2063/00" /* UNSIGNED32 */
#define USER_ACCEL_DENOMINATOR "2064/00" /* UNSIGNED32 */


//++++++++++++++

// -
// TARGET POSITIONS
// -
// 607Ah Target Position
// PP,  Profile Position
// CSP, Cyclic Synchronous Position
#define PP_TARGET_POSITION	"607A/00" /* INTEGER32 */
#define CSP_TARGET_POSITION	"607A/00" /* INTEGER32 */

// -
// TARGET VELOCITIES
// -
// PP, Profile Position
#define PP_VELOCITY		"6081/00" /* UNSIGNED32 */

// Velocity
#define VL_VELOCITY		"6042/00" /* INTEGER16 */

// PV, Profile Velocity
// CSV, Cyclic Synchronous Velocity
#define PV_VELOCITY		"60FF/00" /* INTEGER32 */
#define CSV_VELOCITY		"60FF/00" /* INTEGER32 */

// Homing
#define HM_SPEED_4_SWITCH	"6099/01" /* UNSIGNED32 */
#define HM_SPEED_4_ZERO		"6099/02" /* UNSIGNED32 */

// -
// ACCELERATIONS
// -
// PP, Profile Position
// PV, Profile Velocity
#define PP_ACCELERATION		"6083/00" /* UNSIGNED32 */
#define PV_ACCELERATION		"6083/00" /* UNSIGNED32 */

// Velocity
#define VL_ACCELERATION_DSPEED	"6048/01" /* UNSIGNED32 */
#define VL_ACCELERATION_DTIME	"6048/02" /* UNSIGNED16 */

// Homing
#define HM_ACCEL_DECEL		"609A/00" /* UNSIGNED32 */

// -
// ACCELERATIONS
// -
// PP, Profile Position
// PV, Profile Velocity
#define PP_DECELERATION		"6084/00" /* UNSIGNED32 */
#define PV_DECELERATION		"6084/00" /* UNSIGNED32 */

// Velocity
#define VL_DECELERATION_DSPEED	"6049/01" /* UNSIGNED32 */
#define VL_DECELERATION_DTIME	"6049/02" /* UNSIGNED16 */


//==========================================================
//
//             NANOJ
//
// READ-WRITE OBJECTS
// 2500h NanoJ Outputs 01-20
#define NANOJ_OUT_01		"2500/01" /* INTEGER32 */

// 2400h NanoJ Inputs 
#define NANOJ_IN_01		"2400/01" /* INTEGER32 */
#define ENC_OFFSET_ON_INIT	NANOJ_IN_01
#define NANOJ_IN_02		"2400/02" /* INTEGER32 */
#define AXISINIT_STATE		NANOJ_IN_02	// 0 = NOT / 1 = INITIALIZED

// 2410h NanoJ Init Parameters
#define NANOJ_INIT_01		"2410/01" /* INTEGER32 */
#define MOTORTEMPDIGITTHRES	NANOJ_INIT_01
#define NANOJ_INIT_02		"2410/02" /* INTEGER32 */
#define MOTORBRAKE		NANOJ_INIT_02   // 0 = NO_BRAKE / 1 = real BRAKE_PRESENT

//
// *^*^*^*^* M i c r o c o d i c e *^*^*^*^*
//

#define MicroCode_CC			"2500/05"
#define MicroCode_Pos			"2500/06"
#define MicroCode_Dir			"2500/07"
#define MicroCode_Pc			"2500/08" /* float on INTEGER32 */
#define MicroCode_Bck			"2500/09"
#define MicroCode_NbRtr			"2500/0A"
#define MicroCode_Dbw			"2500/0B"
#define MicroCode_Scale			"2500/0C" /* float on INTEGER32 */
#define MicroCode_Cp			"2500/0D"

#define MicroCode_Err			"2400/05"
#define MicroCode_prevErr		"2400/06"

//=======================================================================
//=======================================================================
//
//
//	N5: 	V E R S I O N >= 2
//

#define SW_VERS_2 2000

// 3203h Feedback Selection
#define FEEDBACK_SELECTION		"3203" // 01, 02, 03 se ci sono 3 encoders...

//2052h Encoder Resolution NON e' indicato esplicitamente nel manuale,
//                         MA sembra contenere il valore di 60E6/03
//                         60E6h Additional Position Encoder Resolution - Encoder Increments
//2050h Encoder Alignment NON e' indicato esplicitamente nel manuale,
//                         MA sembra contenere il valore di 33A0/02
//                         33A0h Feedback Incremental A/B/I 1
//608Fh Position Encoder Resolution se:
//			OPEN   LOOP: e' 60E6/01
//			CLOSED LOOP: e' 60E6/x dove x e' scelto da 3203

//6096h Velocity Factor
#define V2_USER_VELOCITY_NUMERATOR   "6096/01" /* UNSIGNED32 */
#define V2_USER_VELOCITY_DENOMINATOR "6096/02" /* UNSIGNED32 */

//6097h Acceleration Factor
#define V2_USER_ACCEL_NUMERATOR   "6097/01" /* UNSIGNED32 */
#define V2_USER_ACCEL_DENOMINATOR "6097/02" /* UNSIGNED32 */

//60E4h Additional Position Actual Value
#define V2_ENCODER_ACT		  "60E4/03" /* INTEGER32 */ /* encoder */

//606Ch Velocity Actual Value
#define V2_VELOCITY_ACT		  "606C/00" /* INTEGER32 */


} // namespace

#endif /* _N5OBJECTS_H_ */
