#ifndef _N5WEBLINK_H_
#define _N5WEBLINK_H_

#include "Link.h"
#include <curl/curl.h>
#include <string>
#include <algorithm>

#include "N5Objects.h"

namespace Link_ns {

struct data {
  char trace_ascii; /* 1 or 0 */
};

typedef struct {
        std::string str;
        double sec;
} objans_t;


class N5WebLink : public Link 
{
private:
        CURL *curl;
        std::string     readBuffer;
        std::string     _baseurl;

        double          _epos_sw_offset;         // il Nanotec non permette di modificare il valore dell'encoder.... quindi lo si fa' in sw.

        int32_t         get_enc_offset_on_init();

	bool		_with_microcode;

        string          _software_version_str;
        unsigned long   _sw_vers;


public:
	N5WebLink(string target);
	~N5WebLink();

	bool 	with_microcode(){return _with_microcode;};
	void 	with_microcode(bool v){_with_microcode = v;};

	string raw_io(string quest, bool setstate);

        uvalues_t update(string channel);
	bool  refresh();

        bool    defepos(double p);       // define hw encoder position
	bool	homing_method(int8_t h);
	int8_t  homing_method();

	objans_t readobj(std::string obj);
        bool writeobj(std::string obj, std::string value);

        string readSTR(std::string obj, double* t);
        string readSTR(std::string obj);

        int32_t readI32(std::string obj, double* t);
        int32_t readI32(std::string obj);

        uint16_t readU16(std::string obj, double* t);
        uint16_t readU16(std::string obj);

        int16_t  readI16(std::string obj, double* t);
        int16_t  readI16(std::string obj);

        int8_t readI8(std::string obj, double* t);
        int8_t readI8(std::string obj);


        uint8_t readU8(std::string obj, double* t);
        uint8_t readU8(std::string obj);

        uint32_t readU32(std::string obj, double* t);
        uint32_t readU32(std::string obj);

        bool writeU8(std::string obj, uint8_t value);
        bool writeI8(std::string obj, int8_t value);
        bool writeI32(std::string obj, int32_t value);
        bool writeU32(std::string obj, uint32_t value);
        bool writeU16(std::string obj, uint16_t value);

        string          hardware_version();
        string          software_version();

        unsigned long   sw_vers()	{ return _sw_vers; };

private:
        static size_t cb(void *contents, size_t size, size_t nmemb, void *userp);

        static void dump(const char *text, FILE *stream, unsigned char *ptr, size_t size, char nohex);
        static int my_trace(CURL *handle, curl_infotype type, char *data, size_t size, void *userp);


}; /* class N5WebLink */


} /* namespace Link_ns */
#endif /* _N5WEBLINK_H_ */
