#ifndef _SIMULDRV_H_
#define _SIMULDRV_H_

#include "SimulLink.h"
#include "Drv.h"
#include <stdio.h>
#include <math.h>
#include <map>
#include <string>
#include <vector>

#include <pthread.h>
#include <cstdlib>
#include <unistd.h>

namespace Controller_ns {

class SimulDrv : public Drv {

private:
	Link_ns::SimulLink *_simullink;
	SimulThread_ns::SimulThread *simulthreadp;

	mconf_t _mc;

public:
	SimulDrv(string target, string extra);
	~SimulDrv();

	bool  	speed(string channel, double v)	{ return simulthreadp->mspeed((int32_t)v); };
	double  speed(string channel) { return (double)simulthreadp->mspeed(); };

	bool  	acceleration(string channel, double a) { return simulthreadp->maccel((int32_t)a); };
	double  acceleration(string channel) { return (double)simulthreadp->maccel(); };

	bool  	deceleration(string channel, double a) { return simulthreadp->mdecel((int32_t)a); };
	double  deceleration(string channel) { return (double)simulthreadp->mdecel(); };

	bool 	forward(string channel, double v);		// v = speed
	bool    backward(string channel, double v);		// v = speed

	bool    stop(string channel);

	double  mpos(string channel);
	bool    mpos(string channel, double p);		       // mpos : motor position
	bool    xpos(string channel, double p);		       // xpos : motor position in encoder units

	bool    rpos(string channel, double p);		       // rpos : motor relative position

	bool    motor_off(string channel);
        bool    motor_on(string channel);

	bool    mconf(string channel, mconf_t mc);
	mconf_t mconf(string channel);


	bool    econf(string channel, econf_t ec)	{ return true; };
	econf_t econf(string channel)			{ econf_t ec; return ec; };

	bool    defmpos(string channel, double p);       // define motor position
        bool    defepos(string channel, double p);       // define encoder position


	bool    program_running();

        bool    sinit(string channel);         // true = INIT / false = NO INIT
        bool    sinit(string channel, bool b); // true = INIT / false = DEINIT

	Link_ns::rvalues_t      refresh(string channel);

	//================================
	int8_t  		opmode(string channel)  { int8_t v = 0; return v; };
	bool    		opmode(string channel, int8_t op) { return false; };
	bool  			stop_no_brake(string channel) { return false; };
	bool    		findedge(string channel, double v) { return false; };
	bool    		findindex(string channel, double v) { return false; };
	bool  			brake(string channel, bool b) { return false; };
	int     		hw_brake_conf(string channel) { return 0; };
	string  		stop_code2str(unsigned long stop_code) { return "NONE"; };
	long  			get_ls_tolerance(string channel) { return -1; };

	bool                    Cia402HomingStart(int mode) { return false; };
	bool                    Cia402HomingEndRestore(int mode) { return false; };
	Cia402HomingState_t     Cia402HomingState() { Cia402HomingState_t hs = CIA402_HOME_INTERRUPTED; return hs; };
	Cia402HomingParams_t 	Cia402HomingParams() { Cia402HomingParams_t hp; return hp; };
	bool   			Cia402HomingParams(Cia402HomingParams_t p) { return false; };

}; /* class SimulDrv */


} /* namespace Controller_ns */

#endif /* _SIMULDRV_H_ */
