#ifndef _SIMULTHREAD_H_
#define _SIMULTHREAD_H_

#include <stdio.h>
#include <math.h>
#include <map>
#include <string>
#include <vector>

#include <pthread.h>
#include <cstdlib>
#include <unistd.h>

namespace SimulThread_ns {

class SimulThread {
private:

	pthread_attr_t attr;
	pthread_t thread_id;

	void loop();

	int _cycle_time; // ms
	bool _go = true;

	int64_t _mtarget;	//step
	int64_t _mpos;	//step
	double _innermpos;	//fraction
	bool _stop;

	int32_t _mspeed;		//step/s
	double _innermspeed;

	uint32_t _maccel;
	uint32_t _mdecel;

        bool    _isinit;
	bool	_running;
	bool	_ismotor_on;

public:

	static void* loopWrapper (void* thisPtr);
	SimulThread(int cycle_time = 100);
	~SimulThread();

	void 	kill() { _go = false; };

	bool 	defmpos(int64_t p) { /* cout << __func__ << " " << p << endl;*/  _mpos = p; _innermpos = (double)p; _mtarget = p; return true; };
	bool 	mpos(int64_t p);
	int64_t mpos() { return _mpos; };

	bool 	stop() { _stop = true; return true; };

	bool  	mspeed(int32_t v)	{ _mspeed = v; return true; };
	int32_t  mspeed() { return _mspeed; };

	bool  	maccel(int32_t a) { if (a<0) return false; _maccel = a; return true; };
	int32_t  maccel() { return _maccel; };

	bool  	mdecel(int32_t a) { if (a<0) return false; _mdecel = a; return true; };
	int32_t  mdecel() { return _mdecel; };

	bool    isinit() { return _isinit; };
	void 	isinit(bool b) { _isinit = b; };

	bool	isrunning() { return _running; };
	bool	ismotor_on() { return _ismotor_on; };

	bool 	motor_on() { _ismotor_on = true; return true; };	
	bool 	motor_off() { _ismotor_on = false; return true; };	


}; /* class SimulThread  */

} /* namespace SimulThread_ns */

#endif /* _SIMULTHREAD_H_ */
