#ifndef _YAMSDRV_H_
#define _YAMSDRV_H_

#include "GalilDMC2xxxDrv.h"
#include <stdio.h>
#include <math.h>
#include <map>
#include <string>
#include <vector>

#define CHECK_BIT(var,pos) ( ((var) & (1<<(pos)))==0?false:true )
#define BIT_YGO_POS_CMD   0	   //if ( CC_code & 0x1     ) cc.append("[POS]: Positioning Cmd. ");
#define BIT_YGO_PROGRESS  8	   //if ( CC_code & 0x100   ) cc.append("[InPrgrss]: Movement in progress. ");
#define BIT_YGO_END_OK    9        //if ( CC_code & 0x200   ) cc.append("[OK]: Positioning ended without error. ");
#define BIT_YGO_END_NOK  10        //if ( CC_code & 0x400   ) cc.append("[NOK]: Positioning ended with error. ");



namespace Controller_ns {

// values for refresh
typedef struct {
        long target;		// Target position provided by the client in encoder units.
        long nbrtr;		// Number of retries before giving up/ending with movement
        double err;		// Positioning error. Encoder units
        double olderr;		// Previous positioning error. Encoder units
	unsigned long CC_code;
	yconf_t yc;
} yrvalues_t;

typedef enum {
	ycmd_go	  = 5,
	ycmd_stop = 6
} ycmd_t;

typedef enum {
	GALIL,
	YAMS
} yidentity_t;

class YamsDrv : public GalilDMC2xxxDrv {

private:
	int MOT_BRAKE[8];
	std::map<std::string, int> _idxchannel;

	yidentity_t _yidentity;
	
	std::vector<long> _nbrtr;

	int _hw_brake_conf[8];

	bool    ce_econf(string channel, econf_t ec);
	econf_t ce_econf(string channel);

	bool    moffon_econf(string channel, econf_t ec);
	econf_t moffon_econf(string channel);

        double _brake_ONOFF_extra_TMO[8]; // ritardo (microsec) aggiuntivo (bloccante, ybrake()) dopo azione su brake
        double _raw_brakeON_TMO[8]; // ritardo (sec) dopo il quale, nella modalita' RAW, si attiva il freno.


public:
	YamsDrv(string target, string extra);
	//~YamsDrv() { ~GalilDMC2xxxDrv(); };
	//~YamsDrv();

        string  CC_code2str(unsigned long CC_code);

	bool 	forward(string channel, double v);		// v = speed
	bool    backward(string channel, double v);		// v = speed

	bool    findedge(string channel, double v);

	bool    stop(string channel);

	bool    mpos(string channel, double p);		       // mpos : motor position
	bool    xpos(string channel, double p);		       // xpos : motor position in encoder units

	bool    rpos(string channel, double p);		       // rpos : motor relative position

	bool 	brake(string channel, bool b);
	int     hw_brake_conf(string channel);
	bool    motor_off(string channel);
        bool    motor_on(string channel);


	bool yinit(string channel);	    // true = INIT / false = NO INIT
	bool yinit(string channel, bool b); // true = INIT / false = DEINIT

	long ytarget(string channel);		// depends on Cp, Scale....            i.e. in encoder units!!!!
	bool ytarget(string channel, long p); // p: position, nbrtr: number of retries

	bool ystop(string channel);
	bool gstop(string channel);
	bool ygo(string channel, long p, int nbrtr); // p: position, nbrtr: number of retries
	bool yrgo(string channel, long p, int nbrtr); // p: relative position, nbrtr: number of retries

	bool ybrake(string channel);		// true = brake-ON; false = brake-OFF
	bool ybrake(string channel, bool b);

	bool ynbrtr(string channel, int nbrtr);

	bool ybconf(string channel, double etmo); // configura _brake_ONOFF_extra_TMO
	bool yrawbconf(string channel, double rtmo); // configura _raw_brakeON_TMO


	bool yidentity(yidentity_t t) { _yidentity = t; return true; };
	yidentity_t yidentity() { return _yidentity; };

	bool yconf(string channel, yconf_t yc);

	bool ycp(string channel, int e);
	int ycp(string channel);	// encodertype: 
			// 0 = no encoder
			// 1 = incremental
			// 2 = absolute
			// 3 = semiabsolute
			// 4 = analog

	unsigned long 	yCC_code(string channel);
	unsigned long 	yCC_code(string channel, bool do_update);
	bool 		yCC_code(string channel, unsigned long CC_code);

	bool		yCC_cmd(string channel, unsigned char cmd);
	unsigned long	yCC_status(string channel);

	yrvalues_t yrefresh(string channel);

	bool    mconf(string channel, mconf_t mc);
	mconf_t mconf(string channel);


	bool    econf(string channel, econf_t ec);
	econf_t econf(string channel);

	bool    program_running();

        long  get_ls_tolerance(string channel)  { return (long)(-1); };

}; /* class YamsDrv */


} /* namespace Controller_ns */

#endif /* _YAMSDRV_H_ */
