#include "Axis.h"

#include <stdlib.h>
#include <string>
#include <algorithm>
#include <iostream>

namespace Axis_ns {

void  Axis::verbose(int b) {
		#ifdef _DBG_VERBOSE
			cout << "Axis verbose(" << b << ")" << endl;
		#endif
		_verbose = b;
}

void  Axis::lnk_verbose(int b){
		#ifdef _DBG_VERBOSE
			cout << "Axis.lnk_verbose() _controller = " << hex << _controller << dec << endl;
		#endif
		if (_controller){
			#ifdef _DBG_VERBOSE
				cout << "Axis.lnk_verbose()" << endl;
			#endif
			_controller->lnk_verbose(b);
		} else {
			cout << "Axis.lnk_verbose() NO CONTROLLER" << endl;
		}
}

int   Axis::lnk_verbose(){
		return _controller->lnk_verbose();
}

void  Axis::drv_verbose(int b){
		#ifdef _DBG_VERBOSE
			cout << "Axis.drv_verbose()" << endl;
		#endif
		_controller->drv_verbose(b);
}

int   Axis::drv_verbose(){
		return _controller->drv_verbose();
}

/*===============================
*
*
*
*===============================*/
Axis::Axis(string name, string channel, Controller_ns::Controller *controller, bool prtLog): Motor(name, channel, controller), Encoder(name, channel, controller) {
	std::string obj("Axis::Axis()");
	_verbose = 0;

	_name = name;
	_channel = channel;
	_controller = controller;

	_prtLog = prtLog;

	_target = nan("");

	std::stringstream ss("");
	ss << "... " << _name << " ch=" << channel << " ctrl=" << controller->target();
	#ifdef _DBG_CSTOR
		prtverbose(0, obj, ss.str());
	#else
		prtverbose(1, obj, ss.str());
	#endif

	ss.str("");
	if (_controller) {
		_controller->prtLog(_prtLog);

		ss << " _controller->drv_verbose = " << _controller->drv_verbose();
		ss << " _controller->lnk_verbose = " << _controller->lnk_verbose();
	} else {
		ss << " NO CONTROLLER" << endl;
	}

	#ifdef _DBG_CSTOR
		prtverbose(0, obj, ss.str());
	#else
		prtverbose(1, obj, ss.str());
	#endif


#if 0
	_verbose = 1;
	_controller->drv_verbose(1);
	_controller->lnk_verbose(1);
#endif

	_FSM = FSM_WAIT;
	_mInitType = INIT_ON_RLS;
	_mInitFSM = IFSM_IDLE;
	
	_FSMerror = FSM_E000;

	_findlimit_st = FINDLIMIT_UNKNOWN;
/*
typedef struct {
        bool            have_limit;
        AxisLimit_t     limit;
        AxisLimitType_t type;
        union {
                struct {
                        AxisHomeType_t          htype;
                        AxisHomePolarity_t      pol;
                } home;
                struct {
                        int n;
                } ls;
                struct {
                        int n;
                } idx;
        } extra;
} AxisLimitConf_t;
*/

	_home_conf.have_limit 		= true;
	_home_conf.limit 		= LIMIT_HOME;
	_home_conf.type			= LIMIT_NC;
	_home_conf.extra.home.htype     = HOME_POINT;
	_home_conf.extra.home.pol 	= HOME_POL_POS;

	_home_versus   = FIND_HOME_PLUS;

	_fls_conf.have_limit 	= true;
	_fls_conf.limit		= LIMIT_FLS;
	_fls_conf.type		= LIMIT_NC;

	_rls_conf.have_limit 	= true;
	_rls_conf.limit		= LIMIT_RLS;
	_rls_conf.type		= LIMIT_NC;

	_find_speeds.push_back(1000);
	_find_speeds.push_back(100);

	_last_update.tv_sec  = 0;
	_last_update.tv_usec = 0;
	_last_refresh.tv_sec  = 0;
	_last_refresh.tv_usec = 0;

	_chk_enc_enable      = true;
	_chk_enc_wait        = 0;
	_chk_enc_last_ok     = false;
	_chk_enc_last_diff   = 0;
	_chk_enc_wait_on_run = 0;
	_chk_mot_last_diff   = 0;
	_chk_mot_last_ok     = false;
	_chk_diff_on_run     = 0;

	_pos_on_init = 0;
	_position = 0;

        _relative_mpos_preinit = nan("");
        _relative_epos_preinit = nan("");

	_emr = 1;
	_emo = 0;

	_per = 1;
	_peo = 0;

#if 0
        // Axis_sencInit()
	_epos0 = nan("");
	_epos1 = nan("");
        __current_versus=1.0;
	_howmany_success_fi =0;
        _howmany_fi     = 0;
        _howmany_switch = 0;
#endif


/*
typedef struct {
        bool       data_valid;
        double     last_position;
        chk_enc_t  chk_enc;
        int        nb_of_restoration;
        int        max_nb_of_restoration;
} restore_last_position_t;
*/
	_rest_last_pos.data_valid		= false;
	_rest_last_pos.last_position		= 0;
	_rest_last_pos.nb_of_restoration 	= 0;
	_rest_last_pos.max_nb_of_restoration 	= 3;

	_ec.which = Controller_ns::ECONF_NONE;
	_ec.ce = -1;
	_ec.mOffOn.enabled = false;
	_ec.mOffOn.m = nan("");
	_ec.mOffOn.q = nan("");

	Controller_ns::mconf_t mc;
	switch (_controller->controller()){
#ifdef _WITH_SIMULDRV_
		case Controller_ns::SIMUL:
			mc = get_mconf();
			ss.str("");
			ss << "mconf() true/fake on Axis()";
			prtverbose(0, obj, ss.str());
                        break;
#endif
#ifdef _WITH_N5DRV_
		case Controller_ns::N5:
			mc = get_mconf();
			//mc.yc.mt    = -2.5;
			if (!mconf(mc)){
				ss.str("");
				ss << "EXCEPT on Axis()";
				if (_prtLog) prtverbose(0, obj, ss.str());
				throw "AHIA";
			} else {
				ss.str("");
				ss << "mconf() true on Axis()";
				if (_prtLog) prtverbose(0, obj, ss.str());
			}
		
                        break;
#endif
#ifdef _WITH_YAMSDRV_
        	case Controller_ns::Yams:
			mc = get_mconf();
			switch (mc.yc.home_polarity){
				case 1:
					_home_conf.extra.home.pol = HOME_POL_POS;
					break;
				case -1:
					_home_conf.extra.home.pol = HOME_POL_NEG;
					break;
			}
			mc.yc.mt    = -2.5;
			mc.yc.dbw   = 2;
			mc.yc.scale = _emr;
			mc.yc.pc    = 1;
			mc.yc.bck   = 0;
			mc.yc.dir   = 0;
			mc.yc.cp    = 0;
			mc.yc.nbrtr = 10;
			mc.yc.brake_ONOFF_extra_TMO = 1000000;
			mc.yc.raw_brakeON_TMO = 10;
			if (!mconf(mc)){
				ss.str("");
				ss << "EXCEPT on Axis()";
				if (_prtLog) prtverbose(0, obj, ss.str());
				throw "AHIA";
			} else {
				ss.str("");
				ss << "mconf() true on Axis()";
				if (_prtLog) prtverbose(0, obj, ss.str());
			}
		
                        break;
#endif
#ifdef _WITH_DMC2XXXPIEZODRV_
        	case Controller_ns::DMC2xxxPiezo:
			mc = get_mconf();

			ss.str("");
			ss << "Controller_ns::DMC2xxxPiezo.get_mconf.mc.pc.mt=" << mc.pc.mt;
			prtverbose(0, obj, ss.str());

			ss.str("");
			ss << "Controller_ns::DMC2xxxPiezo.get_mconf.mc.pc.mo=" << mc.pc.mo;
			prtverbose(0, obj, ss.str());

			if (!mconf(mc)){
				ss.str("");
				ss << "EXCEPT on Axis()";
				if (_prtLog) prtverbose(0, obj, ss.str());
				throw "AHIA";
			} else {
				ss.str("");
				ss << "mconf() true on Axis()";
				if (_prtLog) prtverbose(0, obj, ss.str());
			}
                        break;
#endif
                default:
                        break;
       	};

        _raw_pos_tmo = mc.yc.raw_brakeON_TMO;

        switch (_controller->controller()){
        	default:
                	break;
                case Controller_ns::N5:
                	opmode(Controller_ns::PROFILE_POSITION);
                        break;
        };


	update();
	refresh();
}


/*===============================
*
*
Link_ns::
// values for update (fast)
typedef struct {
        double motor;
        double encoder;
        double velocity;
        double analog;
        bool fls;
        bool rls;
        bool home;
        unsigned long stop_code;
        unsigned long CC_code;
        status_t status;
} uvalues_t;
typedef enum { STANDSTILL, MOVING, ERROR, OFF } status_t;

Axis_ns::
typedef enum { UNKNOWN, DEACTIVATED, OFFLINE, MOVING, ACCELERATING, DECELERATING, STANDSTILL, BRAKED, ERROR } AxisState_t;
*
*===============================*/
bool    Axis::update(){   // update all relevant 'current' values
        string obj("Axis::update()");
        prtverbose(1, obj, "");
        prtverbose2(-1000, obj, "Entering");

        std::stringstream ss;

	struct timeval t;
	gettimeofday(&t, NULL);
	double t_s = (double)(t.tv_sec-_last_update.tv_sec)+((double)(t.tv_usec-_last_update.tv_usec))/1000000;
	if ( t_s < _controller->updateDelay() ) {
        	prtverbose2(-1000, obj, "Exiting: too early");
		return false;
	}
	_last_update.tv_sec  = t.tv_sec;
	_last_update.tv_usec = t.tv_usec;

	Link_ns::uvalues_t uv = _controller->update(_channel);
	/*===================================================
	* _controller->update() che richiama _drv->update() prima di ottenere i nuovi valori inizializza la struttura uv a:
	*	uv.motor        = nan("");
        *	uv.encoder      = nan("");
        *	uv.velocity     = nan("");
        *	uv.analog       = nan("");
        *	uv.fls          = true;
        *	uv.rls          = true;
        *	uv.home         = true;
        *	uv.stop_code    = 0;
        *	uv.CC_code      = 0;
        *	uv.status       = OFF;
	*  
	* se l'aggiornamento ha avuto successo e': _linkstate = LINK_REACTIVE
	* altrimenti
	* se l'aggiornamento NON ha avuto successo e': _linkstate = LINK_UNREACTIVE / LINK_TIMEOUTED / LINK_OTHEREXCEPT / LINK_GENERALEXCEPT / LINK_CLOSE
	*/

#ifndef _BUG_PHSH_MARZO2015
	if (_controller->linkstate() != Link_ns::LINK_REACTIVE) {
        	prtverbose2(-1000, obj, "Exiting: linkstate() != LINK_REACTIVE");
		return false;
	}
#else
	if (_controller->linkstate() != Link_ns::LINK_REACTIVE) {
        	prtverbose2(-1000, obj, "Continuing but: linkstate() != LINK_REACTIVE");
	}
#endif

	/*
	* il codice originario continuava qui, pero' se lo stato di uscita di _controller->update() non era LINK_REACTIVE, si
	* imponevano/"aggiornavano" i valori _fls/_rls/_home/_mpos/_stop_code/_CC_code/encoder!
	* con i default preimpostati in _controller->update() quindi nan/nan/nan...
	* creando (forse) il problema "xcheck while non moving" dei PHSH quando attivati da SuperGap, ovvero quando la ec aveva
	* una botta di richieste sui PHSH ma anche sui id-srv o id2-srv
	*
	*/
	_fls	   = uv.fls;
	_rls	   = uv.rls;
	_home	   = uv.home;
	_mpos      = uv.motor;
	_stop_code = uv.stop_code;
	_CC_code   = uv.CC_code;

        ss.str("");
	ss << " _stop_code=" <<  _stop_code;
        prtverbose(1, obj, ss.str());

	switch(uv.status){
		case Link_ns::STANDSTILL: 	_mstate = Motor_ns::STANDSTILL; break;
		case Link_ns::MOVING: 		_mstate = Motor_ns::MOVING; 	break;
		case Link_ns::OFF: 		_mstate = Motor_ns::DEACTIVATED;break;
		case Link_ns::SERVO: 		_mstate = Motor_ns::SERVO;	break;
		case Link_ns::SWCTL: 		_mstate = Motor_ns::SWCTL;	break;
		default:			_mstate = Motor_ns::UNKNOWN; 	break;
	}

	eupdate(uv, t_s);

#if 0 /* NON piu' necessario */
	double __epos;
        __epos = epos();
	//cout << "uv.velocity = " << uv.velocity << endl;
	evalues(__epos, uv.velocity, t_s); // update _epos, _evelocity
#endif /* 0 */

	_position = enc2position(epos());

        ss.str("");
	ss << "mpos= " << _mpos << " | mstate= " <<  _mstate << " | mstatus= " <<  mstatus();
        prtverbose2(-1001, obj, ss.str());

	return true;
}

/*===============================
*
*  refresh all 'secondary' values:
*  onexit:
*	true  = effective new values
*	false = time out expired! no new values
*
*===============================*/
bool    Axis::refresh(double tmo){
	prtverbose(1, "Axis::refresh(tmo)", "");

	struct timeval t0, t1;
	double dt = 0;
	gettimeofday(&t0, NULL);
	
	while ( !refresh() ){
		gettimeofday(&t1, NULL);
		dt = (double)(t1.tv_sec-t0.tv_sec)+((double)(t1.tv_usec-t0.tv_usec))/1000000;
		if ( dt >= tmo ) return false;
		usleep( (int)(tmo*1000) ); // _max_tmo / 1000 parte di tmo in sec, parte * 1000000 in usec
	}

	return true;
}

/*===============================
*
*  onexit:
*	true  = effective new values
*	false = time out expired! no new values
*
*===============================*/
bool    Axis::update(double tmo){
	prtverbose(1, "Axis::refresh(tmo)", "");

	struct timeval t0, t1;
	double dt = 0;
	gettimeofday(&t0, NULL);
	
	while ( !update() ){
		gettimeofday(&t1, NULL);
		dt = (double)(t1.tv_sec-t0.tv_sec)+((double)(t1.tv_usec-t0.tv_usec))/1000000;
		if ( dt >= tmo ) return false;
		usleep( (int)(tmo*1000) ); // _max_tmo / 1000 parte di tmo in sec, parte * 1000000 in usec
	}

	return true;
}


/*===============================
*
*
Link_ns::
// values for refresh (slow)
typedef struct {
        double speed;
        double acceleration;
        double deceleration;
} rvalues_t;
*
*===============================*/
bool    Axis::refresh(){  // refresh all 'secondary' values
	prtverbose(1, "Axis::refresh()", "");

	struct timeval t;
	gettimeofday(&t, NULL);
	double t_s = (double)(t.tv_sec-_last_refresh.tv_sec)+((double)(t.tv_usec-_last_refresh.tv_usec))/1000000;
	if ( t_s < _controller->updateDelay() ) return false;
	_last_refresh.tv_sec  = t.tv_sec;
	_last_refresh.tv_usec = t.tv_usec;

	Link_ns::rvalues_t rv = _controller->refresh(_channel);

	_mspeed        = rv.speed;
	_macceleration = rv.acceleration;
	_mdeceleration = rv.deceleration;

	switch (_controller->controller()){
#ifdef _WITH_YAMSDRV_
		case Controller_ns::Yams:
                        if ( !(((Controller_ns::YamsDrv*)_controller->drv())->yinit(_channel)) ) _FSM = FSM_NOTINITED;
			break;
#endif
#ifdef _WITH_DMC2XXXPIEZODRV_
		case Controller_ns::DMC2xxxPiezo:
                        if ( !(((Controller_ns::DMC2xxxPiezoDrv*)_controller->drv())->yinit(_channel)) ) _FSM = FSM_NOTINITED;
			break;
#endif
		default:
			break;
	}

	return true;
}

/*===============================
*
*
typedef enum { LIMIT_FLS, LIMIT_RLS, LIMIT_HOME } AxisLimit_t;
vector<double> _find_speeds;
typedef enum { FINDLIMIT_UNKNOWN, FINDLIMIT_DONE, FINDLIMIT_RUN, FINDLIMIT_ERROR } AxisFindLimit_t; 
*
*===============================*/
AxisFindLimit_t    Axis::findlimit(AxisLimit_t l, AxisFindLimit_t oldstate){
	prtverbose(1, "Axis::findlimit()", "");

	if (_find_speeds.size() < 2) return FINDLIMIT_ERROR;

	if ( (l == LIMIT_HOME) && (home_type() != HOME_POINT) ) return FINDLIMIT_ERROR;

	switch (oldstate){
		case FINDLIMIT_DONE:
			return FINDLIMIT_DONE;
			break;

		case FINDLIMIT_ERROR:
			return FINDLIMIT_ERROR;
			break;

		case FINDLIMIT_UNKNOWN:
			cout << " ==== FINDLIMIT_UNKNOWN ==== " << endl;
			switch (l){
				case LIMIT_FLS:
					if ( fls() ) return FINDLIMIT_DONE;
					else {
						if ( forward(_find_speeds[0]) ) return FINDLIMIT_RUN;
						else return FINDLIMIT_ERROR;
					}
					break;
				case LIMIT_RLS:
					if ( rls() ) return FINDLIMIT_DONE;
					else {
						if ( backward(_find_speeds[0]) ) return FINDLIMIT_RUN;
						else return FINDLIMIT_ERROR;
					}
					break;
				case LIMIT_HOME:
					switch (_home_versus){
						case FIND_HOME_PLUS:
							if ( rls() || !have_rls() ){
								if ( forward(_find_speeds[0]) ) return FINDLIMIT_RUN;
								else return FINDLIMIT_ERROR;
							} else
								if ( backward(_find_speeds[0]) ) return FINDLIMIT_H1;
								else return FINDLIMIT_ERROR;
							break;
						case FIND_HOME_MINUS:
							if ( fls() || !have_fls() ){
								if ( backward(_find_speeds[0]) ) return FINDLIMIT_RUN;
								else return FINDLIMIT_ERROR;
							} else
								if ( forward(_find_speeds[0]) ) return FINDLIMIT_H1;
								else return FINDLIMIT_ERROR;
							break;
						default:
							return FINDLIMIT_ERROR;
							break;
					}
					return FINDLIMIT_ERROR;
					break;
				default:
					return FINDLIMIT_UNKNOWN;
					break;
			}
			break;

		case FINDLIMIT_H1:
			cout << " ==== FINDLIMIT_H1 ==== " << endl;
			switch (l){
				case LIMIT_HOME:
					switch (_home_versus){
						case FIND_HOME_PLUS:
							if ( rls() ){
								if ( forward(_find_speeds[0]) ) return FINDLIMIT_RUN;
								else return FINDLIMIT_ERROR;
							}
							break;
						case FIND_HOME_MINUS:
							if ( fls() ){
								if ( backward(_find_speeds[0]) ) return FINDLIMIT_RUN;
								else return FINDLIMIT_ERROR;
							}
							break;
						default:
							return FINDLIMIT_ERROR;
							break;
					}
					return FINDLIMIT_H1;
					break;
				default:
                                        return FINDLIMIT_UNKNOWN;
                                        break;
			}
			break;

		case FINDLIMIT_RUN:
			cout << " ==== FINDLIMIT_RUN ==== " << endl;
			switch (l){
				case LIMIT_FLS:
					if ( fls() ) return FINDLIMIT_DONE;
					else return FINDLIMIT_RUN;
					break;
				case LIMIT_RLS:
					if ( rls() ) return FINDLIMIT_DONE;
					else return FINDLIMIT_RUN;
					break;
				case LIMIT_HOME:
					if ( home() ) return FINDLIMIT_DONE;
					switch (_home_versus){
						case FIND_HOME_PLUS:
							if ( have_fls() && fls() )  return FINDLIMIT_ERROR;
							break;
						case FIND_HOME_MINUS:
							if ( have_rls() && rls() )  return FINDLIMIT_ERROR;
							break;
						default:
							return FINDLIMIT_ERROR;
							break;
					}
					return FINDLIMIT_RUN;
					//return FINDLIMIT_UNKNOWN;
					break;
				default:
					return FINDLIMIT_UNKNOWN;
					break;
			}
			break;

		default:
			return FINDLIMIT_ERROR;
			break;
	} /* switch (oldstate) */

	return FINDLIMIT_ERROR;
}

/*===============================
*
*       true = inited
*       false = not inited
*
*===============================*/
bool Axis::isinit() {
	return isinit(false);
}

bool Axis::isinit(bool do_update) {
	prtverbose(1, "Axis::isinit()", "");

	switch (_controller->controller()){
#ifdef _WITH_YAMSDRV_
        	case Controller_ns::Yams:
			if ((_controller->state() != Controller_ns::CTRL_UP) || do_update ) update();
			return ((Controller_ns::YamsDrv*)_controller->drv())->yinit(_channel);
                        break;
#endif
#ifdef _WITH_DMC2XXXPIEZODRV_
        	case Controller_ns::DMC2xxxPiezo:
                	if ((_controller->state() != Controller_ns::CTRL_UP) || do_update ) update();
			return ((Controller_ns::DMC2xxxPiezoDrv*)_controller->drv())->yinit(_channel);
                        break;
#endif
#ifdef _WITH_N5DRV_
        	case Controller_ns::N5:
                	if (_controller->state() != Controller_ns::CTRL_UP) return false;
			else if (do_update) ((Controller_ns::N5Drv*)_controller->drv())->get_cinit();
			return ((Controller_ns::N5Drv*)_controller->drv())->cinit(_channel);
			break;
#endif
#ifdef _WITH_SIMULDRV_
        	case Controller_ns::SIMUL:
			if ((_controller->state() != Controller_ns::CTRL_UP) || do_update ) update();
			return ((Controller_ns::SimulDrv*)_controller->drv())->sinit(_channel);
                        break;
#endif
#ifdef _WITH_BCSDRV_
        	case Controller_ns::BCS:
			if ((_controller->state() != Controller_ns::CTRL_UP) || do_update ) update();
			return ((Controller_ns::BcsDrv*)_controller->drv())->bcsinit(_channel);
                        break;
#endif
                default:
			return false;
                        break;
       	};
	return false;
}


/*===============================
*
*       true = success
*       false = failed
*
*===============================*/
bool Axis::set_init(bool b){
        string obj("Axis::set_init()");
        std::stringstream ss;
        ss.str("");
        ss << "b = " << b ;
	#ifdef _DBG_AXISINIT
        	prtverbose(0, obj, ss.str());
	#else
        	prtverbose(1, obj, ss.str());
	#endif

	switch (_controller->controller()){
#ifdef _WITH_YAMSDRV_
        	case Controller_ns::Yams:
                	return ((Controller_ns::YamsDrv*)_controller->drv())->yinit(_channel, b);
                        break;
#endif
#ifdef _WITH_DMC2XXXPIEZODRV_
        	case Controller_ns::DMC2xxxPiezo:
                	return ((Controller_ns::DMC2xxxPiezoDrv*)_controller->drv())->yinit(_channel, b);
                        break;
#endif
#ifdef _WITH_N5DRV_
        	case Controller_ns::N5:
                	return ((Controller_ns::N5Drv*)_controller->drv())->cinit(_channel, b);
                        break;
#endif
#ifdef _WITH_SIMULDRV_
        	case Controller_ns::SIMUL:
                	return ((Controller_ns::SimulDrv*)_controller->drv())->sinit(_channel, b);
                        break;
#endif
#ifdef _WITH_BCSDRV_
        	case Controller_ns::BCS:
                	return ((Controller_ns::BcsDrv*)_controller->drv())->bcsinit(_channel, b);
                        break;
#endif
                default:
			return false;
                        break;
       	};
	return false;
}


/*===============================
*
*       true = success
*       false = failed
*
*===============================*/
bool Axis::set_init(double tmo, bool b){
        string obj("Axis::set_init(tmo)");
        std::stringstream ss;
        ss.str("");
        ss << "tmo = " << tmo << " " ;
        ss << "b = " << b ;
	#ifdef _DBG_AXISINIT
        	prtverbose(0, obj, ss.str());
	#else
        	prtverbose(1, obj, ss.str());
	#endif

	struct timeval t0, t1;
	double dt = 0;
	gettimeofday(&t0, NULL);
	
	while ( !set_init(b) ){
		gettimeofday(&t1, NULL);
		dt = (double)(t1.tv_sec-t0.tv_sec)+((double)(t1.tv_usec-t0.tv_usec))/1000000;
		if ( dt >= tmo ) return false;
		usleep( (int)(tmo*1000) ); // _max_tmo / 1000 parte di tmo in sec, parte * 1000000 in usec
	}

	gettimeofday(&t0, NULL);
	while ( isinit(true) != b ){
		gettimeofday(&t1, NULL);
		dt = (double)(t1.tv_sec-t0.tv_sec)+((double)(t1.tv_usec-t0.tv_usec))/1000000;
		if ( dt >= tmo ) return false;
		usleep( (int)(tmo*1000) ); // _max_tmo / 1000 parte di tmo in sec, parte * 1000000 in usec
	}
	return true;
}


/*===============================
*
*
*
*===============================*/
bool Axis::__position(double p){
        string obj("Axis::__position(.)");
        std::stringstream ss;
        ss.str("");
        ss << "p = " << p ;
        prtverbose(1, obj, ss.str());

	if ( !check_mconf_mt() ) {
		_AXISError = AXIS_E001;
		return false;
	}

	//cout << "__position: : " << p << endl;
#if 1
	double netx_dir = p - position();
	if ( (netx_dir > 0) && fls() ) return false;
	if ( (netx_dir < 0) && rls() ) return false;
	double e = position2enc(p);
#else
// position = per * enc + peo
	if ( _per == 0 ) return false;
	double e = position2enc(p);

        ss.str("");
        ss << "pe = " << e ;
        prtverbose(1, obj, ss.str());

// e = emr * m + emo
	if( _emr == 0 ) return false;
	double netx_dir = (e  - _epos)/_emr ;

	if ( (netx_dir > 0) && _fls ) return false;
	if ( (netx_dir < 0) && _rls ) return false;
#endif

	//cout << "__position: e: " << e << endl;
        ss.str("");
        ss << "__position: e: " << e ;
        prtverbose(1, obj, ss.str());

	double m = position2motor(p);
        ss.str("");
        ss << "__position: m: " << m ;
        prtverbose(1, obj, ss.str());

	switch (_controller->controller()){
#ifdef _WITH_YAMSDRV_
        	case Controller_ns::Yams:
			return xpos(e);
                        break;
#endif
#ifdef _WITH_DMC2XXXPIEZODRV_
        	case Controller_ns::DMC2xxxPiezo:
			return mpos(m);
                        break;
#endif
#ifdef _WITH_N5DRV_
        	case Controller_ns::N5:
			if (_controller->drv()->with_microcode()) return xpos(e);
			else return mpos(m);
                        break;
#endif
#ifdef _WITH_SIMULDRV_
        	case Controller_ns::SIMUL:
			/*
			if (_controller->drv()->with_microcode()) return xpos(e);
			else return mpos(m);
			*/
			return mpos(m);
                        break;
#endif
#ifdef _WITH_BCSDRV_
        	case Controller_ns::BCS:
			return xpos(e);
                        break;
#endif
                default:
			return false;
                        break;
       	};
	return false;
}

/*===============================
*
*
*
*===============================*/
bool Axis::position(double p){
        string obj("Axis::position(.)");
        std::stringstream ss;
        ss.str("");
        ss << "p = " << p << "___";
        prtverbose(1, obj, ss.str());

	if ( !check_mconf_mt() ) {
		_AXISError = AXIS_E001;
		ss.str("");
		ss << AxisErrorName[_AXISError];
        	prtverbose(2, obj, ss.str());
		return false;
	}

	Motor_ns::MotorOnStopConf_t bc = brake_conf();
	Motor_ns::MotorState_t ms = mstate();
	ss.str("");
        ss << "Before brake_conf()/mstate()=" << (int)bc << "/" << Motor_ns::MotorStateName[ms];
        prtverbose(1, obj, ss.str());
	
	switch (bc){
        	case Motor_ns::NONE:
        	case Motor_ns::MOTOROFF:
			switch(ms){
				case Motor_ns::STANDSTILL:
				case Motor_ns::SERVO:
				case Motor_ns::SWCTL:
					break;

				case Motor_ns::DEACTIVATED: //"DEENERGIZED" ex DEACTIVATED
					switch (_controller->controller()){
                                        	default:
							_AXISError = AXIS_E009;
							return false;
                                                       	break;
                                        	case Controller_ns::N5:
							motor_on();
                                                	break;
					}
					break;

				case Motor_ns::UNKNOWN:
				case Motor_ns::OFFLINE:
				case Motor_ns::MOVING:
				case Motor_ns::ACCELERATING:
				case Motor_ns::DECELERATING:
				case Motor_ns::BRAKED:
				case Motor_ns::ERROR:
				default:
					_AXISError = AXIS_E008;
					return false;
			}
			break;
        	case Motor_ns::MOTORBRAKED:
			break;
	}
        ss.str("After brake_conf()/mstate()");
        prtverbose(1, obj, ss.str());

	switch (fsm_state()){
		case FSM_WAIT:
		case FSM_STANDSTILL:
			break;

		case FSM_ERROR:
		case FSM_NOTINITED:
		case FSM_INITIALIZING:
			_AXISError = AXIS_E008;
			return false;
			break;

		default:
			_AXISError = AXIS_E007;
			return false;
			break;
	}
        ss.str("After fsm_state()");
        prtverbose(1, obj, ss.str());


	 _target = p;
	AxisFSM_t  _fsm = fsm(FSMT_POSITION);
	switch (_fsm){
		case FSM_POSITIONING:
		case FSM_START_POSITIONING:
			return true;
			break;
		default:
			if (_FSMerror == FSM_E001) _AXISError = AXIS_E011;
			else _AXISError = AXIS_E010;
			ss.str("");
			ss << AxisErrorName[_AXISError];
        		prtverbose(1, obj, ss.str());
			return false;
			break;
	}
        ss.str("Just before return");
        prtverbose(1, obj, ss.str());
	return false;
}

/*===============================
*
*  onexit:
*	true  = effective new values
*	false = time out expired! no new values
*
*===============================*/
bool    Axis::check_motion_progress(double tmo){
	prtverbose(1, "Axis::check_motion_progress(tmo)", "");

	struct timeval t0, t1;
	double dt = 0;
	gettimeofday(&t0, NULL);
	
	while ( !update() ) { usleep(10000); };
	while ( !Motor::check_motion_progress() ){
		gettimeofday(&t1, NULL);
		dt = (double)(t1.tv_sec-t0.tv_sec)+((double)(t1.tv_usec-t0.tv_usec))/1000000;
		if ( dt >= tmo ) return false;
		usleep( (int)(tmo*1000) ); // _max_tmo / 1000 parte di tmo in sec, parte * 1000000 in usec
		while ( !update() ) { usleep(10000); };
	}
	return true;
}

/*===============================
*
*
*
*===============================*/
double Axis::position2motor(double p){
        string obj("Axis::position2motor(.)");
        std::stringstream ss;
        ss.str("");
        ss << "p = " << p << "___";
        prtverbose(2, obj, ss.str());

#ifdef _DBG_PEM_CONVERSIONS
	double i =  position2enc(p);
	double j = enc2motor(position2enc(p));
	cout << "--------------- position2motor(" << p << ")= ";
	cout << "position2enc(" << p << ")= " << i << " ";
	cout << "enc2motor() = " << j << endl;
#endif /* _DBG_PEM_CONVERSIONS */
	return enc2motor(position2enc(p));
}


/*===============================
*
*
*
*===============================*/
double Axis::motor2position(double m){
#ifdef _DBG_PEM_CONVERSIONS
	cout << "--------------- motor2position(" << m << ")=" << enc2position(motor2enc(m)) << endl;
#endif /* _DBG_PEM_CONVERSIONS */
	return enc2position(motor2enc(m));
}


/*===============================
*
*
*
*===============================*/
double Axis::position2enc(double p){
        string obj("Axis::position2enc(.)");
        std::stringstream ss;
        ss.str("");
        ss << "p = " << p << "___ per=" << _per << " peo=" << _peo;
        prtverbose(2, obj, ss.str());

// position = per * enc + peo
#ifdef _DBG_PEM_CONVERSIONS
	cout << "--------------- position2enc(" << p << ")=" << ((p - _peo) / _per) << endl;
#endif /* _DBG_PEM_CONVERSIONS */
	return (p - _peo) / _per;
}


/*===============================
*
*
*
*===============================*/
double Axis::enc2position(double e){
// position = per * enc + peo
#ifdef _DBG_PEM_CONVERSIONS
	cout << "--------------- enc2position(" << e << ")=" << (_per * e + _peo) << endl;
#endif /* _DBG_PEM_CONVERSIONS */
	return _per * e + _peo;
}


/*===============================
*
*
*
*===============================*/
double Axis::speed2motor(double v){
// e = emr * m + emo
	return speed2enc(v) / _emr;
}



/*===============================
*
*
*
*===============================*/
double Axis::motor2speed(double vm){
// e = emr * m + emo
	return enc2speed(_emr* vm);
}


/*===============================
*
*
*
*===============================*/
double Axis::speed2enc(double v){
// position = per * enc + peo
	return v / _per;
}

/*===============================
*
*
*
*===============================*/
double Axis::enc2speed(double ve){
// position = per * enc + peo
	return  _per * ve;
}


/*===============================
*
*
*
*===============================*/
AxisLimit_t Axis::axis2motorLimit(AxisLimit_t l){
        switch (l){
                case LIMIT_FLS:
			if (( _per * _emr ) > 0) return LIMIT_FLS; else return LIMIT_RLS;
                        break;
                case LIMIT_RLS:
			if (( _per * _emr ) > 0) return LIMIT_RLS; else return LIMIT_FLS;
                        break;
                case LIMIT_HOME:
                default:
                        return l;
                        break;
        };

}


/*===============================
*
*
*
*===============================*/
bool Axis::mconf(Controller_ns::mconf_t mc){
	string obj("Axis::mconf(.)");
	std::stringstream ss;
	ss << "mc.yc.mt = " << mc.yc.mt;
	prtverbose(1, obj, ss.str());

	switch (_controller->controller()){
#ifdef _WITH_YAMSDRV_
        	case Controller_ns::Yams:
#endif
#ifdef _WITH_N5DRV_
        	case Controller_ns::N5:
#endif
#if defined(_WITH_YAMSDRV_) || defined(_WITH_N5DRV_)
			_mc.yc.mt    = mc.yc.mt;
			_mc.yc.dbw   = mc.yc.dbw;
			_mc.yc.scale = mc.yc.scale;
			_mc.yc.pc    = mc.yc.pc;
			_mc.yc.bck   = mc.yc.bck;
			_mc.yc.dir   = mc.yc.dir;
			_mc.yc.cp    = mc.yc.cp;
			_mc.yc.nbrtr = mc.yc.nbrtr;
			_mc.yc.brake_ONOFF_extra_TMO = mc.yc.brake_ONOFF_extra_TMO;
			_mc.yc.raw_brakeON_TMO = mc.yc.raw_brakeON_TMO;
			break;
#endif
#ifdef _WITH_DMC2XXXPIEZODRV_
        	case Controller_ns::DMC2xxxPiezo:
			_mc.pc.mt    = mc.pc.mt;
			_mc.pc.mo    = mc.pc.mo;
			break;
#endif
#ifdef _WITH_SIMULDRV_
        	case Controller_ns::SIMUL:
			break;
#endif
#ifdef _WITH_BCSDRV_
        	case Controller_ns::BCS:
			_mc.yc.scale = mc.yc.scale;
			break;
#endif
		default:
			return false;
			break;
	}
	return _controller->mconf(_channel, _mc);
}

/*===============================
*
typedef struct {
        long dbw;               // Dead Band Width = GalilMotionAccuracy positioning accuracy. Written by the client in encoder units
        double scale;           // EncoderMotorRatio = encoder impulse / motor step. Written by the client
        double pc;              // Percent of the movement to be performed from 0.0 to 1.0
        double bck;             // Backlash. Encoder units
        long dir;               // Direction of the last positioning movement in backlash compensation
        long cp;                // Type/presence of encoder: 0 = no encoder
        long nbrtr;             // Number of retries before giving up/ending with movement
} yconf_t;
*
*
*===============================*/
Controller_ns::mconf_t Axis::get_mconf(){
	return _controller->mconf(_channel);
}

/*===============================
*
*
*
*===============================*/
double Axis::acceleration2motor(double a){
	prtverbose(1, "Axis::acceleration2motor()", "");
// e = emr * m + emo
	return acceleration2enc(a) / _emr;
}

/*===============================
*
*
*
*===============================*/
double Axis::motor2acceleration(double am){
	prtverbose(1, "Axis::motor2acceleration()", "");
// e = emr * m + emo
	return enc2acceleration(_emr * am);
}

/*===============================
*
*
*
*===============================*/
double Axis::acceleration2enc(double a){
	prtverbose(1, "Axis::acceleration2enc()", "");
// position = per * enc + peo
	return a / _per;
}


/*===============================
*
*
*
*===============================*/
double Axis::enc2acceleration(double ae){
	prtverbose(1, "Axis::enc2acceleration()", "");
// position = per * enc + peo
	return  _per * ae;
}

/*===============================
*
*
*
*===============================*/
double Axis::deceleration2motor(double a){
	prtverbose(1, "Axis::deceleration2motor()", "");
// e = emr * m + emo
	return deceleration2enc(a) / _emr;
}

/*===============================
*
*
*
*===============================*/
double Axis::motor2deceleration(double am){
	prtverbose(1, "Axis::motor2deceleration()", "");
// e = emr * m + emo
	return enc2deceleration(_emr * am);
}

/*===============================
*
*
*
*===============================*/
double Axis::deceleration2enc(double a){
	prtverbose(1, "Axis::deceleration2enc()", "");
// position = per * enc + peo
	return a / _per;
}


/*===============================
*
*
*
*===============================*/
double Axis::enc2deceleration(double ae){
	prtverbose(1, "Axis::enc2deceleration()", "");
// position = per * enc + peo
	return  _per * ae;
}


/*===============================
*
*
*
*===============================*/
bool Axis::defposition(double p){
	bool b1, b2;
	double _p;

        switch (etype()){
                case Encoder_ns::ENC_DIGITAL_ABSOLUTE:
                case Encoder_ns::ENC_DIGITAL_SEMIABSOLUTE:
                case Encoder_ns::ENC_ANALOG_ABSOLUTE:
			_p = position();
			break;
		default:
			_p = p;
			break;
	}


	b1 = defmpos(position2motor(_p));
	b2 = defepos(position2enc(_p));
	return (b1 && b2);
}

/*===============================
*
*
*
*===============================*/
double Axis::motor2enc(double m){
// e = emr * m + emo
	return (_emr * m + _emo);
}


/*===============================
*
*
*
*===============================*/
double Axis::enc2motor(double e){
        string obj("Axis::enc2motor(.)");
        std::stringstream ss;
        ss.str("");
        ss << "e = " << e << "___ emr=" << _emr << " emo=" << _emo;
        prtverbose(2, obj, ss.str());
// e = emr * m + emo
	return (e - _emo)/_emr;
}


/*===============================
*
*
*
*===============================*/
bool Axis::forward(double v){
	bool b1;
	if ( !check_mconf_mt() ) {
                _AXISError = AXIS_E001;
		return false;
	}
	if ( pmr() > 0 ) b1 = Motor::forward (speed2motor(fabs(v)));
	else             b1 = Motor::backward(speed2motor(fabs(v)));
	if (!b1) _AXISError = AXIS_E006;
	else fsm_error_clear();
	return b1;
}

/*===============================
*
*
*
*===============================*/
bool Axis::backward(double v){
	bool b1;
	if ( !check_mconf_mt() ) {
		_AXISError = AXIS_E001;
		return false;
	}
	if ( pmr() > 0 ) b1 = Motor::backward(speed2motor(fabs(v)));
	else             b1 = Motor::forward (speed2motor(fabs(v)));
	if (!b1) _AXISError = AXIS_E006;
	else fsm_error_clear();
	return b1;
}

/*===============================
*
typedef enum {
	CHK_ENC_UNKNOWN,
        CHK_ENC_OK,
	CHK_ENC_ERROR,
        CHK_ENC_ERROR_ONSTAY,
        CHK_ENC_ERROR_ONRUN
} chk_enc_t;
typedef enum {
        UNKNOWN,
        DEACTIVATED,
        OFFLINE,
        MOVING,
        ACCELERATING,
        DECELERATING,
        STANDSTILL,
        BRAKED,
        ERROR
} MotorState_t;
*
*AAA
*===============================*/
chk_enc_t Axis::check_encoder(){
        string obj("Axis::check_encoder()");
	chk_enc_t v = CHK_ENC_UNKNOWN;

	if (_chk_enc_enable){
		std::stringstream ss;

		double epos_actual = epos();
		double mpos_actual = motor2enc(mpos());
		Motor_ns::MotorState_t l_mstate = mstate();

		ss.str("");
		ss << "mstate() = " << l_mstate << " [" << Motor_ns::MotorStateName[l_mstate] << "]";
		switch (_controller->controller()){
#ifdef _WITH_YAMSDRV_
        		case Controller_ns::Yams:
				if (Motor::check_motion_progress()) l_mstate = Motor_ns::MOVING;
				break;
#endif
			default:
				break;
		}
		ss << " l_mstate = " << l_mstate << " [" << Motor_ns::MotorStateName[l_mstate] << "]";
        	prtverbose2(-1101, obj, ss.str());

		#ifdef __CHK_ENC_DBG__
			//-----------------------|
			cout << "CHECK ENCODER:: ";
			cout << "mstate()/l_mstate= " << mstate() << "/" << l_mstate << "[ " << mstatus() << " ]" << " check_motion_progress()= " << Motor::check_motion_progress() << endl;
			cout << "                epos_actual / _chk_enc_last_value = " << epos_actual << " / " << _chk_enc_last_value << endl;
			cout << "                mpos_actual / _chk_mot_last_value = " << mpos_actual << " / " << _chk_mot_last_value << endl;
		#endif

		ss.str("");
		ss << "PRE: _chk_enc_wait_on_run= " << _chk_enc_wait_on_run << " | _chk_enc_wait= " << _chk_enc_wait;
        	prtverbose2(-1101, obj, ss.str());

		switch (l_mstate){
			case Motor_ns::ACCELERATING:
			case Motor_ns::DECELERATING:
			case Motor_ns::MOVING:
				_chk_mot_last_diff  = mpos_actual - _chk_mot_last_value;
				_chk_mot_last_value = mpos_actual;
				_chk_enc_last_diff  = epos_actual - _chk_enc_last_value;
				_chk_enc_last_value = epos_actual;
				_chk_enc_wait = -1; // era 0
				_chk_enc_last_ok = true;
				_chk_mot_last_ok = true;
				if (_chk_enc_wait_on_run < n_delay_on_check() ) {
					_chk_enc_wait_on_run++;
				}
				break;
			case Motor_ns::DEACTIVATED:
			case Motor_ns::OFFLINE:
			case Motor_ns::STANDSTILL:
			case Motor_ns::BRAKED:
				_chk_enc_wait_on_run = -1; // era 0
				if ( !_chk_enc_last_ok){
					_chk_enc_last_value = epos_actual;
					_chk_enc_wait = -1; //era 0
					_chk_enc_last_ok = true;
				}
				if ( !_chk_mot_last_ok){
					_chk_mot_last_value = mpos_actual;
					_chk_enc_wait = -1; //era 0
					_chk_mot_last_ok = true;
				}
				if (_chk_enc_wait < n_delay_on_check() ) {
					 _chk_enc_last_value = epos_actual;
					 _chk_mot_last_value = mpos_actual;
					_chk_enc_wait++;
				}
				break;
			default:
				_chk_enc_wait = -1; //era 0
				_chk_enc_wait_on_run = -1; // non c'era
				v = CHK_ENC_ERROR;
				break;
		}
	
		if (_chk_enc_wait == n_delay_on_check()){
			#ifdef __CHK_ENC_DBG__
				//-----------------------|
				cout << "CHECK ON STAY:: " << endl;
			#endif
	
			if ( fabs(epos_actual - _chk_enc_last_value) > ripple_on_stay() ) v = CHK_ENC_ERROR_ONSTAY;
			else v = CHK_ENC_OK;
		} else {
			#ifdef __CHK_ENC_DBG__
				//-----------------------|
				cout << "NO check on STAY" << endl;
			#endif
		}
	
		if (_chk_enc_wait_on_run == n_delay_on_check()){
			#ifdef __CHK_ENC_DBG__
				//-----------------------|
				cout << "CHECK ON RUN::  " << endl;
			#endif
	
			_chk_diff_on_run = _chk_mot_last_diff - _chk_enc_last_diff;
			if ( fabs(_chk_diff_on_run)  > ripple_on_run() ) v = CHK_ENC_ERROR_ONRUN;
			else v = CHK_ENC_OK;
		} else {
			#ifdef __CHK_ENC_DBG__
				//-----------------------|
				cout << "NO check on RUN " << endl;
			#endif
		}
	
		ss.str("");
		ss << "POST: _chk_enc_wait_on_run= " << _chk_enc_wait_on_run << " | _chk_enc_wait= " << _chk_enc_wait;
        	prtverbose2(-1101, obj, ss.str());
	
		#ifdef __CHK_ENC_DBG__
			//-----------------------|
			cout << "................on stay:" << endl;
			cout << "                fabs(epos_actual - _chk_enc_last_value) = " << fabs(epos_actual - _chk_enc_last_value);
				cout << " / ripple_on_stay() = " << ripple_on_stay() << endl;
			cout << "................on run:" << endl;
			cout << "                _chk_mot_last_diff = " << _chk_mot_last_diff << endl;
			cout << "                _chk_enc_last_diff = " << _chk_enc_last_diff << endl;
			cout << "                XXX _chk_mot_last_diff - _chk_enc_last_diff = " << _chk_diff_on_run;
				cout << " / ripple_on_run() = " << ripple_on_run() << endl;
			cout << " >> >> >> >> >> check_encoder= " << CheckEncName[v] << " i/n= " << _chk_enc_wait << "/" << n_delay_on_check() << endl;
			cout << endl;
		#endif
	} else { /* _chk_enc_enable */
		#ifdef __CHK_ENC_DBG__
			cout << "===== NO CHECK ENCODER =======" << endl;
		#endif
	} /* _chk_enc_enable */

	return v;
}

/*===============================
*
*
*
*===============================*/
bool Axis::force_init(double p){ // force init state with position p

	_target = p;
	AxisFSM_t  _fsm = fsm(FSMT_DEF_POSITION);
	switch (_fsm){
		case FSM_ERROR:
			return false;
			break;
		default:
			return true;
			break;
	}
	return false;
}

/*===============================
*
*
*
*===============================*/
bool Axis::home(){
	bool v = false;

	switch(_home_conf.type){
		case LIMIT_NC:
			switch(_home_conf.extra.home.pol){
				case HOME_POL_POS:
					v = _home;
					break;
				case HOME_POL_NEG:
					v = !_home;
					break;
			}
			break;
		case LIMIT_NO:
			switch(_home_conf.extra.home.pol){
				case HOME_POL_POS:
					v = !_home;
					break;
				case HOME_POL_NEG:
					v = _home;
					break;
			}
			break;
	}

	return v;
}

/*===============================
*
*
*
*===============================*/
bool Axis::findedge(double v){

	if ( !check_mconf_mt() ) {
                _AXISError = AXIS_E001;
		return false;
	}
        return Motor::findedge(speed2motor(fabs(v)));
}

/*===============================
*
*
*
*===============================*/
bool  Axis::check_mconf_mt(){
	string obj("Axis::check_mconf_mt()");
	std::stringstream ss;
        prtverbose(2, obj, "");

	Controller_ns::mconf_t mc;
        switch (_controller->controller()){
#ifdef _WITH_YAMSDRV_
                case Controller_ns::Yams:
                        mc = get_mconf();
			if ( mc.yc.mt == _mc.yc.mt ){
				return true;
			} else {
				ss.str("");
				ss << "mc.yc.mt =" << mc.yc.mt;
				ss << " ";
				ss << "_mc.yc.mt =" << _mc.yc.mt;
				prtverbose(0, obj, ss.str());
				return false;
			}
                        break;
#endif
#ifdef _WITH_DMC2XXXPIEZODRV_
        	case Controller_ns::DMC2xxxPiezo:
                        mc = get_mconf();
			ss.str("");
			ss << " mc.pc.mt = " << mc.pc.mt << " ";
			ss << "_mc.pc.mt = " << _mc.pc.mt;
			prtverbose(1, obj, ss.str());
			if ( mc.pc.mt == _mc.pc.mt ) return true; else return false;
                        break;
#endif
#ifdef _WITH_N5DRV_
                case Controller_ns::N5:
                        mc = get_mconf();
			if ( mc.yc.mt == _mc.yc.mt ){
				return true;
			} else {
				ss.str("");
				ss << "mc.yc.mt =" << mc.yc.mt;
				ss << " ";
				ss << "_mc.yc.mt =" << _mc.yc.mt;
				prtverbose(0, obj, ss.str());
				return false;
			}
                        break;
#endif
#ifdef _WITH_SIMULDRV_
                case Controller_ns::SIMUL:
                        mc = get_mconf();
			if ( mc.yc.mt == _mc.yc.mt ){
				return true;
			} else {
				ss.str("");
				ss << "mc.yc.mt =" << mc.yc.mt;
				ss << " ";
				ss << "_mc.yc.mt =" << _mc.yc.mt;
				prtverbose(0, obj, ss.str());
				return false;
			}
                        break;
#endif
#ifdef _WITH_BCSDRV_
                case Controller_ns::BCS:
                        mc = get_mconf();
			if ( mc.yc.scale == _mc.yc.scale ){
				return true;
			} else {
				ss.str("");
				ss << "mc.yc.scale =" << mc.yc.scale;
				ss << " ";
				ss << "_mc.yc.scale =" << _mc.yc.scale;
				prtverbose(0, obj, ss.str());
				return false;
			}
                        break;
#endif
                default:
			ss.str("");
			ss << "|";
			ss << _controller->controller();
			ss << "| ";
			ss << _controller->controller_typeName();
			ss << ": neither YAMS, nor DMCPIEZO, nor N5";
        		prtverbose(2, obj, ss.str());
                        break;
        };

	return false;
}

/*===============================
*
*
*
*===============================*/
bool Axis::restore_last_position(){
	bool b1;

	if ( _rest_last_pos.nb_of_restoration >= _rest_last_pos.max_nb_of_restoration ){
		_AXISError = AXIS_E002;
		return false;
	}
	if ( !_rest_last_pos.data_valid ) {
		_AXISError = AXIS_E004;
		return false;
	}
	_rest_last_pos.nb_of_restoration++;
	b1 = defposition(_rest_last_pos.last_position);
	if (b1) return true;

	_AXISError = AXIS_E000;
	return false;
}

/*===============================
*
*
*
*===============================*/
bool Axis::findindex(double v){

	if ( !check_mconf_mt() ) {
                _AXISError = AXIS_E001;
		return false;
	}
        return Motor::findindex(speed2motor(v));
}



/*===============================
*
*
*
*===============================*/
bool Axis::econf(Controller_ns::econf_t ec){
	string obj("Axis::econf(.)");
	std::stringstream ss;
	prtverbose(1, obj, "");

	switch (_controller->controller()){
#ifdef _WITH_YAMSDRV_
        	case Controller_ns::Yams:
			_ec.which = ec.which;
			_ec.ce = ec.ce;
			_ec.mOffOn.enabled = ec.mOffOn.enabled;
			_ec.mOffOn.m = ec.mOffOn.m;
			_ec.mOffOn.q = ec.mOffOn.q;
			break;
#endif
#ifdef _WITH_DMC2XXXPIEZODRV_
        	case Controller_ns::DMC2xxxPiezo:
			break;
#endif
#ifdef _WITH_N5DRV_
        	case Controller_ns::N5:
			_ec.which = ec.which;
			_ec.ce = ec.ce;
			break;
#endif
		default:
			return false;
			break;
	}
	return _controller->econf(_channel, _ec);
}

/*===============================
*
*
*
*===============================*/
Controller_ns::econf_t Axis::get_econf(){
	return _controller->econf(_channel);
}


/*===============================
*
*
*
*===============================*/
double Axis::get_ls_tolerance(){
	long hwlst = _controller->drv()->get_ls_tolerance(_channel);
	if ( hwlst <0 ) return nan("");
	return fabs(motor2position((double)hwlst));
}

} /* namespace Axis_ns */
