#include "Axis.h"

#include <stdlib.h>
#include <string>
#include <algorithm>
#include <iostream>

namespace Axis_ns {

/*===============================
*
*
*
*===============================*/
AxisFSM_t    Axis::fsm(AxisFSMTransitions_t tr){          // proceed with FSM
	std::stringstream ss;

	#ifdef _DBG_TRACE_FSM
		ss.str("");
		ss << "== == == FSM: " << AxisFSMName[_FSM] << " & new FSMT: " << AxisFSMTransitions[tr];
		prtverbose(0, "Axis::fsm()", ss.str());
	#endif /* _DBG_TRACE_FSM */

	
	ss.str("");
	ss << "FSM(new= " << AxisFSMTransitions[tr] << ")";
	prtverbose(1, "Axis::fsm()", ss.str());

	if (tr == FSMT_ABORT){
			_oldtr    = FSMT_NOP;
			_FSM      = FSM_WAIT;
			_mInitFSM = IFSM_IDLE;
	//		_FSMerror = FSM_E000;
			chk_enc_enable(true);
//N5N5
#if 0
        		switch (_controller->controller()){
				default:
					break;
                		case Controller_ns::N5:
					opmode(Controller_ns::PROFILE_POSITION);
					break;
			}
#endif
			return _FSM;
	}

	if ( _mstate == Motor_ns::MOVING ) _rest_last_pos.data_valid = false;

	switch (_FSM){
		case FSM_ERROR:
		case FSM_NOTINITED:
		case FSM_WAIT:
			_oldtr = tr;
			if ( tr != FSMT_NOP ) _FSMerror = FSM_E000;
			break;

		case FSM_INITIALIZING:
		case FSM_STANDSTILL:
		case FSM_MOVING:
		case FSM_START_POSITIONING:
		case FSM_POSITIONING:
		case FSM_STOP_POSITIONING:
		case FSM_POSITIONING_END:
		case FSM_FORWARD:
		case FSM_BACKWARD:
		case FSM_FINDHOME:
		case FSM_FINDRLS:
		case FSM_FINDFLS:
		case FSM_FINDINDEX:
		case FSM_FINDDONE:
		case FSM_FINDENCSIGN_LS:
		case FSM_FINDENCSIGN:
			if (tr == FSMT_ABORT) _oldtr = tr;
			break;

		case FSM_RAWPOS_WAITTMO:
			if (tr == FSMT_NOP) { _oldtr = FSMT_RAWPOS_WAITTMO; break;};
			if (tr != FSMT_RAWPOS_WAITTMO) {Motor::stop(); _oldtr = tr;};
			break;
	}

	ss.str("");
	ss << "FSM(old= " << AxisFSMTransitions[_oldtr] << ")";
	prtverbose(1, "Axis::fsm()", ss.str());


	switch (_oldtr) {

		default:
		case FSMT_NOP:
			break;

		case FSMT_ABORT:
				_FSM = FSM_WAIT;
				_mInitFSM = IFSM_IDLE;
			break;

		case FSMT_INIT:

			#ifdef _DBG_TRACE_FSM
				ss.str("");
				ss << "== == == FSM: " << AxisFSMName[_FSM];
				prtverbose(0, "Axis::fsm()", ss.str());
			#endif /* _DBG_TRACE_FSM */

			ss.str("");
			ss << "FSMT_INIT";
			#ifdef _DBG_AXISINIT
				prtverbose(0, "Axis::fsm()", ss.str());
			#else
				prtverbose(1, "Axis::fsm()", ss.str());
			#endif

			if (_FSM != FSM_INITIALIZING){

				#ifdef _DBG_TRACE_FSM
					ss.str("");
					ss << "== == == FSM: " << AxisFSMName[_FSM] << " -- _FSM != FSM_INITIALIZING ";
					prtverbose(0, "Axis::fsm()", ss.str());
				#endif /* _DBG_TRACE_FSM */

				_FSMerror = FSM_E000;
				// REFRESH some vital data for init phase:
		        	Controller_ns::mconf_t mc;
        			switch (_controller->controller()){
                			case Controller_ns::Yams:
                        			mc = get_mconf();
                        			switch (mc.yc.home_polarity){
                                			case 1:
                                        			_home_conf.extra.home.pol = HOME_POL_POS;
                                        			break;
                                			case -1:
                                        			_home_conf.extra.home.pol = HOME_POL_NEG;
                                        			break;
							default:
                                        			break;
						}
						break;
					default:
						break;
                        	}
				// refresh END

				set_init(_controller->max_tmo(), false);
				refresh(_controller->max_tmo());
				_speed_save = speed();
				__save_mpos.clear(); __save_epos.clear();
				__save_mpos.push_back(mpos()); __save_epos.push_back(epos());
				chk_enc_enable(false);
			}
			_FSM = FSM_INITIALIZING;

			#if  defined(_DBG_AXISINIT)
				ss.str("... going to mInit() ...");
				prtverbose(0, "Axis::fsm()", ss.str());
			#endif /* */

			if (mInit()) {

				#if defined(_DBG_TRACE_FSM) || defined(_DBG_AXISINIT)
					ss.str("");
					ss << "== == == FSM: " << AxisFSMName[_FSM] << " -- mInit() == true ";
					prtverbose(0, "Axis::fsm()", ss.str());
				#endif /* _DBG_TRACE_FSM */

				_FSM = FSM_WAIT;
				_oldtr = FSMT_NOP;
				_mInitFSM = IFSM_IDLE;
				set_init(_controller->max_tmo(), true);
				speed(_speed_save);
				chk_enc_enable(true);
				//refresh(_controller->max_tmo());

		// posso togliere l'update di sotto? altrimenti ce ne sono due di fila?
#if 1
				update(_controller->max_tmo());
#else
#warning
#warning un update in meno
#warning
#endif

				#ifdef _DBG_TRACE_FSM
					ss.str("");
					ss << "== == == FSM: " << AxisFSMName[_FSM] << " -- mInit() == true 2";
					prtverbose(0, "Axis::fsm()", ss.str());
				#endif /* _DBG_TRACE_FSM */

				__save_mpos.push_back(mpos());
				__save_epos.push_back(epos());
				if ( __save_mpos.size() == 2 ) _relative_mpos_preinit = __save_mpos[0] - __save_mpos[1];
				else _relative_mpos_preinit = nan("");
				if ( __save_epos.size() == 2 ) _relative_epos_preinit = __save_epos[0] - __save_epos[1];
				else _relative_epos_preinit = nan("");

				//defmpos(mpos_on_init());
				defposition(pos_on_init());
				refresh(_controller->max_tmo());
				update(_controller->max_tmo());
				check_encoder_restart();

				#ifdef _DBG_TRACE_FSM
					ss.str("");
					ss << "== == == FSM: " << AxisFSMName[_FSM] << " -- mInit() == true 3";
					prtverbose(0, "Axis::fsm()", ss.str());
				#endif /* _DBG_TRACE_FSM */

				_rest_last_pos.nb_of_restoration = 0;
				_rest_last_pos.data_valid    = true;
				_rest_last_pos.last_position = position();
			}
			else if (_mInitFSM == IFSM_ERROR){

				#ifdef _DBG_TRACE_FSM
					ss.str("");
					ss << "== == == FSM: " << AxisFSMName[_FSM] << " -- mInit() == false  && IFSM_ERROR";
					prtverbose(0, "Axis::fsm()", ss.str());
				#endif /* _DBG_TRACE_FSM */

				ss.str("");
				ss << " mInit() IFSM_ERROR _FSMerror= " << _FSMerror << " : " << fsm_error_status();
				prtverbose(0, "Axis::fsm()", ss.str());

				_mInitFSM = IFSM_IDLE;
				speed(_speed_save);
				chk_enc_enable(true);
				check_encoder_restart();
				_oldtr = FSMT_NOP;
				_FSM = FSM_ERROR;
				//_FSMerror = FSM_E008;
			} // forse bisognerebbe far diventare anche FSM_ERROR

			#if defined(_DBG_TRACE_FSM) || defined(_DBG_AXISINIT)
				ss.str("");
				ss << "== == == FSM: " << AxisFSMName[_FSM] << " -- mInit() == false  && !IFSM_ERROR";
				prtverbose(0, "Axis::fsm()", ss.str());
			#endif /* _DBG_TRACE_FSM */
			break;

		case FSMT_POSITION:
			switch (_FSM){
				case FSM_WAIT:
					#ifdef _DBG_POSITIONING
						cout << "FSMT_POSITION: 1 " << AxisFSMName[_FSM] << endl;
					#endif /* _DBG_POSITIONING */
					prtverbose(1, "Axis::fsm()", "FSMT_POSITION: FSM_WAIT->?");
					if ( !_controller->drv()->program_running() ){
						_FSM = FSM_ERROR;
						_FSMerror = FSM_E003;
					} else {
						_FSMerror = FSM_E000;
						if ( __position(_target)){
							_FSM = FSM_START_POSITIONING;
							//update(_controller->max_tmo());
							prtverbose(2, "Axis::fsm()", "FSM_WAIT->FSM_POSITIONING");
						} else {
							_FSM = FSM_ERROR;
							_FSMerror = FSM_E001;
						}
					}
					break;

				case FSM_START_POSITIONING:
					#ifdef _DBG_POSITIONING
						cout << "FSMT_POSITION: 2 " << AxisFSMName[_FSM] << endl;
					#endif /* _DBG_POSITIONING */
					if ( _mstate  == Motor_ns::MOVING ){
						_FSM = FSM_POSITIONING;
					} else {
						if ( (_stop_code  == 2) || (_stop_code  == 3) ){
							_FSM = FSM_ERROR;
							_FSMerror = FSM_E002;
							_oldtr = FSMT_NOP;
							Motor::stop();
							_rest_last_pos.data_valid = false;
						} else
						if ( Motor::check_motion_progress() ) {
							#ifdef _DBG_POSITIONING
								cout << "FSMT_POSITION: 2a " << AxisFSMName[_FSM] << endl;
							#endif /* _DBG_POSITIONING */
						} else {
							check_encoder_restart();
							_FSM = FSM_POSITIONING;
						}
					}
					break;

				case FSM_POSITIONING:
					#ifdef _DBG_POSITIONING
						cout << "FSMT_POSITION: 3 " << AxisFSMName[_FSM] << " ";
						cout << "check_motion_progress() = " << Motor::check_motion_progress() << " ";
						cout << "check_motion_cmd() = " << check_motion_cmd() << " ";
						cout << "_mstate = " << _mstate << " " <<  mstatus() << endl;
					#endif /* _DBG_POSITIONING */
					//if ( Motor::check_motion_progress() && check_motion_cmd() && (_mstate == Motor_ns::MOVING) ){
					if ( check_motion_cmd() && Motor::check_motion_progress() ){
						#ifdef _DBG_POSITIONING
							cout << "FSMT_POSITION: 3a " << AxisFSMName[_FSM] << endl;
						#endif /* _DBG_POSITIONING */
						prtverbose(2, "Axis::fsm()", "FSM_POSITIONING: PROGRESS & MOVING");
						if ( (_stop_code  == 2) || (_stop_code  == 3) ){
							_FSM = FSM_STOP_POSITIONING;
						}
					} else {
						_FSM = FSM_STOP_POSITIONING;
					}
					break;

				case FSM_STOP_POSITIONING:
					#ifdef _DBG_POSITIONING
						cout << "FSMT_POSITION: 4 " << AxisFSMName[_FSM] << endl;
					#endif /* _DBG_POSITIONING */
					_FSM = FSM_POSITIONING_END;
					break;

				case FSM_POSITIONING_END:
					if ( check_motion_end() ) {
						#ifdef _DBG_POSITIONING
							cout << "FSMT_POSITION: " << AxisFSMName[_FSM] << " check_motion_end(0x" << hex << _CC_code << dec <<") true" << endl;
						#endif /* _DBG_POSITIONING */
						prtverbose(2, "Axis::fsm()", "FSM_POSITIONING: POSITIONING OK");
						_oldtr = FSMT_NOP;
						_FSM = FSM_WAIT;
						_rest_last_pos.data_valid    = true;
						_rest_last_pos.last_position = position();
					} else {
						#ifdef _DBG_POSITIONING
							cout << "FSMT_POSITION: " << AxisFSMName[_FSM] << " check_motion_end(0x" << hex << _CC_code << dec <<") false" << endl;
						#endif /* _DBG_POSITIONING */
						prtverbose(2, "Axis::fsm()", "FSM_POSITIONING: POSITIONING NOT OK");
						_FSM = FSM_ERROR;
						_FSMerror = FSM_E002;
						_oldtr = FSMT_NOP;
						Motor::stop();
						_rest_last_pos.data_valid = false;
					}
					break;

				default:
					break;
			}
			break;

		case FSMT_DEF_POSITION:
			prtverbose(1, "Axis::fsm()", "FSMT_DEF_POSITION: ->FSM_WAIT");
			{
			  bool b1, b2;
	        	  b1 = defposition(_target);
        		  b2 = set_init(_controller->max_tmo(), true);
			  update(_controller->max_tmo());
			  check_encoder_restart();
			  if ( b1 && b2 ){
			  	_oldtr = FSMT_NOP;
			  	_FSM = FSM_WAIT;
				_FSMerror = FSM_E000;
			  } else {
				_FSM = FSM_ERROR;
				_FSMerror = FSM_E004;
				_oldtr = FSMT_NOP;
				if (!b1) prtverbose(0, "Axis::fsm()", "FSMT_DEF_POSITION: defposition() error");
				if (!b2) prtverbose(0, "Axis::fsm()", "FSMT_DEF_POSITION: set_init() error");
			  }
			}
			break;

		case FSMT_FINDRLS:
				if ( _FSM != FSM_FINDRLS ) _FSMerror = FSM_E000;
				_FSM = FSM_FINDRLS;
				if ( rflsInit(LIMIT_RLS) ) {
					_FSM = FSM_WAIT;
					_oldtr = FSMT_NOP;
					_mInitFSM = IFSM_IDLE;
				}
				else if (_mInitFSM == IFSM_ERROR){
					cout << " mInit() IFSM_ERROR " << endl;
					_FSM = FSM_ERROR;
					_FSMerror = FSM_E005;
                                        _oldtr = FSMT_NOP;
					Motor::stop();
				}
			break;

		case FSMT_FINDFLS:
				if ( _FSM != FSM_FINDFLS ) _FSMerror = FSM_E000;
				_FSM = FSM_FINDFLS;
				if ( rflsInit(LIMIT_FLS) ) {
					_FSM = FSM_WAIT;
					_oldtr = FSMT_NOP;
					_mInitFSM = IFSM_IDLE;
				}
				else if (_mInitFSM == IFSM_ERROR){
					cout << " mInit() IFSM_ERROR:: " << AxisFSMErrorName[_FSMerror] << endl;
					_FSM = FSM_ERROR;
					_FSMerror = FSM_E005;
                                        _oldtr = FSMT_NOP;
					Motor::stop();
				}
			break;

		case FSMT_FINDHOME:
				if ( _FSM != FSM_FINDHOME ) _FSMerror = FSM_E000;
				_FSM = FSM_FINDHOME;
				if ( homeInit(findedge_versus()) ) {
					_FSM = FSM_WAIT;
					_oldtr = FSMT_NOP;
					_mInitFSM = IFSM_IDLE;
				}
				else if (_mInitFSM == IFSM_ERROR){
					cout << " mInit() IFSM_ERROR " << endl;
					_FSM = FSM_ERROR;
					_FSMerror = FSM_E005;
                                        _oldtr = FSMT_NOP;
					Motor::stop();
				}
			break;

		case FSMT_FINDENCSIGN:
				if ( _FSM != FSM_FINDHOME ) _FSMerror = FSM_E000;
				_FSM = FSM_FINDENCSIGN;
				if ( get_SencSignature() ){
					// signature procedure ended with success!
					_FSM = FSM_WAIT;
					_oldtr = FSMT_NOP;
					_mInitFSM = IFSM_IDLE;
					Motor::stop();
				} else {
					if (_mInitFSM == IFSM_ERROR){
						cout << " mInit() IFSM_ERROR " << endl;
						_FSM = FSM_ERROR;
						_FSMerror = FSM_E009;
                                       		_oldtr = FSMT_NOP;
						Motor::stop();
					} // else signature procedure in progress..
				}
			break; // of FSMT_FINDENCSIGN

		case FSMT_RAWPOS_WAITTMO:
			_FSM =  FSM_RAWPOS_WAITTMO;
			
			switch (Motor::mstate()) {
				case Motor_ns::MOVING:
					{
					  Controller_ns::mconf_t mc = mconf();
				          _raw_pos_tmo = mc.yc.raw_brakeON_TMO;
					}
					//cout << "FSMT_RAWPOS_WAITTMO:_raw_pos_tmo= " << _raw_pos_tmo << endl;
					break;
				default:
					_raw_pos_tmo -= 1;
					//cout << "FSMT_RAWPOS_WAITTMO:_raw_pos_tmo= " << _raw_pos_tmo << endl;
					if (_raw_pos_tmo <= 0) {
						Motor::stop();
						{
						  Controller_ns::mconf_t mc = mconf();
				        	  _raw_pos_tmo = mc.yc.raw_brakeON_TMO;
						}
	                                        _FSM = FSM_WAIT;
       		                                _oldtr = FSMT_NOP;
					}
					break;
			};
			
			break; // of FSMT_RAWPOS_WAITTMO

	}

	return _FSM;
}


} /* namespace Axis_ns */
