#include "Axis.h"

#include <stdlib.h>
#include <string>
#include <algorithm>
#include <iostream>

namespace Axis_ns {

/*============================================================================================
*
*	start on IFSM_IDLE
*
*	return true/false
*	change _mInitFSM
*
*	true  + IFSM_DONE	=> done with success
*	false + IFSM_ERROR 	=> end with error
*
*	false + ....  in progress
*
*============================================================================================*/
bool Axis::indexInit(AxisInitType_t t){
	string obj("Axis::indexInit()");

	bool b1 = false;

	std::stringstream ss;
	
	ss.str("");
	prtverbose(1, obj, ss.str());

	switch (t){
		case INIT_ON_RLSIDX:
        	case INIT_ON_FLSIDX:
			break;
		default:
			_mInitFSM = IFSM_ERROR;
			break;
	}

	switch (_mInitFSM){

		case IFSM_ERROR:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif
			prtverbose(0, obj, "IFSM_ERROR");
			break;

		case IFSM_IDLE:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif
			switch (t){
				case INIT_ON_FLSIDX: b1 = fls(); break;
				case INIT_ON_RLSIDX: b1 = rls(); break;
				default:        b1 = false; break;
			}
			ss.str("");
			ss << "b1= " << b1;
			prtverbose(1, obj, "IFSM_IDLE: "+ss.str());
			if (b1) {
				switch (t){
					case INIT_ON_FLSIDX: b1 = backward(_find_speeds[0]); break;
					case INIT_ON_RLSIDX: b1 = forward(_find_speeds[0]); break;
					default:        b1 = false; break;
				}
				if (b1) _mInitFSM = IFSM_LEAVESWSTART;
				else    _mInitFSM = IFSM_ERROR;
			} else {
				switch (t){
					case INIT_ON_FLSIDX: b1 = forward(_find_speeds[0]); break;
					case INIT_ON_RLSIDX: b1 = backward(_find_speeds[0]); break;
					default:        b1 = false; break;
				}
				if (b1) _mInitFSM = IFSM_FASTTOSWRUNSTART;
				else    _mInitFSM = IFSM_ERROR;
			}
			ss.str(AxisIFSMName[_mInitFSM]);
			prtverbose(1, obj, "IFSM_IDLE: "+ss.str());
			break;

	//---------
		case IFSM_LEAVESWSTART:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif
			if ( _mstate != Motor_ns::MOVING );
			else _mInitFSM = IFSM_WAIT_LEAVESW;
			break;

		case IFSM_WAIT_LEAVESW:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif
			switch (t){
				case INIT_ON_FLSIDX: b1 = fls(); break;
				case INIT_ON_RLSIDX: b1 = rls(); break;
				default:        b1 = true; break;
			}
			if (!b1) {
				Motor::stop();
				_mInitFSM = IFSM_LEAVESWDONE;
			}			
			break;

		case IFSM_LEAVESWDONE:
			switch (t){
				case INIT_ON_FLSIDX: b1 = forward(_find_speeds[0]); break;
				case INIT_ON_RLSIDX: b1 = backward(_find_speeds[0]); break;
				default:        b1 = false; break;
			}
			if (b1) _mInitFSM = IFSM_FASTTOSWRUNSTART;
			else    _mInitFSM = IFSM_ERROR;
			break;
	//---------

		case IFSM_FASTTOSWRUNSTART:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif

			switch (t){
				case INIT_ON_FLSIDX: b1 = fls(); break;
				case INIT_ON_RLSIDX: b1 = rls(); break;
				default:        b1 = false; break;
			}
			if (b1) _mInitFSM = IFSM_FASTTOSWDONE;
			else if ( _mstate != Motor_ns::MOVING ) { /* stay in IFSM_FASTTOSWRUNSTART */ };

			break;

		case IFSM_FASTTOSWRUN:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif
			if ( _mstate != Motor_ns::MOVING ) {
				switch (t){
					case INIT_ON_FLSIDX: b1 = fls(); break;
					case INIT_ON_RLSIDX: b1 = rls(); break;
					default:        b1 = false; break;
				}
				if (b1) _mInitFSM = IFSM_FASTTOSWDONE;
				else if ( (_stop_code  == 2) || (_stop_code  == 3) ) _mInitFSM = IFSM_FASTTOSWDONE;
				else {
					_mInitFSM = IFSM_ERROR;
					_FSMerror = FSM_E104;
				}
			}
			break;

		case IFSM_FASTTOSWDONE:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif
			// fall down to: IFSM_START_FI
			_mInitFSM = IFSM_START_FI;

		case IFSM_START_FI:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif
			switch (t){
				case INIT_ON_FLSIDX: b1 = findindex(-_find_speeds[1]); break;
				case INIT_ON_RLSIDX: b1 = findindex( _find_speeds[1]); break;
				default:        b1 = false; break;
			}
			if (b1) _mInitFSM = IFSM_WAIT_FISTART;
			else    _mInitFSM = IFSM_ERROR;

			break;

		case IFSM_WAIT_FISTART:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif
			//===========update();
			if ( _mstate != Motor_ns::MOVING );
			else _mInitFSM = IFSM_WAIT_FI;
			break;

		case IFSM_WAIT_FI:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif
			if ( _mstate == Motor_ns::MOVING ) break;
			switch (t){
				case INIT_ON_FLSIDX: b1 = rls(); break;
				case INIT_ON_RLSIDX: b1 = fls(); break;
				default:        b1 = false; break;
			}
			if (b1) _mInitFSM = IFSM_ERROR;
			else {
				_mInitFSM = IFSM_WAIT_FIDONE;
			}
			break;

		case IFSM_WAIT_FIDONE:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif
			Motor::stop();
			_mInitFSM =IFSM_DONE;
			return true;
			break;

		default:
			#ifdef _DBG_IDXINIT_IFSM
				cout << "default" << endl;
			#endif
			break;
	};

	return false;
}

} /* namespace Axis_ns */
