#include "Axis.h"

#include <stdlib.h>
#include <string>
#include <algorithm>
#include <iostream>

namespace Axis_ns {

/*============================================================================================
*
*	start on IFSM_IDLE
*
*	return true/false
*	change _mInitFSM
*
*	true  + IFSM_DONE	=> done with success
*	false + IFSM_ERROR 	=> end with error
*
*	false + IFSM_IDLE/IFSM_FASTTOSWRUN/IFSM_FASTTOSWDONE		  |
*	        IFSM_FASTLEAVESWRUN/IFSM_FASTLEAVESWDONE/IFSM_SLOWTOSWRUN | .... in progress
*
*============================================================================================*/
bool Axis::rflsInit(AxisLimit_t l){
	string obj("Axis::rflsInit()");

	bool b1 = false;

	std::stringstream ss;
	
	ss.str("");
	ss << "rflsInit f/r = " << fls() << "/" << rls();
	prtverbose(1, obj, ss.str());
	
	if (_find_speeds.size() < 2) {
		_mInitFSM = IFSM_ERROR;
		return false;
	};

	switch (_mInitFSM){
		case IFSM_ERROR:
			prtverbose(0, obj, "IFSM_ERROR");
			break;
		case IFSM_STOP:
				b1 = Motor::stop();
				_mInitFSM = IFSM_ERROR;
				_FSMerror = FSM_E101;
			break;
		case IFSM_IDLE:
			#ifdef _DBG_RFLSINIT
				ss.str("");
				ss << AxisIFSMName[_mInitFSM];
				prtverbose(0, obj, ss.str());
			#endif /* _DBG_RFLSINIT */
			if (motor_on()) _mInitFSM = IFSM_CHECK_SWITCH1;
			else {
				_mInitFSM = IFSM_ERROR;
				_FSMerror = FSM_E102;
			}
			break;

		case IFSM_CHECK_SWITCH1:
			#ifdef _DBG_RFLSINIT
				ss.str("");
				ss << AxisIFSMName[_mInitFSM];
				prtverbose(0, obj, ss.str());
			#endif /* _DBG_RFLSINIT */
			switch (l){
				case LIMIT_FLS: b1 = fls(); break;
				case LIMIT_RLS: b1 = rls(); break;
				default:        b1 = false; break;
			}
			ss.str("");
			ss << "b1= " << b1;
			prtverbose(1, obj, "IFSM_IDLE: "+ss.str());
			if (b1) {
				_mInitFSM = IFSM_FASTTOSWDONE;
			} else {
				switch (l){
					case LIMIT_FLS: b1 = forward(_find_speeds[0]); break;
					case LIMIT_RLS: b1 = backward(_find_speeds[0]); break;
					default:        b1 = false; break;
				}
				if (b1) _mInitFSM = IFSM_WAIT_FAST_MOVE;
				else {
					_mInitFSM = IFSM_ERROR;
					_FSMerror = FSM_E103;
				}
			}
			ss.str(AxisIFSMName[_mInitFSM]);
			prtverbose(1, obj, "IFSM_CHECK_SWITCH1: "+ss.str());
			break;

		case IFSM_WAIT_FAST_MOVE:
			#ifdef _DBG_RFLSINIT
				ss.str("");
				ss << AxisIFSMName[_mInitFSM];
				prtverbose(0, obj, ss.str());
			#endif /* _DBG_RFLSINIT */
			if ( _mstate == Motor_ns::MOVING ) _mInitFSM = IFSM_FASTTOSWRUN;
			else {
				switch (l){
					case LIMIT_FLS: b1 = fls(); break;
					case LIMIT_RLS: b1 = rls(); break;
					default:        b1 = false; break;
				}
				if (b1) _mInitFSM = IFSM_FASTTOSWRUN;
			}
			break;

		case IFSM_FASTTOSWRUN:
			#ifdef _DBG_RFLSINIT
				ss.str("");
				ss << AxisIFSMName[_mInitFSM];
				prtverbose(0, obj, ss.str());
			#endif /* _DBG_RFLSINIT */
			if ( _mstate != Motor_ns::MOVING ) {
				switch (l){
					case LIMIT_FLS: b1 = fls(); break;
					case LIMIT_RLS: b1 = rls(); break;
					default:        b1 = false; break;
				}
				if (b1) _mInitFSM = IFSM_FASTTOSWDONE;
				else if ( (_stop_code  == 2) || (_stop_code  == 3) ) _mInitFSM = IFSM_FASTTOSWDONE;
				else {
					_mInitFSM = IFSM_ERROR;
					_FSMerror = FSM_E104;
				}
			}
			break;

		case IFSM_FASTTOSWDONE:
			#ifdef _DBG_RFLSINIT
				ss.str("");
				ss << AxisIFSMName[_mInitFSM];
				prtverbose(0, obj, ss.str());
			#endif /* _DBG_RFLSINIT */
			switch (l){
				case LIMIT_FLS: b1 = backward(_find_speeds[0]); break;
				case LIMIT_RLS: b1 = forward(_find_speeds[0]); break;
				default:        b1 = false; break;
			}
			if (b1) _mInitFSM = IFSM_WAIT_FASTLEAVE_MOVE;
			else {
				_mInitFSM = IFSM_ERROR;
				_FSMerror = FSM_E103;
				break;
			}
			break;
		
		case IFSM_WAIT_FASTLEAVE_MOVE:
			#ifdef _DBG_RFLSINIT
				ss.str("");
				ss << AxisIFSMName[_mInitFSM] << " mstate/mstatus = " << _mstate << "/" << mstatus() << "[" << mstate() << "]";
				prtverbose(0, obj, ss.str());
			#endif /* _DBG_RFLSINIT */
			if ( _mstate == Motor_ns::MOVING ) _mInitFSM = IFSM_FASTLEAVESWRUN;
/*
*
*  Supponiamo che:
*  1) l'asse sia gia' sul limitswitch di inizializzazione
*  2) si spenga e riaccenda il cestello Yams
*  3) si faccia partire il server axisg2-srv
*  4) si invii il cmd axis_init()
*
*  Allora:
*  quando la fsm transita per questo punto e' ancora in STANDSTILL (ovvero !MOVING) 
*  MA
*  il limit switch e' ancora ATTIVO!
*  generando quindi errore (FSM_E104).
*  ma controllare il limitswitch (come da codice sottostante 'commentato')
*  non solo e' inutile ma dannoso.
*  Inoltre e' provato che col codice commentato, funziona!
*/
#if 0

			else {
				switch (l){
					case LIMIT_FLS: b1 = fls(); break;
					case LIMIT_RLS: b1 = rls(); break;
					default:        b1 = true; break;
				}
				#ifdef _DBG_RFLSINIT
					ss.str("");
					switch (l){
						case LIMIT_FLS: ss << " fls = "; break;
						case LIMIT_RLS: ss << " rls = "; break;
						default:        ss << " xls = "; break;
					}
					ss << (b1?"true":"false");
					prtverbose(0, obj, ss.str());
				#endif /* _DBG_RFLSINIT */
				if (!b1) _mInitFSM = IFSM_FASTLEAVESWRUN;
				else {
					_mInitFSM = IFSM_ERROR;
					_FSMerror = FSM_E104;
				}
			}
#endif
			break;

		case IFSM_FASTLEAVESWRUN:
			#ifdef _DBG_RFLSINIT
				ss.str("");
				ss << AxisIFSMName[_mInitFSM];
				prtverbose(0, obj, ss.str());
			#endif /* _DBG_RFLSINIT */
			switch (l){
				case LIMIT_FLS: b1 = fls(); break;
				case LIMIT_RLS: b1 = rls(); break;
				default:        b1 = true; break;
			}
			if (!b1) {
				if (!Motor::stop_no_brake()) {
					_mInitFSM = IFSM_ERROR;
					_FSMerror = FSM_E105;
				}
				_mInitFSM = IFSM_FASTLEAVESWDONE;
			}	
			break;

		case IFSM_FASTLEAVESWDONE:
			#ifdef _DBG_RFLSINIT
				ss.str("");
				ss << AxisIFSMName[_mInitFSM];
				prtverbose(0, obj, ss.str());
			#endif /* _DBG_RFLSINIT */
			switch (l){
				case LIMIT_FLS: b1 = forward(_find_speeds[1]); break;
				case LIMIT_RLS: b1 = backward(_find_speeds[1]); break;
				default:        b1 = false; break;
			}	
			if (b1) _mInitFSM = IFSM_WAIT_SLOW_MOVE;
			else {
				_mInitFSM = IFSM_ERROR;
				_FSMerror = FSM_E103;
				break;
			}
			break;

		case IFSM_WAIT_SLOW_MOVE:
			#ifdef _DBG_RFLSINIT
				ss.str("");
				ss << AxisIFSMName[_mInitFSM];
				prtverbose(0, obj, ss.str());
			#endif /* _DBG_RFLSINIT */
			if ( _mstate == Motor_ns::MOVING ) _mInitFSM = IFSM_SLOWTOSWRUN;
			else {
				switch (l){
					case LIMIT_FLS: b1 = fls(); break;
					case LIMIT_RLS: b1 = rls(); break;
					default:        b1 = false; break;
				}	
				if (b1) _mInitFSM = IFSM_SLOWTOSWRUN;
			}
			break;

		case IFSM_SLOWTOSWRUN:
			#ifdef _DBG_RFLSINIT
				ss.str("");
				ss << AxisIFSMName[_mInitFSM];
				prtverbose(0, obj, ss.str());
			#endif /* _DBG_RFLSINIT */
			if ( _mstate != Motor_ns::MOVING ) {
				switch (l){
					case LIMIT_FLS: b1 = fls(); break;
					case LIMIT_RLS: b1 = rls(); break;
					default:        b1 = false; break;
				}
				if (b1) _mInitFSM = IFSM_SLOWTOSWDONE;
				else if ( (_stop_code  == 2) || (_stop_code  == 3) ) _mInitFSM = IFSM_SLOWTOSWDONE;
				else {
					_mInitFSM = IFSM_ERROR;
					_FSMerror = FSM_E104;
				}
			}
			ss.str(AxisIFSMName[_mInitFSM]);
			ss << "b1= " << b1;
			prtverbose(1, obj, "IFSM_SLOWTOSWRUN: "+ss.str());
			break;
		case IFSM_SLOWTOSWDONE:
			#ifdef _DBG_RFLSINIT
				ss.str("");
				ss << AxisIFSMName[_mInitFSM];
				prtverbose(0, obj, ss.str());
			#endif /* _DBG_RFLSINIT */
			_mInitFSM = IFSM_DONE;
			Motor::stop();
			return true;
			break;
		default:
			break;
	};
	return false;
}

} /* namespace Axis_ns */
