#include "Axis.h"

#include <stdlib.h>
#include <string>
#include <algorithm>
#include <iostream>

namespace Axis_ns {

/*============================================================================================
*
*  Init for rotative axis, NO limit switches, useless HOME, just INDEX
*
*
*	start on IFSM_IDLE
*
*	return true/false
*	change _mInitFSM
*
*	true  + IFSM_DONE	=> done with success
*	false + IFSM_ERROR 	=> end with error
*
*	false + ....  in progress
*
*============================================================================================*/
bool Axis::rotIdxInit(AxisInitType_t t){
	string obj("Axis::rotIdxInit()");

	bool b1 = false;

	std::stringstream ss;
	
	ss.str("");
	prtverbose(1, obj, ss.str());

        switch (t){
                case INIT_ON_ROTIDXPOS:
                case INIT_ON_ROTIDXNEG:
                        break;
                default:
                        _mInitFSM = IFSM_ERROR;
                        break;
        }


	switch (_mInitFSM){
		case IFSM_ERROR:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif
			prtverbose(0, obj, "IFSM_ERROR");
			break;
		case IFSM_IDLE:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif
			b1 = motor_on();
			if (b1) _mInitFSM = IFSM_START_FI; else _mInitFSM = IFSM_ERROR;
			ss.str(AxisIFSMName[_mInitFSM]);
			prtverbose(1, obj, "IFSM_IDLE: "+ss.str());
			break;

		case IFSM_START_FI:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif
			switch (t){
				case INIT_ON_ROTIDXPOS: b1 = findindex(_find_speeds[1]); break;
				case INIT_ON_ROTIDXNEG: b1 = findindex(- _find_speeds[1]); break;
				default:        b1 = false; break;
			}
			if (b1) _mInitFSM = IFSM_WAIT_FISTART;
			else    _mInitFSM = IFSM_ERROR;

			break;

		case IFSM_WAIT_FISTART:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif
			//===========update();
			if ( _mstate != Motor_ns::MOVING );
			else _mInitFSM = IFSM_WAIT_FI;
			break;

		case IFSM_WAIT_FI:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif
			if ( _mstate == Motor_ns::MOVING ) break;
			_mInitFSM = IFSM_WAIT_FIDONE;
			break;

		case IFSM_WAIT_FIDONE:
			#ifdef _DBG_IDXINIT_IFSM
				cout << AxisIFSMName[_mInitFSM] << endl;
			#endif
			Motor::stop();
			_mInitFSM =IFSM_DONE;
			return true;
			break;

		default:
			#ifdef _DBG_IDXINIT_IFSM
				cout << "default" << endl;
			#endif
			break;
	};

	return false;
}

} /* namespace Axis_ns */
