#include "Axis.h"

#include <stdlib.h>
#include <string>
#include <algorithm>
#include <iostream>

namespace Axis_ns {

/*============================================================================================
*
*       start on IFSM_IDLE
*
*       return true/false
*       change _mInitFSM
*
*       true  + IFSM_DONE       => done with success
*       false + IFSM_ERROR      => end with error
*
*
*============================================================================================*/

bool Axis::get_SencSignature(){
        #if defined(_DBG_SENCSIGN_IFSM) && (0)
                cout << " - - - - - - - get_SencSignature() .... BEGIN - - - - - - - -" << endl;
        #endif
        string obj("Axis::get_SencSignature()");

        bool b1 = false;

        std::stringstream ss;

	Encoder_ns::EncoderLimit_t t = SencSignature().startPoint;
	//cout << "get_SencSignature() startPoint = " << t << endl;

        ss.str("");
        prtverbose(1, obj, ss.str());

        switch (_mInitFSM){
                case IFSM_ERROR:
                        #ifdef _DBG_SENCSIGN_IFSM
                                cout << "get_SencSignature(): " << AxisIFSMName[_mInitFSM] << endl;
                        #endif
                        prtverbose(0, obj, "IFSM_ERROR");
                        break;

                case IFSM_IDLE:
                        #ifdef _DBG_SENCSIGN_IFSM
                                cout << "get_SencSignature(): " << AxisIFSMName[_mInitFSM] << endl;
                        #endif
                        if (motor_on()) _mInitFSM = IFSM_FASTTOSWRUNSTART;
                        else _mInitFSM = IFSM_ERROR;
                        break;

		case IFSM_FASTTOSWRUNSTART:
                        switch (t){
                                case Encoder_ns::LIMIT_RLS: b1 = rls(); break;
                                case Encoder_ns::LIMIT_FLS: b1 = fls(); break;
                                default:        b1 = false; break;
                        }
                        ss.str("");
                        ss << "b1= " << b1;
                        prtverbose(1, obj, "IFSM_IDLE: "+ss.str());
                        if (b1) {
                                _mInitFSM = IFSM_FASTTOSWDONE;
                        } else {
                                switch (t){
                                        case Encoder_ns::LIMIT_FLS: b1 = forward(_find_speeds[0]); break;
                                        case Encoder_ns::LIMIT_RLS: b1 = backward(_find_speeds[0]); break;
                                        default:        b1 = false; break;
                                }
                                if (b1) _mInitFSM = IFSM_FASTTOSWRUNWAIT;
                                else    _mInitFSM = IFSM_ERROR;
                        }
                        break;

                case IFSM_FASTTOSWRUNWAIT:
                        #ifdef _DBG_SENCSIGN_IFSM
                                cout << "get_SencSignature(): " << AxisIFSMName[_mInitFSM] << endl;
                        #endif
                        if ( _mstate != Motor_ns::MOVING );
                        else _mInitFSM = IFSM_FASTTOSWRUN;
                        break;

                case IFSM_FASTTOSWRUN:
                        #ifdef _DBG_SENCSIGN_IFSM
                                cout << "get_SencSignature(): " << AxisIFSMName[_mInitFSM] << endl;
                        #endif
                        if ( _mstate != Motor_ns::MOVING ) {
                                switch (t){
                                        case Encoder_ns::LIMIT_FLS: b1 = fls(); break;
                                        case Encoder_ns::LIMIT_RLS: b1 = rls(); break;
                                        default:        b1 = false; break;
                                }
                                if (b1) _mInitFSM = IFSM_FASTTOSWDONE;
                                else if ( (_stop_code  == 2) || (_stop_code  == 3) ) _mInitFSM = IFSM_FASTTOSWDONE;
                                else {
                                        _mInitFSM = IFSM_ERROR;
                                }
                        }
                        break;

                case IFSM_FASTTOSWDONE:
                        #ifdef _DBG_SENCSIGN_IFSM
                                cout << "get_SencSignature(): " << AxisIFSMName[_mInitFSM] << endl;
                        #endif
			{
			  double v = epos();
			  _sencsign.v0 = v;

                       	  #if defined(_DBG_SENCSIGN_IFSM_VALUES) || defined(_DBG_SENCSIGN_IFSM)
				cout << "get_SencSignature(): " << "v0= " << v << endl;
			  #endif
			}
                        // fall down to: IFSM_START_FI
                        _mInitFSM = IFSM_START_FI;

                case IFSM_START_FI:
                        #ifdef _DBG_SENCSIGN_IFSM
                                cout << "get_SencSignature(): " << AxisIFSMName[_mInitFSM] << endl;
                        #endif
                        switch (t){
                                case Encoder_ns::LIMIT_FLS: b1 = findindex(-_find_speeds[1]); break;
                                case Encoder_ns::LIMIT_RLS: b1 = findindex( _find_speeds[1]); break;
                                default:        b1 = false; break;
                        }
                        if (b1) _mInitFSM = IFSM_WAIT_FISTART;
                        else    _mInitFSM = IFSM_ERROR;

                        break;

                case IFSM_WAIT_FISTART:
                        #ifdef _DBG_SENCSIGN_IFSM
                                cout << "get_SencSignature(): " << AxisIFSMName[_mInitFSM] << endl;
                        #endif
                        //===========update();
                        if ( _mstate != Motor_ns::MOVING );
                        else _mInitFSM = IFSM_WAIT_FI;
                        break;

                case IFSM_WAIT_FI:
                        #ifdef _DBG_SENCSIGN_IFSM
                                cout << "get_SencSignature(): " << AxisIFSMName[_mInitFSM] << endl;
                        #endif
                        if ( _mstate == Motor_ns::MOVING ) break; // if moving wait!
                        switch (t){
                                case Encoder_ns::LIMIT_FLS: b1 = rls(); break;
                                case Encoder_ns::LIMIT_RLS: b1 = fls(); break;
                                default:        b1 = false; break;
                        }
                        if (b1) _mInitFSM = IFSM_FIDONE;
                        else {
				// motor not moving, not limit switch, i.e.: stopped on index.
				// if stopped on index motor step == 0
				// push actual encoder value onto signature
				double v = epos();
				v = v - _sencsign.v0;
				_sencsign.values.push_back(v);

                        	#if defined(_DBG_SENCSIGN_IFSM_VALUES) || defined(_DBG_SENCSIGN_IFSM)
					cout << "get_SencSignature(): " << "v= " << v << endl;
				#endif

                                _mInitFSM = IFSM_LEAVEFI;
                        }
                        break;

                case IFSM_LEAVEFI:
                        #ifdef _DBG_SENCSIGN_IFSM
                                cout << AxisIFSMName[_mInitFSM];
                                cout << "  enc2motor((double)refwidth()) = " << enc2motor((double)refwidth());
                                cout << "  enc2motor(0) = " << enc2motor(0);
                                cout << " pmr() = " << pmr();
                                cout << endl;
                        #endif
                        {
                                double mot_inc;
				double versus;
                                switch (t){
                                        case Encoder_ns::LIMIT_FLS: versus = -1.0; break;
                                        case Encoder_ns::LIMIT_RLS: versus = 1.0; break;
                                        default:        versus = 0; break;
                                }
                                mot_inc =  versus * 5 * fabs(enc2motor((double)refwidth()) - enc2motor(0));
                                if (pmr() < 0) mot_inc *= -1;
                                #ifdef _DBG_SENCSIGN_IFSM
                                        cout << "LEAVEFI values:" ;
                                        cout << "  versus= " << versus;
                                        cout << "  mot_inc= " << mot_inc;
                                        cout << endl;
                                #endif
                                if (mpos(mot_inc))  _mInitFSM = IFSM_WAIT_LEAVEFI;
                                else _mInitFSM = IFSM_ERROR;
                        };
                        break;

                case IFSM_WAIT_LEAVEFI:
                        #if defined(_DBG_SENCSIGN_IFSM)
                                cout << AxisIFSMName[_mInitFSM] ;
                                cout << "  mpos()= " << mpos();
                                cout << "  epos()= " << epos();
                                cout << "  _mstate= " << _mstate;
                                cout << "  STOP_CODE: " << stop_code2str();
                                cout << endl;
                        #endif
                        switch (_mstate){
                                case Motor_ns::STANDSTILL:
                                case Motor_ns::BRAKED:
                                        _mInitFSM = IFSM_LEAVEFIDONE;
                                        break;
                                default:
                                        break;
                        };
                        break;

                case IFSM_LEAVEFIDONE:
                        #ifdef _DBG_SENCSIGN_IFSM
                                cout << AxisIFSMName[_mInitFSM] << endl;
                        #endif
                        _mInitFSM = IFSM_START_FI;
                        break;

                case IFSM_FIDONE:
                        #ifdef _DBG_SENCSIGN_IFSM
                                cout << "get_SencSignature(): " << AxisIFSMName[_mInitFSM] << endl;
                        #endif
                        Motor::stop();
                        _mInitFSM =IFSM_DONE;

			// optionally calculate and/or adjust signature.
                       	#if defined(_DBG_SENCSIGN_IFSM_VALUES) || defined(_DBG_SENCSIGN_IFSM)
				printSignature();
                        #endif

                        return true;
                        break;

		default:
                        #ifdef _DBG_SENCSIGN_IFSM
                                cout << "get_SencSignature(): " << "default" << endl;
                        #endif
                        break;
	} // switch (_mInitFSM)
	return false;

}

} // namespace
