#ifdef _WITH_BCSDRV_
#include <iostream>
#include "BcsDrv.h"

#include <stdlib.h>
#include <string>
#include <algorithm>

#include <unistd.h>


namespace Controller_ns {


BcsDrv::BcsDrv(string target, string extra): Drv (target, extra){
	#ifdef _DBG_BCSDRV
		_verbose = 1;
	#else
		_verbose = 0;
	#endif

	_name = target;

	std::stringstream ss("");

        ss << " _verbose = " << _verbose;

        #ifdef _DBG_CSTOR
                prtverbose(0, "BcsDrv::BcsDrv()", ss.str());
        #else
                prtverbose(1, "BcsDrv::BcsDrv()", ss.str());
        #endif

	_link = NULL;

	memset(&_mc, 0, sizeof(_mc));

	Link_ns::LinkType t;

        size_t found;
        found = _target.find_first_of("tango://");
        if (found != string::npos ) t = Link_ns::TANGO_CHAN;
        else {
                found = _target.find_first_of("tcp://");
                if (found != string::npos ) t = Link_ns::TCP_CHAN;
                else {
                        found = _target.find_first_of("serial://");
                        if (found != string::npos ) t = Link_ns::SERIAL_CHAN;
                        else t = Link_ns::NULL_CHAN;
                }
        }

	_target = target;

        switch (t){
                case Link_ns::TANGO_CHAN:
                        ss.str("");
                        ss << "new Link_ns::BcsLink(" << _target << ")";
                        prtverbose(1, "BcsDrv::BcsDrv()", ss.str());
                        _bcslink = new Link_ns::BcsTangoLink(_target);
                        _link = (Link_ns::Link*)_bcslink;
                        break;
                case Link_ns::TCP_CHAN:
			#if 0
                        ss.str("");
                        ss << "new Link_ns::BcsLink(" << _target << ")";
                        prtverbose(1, "BcsDrv::BcsDrv()", ss.str());
                        _bcslink = new Link_ns::BcsTcpLink(_target);
                        _link = (Link_ns::Link*)_bcslink;
			#else
                        _bcslink = NULL;
                        _link = NULL;
			#endif
                        break;
                default:
                case Link_ns::NULL_CHAN:
                        ss.str("");
                        ss << "new Link_ns::NULL_CHAN";
                        prtverbose(1, "BcsDrv::BcsDrv()", ss.str());
                        _link = NULL;
                        break;
        }

}

BcsDrv::~BcsDrv(){

	if (_bcslink) {
		delete _bcslink;
		_bcslink = NULL;
		_link = NULL;
	};

}

double  BcsDrv::mpos(string channel){
        std::stringstream ss;
        string obj("BcsDrv::mpos("+channel+")");
        prtverbose(1, obj, "");

        if (_bcslink) {
		Link_ns::uvalues_t uv = _bcslink->update("");

        	cout << "uv.motor   = " << uv.motor << endl;

		return uv.motor;
		
        }
        return nan("");
}

bool    BcsDrv::mpos(string channel, double p){                // mpos : motor position
        std::stringstream ss;
        ss << "p=" << p;
        string obj("BcsDrv::mpos("+channel+",.)");
        prtverbose(1, obj, ss.str());

        //if (!__hw_write_enable) return true;

	if (_bcslink) 
		return _bcslink->double_cmd(channel, "mpos", p);
	else
		return false;
}


bool    BcsDrv::xpos(string channel, double p){                // xpos : motor position in encoder units
        std::stringstream ss;
        ss << "p=" << p;
        string obj("BcsDrv::xpos("+channel+",.)");
        prtverbose(1, obj, ss.str());

        //if (!__hw_write_enable) return true;

	if (_bcslink) 
		return _bcslink->double_cmd(channel, "xpos", p);
	else
		return false;
}


bool    BcsDrv::rpos(string channel, double p){                // rpos : motor relative position
        std::stringstream ss;
        ss << "p=" << p;
        string obj("BcsDrv::rpos("+channel+",.)");
        prtverbose(1, obj, ss.str());

        //if (!__hw_write_enable) return true;

	if (_bcslink) 
		return _bcslink->double_cmd(channel, "rpos", p);
	else
		return false;
}



bool    BcsDrv::mconf(string channel, mconf_t mc){
        std::stringstream ss;
        ss << "BcsDrv::mconf(" << channel << ", mc)";
        string obj(ss.str());
        prtverbose(0, obj, "");
        ss.str("");

        _mc = mc;

   #if 1
	if (_bcslink) 
		return _bcslink->dbl_write(channel, "scale", _mc.yc.scale);
	else
		return false;
   #endif
        return true;
}

mconf_t BcsDrv::mconf(string channel){
        std::stringstream ss;
        ss << "BcsDrv::mconf(" << channel << ")";
        string obj(ss.str());
        prtverbose(1, obj, "");
        ss.str("");

        return _mc;
}


unsigned long BcsDrv::yCC_code(string channel){
        return yCC_code(channel, true);
}

unsigned long BcsDrv::yCC_code(string channel, bool do_update){
        std::stringstream ss;
        string obj("BcsDrv::yCC_code("+channel+")");
        prtverbose(1, obj, "");

        if (do_update) update(channel);
        unsigned long a = uv().CC_code;

        return (unsigned long)a;
}

bool BcsDrv::yCC_code(string channel, unsigned long CC_code){
        std::stringstream ss;
        ss << " new code: 0x" << hex << CC_code << dec << "[" << CC_code << "]";
        string obj("BcsDrv::yCC_code("+channel+",)");
        prtverbose(1, obj, ss.str());

	if (_bcslink) 
		return _bcslink->ul_write(channel, "CC", CC_code);
	else
		return false;
}


bool    BcsDrv::bcsinit(string channel) { // true = INIT / false = NO INIT
#if 0
	return false;
#else
	return (yCC_code(channel, false)) & 0x10000;
#endif
}

bool    BcsDrv::bcsinit(string channel, bool b) {      // true = INIT / false = DEINIT
        std::stringstream ss;
	ss << " " << (b?"true":"false");
        string obj("BcsDrv::bcsinit("+channel+",)");
        prtverbose(1, obj, ss.str());
#if 0
	return b;
#else
        unsigned long CC_code;
        if (b) CC_code = 0x10000; else CC_code = 0x00000;

        ss.str("");
        ss << "new CC= 0x" << hex << ((yCC_code(channel) & 0xE0000) | CC_code) << dec;
        prtverbose(1, obj, ss.str());

        return yCC_code(channel, (yCC_code(channel) & 0xE0000) | CC_code );

#endif
}

bool    BcsDrv::defmpos(string channel, double p) { // define motor position
#if 0
	return true;
#else
	if (_bcslink) 
		return _bcslink->double_cmd(channel, "defmpos", p);
	else
		return false;

#endif
}  


} /* namespace Controller_ns */
#endif /* _WITH_BCSDRV_ */
