#include "BcsTangoLink.h"

#include <sys/time.h>
#include <stdlib.h>




namespace Link_ns {


BcsTangoLink::BcsTangoLink(string target):Link(Link_ns::TANGO_CHAN, target) {
	#ifdef _DBG_BCSTANGOLINK
		_verbose = 1;
	#else
		_verbose = 0;
	#endif

	_name = target;

	std::stringstream ss("");

	ss << " _verbose = " << _verbose;


        string obj("BcsTangoLink()");
        prtverbose(1, obj, ss.str());

	#ifdef _DBG_BCSTANGOLINK_PRTLOG
		ss.str("");
		if (_prtLog) ss << "================  _prtLog TRUE!";
		else         ss << "================  _prtLog FALSE!";
        	prtverbose(-1, obj, ss.str());
	#endif

	_flag = 0;
	dev = NULL;

	try {
		#ifdef _DBG_BCSTANGOLINK_2
			cout << "BcsTangoLink().dev old dev = " << dev << endl;
		#endif

		_linkstate = LINK_UNREACTIVE;
		dev = new Tango::DeviceProxy(target);

		#ifdef _DBG_BCSTANGOLINK_2
			cout << "BcsTangoLink().dev new dev = " << dev << endl;
		#endif

                dev->set_timeout_millis(10*1000);


		ss.str("");
/*
		int ms;
		ms = dev->ping();
		ss << "ping " << ms;
*/

		Tango::DevState	ustate;
		try {
			ustate = dev->state();
			_linkstate = LINK_REACTIVE;
			ss.str("");
			switch (ustate){
				case Tango::ON:
				case Tango::OFF:
				case Tango::CLOSE:
				case Tango::OPEN:
				case Tango::INSERT:
				case Tango::EXTRACT:
				case Tango::MOVING:
				case Tango::STANDBY:
				case Tango::FAULT:
				case Tango::INIT:
				case Tango::RUNNING:
				case Tango::ALARM:
				case Tango::DISABLE:
				case Tango::UNKNOWN:
					ss << " :: " << Tango::DevStateName[ustate] << " :: ";
					break;
				default:
					ss << "++++++++++++++++++++" << ustate;
					break;
			};
        		if (_prtLog) prtverbose(0, obj, ss.str());

			switch (ustate){
				case Tango::CLOSE: _linkstate = LINK_UNREACTIVE; break;
				default: break;
			};

		} catch(Tango::DevFailed &e) {
			ss.str("");

			ss << e.errors.length() ;
			for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
       			if (_prtLog) prtverbose(0, obj, "Tango except!: "+ss.str());

			//2 [0] API_CorbaException [1] API_DeviceTimedOut
			//2 [0] API_CantConnectToDevice [1] API_CommandFailed

			for (size_t i = 0; i < e.errors.length(); i++) {
				if ( strcmp(e.errors[i].reason, API_CantConnectToDevice) == 0 ){ _linkstate = LINK_UNREACTIVE; break; } else
				if ( strcmp(e.errors[i].reason, API_DeviceTimedOut)      == 0 ){ _linkstate = LINK_TIMEOUTED;  break; } else
				{ _linkstate = LINK_OTHEREXCEPT; }
			}
		} catch (...) {
			_linkstate = LINK_GENERALEXCEPT;
       			if (_prtLog) prtverbose(0, obj, "General except!");
		}

		ss.str("");
		ss << linkstatus();
        	prtverbose(1, obj, ss.str());

	} catch(Tango::DevFailed &e) {
		ss.str("");

		ss << e.errors.length() ;
		for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
       		if (_prtLog) prtverbose(0, obj, "Tango except!: "+ss.str());

		#ifdef _DBG_BCSTANGOLINK_2
			cout << "BcsTangoLink().dev = " <<  ss.str() << endl;
		#endif

		if (dev) delete dev;
		dev = NULL;
		_linkstate = LINK_CLOSE;

		ss.str("");
		ss << linkstatus();
        	if (_prtLog) prtverbose(1, obj, ss.str());

	} catch (...) {
		if (dev) delete dev;
		dev = NULL;
		_linkstate = LINK_CLOSE;

		ss.str("");
		ss << linkstatus();
        	if (_prtLog) prtverbose(1, obj, ss.str());
	}

	if ( !refresh() ) _updateDelay = 1.5;

	#ifdef _DBG_BCSTANGOLINK_2
		cout << "BcsTangoLink().dev exiting with = " << dev << endl;
	#endif
}

BcsTangoLink::~BcsTangoLink(){
        #ifdef _DBG_CLASSDESCTRUCTOR
                cout << "~BcsTangoLink() : ";
        #endif
	if (dev) {
        	#ifdef _DBG_CLASSDESCTRUCTOR
                	cout << "going to delete dev ... ";
        	#endif
		delete dev;
	}
        #ifdef _DBG_CLASSDESCTRUCTOR
                cout << endl;
        #endif
}

bool BcsTangoLink::refresh(){
        string obj("BcsTangoLink::refresh()");
        prtverbose(1, obj, "");
	std::stringstream ss;

	int pool_period_ms;

	if (dev){
		try {
			pool_period_ms = dev->get_attribute_poll_period("Motor");
			if ( pool_period_ms <= 0 ) return false;
			_updateDelay = ((double) pool_period_ms) * 1.1 / 1000;
			_linkstate = LINK_REACTIVE;
			//bf->log();
			return true;

		} catch(Tango::DevFailed &e) {
			ss.str("");

			ss << e.errors.length() ;
			for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
        		//prtverbose(0, obj, "Tango except!: "+ss.str());
			if (_prtLog) bf->log(obj, "Tango except!: "+ss.str());

			//2 [0] API_CorbaException [1] API_DeviceTimedOut
			//2 [0] API_CantConnectToDevice [1] API_CommandFailed

			for (size_t i = 0; i < e.errors.length(); i++) {
				if ( strcmp(e.errors[i].reason, API_CantConnectToDevice) == 0 ){ _linkstate = LINK_UNREACTIVE; break; } else
				if ( strcmp(e.errors[i].reason, API_DeviceTimedOut)      == 0 ){ _linkstate = LINK_TIMEOUTED;  break; } else
				{ _linkstate = LINK_OTHEREXCEPT; }
			}

		} catch (...) {
			_linkstate = LINK_GENERALEXCEPT;
        		//prtverbose(0, obj, "except!");
			bf->log(obj, "Tango except!: "+ss.str());
			return false;
		}
	} else { /* if no dev */
		_linkstate = LINK_CLOSE;
        	prtverbose(1, obj, "no dev");
		return false;
	}

	return false;
}

string BcsTangoLink::raw_io(string quest, bool setstate){
        string obj("BcsTangoLink::raw_io()");
        prtverbose(1, obj, "quest="+quest);
	std::stringstream ss;

	string ans("?");
        prtverbose(1, obj, "ans="+ans);
	return ans;
}

uvalues_t BcsTangoLink::update(string channel){
	std::stringstream ss;
        string obj("BcsTangoLink::update("+channel+")");
        prtverbose(1, obj, "");

	timeval start, end;
	gettimeofday(&start, NULL);

	uvalues_t uv;
	// uvalues_t:
        uv.motor 	= nan("");
        uv.encoder	= nan("");
        uv.velocity	= nan("");
        uv.analog	= nan("");
        uv.fls		= true;
        uv.rls		= true;
        uv.home		= true;
        uv.stop_code	= 0;
        uv.CC_code	= 0;
    	#if 0
        	uv.status	= OFF;
	#else
        	uv.status	= STANDSTILL;
	#endif

	vector<Tango::DeviceAttribute> *devattr = NULL;

	Tango::DevState	ustate;
	try {
		ustate = dev->state();
		bf->log();
	} catch(Tango::DevFailed &e) {
		ss.str("");

		ss << " " << __FILE__ << " " << __LINE__ << " ";
		ss << e.errors.length() ;
		for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
       		//prtverbose(0, obj, "Tango except!: "+ss.str());
		if (_prtLog) bf->log(obj, "Tango except!: "+ss.str());

		//2 [0] API_CorbaException [1] API_DeviceTimedOut
		//2 [0] API_CantConnectToDevice [1] API_CommandFailed

		for (size_t i = 0; i < e.errors.length(); i++) {
			if ( strcmp(e.errors[i].reason, API_CantConnectToDevice) == 0 ){ _linkstate = LINK_UNREACTIVE; break; } else
			if ( strcmp(e.errors[i].reason, API_DeviceTimedOut)      == 0 ){ _linkstate = LINK_TIMEOUTED;  break; } else
			{ _linkstate = LINK_OTHEREXCEPT; }
		}
		goto end_label;

	} catch (...) {
		_linkstate = LINK_GENERALEXCEPT;
       		//prtverbose(0, obj, "General except!");
       		bf->log(obj, "General except!");
		goto end_label;
	}


	//if (dev){
	if (dev && (ustate != Tango::CLOSE)){ 
			try {
				vector<string> attr_names;
				attr_names.clear();

				string n;

				n.assign("Motor");
				attr_names.push_back(n);

				n.assign("Encoder");
				attr_names.push_back(n);

				n.assign("RLS");
				attr_names.push_back(n);

				n.assign("FLS");
				attr_names.push_back(n);

                                n.assign("CC");
                                attr_names.push_back(n);

				#ifdef _DBG_BCSTANGOLINK_2
					vector<string>::iterator it;
					for (it=attr_names.begin(); it!=attr_names.end(); ++it){
						cout << "|" << *it;
					}
					cout << "|" << endl;
				#endif

				devattr = dev->read_attributes(attr_names);

				Tango::DevLong l(0);
				Tango::DevDouble f(nan(""));
				Tango::DevBoolean b(false);
				Tango::DevUChar uc(0);
				Tango::DevULong ul(0);

				(*devattr)[0] >> f;  uv.motor = f;
				(*devattr)[1] >> f;  uv.encoder = f;
				(*devattr)[2] >> b;  uv.rls = b;
				(*devattr)[3] >> b;  uv.fls = b;
				(*devattr)[4] >> ul; uv.CC_code = ul;

				switch (ustate){
					case Tango::MOVING:
						uv.status = Link_ns::MOVING;
						break;
					case Tango::STANDBY:
						uv.status = Link_ns::STANDSTILL;
						break;
					default:
						break;
				}

				_linkstate = LINK_REACTIVE;

				bf->log();

			} catch(Tango::DevFailed &e) {
				ss.str("");

				ss << " " << __FILE__ << " " << __LINE__ << " ";
				ss << e.errors.length() ;
				for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
        			//prtverbose(0, obj, "Tango except!: "+ss.str());
				bf->log(obj, "Tango except!: "+ss.str());

				//2 [0] API_CorbaException [1] API_DeviceTimedOut
				//2 [0] API_CantConnectToDevice [1] API_CommandFailed

				for (size_t i = 0; i < e.errors.length(); i++) {
					if ( strcmp(e.errors[i].reason, API_CantConnectToDevice) == 0 ){ _linkstate = LINK_UNREACTIVE; break; } else
					if ( strcmp(e.errors[i].reason, API_DeviceTimedOut)      == 0 ){ _linkstate = LINK_TIMEOUTED;  break; } else
					{ _linkstate = LINK_OTHEREXCEPT; }
				}

			} catch (...) {
				_linkstate = LINK_GENERALEXCEPT;
        			//prtverbose(0, obj, "General except!");
				bf->log(obj, "Tango except!: "+ss.str());
			}

	
	} else { /* if (dev) */
		_linkstate = LINK_CLOSE;
        	prtverbose(1, obj, "LINK_CLOSE");
	}

/*
	sopra:
	vector<Tango::DeviceAttribute> *devattr = NULL;
*/
end_label:
	if (devattr) delete devattr;

	gettimeofday(&end, NULL);
	_last_update_ms = (double)(end.tv_sec - start.tv_sec)*1000 + ((double)(end.tv_usec - start.tv_usec))/1000;

	ss.str("");
	ss << "update time: " << _last_update_ms << "ms";
       	prtverbose(2, obj, ss.str());

	_uv = uv;
	return uv;
}


bool BcsTangoLink::ul_write(string channel, string attrname, Tango::DevULong ul){
	std::stringstream ss;
	ss.str("");
	ss << "BcsTangoLink::ul_write(" << channel << ", " << attrname << ", " << ul << ")";
        string obj(ss.str());
        prtverbose(1, obj, "");

	bool retval(false);

	timeval start, end;
	gettimeofday(&start, NULL);

	Tango::DevState	ustate;
	try {
		ustate = dev->state();
		bf->log();
	} catch(Tango::DevFailed &e) {
		ss.str("");

		ss << " " << __FILE__ << " " << __LINE__ << " ";
		ss << e.errors.length() ;
		for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
       		//prtverbose(0, obj, "Tango except!: "+ss.str());
		if (_prtLog) bf->log(obj, "Tango except!: "+ss.str());

		//2 [0] API_CorbaException [1] API_DeviceTimedOut
		//2 [0] API_CantConnectToDevice [1] API_CommandFailed

		for (size_t i = 0; i < e.errors.length(); i++) {
			if ( strcmp(e.errors[i].reason, API_CantConnectToDevice) == 0 ){ _linkstate = LINK_UNREACTIVE; break; } else
			if ( strcmp(e.errors[i].reason, API_DeviceTimedOut)      == 0 ){ _linkstate = LINK_TIMEOUTED;  break; } else
			{ _linkstate = LINK_OTHEREXCEPT; }
		}
		goto end_label;

	} catch (...) {
		_linkstate = LINK_GENERALEXCEPT;
       		//prtverbose(0, obj, "General except!");
       		bf->log(obj, "General except!");
		goto end_label;
	}

	if (dev && (ustate != Tango::CLOSE)){ 
		try {

                        Tango::DeviceAttribute  daN(attrname, ul);
                        dev->write_attribute(daN);

			retval = true;

		} catch(Tango::DevFailed &e) {
			ss.str("");

			ss << " " << __FILE__ << " " << __LINE__ << " ";
			ss << e.errors.length() ;
			for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
        			//prtverbose(0, obj, "Tango except!: "+ss.str());
			bf->log(obj, "Tango except!: "+ss.str());

			//2 [0] API_CorbaException [1] API_DeviceTimedOut
			//2 [0] API_CantConnectToDevice [1] API_CommandFailed

			for (size_t i = 0; i < e.errors.length(); i++) {
				if ( strcmp(e.errors[i].reason, API_CantConnectToDevice) == 0 ){ _linkstate = LINK_UNREACTIVE; break; } else
				if ( strcmp(e.errors[i].reason, API_DeviceTimedOut)      == 0 ){ _linkstate = LINK_TIMEOUTED;  break; } else
				{ _linkstate = LINK_OTHEREXCEPT; }
			}

		} catch (...) {
			_linkstate = LINK_GENERALEXCEPT;
        			//prtverbose(0, obj, "General except!");
			bf->log(obj, "Tango except!: "+ss.str());
		}
	} else { /* if (dev) */
		_linkstate = LINK_CLOSE;
        	prtverbose(1, obj, "LINK_CLOSE");
	}


end_label:
	gettimeofday(&end, NULL);
	double ms = (double)(end.tv_sec - start.tv_sec)*1000 + ((double)(end.tv_usec - start.tv_usec))/1000;

	ss.str("");
	ss << "time: " << ms << "ms";
       	prtverbose(2, obj, ss.str());
	return retval;
}

bool BcsTangoLink::double_cmd(string channel, string cmdname, Tango::DevDouble v){
	std::stringstream ss;
	ss.str("");
	ss << "BcsTangoLink::double_cmd(" << channel << ", " << cmdname << ", " << v << ")";
        string obj(ss.str());
        prtverbose(1, obj, "");

	bool retval(false);

	timeval start, end;
	gettimeofday(&start, NULL);

	Tango::DevState	ustate;
	try {
		ustate = dev->state();
		bf->log();
	} catch(Tango::DevFailed &e) {
		ss.str("");

		ss << " " << __FILE__ << " " << __LINE__ << " ";
		ss << e.errors.length() ;
		for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
       		//prtverbose(0, obj, "Tango except!: "+ss.str());
		if (_prtLog) bf->log(obj, "Tango except!: "+ss.str());

		//2 [0] API_CorbaException [1] API_DeviceTimedOut
		//2 [0] API_CantConnectToDevice [1] API_CommandFailed

		for (size_t i = 0; i < e.errors.length(); i++) {
			if ( strcmp(e.errors[i].reason, API_CantConnectToDevice) == 0 ){ _linkstate = LINK_UNREACTIVE; break; } else
			if ( strcmp(e.errors[i].reason, API_DeviceTimedOut)      == 0 ){ _linkstate = LINK_TIMEOUTED;  break; } else
			{ _linkstate = LINK_OTHEREXCEPT; }
		}
		goto end_label;

	} catch (...) {
		_linkstate = LINK_GENERALEXCEPT;
       		//prtverbose(0, obj, "General except!");
       		bf->log(obj, "General except!");
		goto end_label;
	}

	if (dev && (ustate != Tango::CLOSE)){ 
		try {

			Tango::DeviceData Din, Dout;
        		Din << v;

                	Dout = dev->command_inout(cmdname, Din);

			retval = true;

		} catch(Tango::DevFailed &e) {
			ss.str("");

			ss << " " << __FILE__ << " " << __LINE__ << " ";
			ss << e.errors.length() ;
			for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
        			//prtverbose(0, obj, "Tango except!: "+ss.str());
			bf->log(obj, "Tango except!: "+ss.str());

			//2 [0] API_CorbaException [1] API_DeviceTimedOut
			//2 [0] API_CantConnectToDevice [1] API_CommandFailed

			for (size_t i = 0; i < e.errors.length(); i++) {
				if ( strcmp(e.errors[i].reason, API_CantConnectToDevice) == 0 ){ _linkstate = LINK_UNREACTIVE; break; } else
				if ( strcmp(e.errors[i].reason, API_DeviceTimedOut)      == 0 ){ _linkstate = LINK_TIMEOUTED;  break; } else
				{ _linkstate = LINK_OTHEREXCEPT; }
			}

		} catch (...) {
			_linkstate = LINK_GENERALEXCEPT;
        			//prtverbose(0, obj, "General except!");
			bf->log(obj, "Tango except!: "+ss.str());
		}
	} else { /* if (dev) */
		_linkstate = LINK_CLOSE;
        	prtverbose(1, obj, "LINK_CLOSE");
	}

end_label:
	gettimeofday(&end, NULL);
	double ms = (double)(end.tv_sec - start.tv_sec)*1000 + ((double)(end.tv_usec - start.tv_usec))/1000;

	ss.str("");
	ss << "time: " << ms << "ms";
       	prtverbose(2, obj, ss.str());
	return retval;

	return retval;
}


bool BcsTangoLink::dbl_write(string channel, string attrname, Tango::DevDouble dbl){
	std::stringstream ss;
	ss.str("");
	ss << "BcsTangoLink::dbl_write(" << channel << ", " << attrname << ", " << dbl << ")";
        string obj(ss.str());
        prtverbose(1, obj, "");

	bool retval(false);

	timeval start, end;
	gettimeofday(&start, NULL);

	Tango::DevState	ustate;
	try {
		ustate = dev->state();
		bf->log();
	} catch(Tango::DevFailed &e) {
		ss.str("");

		ss << " " << __FILE__ << " " << __LINE__ << " ";
		ss << e.errors.length() ;
		for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
       		//prtverbose(0, obj, "Tango except!: "+ss.str());
		if (_prtLog) bf->log(obj, "Tango except!: "+ss.str());

		//2 [0] API_CorbaException [1] API_DeviceTimedOut
		//2 [0] API_CantConnectToDevice [1] API_CommandFailed

		for (size_t i = 0; i < e.errors.length(); i++) {
			if ( strcmp(e.errors[i].reason, API_CantConnectToDevice) == 0 ){ _linkstate = LINK_UNREACTIVE; break; } else
			if ( strcmp(e.errors[i].reason, API_DeviceTimedOut)      == 0 ){ _linkstate = LINK_TIMEOUTED;  break; } else
			{ _linkstate = LINK_OTHEREXCEPT; }
		}
		goto end_label;

	} catch (...) {
		_linkstate = LINK_GENERALEXCEPT;
       		//prtverbose(0, obj, "General except!");
       		bf->log(obj, "General except!");
		goto end_label;
	}

	if (dev && (ustate != Tango::CLOSE)){ 
		try {

                        Tango::DeviceAttribute  daN(attrname, dbl);
                        dev->write_attribute(daN);

			retval = true;

		} catch(Tango::DevFailed &e) {
			ss.str("");

			ss << " " << __FILE__ << " " << __LINE__ << " ";
			ss << e.errors.length() ;
			for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
        			//prtverbose(0, obj, "Tango except!: "+ss.str());
			bf->log(obj, "Tango except!: "+ss.str());

			//2 [0] API_CorbaException [1] API_DeviceTimedOut
			//2 [0] API_CantConnectToDevice [1] API_CommandFailed

			for (size_t i = 0; i < e.errors.length(); i++) {
				if ( strcmp(e.errors[i].reason, API_CantConnectToDevice) == 0 ){ _linkstate = LINK_UNREACTIVE; break; } else
				if ( strcmp(e.errors[i].reason, API_DeviceTimedOut)      == 0 ){ _linkstate = LINK_TIMEOUTED;  break; } else
				{ _linkstate = LINK_OTHEREXCEPT; }
			}

		} catch (...) {
			_linkstate = LINK_GENERALEXCEPT;
        			//prtverbose(0, obj, "General except!");
			bf->log(obj, "Tango except!: "+ss.str());
		}
	} else { /* if (dev) */
		_linkstate = LINK_CLOSE;
        	prtverbose(1, obj, "LINK_CLOSE");
	}


end_label:
	gettimeofday(&end, NULL);
	double ms = (double)(end.tv_sec - start.tv_sec)*1000 + ((double)(end.tv_usec - start.tv_usec))/1000;

	ss.str("");
	ss << "time: " << ms << "ms";
       	prtverbose(2, obj, ss.str());
	return retval;
}


} /* namespace Link_ns */
