#include "BcsTcpLink.h"

#include <sys/time.h>
#include <stdlib.h>

static vector<string> splitby(const char *str, char c){
  vector<string> v;
	#ifdef _DBG_PARSE
		cout << "splitby():: " << str << endl;
	#endif /* _DBG_PARSE */

    	do {
        	const char *begin = str;

        	while(*str != c && *str) str++;

        	v.push_back(string(begin, str));
    	} while (0 != *str++);

	return v;
}



namespace Link_ns {


BcsTcpLink::BcsTcpLink(string target): Link(TCP_CHAN, target) {
	_linkstate = LINK_CLOSE;
	_channel_type  = TCP_CHAN;
	_name  = _target;

	stringstream ss;
        string obj("TcpLink()");
	ss << "[" << target << "]";
        DBG_LINK(obj, ss.str());

	_TCPSocket = -1;

	size_t f1, f2;
	string proto("tcp://");
	string host("");
	string sport("");
	int port = 0;
		
        f1 = target.find(proto);
        if (f1 == string::npos ) {
		ss.str("");
        	ss << "target proto err at " << __FILE__ << ":" << __LINE__;
		LOG_LINK(obj, ss.str());

		throw LinkExcpt(-1, "TcpLink:: error in target, 1");
	};
	
	f2 = target.find(":", f1+proto.size());
        if (f2 == string::npos ) {
		ss.str("");
        	ss << "target proto err at " << __FILE__ << ":" << __LINE__;
		LOG_LINK(obj, ss.str());

		throw LinkExcpt(-1, "TcpLink:: error in target, 2");
	};
	
	host = target.substr(f1+proto.size(), f2 - f1 - proto.size());
	sport = target.substr(f2+1);
	port = atoi(sport.c_str());
	
	if ( port == 0 ) {
		ss << "port= " << port << " ??";
		LOG_LINK(obj, ss.str());
               	throw LinkExcpt(-1, ss.str());
	}
	
	ss.str("");
	ss << "Going to gethostbyname(" << host << ")";
	DBG_LINK(obj, ss.str());

	struct hostent *hp = NULL;
        struct sockaddr_in sa;

	if ((hp = gethostbyname(host.c_str())) == NULL) {
		ss.str("");
       		ss << "target=" << target << " => host=" << host << ": non esiste tale host?";
		LOG_LINK(obj, ss.str());

               	throw LinkExcpt(-1, ss.str());
	}
	
	bcopy( (char *)hp->h_addr, (char *)&sa.sin_addr, hp->h_length);
	sa.sin_family = hp->h_addrtype;
	sa.sin_port = htons(port);
	
	if ( (_TCPSocket = socket(hp->h_addrtype, SOCK_STREAM, 0) ) < 0) {
		ss.str("");
		ss << "socket() "<< strerror(errno);
		LOG_LINK(obj, ss.str());

               	throw LinkExcpt(-1, ss.str());
	}

	int on = 1;
	int rc = setsockopt(_TCPSocket, SOL_SOCKET,  SO_REUSEADDR, (char *)&on, sizeof(on));
   	if (rc < 0) {
		ss.str("");
      		ss << "setsockopt() failed";
		LOG_LINK(obj, ss.str());

		if (_TCPSocket) close(_TCPSocket);
               	throw LinkExcpt(-1, ss.str());
   	}

	//=============================================================
	// check il host:port is reachable/connectable
	
	long arg;
	// Save
  	if( (arg = fcntl(_TCPSocket, F_GETFL, NULL)) < 0) { 
		ss.str("");
		ss  << "Error fcntl(..., F_GETFL) (" << strerror(errno) << ")";
		LOG_LINK(obj, ss.str());

		if (_TCPSocket) close(_TCPSocket);
               	throw LinkExcpt(-1, ss.str());
  	} 

	// Set non-blocking 
  	arg |= O_NONBLOCK; 
  	if( fcntl(_TCPSocket, F_SETFL, arg) < 0) { 
		ss.str("");
		ss << "Error fcntl(..., F_SETFL) (" << strerror(errno) << ")";
		LOG_LINK(obj, ss.str());

		if (_TCPSocket) close(_TCPSocket);
               	throw LinkExcpt(-1, ss.str());
  	} 
	
	// Trying to connect with timeout 
	struct timeval tv;
	fd_set fdset;
	socklen_t lon;
	int valopt;

	int res;

	// la connect puo' avere bisogno di tempo per completarsi
	// quindi in modo NOBLOCK (necessario per non bloccare qui tutto il server)
	// devo verificare con una select quando la connect e' pronta,
	// cosi' posso usare un time-out minore di quello della connect
	//
	res = connect(_TCPSocket, (struct sockaddr *) & sa, sizeof(sa));

	if (res < 0) {
		ss.str("");
		ss << "res= " << res  << " " << strerror(errno);
                if (errno != EINPROGRESS) LOG_LINK(obj, ss.str());

		_linkstate = LINK_CLOSE;

		// attesa che la connect sia pronta:
		//
		ss.str("");
               	ss << "[" << target << "] " << "IF0: begin";
		DBG_LINK(obj, ss.str());

                if (errno == EINPROGRESS) {
                        do {
				ss.str("");
                         	ss << "[" << target << "] " << "DO: 0";
				DBG_LINK(obj, ss.str());

                           	tv.tv_sec = 1;
                           	tv.tv_usec = 0;

                           	FD_ZERO(&fdset);
                           	FD_SET(_TCPSocket, &fdset);

                           	res = select(_TCPSocket+1, NULL, &fdset, NULL, &tv);

			   	if (res < 0) {
					ss.str("");
					ss << "res= " << res  << " " << strerror(errno);
					LOG_LINK(obj, ss.str());
			   	}

                           if (res < 0 && errno != EINTR) {
				ss.str("");
                               	ss << " [" << target << "] " << "DO: 1" << endl;
				DBG_LINK(obj, ss.str());
                                ss.str("");
                                ss << "Error connecting (" << strerror(errno) << ")";
				LOG_LINK(obj, ss.str());

				if (_TCPSocket) close(_TCPSocket);
                                throw LinkExcpt(-1, ss.str());
                           } else if (res > 0) {
				ss.str("");
                               	ss << "[" << target << "] " << "DO: 2";
				DBG_LINK(obj, ss.str());

                              	// Socket selected for write
                              	lon = sizeof(int);
                              	if (getsockopt(_TCPSocket, SOL_SOCKET, SO_ERROR, (void*)(&valopt), &lon) < 0) {
                                	ss.str("");
                                	ss << "Error in getsockopt() (" << strerror(errno) << ")";
					LOG_LINK(obj, ss.str());

					if (_TCPSocket) close(_TCPSocket);
                                	throw LinkExcpt(-1, ss.str());
                              	}
				ss.str("");
                              	ss << "[" << target << "] " << "DO: 3";
				DBG_LINK(obj, ss.str());

                              	// Check the value returned...
                              	if (valopt) {
                                	ss.str("");
                                	ss << "Error in delayed connection() " << valopt << " (" << strerror(errno) << ")";
					LOG_LINK(obj, ss.str());

					if (_TCPSocket) close(_TCPSocket);
                                	throw LinkExcpt(-1, ss.str());
                              	}
				ss.str("");
                              	ss  << "[" << target << "] " << "DO: 4";
				DBG_LINK(obj, ss.str());

			      	_linkstate = LINK_CONNECTED;
                              	break;  // now OK, proceed

                           } else {
                                ss.str("");
                                ss << "Timeout in select() - Cancelling!";
				LOG_LINK(obj, ss.str());

				if (_TCPSocket) close(_TCPSocket);
                                throw LinkExcpt(-1, ss.str());
                           }
                        } while (1);
                } else {
                        ss.str("");
                        ss  << "Error connecting (" << strerror(errno) << ")";
			LOG_LINK(obj, ss.str());

			if (_TCPSocket) close(_TCPSocket);
                        throw LinkExcpt(-1, ss.str());
                }
		ss.str("");
               	ss  << "[" << target << "] " << "IF0: end";
		DBG_LINK(obj, ss.str());

	} else {
		ss.str("");
		ss << "connect";
		DBG_LINK(obj, ss.str());

		_linkstate = LINK_CONNECTED;
	}

	if ( _linkstate == LINK_CONNECTED ){
		ss.str("");
		ss << "connected !!";
		LOG_LINK(obj, ss.str());
	}

        // Set to blocking mode again...
        if( (arg = fcntl(_TCPSocket, F_GETFL, NULL)) < 0) {
                        ss.str("");
                        ss << "Error fcntl(..., F_GETFL) (" << strerror(errno) << ")";
			LOG_LINK(obj, ss.str());

			if (_TCPSocket) close(_TCPSocket);
                        throw LinkExcpt(-1, ss.str());
        }
        arg &= (~O_NONBLOCK);
        if( fcntl(_TCPSocket, F_SETFL, arg) < 0) {
                        ss.str("");
                        ss  << "Error fcntl(..., F_SETFL) (" << strerror(errno) << ")";
			LOG_LINK(obj, ss.str());

			if (_TCPSocket) close(_TCPSocket);
                        throw LinkExcpt(-1, ss.str());
        }
		
	ss.str("");
	ss << "exit status: " << linkstatus();
	DBG_LINK(obj, ss.str());
}

BcsTcpLink::~BcsTcpLink(){
	stringstream ss;
        string obj("~TcpLink()");
        DBG_LINK(obj, ss.str());

	ss.str("");
	ss << "close";
        DBG_LINK(obj, ss.str());

	if (_TCPSocket) close(_TCPSocket);
	_linkstate = LINK_CLOSE;
}

string BcsTcpLink::raw_io(string quest, bool setstate){
        string obj("BcsTcpLink::raw_io()");
        prtverbose(1, obj, "quest="+quest);
	std::stringstream ss;

        string msg(quest);
        int _len    = 0;

        _len = write(_TCPSocket, msg.c_str(), msg.size());

        if ( _len < 0 ) {
                ss.str("");
                ss << "write(,.,"<<msg.size()<<")="<<_len<<" !! " << strerror(errno);
                LOG_LINK(obj, ss.str());
                return "REM ERROR ON WRITE!";
        }
        if (  (size_t)_len != msg.size() ) {
                ss.str("");
                ss << "write(,.,"<<msg.size()<<")="<<_len<<" <!!";
                LOG_LINK(obj, ss.str());
                return "REM ERROR ON WRITE, LESS!";
        }


	string ans("");
	usleep(1000 * BCS_MIN_DELAY);

	ans = rx3(1000, 0);

	return ans;
}

string BcsTcpLink::rx3(double tmo /*ms*/, __attribute__((unused)) double nexttmo /*ms*/){
        string obj("rx3()");
        std::stringstream ss;
        ss << "Entering... ";
        DBG_LINK_RX(obj, ss.str());


        fd_set fdset;
        int retval  = 0;

        if ( tmo < 0 ) {
                ss.str("");
                ss << "tmo error (<0)";
                LOG_LINK(obj, ss.str());

                throw LinkExcpt(-1, ss.str());
        }

        struct timeval tv, _tmo;
        __attribute__((unused)) struct timeval _nexttmo;

        _tmo.tv_sec  = (int)(tmo/1000);
        _tmo.tv_usec = (int)((tmo - _tmo.tv_sec*1000)*1000);


        unsigned char msgrx[TCPMSGSIZE+1] = "";
        string ans;

        int _len;

        retval  = 0;
        do {
                _len = 0;
                bzero (msgrx, TCPMSGSIZE+1);
                tv.tv_sec = _tmo.tv_sec;
                tv.tv_usec = _tmo.tv_usec;

                FD_ZERO (&fdset);
                FD_SET (_TCPSocket, &fdset);
                retval = select (_TCPSocket+1, &fdset, NULL, NULL, &tv);
                if (retval > 0){
                        _len = recv(_TCPSocket, msgrx, TCPMSGSIZE, 0);
                        if (_len < 0 ) {
                                ss.str("");
                                ss << "rx error " << strerror(errno);
                                LOG_LINK(obj, ss.str());

                                throw LinkExcpt(-1, ss.str());
                        } else
                        if (_len > 0 ) {
                                ans.insert(ans.end(), msgrx, msgrx+_len);
                        }
                }
                if (retval == 0) break; //TIMEOUT!
                if (_len == 0) break; //no more rx!
        } while ( (retval == -1 && errno == EINTR) );


        ss.str("");
        ss << "Leaving... with retval=" << retval ;
        DBG_LINK_RX(obj, ss.str());
        return ans;
}

double BcsTcpLink::abactpos(string channel){
	std::stringstream ss;
        string obj("BcsTcpLink::abactpos("+channel+")");
        prtverbose(1, obj, "");

	/*
	*
	* channel = encoder;motore;inshst
	*
	*/
	vector<string> bcspoints = splitby(channel.c_str(), ';');
	vector<string> rxvalues;

	string quest, ans;
	size_t f1;
	double v = nan("");

        f1 = (bcspoints[1]).find("ABACTPO");
        if (f1 != string::npos ) {
		usleep(1000 * BCS_MIN_DELAY);
		quest = "GET " + bcspoints[1];
		ans = raw_io(quest, false);
		DBG_LINK_RX(obj, ans);
		rxvalues = splitby(ans.c_str(), ',');
		v  = atof(rxvalues[3].c_str());
	}

	return v;
}


uvalues_t BcsTcpLink::update(string channel){
	std::stringstream ss;
        string obj("BcsTcpLink::update("+channel+")");
        prtverbose(1, obj, "");

	/*
	*
	* channel = encoder;motore;inshst
	*
	*/
	vector<string> bcspoints = splitby(channel.c_str(), ';');
	vector<string> rxvalues;

	string quest, ans;
	size_t f1;
	unsigned int v;

	timeval start, end;
	gettimeofday(&start, NULL);

	uvalues_t uv;
	// uvalues_t:
        uv.motor 	= nan("");
        uv.encoder	= nan("");
        uv.velocity	= nan("");
        uv.analog	= nan("");
        uv.fls		= true;
        uv.rls		= true;
        uv.home		= true;
        uv.stop_code	= 0;
        uv.CC_code	= 0;
        uv.status	= OFF;

	/*
	R,A,-----------,9.7509,mm
	| | |           |      |
	| | |		|      +descr_valore
        | | |           +valore
        | |  +stato:  11 caratteri ASCII
        | +tipo_punto:
	|		A:      Analogico Floating
	|		I:      Analogico Intero
	|		D:      Digitale
	|		B:      Bitmap
	+tipo_acquisizione

	INSHST 16bit, numerati da 1:
	bit 5 ... X-   (pulse)
	bit 6 ... X+   (direction)
	bit 7 ... STOP (PWM)
	bit 8 Motor idle"    "Desired speed not= 0
	*/

	usleep(1000 * BCS_MIN_DELAY);
	quest = "GET " + bcspoints[0];
	ans = raw_io(quest, false);
	DBG_LINK_RX(obj, ans);

        f1 = ans.find(BCS_ERROR_REPLY);
        if (f1 != string::npos ) {
		goto end_label; 
	}
	rxvalues = splitby(ans.c_str(), ',');
	uv.encoder = atof(rxvalues[3].c_str());

        f1 = (bcspoints[1]).find("ABACTPO");
        if (f1 == string::npos ) {
		goto end_label; 
	}
	usleep(1000 * BCS_MIN_DELAY);
	quest = "GET " + bcspoints[1];
	ans = raw_io(quest, false);
	DBG_LINK_RX(obj, ans);
	rxvalues = splitby(ans.c_str(), ',');
	uv.motor = atof(rxvalues[3].c_str());


        f1 = (bcspoints[2]).find("INSHST");
        if (f1 == string::npos ) {
		goto end_label; 
	}
	usleep(1000 * BCS_MIN_DELAY);
	quest = "GET " + bcspoints[2];
	ans = raw_io(quest, false);
	DBG_LINK_RX(obj, ans);
	rxvalues = splitby(ans.c_str(), ',');
	v = atoi(rxvalues[3].c_str());

	if ( v & 0x10 ) uv.rls = true; else uv.rls = false;
	if ( v & 0x20 ) uv.fls = true; else uv.fls = false;
	

	bf->log();
end_label:

	gettimeofday(&end, NULL);
	_last_update_ms = (double)(end.tv_sec - start.tv_sec)*1000 + ((double)(end.tv_usec - start.tv_usec))/1000;

	ss.str("");
	ss << "update time: " << _last_update_ms << "ms";
       	prtverbose(2, obj, ss.str());

	_uv = uv;
	return uv;
}


	
} /* namespace Link_ns */

//=============================================================================
#if 0
		//if (_simulthread->isinit()) uv.CC_code |= 0x10000;
		//else			    uv.CC_code &= 0xEFFFF;

		//if (_simulthread->isrunning()) 	uv.status = MOVING;
		//else {
	//		if (_simulthread->ismotor_on() ) uv.status = STANDSTILL;
	//		else				 uv.status = OFF;
		}

		if ( uv.status == MOVING ) uv.CC_code |= 0x100;
                else uv.CC_code &= !((unsigned long)0x100);

#endif


