#include "Axis.h"

#include <stdlib.h>
#include <string>
#include <algorithm>
#include <iostream>

namespace Axis_ns {

/*============================================================================================
*
*	start on IFSM_IDLE
*
*	return true/false
*	change _mInitFSM
*
*	true  + IFSM_DONE	=> done with success
*	false + IFSM_ERROR 	=> end with error
*
typedef enum {
         CIA402_HOME_PERFORMED
        ,CIA402_HOME_INTERRUPTED
        ,CIA402_HOME_PROGRESS
        ,CIA402_HOME_COMPLETED
        ,CIA402_HOME_ERROR_MOVING
        ,CIA402_HOME_ERROR_STANDSTILL
} Cia402HomingState_t;

        INIT_ON_DEFPOS = 0,
        INIT_ON_HOME = 1,
        INIT_ON_RLS = 2,
        INIT_NO_3,
        INIT_ON_FLS = 4,
        INIT_ON_CUSTOM = 5,
        INIT_NO_6,
        INIT_ON_RLSIDX = 7,
        INIT_ON_FLSIDX = 8,
        INIT_ON_EDGEPOSDEF = 9,
        INIT_ON_EDGENEGDEF = 10,
        INIT_ON_ABSOLUTE,
        INIT_ON_SENCPOS,
        INIT_ON_SENCNEG,
        INIT_ON_ROTIDXPOS,
        INIT_ON_ROTIDXNEG,
        CIA402_INIT_ON_RLS,
        CIA402_INIT_ON_FLS,
        CIA402_INIT_ON_RLSIDX,
        INIT_VOID


*
*============================================================================================*/
bool Axis::Cia402Init(AxisInitType_t l){
	std::stringstream ss;
	string obj("Axis::Cia402Init()");
	
	ss.str("");
	ss << "Cia402Init f/r = " << fls() << "/" << rls();
	#ifdef _DBG_CIA402INIT
		prtverbose(0, obj, ss.str());
	#else
		prtverbose(1, obj, ss.str());
	#endif
	
	bool b1 = false;

	if (_find_speeds.size() < 2) {
		_mInitFSM = IFSM_ERROR;
		return false;
	};

	double home_speed_switch = fabs(speed2motor(_find_speeds[0]));
       	double home_speed_reference = fabs(speed2motor(_find_speeds[1]));

	if ( (home_speed_switch <= 0.0) || (home_speed_reference <= 0.0) ){
		_mInitFSM = IFSM_ERROR;
		return false;
	}

	Controller_ns::Drv *drvp = NULL;
	drvp = (Controller_ns::Drv *)(_controller->drv());
	if (drvp == NULL) {
		_mInitFSM = IFSM_ERROR;
		return false;
	}


	switch (_mInitFSM){
		case IFSM_ERROR:
			#ifdef _DBG_CIA402INIT
                                ss.str("");
                                ss << AxisIFSMName[_mInitFSM];
                                prtverbose(0, obj, ss.str());
                        #endif /* _DBG_CIA402INIT */

			prtverbose(0, obj, "IFSM_ERROR");
			break;

		case IFSM_IDLE:
			#ifdef _DBG_CIA402INIT
				ss.str("");
				ss << AxisIFSMName[_mInitFSM];
				prtverbose(0, obj, ss.str());
			#endif /* _DBG_CIA402INIT */
			{
		        Motor::stop();
			motor_off();

			// salvo il corrente opmode
			_opmode = opmode();
			
			#ifdef _DBG_CIA402INIT
				cout << "OPMODE = " << _opmode << endl;
			#endif /* _DBG_CIA402INIT */

			// 
			// imposto i valori e
			//
			Controller_ns::Cia402HomingParams_t p = drvp->Cia402HomingParams();
		        p.home_speed_switch = home_speed_switch;
        		p.home_speed_reference = home_speed_reference;
			if (drvp->Cia402HomingParams(p)){

				//
				// avvio la procedura di homing
				//
				ss.str("");
				ss << "Going to perform init method: " << AxisInitTypeName[l];
				prtverbose(0, obj, ss.str());
				switch (l){
	                                case CIA402_INIT_ON_FLS:    b1 = drvp->Cia402HomingStart(18); break;
	                                case CIA402_INIT_ON_RLS:    b1 = drvp->Cia402HomingStart(17); break;
	                                case CIA402_INIT_ON_RLSIDX: b1 = drvp->Cia402HomingStart(1); break;
	                                case CIA402_INIT_ON_FLSIDX: b1 = drvp->Cia402HomingStart(2); break;
					case CIA402_INIT_ON_ROTIDXPOS: b1 = drvp->Cia402HomingStart(34); break;
					case CIA402_INIT_ON_ROTIDXNEG: b1 = drvp->Cia402HomingStart(33); break;

	                                default:        b1 = false; break;
	                        }
				drvp->Cia402HomingState();
				if (b1) _mInitFSM = IFSM_STEP4RUN;
	                        else {
	                        	_mInitFSM = IFSM_ERROR;
	                                _FSMerror = FSM_E008;
	                        }
			} else {
	                        	_mInitFSM = IFSM_ERROR;
	                                _FSMerror = FSM_E008;
	                }
			}
			break;

		case IFSM_STEP4RUN:
			#ifdef _DBG_CIA402INIT
                                ss.str("----------------");
                                ss << AxisIFSMName[_mInitFSM];
                                prtverbose(0, obj, ss.str());
                        #endif /* _DBG_CIA402INIT */
			switch (drvp->Cia402HomingState()){
				default:
        			case Controller_ns::CIA402_HOME_COMPLETED:
        			case Controller_ns::CIA402_HOME_PROGRESS:
					break;
        			case Controller_ns::CIA402_HOME_PERFORMED:
	                        	_mInitFSM = IFSM_WAIT;
					break;
			}
			break;

		case IFSM_WAIT:
			#ifdef _DBG_CIA402INIT
                                ss.str("");
                                ss << AxisIFSMName[_mInitFSM];
                                prtverbose(0, obj, ss.str());
                        #endif /* _DBG_CIA402INIT */

			// 
			// se la procedura sta' lavorando: aspetto
			//
			switch (drvp->Cia402HomingState()){
        			case Controller_ns::CIA402_HOME_PERFORMED:
        			case Controller_ns::CIA402_HOME_PROGRESS:
				default:
					break;

				// 
				// la procedura e' terminata con successo
				//
        			case Controller_ns::CIA402_HOME_COMPLETED:
					drvp->Cia402HomingEndRestore(_opmode);
                        		_mInitFSM = IFSM_DONE;
					return true;
					break;

				// 
				// la procedura e' terminata con errore
				//
        			case Controller_ns::CIA402_HOME_INTERRUPTED:
        			case Controller_ns::CIA402_HOME_ERROR_MOVING:
		                        Motor::stop();
        			case Controller_ns::CIA402_HOME_ERROR_STANDSTILL:
					drvp->Cia402HomingEndRestore(_opmode);
                                        _mInitFSM = IFSM_ERROR;
                                        _FSMerror = FSM_E106;
					break;
			}
			break;
		default:
			break;
	};

	return false; // false = deve ancora lavorare, oppure errore
}

} /* namespace Axis_ns */
