#include "Controller.h"

namespace Controller_ns {

						// 
						// controller = GALILDMC2xxx
						// 
						// target = tango://db_host:db_port/domain/family/member
						// target = tcp://host:port
						// target = serial://port:config
						// channel = A | B | C | D | E | F | G | H
						// extra: dipende dal tipo di controller che ne fara' il parsing 

void Controller::ControllerConstruct(ControllerType_t controller, string target, string extra, bool prtLog) {	
        	std::string obj("Controller::Controller()");
        	std::stringstream ss("");

        	_name = "CTRL";

        	#ifdef _DBG_CSTOR
                	prtverbose(0, obj, target);
        	#else
                	prtverbose(1, obj, target);
        	#endif

		_controller = controller;
		_target = target;
		_verbose = 0;
		_max_tmo = 10;	//max timeout : 10 sec
                _prtLog = prtLog;
		_drv = NULL;

		#ifdef _DBG_CTRL
	        	cout << "Controller::Controller()" << endl;
		#endif /* _DBG_CTRL */
		
		switch (_controller){
		  #ifdef _WITH_GALILDMC2XXXDRV_
                        case GALILDMC2xxx:
                                        _drv = new GalilDMC2xxxDrv(target, extra);
                                break;
		  #endif
		  #ifdef _WITH_YAMSDRV_
                        case Yams:
                                        _drv = new YamsDrv(target, extra);
                                        ((Controller_ns::YamsDrv*)_drv)->yidentity(Controller_ns::YAMS);
                                        _drv->lnk_flag(TANGO_GROUP_ATTRIBUTE_2);
                                break;
		  #endif
		  #ifdef _WITH_DMC2XXXPIEZODRV_
                        case DMC2xxxPiezo:
                                        _drv = new DMC2xxxPiezoDrv(target, extra);
                                break;
		  #endif
			default:
				break;
		  #ifdef _WITH_N5DRV_
			case N5:
					_drv = new N5Drv(target, extra);
				break;
		  #endif
		  #ifdef _WITH_SIMULDRV_
			case SIMUL:
					_drv = new SimulDrv(target, extra);
				break;
		  #endif
		  #ifdef _WITH_BCSDRV_
			case BCS:
					_drv = new BcsDrv(target, extra);
				break;
		  #endif
		}
		if (_drv) _drv->lnk_prtLog(_prtLog);
		#if defined(_DBG_CTRL) || defined(_DBG_CSTOR)
			cout << "Controller() status at creat: " << status() << ", drv pointer:" << _drv << endl;
			cout << " drv(): " << drv() << endl;
		#endif /* _DBG_CTRL */
}

Controller::~Controller() {	
	#ifdef _DBG_CLASSDESCTRUCTOR
		cout << "~Controller() : ";
	#endif
	if(_drv) {
		#ifdef _DBG_CLASSDESCTRUCTOR
			cout << "going to delete _drv ...  ";
		#endif
		delete _drv;
	}
	#ifdef _DBG_CLASSDESCTRUCTOR
		cout << endl;
	#endif
}

bool Controller::prtLog()      { return _prtLog; }
void Controller::prtLog(bool v){ _prtLog = v; if (_drv) _drv->lnk_prtLog(_prtLog);}


ControllerType_t Controller::controller()		{ return _controller; }
string	Controller::controller_typeName()		{
						  switch (_controller){
							case GALILDMC2xxx:
							case Yams:
							case DMC2xxxPiezo:
							case N5:
								return ControllerTypeName[_controller];
							default: return "Unknown controller type";
						  }
						}
	
string		Controller::target()		{ return _target; }	

string 			Controller::raw_io(string quest) 			{return _drv->raw_io(quest); }

opmode_t        Controller::opmode(string channel)		{ return (opmode_t)_drv->opmode(channel); }
bool            Controller::opmode(string channel, opmode_t op)	{ return _drv->opmode(channel, (int8_t)op); }

std::string     Controller::OpMode2str(opmode_t op)	{
        switch (op) {
                case -2 : return "Auto setup"; break;
                case -1 : return "Clock-direction mode"; break;
                case 0  : return "No mode change/no mode assigned"; break;
                case 1  : return "Profile Position Mode"; break;
                case 2  : return "Velocity Mode"; break;
                case 3  : return "Profile Velocity Mode"; break;
                case 4  : return "Profile Torque Mode"; break;
                case 5  : return "Reserved"; break;
                case 6  : return "Homing Mode"; break;
                case 7  : return "Interpolated Position Mode"; break;
                case 8  : return "Cyclic Synchronous Position Mode"; break;
                case 9  : return "Cyclic Synchronous Velocity Mode"; break;
                case 10 : return "Cyclic Synchronous Torque Mode"; break;
                default : return "Unknown mode!"; break;
        }
        return "Unknown mode!";
}


Link_ns::uvalues_t 	Controller::update(string channel)			{return _drv->update(channel);        }
Link_ns::rvalues_t 	Controller::refresh(string channel)			{return _drv->refresh(channel);       }

bool 			Controller::stop(string channel) 			{return _drv->stop(channel);          }
bool 			Controller::stop_no_brake(string channel) 		{return _drv->stop_no_brake(channel); }

bool 			Controller::speed(string channel, double v) 		{return _drv->speed(channel, v);      }
double 			Controller::speed(string channel) 			{return _drv->speed(channel);         }

bool 			Controller::acceleration(string channel, double a) 	{return _drv->acceleration(channel, a);}
double 			Controller::acceleration(string channel) 		{return _drv->acceleration(channel);   }
bool 			Controller::deceleration(string channel, double a) 	{return _drv->deceleration(channel, a);}
double 			Controller::deceleration(string channel) 		{return _drv->deceleration(channel);   }

bool 			Controller::forward(string channel, double v) 		{return _drv->forward(channel, v);    }
bool 			Controller::backward(string channel, double v) 		{return _drv->backward(channel, v);   }

bool 			Controller::findedge(string channel, double v) 		{return _drv->findedge(channel, v);   }
bool 			Controller::findindex(string channel, double v) 	{return _drv->findindex(channel, v);   }

double 			Controller::mpos(string channel) 			{return _drv->mpos(channel);          }
bool 			Controller::mpos(string channel, double p) 		{return _drv->mpos(channel, p);       }  // mpos : motor position

bool 			Controller::xpos(string channel, double p) 		{return _drv->xpos(channel, p);       }  // xpos : motor position in encoder units

bool 			Controller::rpos(string channel, double p) 		{return _drv->rpos(channel, p);       }  // rpos : motor relative position

bool            	Controller::defmpos(string channel, double p)		{return _drv->defmpos(channel, p); }
bool            	Controller::defepos(string channel, double p)		{return _drv->defepos(channel, p); }

bool            	Controller::brake(string channel, bool b)		{return _drv->brake(channel, b); }
bool			Controller::motor_on(string channel)			{ return _drv->motor_on(channel); }
bool			Controller::motor_off(string channel)			{ return _drv->motor_off(channel); }

int 			Controller::hw_brake_conf(string channel)		{ if (_drv) return _drv->hw_brake_conf(channel);
										  else {
											cout << "NULL DRV!" << endl;
											return 0;
										  }
									 	}

int 			Controller::verbose()					{ return _verbose;}
void 			Controller::verbose(int b)				{ _verbose = b;}

int 			Controller::drv_verbose()				{ return _drv->verbose();}
void 			Controller::drv_verbose(int b)				{
					                        		#ifdef _DBG_VERBOSE
											cout << "Controller.drv_verbose()" << endl;
										#endif
								 		_drv->verbose(b);
										}

int 			Controller::lnk_verbose()				{ return _drv->lnk_verbose();}
void 			Controller::lnk_verbose(int b){
				if (_drv){
		                        #ifdef _DBG_VERBOSE
						cout << "Controller.lnk_verbose()" << endl;
					#endif
					_drv->lnk_verbose(b);
				} else {
					cout << "Controller.lnk_verbose() NO DRIVER" << endl;
				}
}

int 			Controller::lnk_flag()					{ return _drv->lnk_flag();}
void 			Controller::lnk_flag(int b)				{ _drv->lnk_flag(b);}

string  		Controller::stop_code2str(unsigned long stop_code) 	{return _drv->stop_code2str(stop_code);}
string 			Controller::CC_code2str(unsigned long CC_code) 		{return _drv->CC_code2str(CC_code);}

Drv*			Controller::drv()					{ return _drv; }

bool 			Controller::hw_write_enable() 				{ return _drv->hw_write_enable(); }
void 			Controller::hw_write_enable(bool b) 			{ _drv->hw_write_enable(b); }

double 			Controller::max_tmo()					{ return _max_tmo; }

double 			Controller::updateDelay()				{ return _drv->updateDelay(); }

Link_ns::LinkState_t 	Controller::linkstate()					{ return _drv->linkstate(); }
string 			Controller::linkstatus()				{ return _drv->linkstatus(); }

ControllerState_t 	Controller::state()		{ switch (_drv->linkstate()){
							case Link_ns::LINK_CLOSE:
						  		return CTRL_CLOSE;
								break;
        						case Link_ns::LINK_TIMEOUTED:
        						case Link_ns::LINK_UNREACTIVE:
        						case Link_ns::LINK_OTHEREXCEPT:
        						case Link_ns::LINK_GENERALEXCEPT:
						  		return CTRL_DOWN;
								break;
							default:
								break;
						  }
						  return CTRL_UP;
						}
string			Controller::status()		{
						  switch(state()){
							case CTRL_CLOSE:
							case CTRL_UP:
							case CTRL_DOWN:
						  		return ControllerStateName[state()];
								break;
							default:
								return "CTRL_UNKNOWN";
								break;
						  };
						}
	

bool 			Controller::mconf(string channel, mconf_t mc) 		{ return _drv->mconf(channel, mc);}
mconf_t			Controller::mconf(string channel) 		  	{ return _drv->mconf(channel);}

bool 			Controller::econf(string channel, econf_t ec) 		{ return _drv->econf(channel, ec);}
econf_t			Controller::econf(string channel) 		  	{ return _drv->econf(channel);}


bool                    Controller::check_motion_end(unsigned long CC_code) {
                                                                  switch(_controller){
								   #ifdef _WITH_YAMSDRV_
                                                                    case Yams: return !((bool)CHECK_BIT(CC_code, BIT_YGO_END_NOK)); break;
								   #endif
								   #ifdef _WITH_DMC2XXXPIEZODRV_
                                                                    case DMC2xxxPiezo: return true; break;
							   	   #endif
								   #ifdef _WITH_N5DRV_
                                                                    case N5: return true; break;
							   	   #endif
								   #ifdef _WITH_SIMULDRV_
                                                                    case SIMUL: return true; break;
							   	   #endif
								   #ifdef _WITH_BCSDRV_
                                                                    case BCS: return !((bool)CHECK_BIT(CC_code, BIT_YGO_END_NOK)); break;
								   #endif
                                                                    default:
                                                                      break;
                                                                  }
                                                                  return false;
                                                                }

bool                    Controller::check_motion_progress(unsigned long CC_code) {
                                                                  switch(_controller){
								   #ifdef _WITH_YAMSDRV_
                                                                    case Yams:
								   #endif
								   #ifdef _WITH_N5DRV_
                                                                    case N5:
								   #endif
								   #ifdef _WITH_SIMULDRV_
                                                                    case SIMUL:
								   #endif
								   #ifdef _WITH_BCSDRV_
                                                                    case BCS:
								   #endif
								   #if defined(_WITH_YAMSDRV_) || defined(_WITH_N5DRV_) || defined(_WITH_SIMULDRV_) || defined(_WITH_BCSDRV_)
									return (bool) CHECK_BIT(CC_code, BIT_YGO_PROGRESS); break;
								   #endif
                                                                    default:
                                                                      break;
                                                                  }
                                                                  return false;
                                                                }

bool                    Controller::check_motion_cmd(unsigned long CC_code) {
                                                                  switch(_controller){
								   #ifdef _WITH_YAMSDRV_
                                                                    case Yams: return (bool) CHECK_BIT(CC_code, BIT_YGO_POS_CMD); break;
								   #endif
								   #ifdef _WITH_N5DRV_
								    case N5: return true; break;
								   #endif
								   #ifdef _WITH_SIMULDRV_
								    case SIMUL: return true; break;
								   #endif
								   #ifdef _WITH_BCSDRV_
								    //case BCS: return true; break;
								    case BCS: return (bool) CHECK_BIT(CC_code, BIT_YGO_POS_CMD); break;
								   #endif
                                                                    default:
                                                                      break;
                                                                  }
                                                                  return false;
                                                                }


} /* namespace Controller_ns */
