#ifdef _WITH_DMC2XXXPIEZODRV_
#include <iostream>
#include "DMC2xxxPiezoDrv.h"
#include "GalilTangoLink.h"

namespace Controller_ns {


DMC2xxxPiezoDrv::DMC2xxxPiezoDrv(string target, string extra): Drv(target, extra) {
//cout << "DMC2xxxPiezoDrv()" << endl;
        prtverbose(0, "DMC2xxxPiezoDrv::DMC2xxxPiezoDrv()", "");

        AXIS_OUT[0]=5;
        AXIS_OUT[1]=6;
        AXIS_OUT[2]=7;
        AXIS_OUT[3]=8;
        AXIS_OUT[4]=13;
        AXIS_OUT[5]=14;
        AXIS_OUT[6]=15;
        AXIS_OUT[7]=16;

        _idxchannel["A"] = 0;
        _idxchannel["B"] = 1;
        _idxchannel["C"] = 2;
        _idxchannel["D"] = 3;
        _idxchannel["E"] = 4;
        _idxchannel["F"] = 5;
        _idxchannel["G"] = 6;
        _idxchannel["H"] = 7;



	__hw_write_enable = false;

        Link_ns::LinkType t;
        size_t found;
        found = _target.find_first_of("tango://");
        if (found != string::npos ) t = Link_ns::TANGO_CHAN;
        else {
                found = _target.find_first_of("tcp://");
                if (found != string::npos ) t = Link_ns::TCP_CHAN;
                else {
                        found = _target.find_first_of("serial://");
                        if (found != string::npos ) t = Link_ns::SERIAL_CHAN;
                        else t = Link_ns::NULL_CHAN;
                }
        }

	switch (t){
        	case Link_ns::TANGO_CHAN:
                	_link = new Link_ns::GalilTangoLink(_target);
                        break;
		default:
        	case Link_ns::NULL_CHAN:
        	case Link_ns::SERIAL_CHAN:
        	case Link_ns::TCP_CHAN:
                	//_link = new Link_ns::GalilTcpLink(_target);
                        break;
        }


}

DMC2xxxPiezoDrv::~DMC2xxxPiezoDrv(){
	//cout << "~DMC2xxxPiezoDrv()" << endl;
	if (_link) delete _link;
}

bool 	DMC2xxxPiezoDrv::yinit(string channel){         // true = INIT / false = NO INIT
	std::stringstream ss;
	string obj("DMC2xxxPiezoDrv::yinit("+channel+")");
        prtverbose(1, obj, "");

        string n, ans;
        n.assign("Iinit");
        n.append(channel);
        n.append("=?");
        RAW_IO(1, 2, obj, n, ans, false)

	int r;
	float v;
	r = sscanf(ans.c_str(), "%f", &v);
	if (r == 1){
		if (v == 0) return false;
		else return true;
	}
	return false;
}

bool 	DMC2xxxPiezoDrv::yinit(string channel, bool b){ // true = INIT / false = DEINIT
	std::stringstream ss;
	string obj("DMC2xxxPiezoDrv::yinit("+channel+",.)");
        prtverbose(1, obj, "");

        string n, ans;
        n.assign("Iinit");
        n.append(channel);
        n.append("=");
        char buff[10];
        if (b) snprintf(buff, 10, "1");
	else   snprintf(buff, 10, "0");
        n.append(buff);
        RAW_IO(1, 2, obj, n, ans, false)

	return true;
}



bool    DMC2xxxPiezoDrv::mconf(string channel, mconf_t mc){
	std::stringstream ss;
	string obj("DMC2xxxPiezoDrv::mconf("+channel+",.)");
        prtverbose(1, obj, "");

        string n, ans;
        n.assign("MT");
        n.append(channel);
        n.append("=1");
        n.append(";");
        n.append("MO");
        n.append(channel);

        RAW_IO(1, 2, obj, n, ans, false)

	return true;
}

mconf_t DMC2xxxPiezoDrv::mconf(string channel){
	std::stringstream ss;
	string obj("DMC2xxxPiezoDrv::mconf("+channel+")");
        prtverbose(1, obj, "");

	mconf_t mc;

	mc.pc.mt = nan("");
	mc.pc.mo = false;

        string n, ans;
        n.assign("MT");
        n.append(channel);
        n.append("=?");
        RAW_IO(1, 2, obj, n, ans, mc)

	int r;
	float v;
	r = sscanf(ans.c_str(), "%f", &v);
	if (r == 1)
		mc.pc.mt = (double)v;

        n.assign("MG _MO");
        n.append(channel);
        RAW_IO(1, 2, obj, n, ans, mc)

	r = sscanf(ans.c_str(), "%f", &v);
	if (r == 1){
		if ( v == 1 ) mc.pc.mo = true; 	// motor OFF
		else mc.pc.mo = false;		// motor ON
	}

	ss.str("");
	ss <<  mc.pc.mt << " " << mc.pc.mo;
        prtverbose(1, obj, ss.str());
	return mc;
}

Link_ns::rvalues_t DMC2xxxPiezoDrv::refresh(string channel){
	std::stringstream ss;
	string obj("DMC2xxxPiezoDrv::refresh("+channel+")");
        prtverbose(1, obj, "");

        timeval start, end;
        gettimeofday(&start, NULL);

        Link_ns::rvalues_t rv;

        gettimeofday(&end, NULL);
        _last_refresh_ms = (double)(end.tv_sec - start.tv_sec)*1000 + ((double)(end.tv_usec - start.tv_usec))/1000;

        return rv;
}

Link_ns::uvalues_t DMC2xxxPiezoDrv::update(string channel){
	std::stringstream ss;
	string obj("DMC2xxxPiezoDrv::update("+channel+")");
        prtverbose(1, obj, "");

	Link_ns::uvalues_t uv;
	uv = Drv::update(channel);
	uv.motor = nan("");

        string n, ans;
        n.assign("MG _OF");
        n.append(channel);

        RAW_IO(1, 2, obj, n, ans, uv)

	int r;
	float v;
	r = sscanf(ans.c_str(), "%f", &v);
	if (r == 1)
		uv.motor = (double)v * 10000;


	//++++++
	uv.status = Link_ns::ERROR;

        char buff[100];
        char nl='\n';
        char cr='\r';

        n.assign("MG @OUT[");
        snprintf(buff, 100, "%d", AXIS_OUT[_idxchannel[channel]]);
        n.append(buff); n.append("]");

        RAW_IO(1, 2, obj, n, ans, uv)

        ans.erase(std::remove(ans.begin(),ans.end(),nl),ans.end());
        ans.erase(std::remove(ans.begin(),ans.end(),cr),ans.end());
        ans.erase(std::remove(ans.begin(),ans.end(),' '),ans.end());
        prtverbose(2, obj, "ans="+ans);

        float a1;
        sscanf(ans.c_str(), "%f", &a1);
        if ( a1 == 0 ) {
		uv.status = Link_ns::SERVO;
		//cout << "Link_ns::SERVO" << endl;
	} else {
		uv.status = Link_ns::SWCTL;
		//cout << "Link_ns::SWCTL" << endl;
	}

	//cout << "-------------------- OUT = " << a1 << endl;

	//++++++

	uv.fls    = false;
        uv.rls    = false;
	uv.stop_code = 1;

	return uv;
}

bool    DMC2xxxPiezoDrv::mpos(string channel, double p){       // mpos : motor position
	string obj("DMC2xxxPiezoDrv::mpos("+channel+",.)");
	std::stringstream ss;
	ss << "(p=" << p << ")";
        prtverbose(1, obj, ss.str());

	if (!__hw_write_enable) return true;

        string n, ans;
        n.assign("OF");
        n.append(channel);
        n.append("=");

        char buff[100];
        snprintf(buff, 100, "%7.4f", ((float)p/10000));
        n.append(buff);

        RAW_IO(1, 2, obj, n, ans, false)

	return true;
}

double  DMC2xxxPiezoDrv::mpos(string channel){
	std::stringstream ss;
	string obj("DMC2xxxPiezoDrv::mpos("+channel+")");
        prtverbose(1, obj, "");

	if (!__hw_write_enable) return true;

	double mpos=nan("");

        string n, ans;
        n.assign("MG _OF");
        n.append(channel);

        RAW_IO(1, 2, obj, n, ans, nan(""))

	int r;
	float v;
	r = sscanf(ans.c_str(), "%f", &v);
	if (r == 1)
		mpos = (double)v * 10000;
	return mpos;
}

bool    DMC2xxxPiezoDrv::rpos(string channel, double p){       // rpos : motor relative position
	string obj("DMC2xxxPiezoDrv::rpos("+channel+",.)");
	std::stringstream ss;
	ss << "(p=" << p << ")";
        prtverbose(1, obj, ss.str());

	if (!__hw_write_enable) return true;

	return false;
}


int     DMC2xxxPiezoDrv::cnt_io(string quest){
	string obj("DMC2xxxPiezoDrv::cnt_io("+quest+")");
        prtverbose(1, obj, "");

	if (quest.size() == 0) return 1;  //almeno il crnl viene poi inviato.....
	return 1+std::count(quest.begin(), quest.end()-1, ';'); // un ; alla fine non conta
}


int     DMC2xxxPiezoDrv::chk_io(string ans){
	string obj("DMC2xxxPiezoDrv::chk_io()");
        prtverbose(1, obj, "on "+ans);

	if (ans.size() == 0) return 0;
	size_t question = std::count(ans.begin(), ans.end(), '?');
	size_t colon = std::count(ans.begin(), ans.end(), ':');

	std::stringstream ss;
	ss << "ans=" << ans << " || question= " << question << " colon= " << colon;
        prtverbose(2, obj, ss.str());

	if (question !=0) return -question;
	else return colon;
}

string  DMC2xxxPiezoDrv::raw_io(string quest)    {
	string obj("DMC2xxxPiezoDrv::raw_io()");
        prtverbose(1, obj, "cmd="+quest);
	std::stringstream ss;

	int c, r;

	c = cnt_io(quest);

	ss.str("");
	ss << "#; = " << c;
        prtverbose(2, obj, ss.str());

	string ans = _link->raw_io(quest);
	r = chk_io(ans);

	ss.str("");
	ss << "#(:?) = " << r;
        prtverbose(2, obj, ss.str());

	// facendo cosi' in "due parti", ovvero se c'e' un ? allora faccio una altra
	// raw_io di lettura codice di errore. puo' essre che tra la prima e questa seconda
	// raw_io, il server axisg2drv riceva una raw_io da un altro e ce quindi il codice di 
	// errore non si riferisca piu' alla prima raw_io fatta in questa funzione.
	// sarebbe meglio fare una unica raw_io con ache TC...

	if ( r < 0 ) {
		string errmsg = _link->raw_io("TC 1", false);
		char buff[100];
		snprintf(buff, 100, "Sent %d cmds, rcved %d bad", c, -r);
		throw Controller_ns::DrvExcpt(Controller_ns::RAWIO, buff, ans, errmsg);
	};
	return ans;
}

bool    DMC2xxxPiezoDrv::defmpos(string channel, double p){
	std::stringstream ss;
	string obj("DMC2xxxPiezoDrv::defmpos("+channel+",.)");
	ss << "p= " << p;
        prtverbose(1, obj, ss.str());

	if (!__hw_write_enable) return true;

	return true;
}


// define hw encoder position
bool    DMC2xxxPiezoDrv::defepos(string channel, double p){
	std::stringstream ss;
	string obj("DMC2xxxPiezoDrv::defepos("+channel+",.)");
	ss << "p= " << p;
        prtverbose(1, obj, ss.str());

	if (!__hw_write_enable) return true;

	return true;
}


bool    DMC2xxxPiezoDrv::program_running(){
	std::stringstream ss;
	string obj("DMC2xxxPiezoDrv::program_running()");
        prtverbose(1, obj, "");


	string n, ans;
	n.assign("TB");

	RAW_IO(1, 2, obj, n, ans, false) 

	int v;
	if ( 1 != sscanf(ans.c_str(), "%d", &v) ) return false;
		
	if (v & 0x80) return true;

	return false;
}

bool    DMC2xxxPiezoDrv::xpos(string channel, double p) {  // xpos : motor position in encoder units
	return true;
}      


} /* namespace Controller_ns */
#endif /* _WITH_DMC2XXXPIEZODRV_ */
