#include <iostream>
#include "Encoder.h"

#include <stdlib.h>
#include <string>
#include <algorithm>
#include <values.h>

namespace Encoder_ns {

/*===============================
*
*
*
*===============================*/
Encoder::Encoder(string name, string channel, Controller_ns::Controller *controller) {

	_verbose = 0;
        _name = name;

        #ifdef _DBG_CSTOR
                prtverbose(0, "Encoder::Encoder()", "");
        #else
                prtverbose(1, "Encoder::Encoder()", "");
        #endif

	_channel = channel;
	_controller = controller;

	//_controller->drv_verbose(true);

	_last_update.tv_sec  = 0;
	_last_update.tv_usec = 0;

	_epos = 0;
	_epos_old = 0;

	_oldevelocity.resize(3);
	_oldevelocity[0] = 0;
	_oldevelocity[1] = 0;
	_oldevelocity[2] = 0;
		
	_etype = ENC_NO;
	_precision = 1;

	_ear = 1;
	_eao = 0;

        _mOffOn.enabled = false;
        _mOffOn.m = nan("");
        _mOffOn.q = nan("");

	_ripple_on_stay = 0;
	_ripple_on_run = 0;

	_sencsign.startPoint=LIMIT_RLS;
/*
typedef struct {
        EncoderLimit_t startPoint;
        double SencRes;
        double SencPitch;
} SencParams_t;
*/
	_sencparams.startPoint=LIMIT_RLS;
	_sencparams.SencRes = 0.5;
	_sencparams.SencPitch = 20;

	_refwidth = 1;

}

/*===============================
*
*
*===============================*/
bool    Encoder::eupdate(Link_ns::uvalues_t uv, double dt){
	string obj("Encoder::eupdate()");
	stringstream ss;
	

	// ENCODER VALUE
	switch (_etype){
        	case ENC_DIGITAL_INCREMENTAL:
        	case ENC_DIGITAL_ABSOLUTE:
        	case ENC_DIGITAL_SEMIABSOLUTE:
			epos(uv.encoder);
			break;

        	case ENC_ANALOG_ABSOLUTE:
			ss.str("");
			ss << "_mOffOn.enabled=" << _mOffOn.enabled << " _mOffOn.m=" << _mOffOn.m << " _mOffOn.q=" << _mOffOn.q;
			prtverbose(1, obj, ss.str());

			ss.str("");
			if( (_mOffOn.enabled) && (uv.status==Link_ns::OFF) ){
				ss << "IN= " << uv.analog << " OUT= " << ( _mOffOn.m * uv.analog + _mOffOn.q );
				prtverbose(1, obj, ss.str());
				epos( _mOffOn.m * uv.analog + _mOffOn.q );
			} else {
				ss << "IN= " << uv.analog << " OUT= " << uv.analog ;
				prtverbose(1, obj, ss.str());
				epos(uv.analog);
			}
			break;

		default:
		case ENC_NO:
			epos(uv.motor);
			break;
	}
	// ENCODER SPEED
	switch (_etype){
        	case ENC_NO:
        	case ENC_ANALOG_ABSOLUTE:
			if (dt > (double)0 ){
				_oldevelocity[2] = _oldevelocity[1];
				_oldevelocity[1] = _oldevelocity[0];
				_oldevelocity[0] = (_epos - _epos_old)/dt;
				_evelocity = (_oldevelocity[0] + _oldevelocity[1] + _oldevelocity[2])/3;
			} else {
				_evelocity = nan("");
				_oldevelocity[2] = _oldevelocity[1] = _oldevelocity[0] = 0;
			}
			break;

        	case ENC_DIGITAL_ABSOLUTE:
        	case ENC_DIGITAL_SEMIABSOLUTE:
        	case ENC_DIGITAL_INCREMENTAL:
			if (dt > (double)0 ){
				_evelocity = uv.velocity;
			}
			break;
		default:
			break;
	}
	_epos_old = _epos;

	return true;
}

/*===============================
*
Link_ns::
typedef struct {
        double motor;
        double encoder;
        double velocity;
        double analog;
        bool fls;
        bool rls;
        bool home;
        unsigned long stop_code;
        unsigned long CC_code;
        status_t status;
} uvalues_t;
*
*
*===============================*/
bool    Encoder::update(){   // update all relevant 'current' values
	prtverbose(1, "Encoder::update()", "");

	struct timeval t;
	gettimeofday(&t, NULL);
	double t_s = (double)(t.tv_sec-_last_update.tv_sec)+((double)(t.tv_usec-_last_update.tv_usec))/1000000;
	if ( t_s < _controller->updateDelay() ) return false;

	_last_update.tv_sec  = t.tv_sec;
	_last_update.tv_usec = t.tv_usec;

	Link_ns::uvalues_t uv = _controller->update(_channel);
	return eupdate(uv, t_s);
}

/*===============================
*
*
*
*===============================*/
bool Encoder::epos(double v){
	string obj("Encoder::epos(.)");
	stringstream ss;
	ss << "v=" << v;
	prtverbose(1, obj, ss.str());

	switch (_etype){
        	case ENC_ANALOG_ABSOLUTE:
			_epos = analog2enc(v);
			break;

		default:
        	case ENC_DIGITAL_INCREMENTAL:
        	case ENC_DIGITAL_ABSOLUTE:
        	case ENC_DIGITAL_SEMIABSOLUTE:
		case ENC_NO:
			_epos = v;
			break;
	}
	return true;
}


/*===============================
*
*
*
*===============================*/
bool Encoder::defepos(double p){
	string obj("Encoder::defepos(.)");
	std::stringstream ss;
	ss << "e= " << p;
	prtverbose(1, obj, ss.str());

	switch (_etype){
        	case ENC_NO:
        	case ENC_DIGITAL_ABSOLUTE:
        	case ENC_DIGITAL_SEMIABSOLUTE:
        	case ENC_ANALOG_ABSOLUTE:
			return true;
			break;
        	case ENC_DIGITAL_INCREMENTAL:
			return _controller->defepos(_channel, p);
			break;
		default:
			return false;
			break;
	}
	return false;
}

/*===============================
*
*
*
*===============================*/
#if 0 /* NON serve piu', funzionalita' spostate in eupdate(.,.) */
bool Encoder::evalues(double e, double ev, double dt){
	string obj("Encoder::epos(.,.)");
	std::stringstream ss;
	ss << "e= " << e;
	ss << "; dt = " << dt;
	prtverbose(1, obj, ss.str());

	_epos = e;

	switch (_etype){
        	case ENC_NO:
        	case ENC_ANALOG_ABSOLUTE:
			if (dt != (double)0 ){
				_oldevelocity[2] = _oldevelocity[1];
				_oldevelocity[1] = _oldevelocity[0];
				_oldevelocity[0] = (_epos - _epos_old)/dt;
				_evelocity = (_oldevelocity[0] + _oldevelocity[1] + _oldevelocity[2])/3;
			} else {
				_evelocity = nan("");
				_oldevelocity[2] = _oldevelocity[1] = _oldevelocity[0] = 0;
			}
			break;

        	case ENC_DIGITAL_ABSOLUTE:
        	case ENC_DIGITAL_SEMIABSOLUTE:
        	case ENC_DIGITAL_INCREMENTAL:
			_evelocity = ev;
			break;
		default:
			return false;
			break;
	}
	_epos_old = _epos;
	return true;
}
#endif /* 0 */

/*===============================
*
*
*
*===============================*/
bool Encoder:: delay_on_check(double delay, double update_cycle){
	double v;
	int w;

	_delay_on_check = delay;

	if ( update_cycle > 0 ) v = delay/update_cycle;
	else v = 3;
	w = (int)v;
	w = w>0?w:1;
	_n_delay_on_check = w;
	return true;
}

/*===============================
*
*
*
*===============================*/
double Encoder::analog2enc(double a){

	return (_ear * a + _eao);
}

/*===============================
*
*
*
*===============================*/
double Encoder::enc2analog(double e) {

	return (e - _eao)/_ear;
}


/*===============================
*
*
*
*===============================*/
bool Encoder::SencAbsPos(double _epos0, double _epos1){
	string obj("Encoder::SencAbsPos(.,.,.)");
	std::stringstream ss;
	ss << " _epos0= " << _epos0;
	ss << " _epos1= " << _epos1;
	prtverbose(1, obj, ss.str());

	#ifdef _DBG_SENC_ABS
		cout << "_epos0= " << _epos0 << " _epos1= " << _epos1 << endl;
	#endif

	SencParams_t sp = SencParams();
	double SencRes = sp.SencRes;
	double SencPitch = sp.SencPitch;

	double p;
	double diff = _epos1 - _epos0;
	double nc;


	if (diff <0) {
		nc =round((diff*SencRes+10000)/SencPitch);
		if ( nc<0 ) p = -nc*20000 ; else
		if ( nc>0 ) p = (nc*(20000+SencPitch)+10000) ; else p = nan("");
	} else 
	if (diff >0) {
		nc =round((diff*SencRes-10000)/SencPitch);
		if ( nc<0 ) p = ((-nc+1)*20000) ; else
		if ( nc>0 ) p = (nc*(20000+SencPitch)+10000) ; else p = nan("");
	} else return false;
	
	p /= SencRes;

	#ifdef _DBG_SENC_ABS
		cout << "nc = " << nc << endl;
		cout << "p/SencRes= " << p << endl;
	#endif
	
	if ( !isnan(p) ) return _controller->defepos(_channel, p); else return false;
}

/*===============================
*
*
*
*===============================*/
SencSignature_t Encoder::SencSignature(){

	return _sencsign;
}

/*===============================
*
*
*
*===============================*/
bool Encoder::SencSignature(SencSignature_t ss){

	//_sencsign = ss;
	return false;
}

/*===============================
*
*
*
*===============================*/
/*
typedef struct {
        EncSignatureError_t err;
        double abs_error;
        int enc_dir;
        int ix_dir;
} CheckSignature_t;
*/
CheckSignature_t Encoder::checkSencSignature(SencSignature_t ss){

	CheckSignature_t ret;
	ret.err = ENCSIGN_GENERIC_ERROR;
	ret.abs_error = nan("");
	ret.enc_dir = 0;
	ret.ix_dir = 0;

	std::vector<double> diff;

	std::vector<double>::iterator previt;
	std::vector<double>::iterator it=(ss.values).begin();
	#ifdef _DBG_SENCSIGN_CHECK
        	cout << *it << endl;
		cout << "-----" << endl;
	#endif
	diff.push_back(nan(""));
	previt = it;
	it++;
	
	for (; it != (ss.values).end(); ++it){
		#ifdef _DBG_SENCSIGN_CHECK
        		cout << *it << endl;
		#endif
		diff.push_back((*it) - (*previt));
		previt = it;
	}
	#ifdef _DBG_SENCSIGN_CHECK
		cout << " -- diff -- " << endl;
		for (it=diff.begin(); it != diff.end(); ++it){
        		cout << *it << endl;
		}
	#endif

	// cerco se "tutti" positivi o negativi:
	// scarto il primo = nan
	// scarto il secondo che puo' non essere completo = LS troppo vicino alla tacca
	// scarto l'ultimo che puo' non essere completo = tacca al di la' del LS

	if (diff.size() < 4) {
		cout << "Unable to check signature, too few elements!" << endl;
		ret.err = ENCSIGN_FEWELEMENTS_ERROR;
		return ret;
	}

	int s = 0;
	for (it=diff.begin()+2; it != diff.end()-1; ++it){
		if (s==0) {
			if ( (*it) < 0 ) s = -1;
			if ( (*it) > 0 ) s = 1;
		} else {
			if ( (s == 1) && ((*it) < 0) ) {
				cout << "Error: all signs must be equals!" << endl;
				ret.err =  ENCSIGN_DIFFSIGNS_ERROR;
				return ret;
			}
			if ( (s == -1) && ((*it) > 0) ) {
				cout << "Error: all signs must be equals!" << endl;
				ret.err =  ENCSIGN_DIFFSIGNS_ERROR;
				return ret;
			}
		}
        }
	cout << "All signs are consistent!: " << (s==1?"POSITIVE":"NEGATIVE") << endl;
	
	ret.enc_dir = s;

	std::vector<int> nc;
	int _nc;

	SencParams_t sp = SencParams();
	double SencRes = sp.SencRes;
	double SencPitch = sp.SencPitch;

	double tmp, error, abs_error;
	double min_err = MAXDOUBLE;
	double max_err = -MAXDOUBLE;
	nc.push_back(0);
	for (it=diff.begin()+1; it != diff.end()-1; ++it){
		tmp = ((*it)*SencRes-s*10000)/SencPitch;
		_nc = round(tmp);
		nc.push_back(_nc);
		if (tmp != 0 ) {
			error = ( tmp - (double)_nc ) / tmp * 100 ;
			if ( error < min_err ) min_err = error ;
			if ( error > max_err ) max_err = error ;
		} else error = nan("");
		#ifdef _DBG_SENCSIGN_CHECK
			cout << "n = " << tmp << " ";
			cout << "nc = " << _nc << " ";
			cout << "error " << error << "[%] ";
			cout << endl;
		#else
			cout << "diff= " << (*it) << " nc= " << _nc << endl;
		#endif

	}
	abs_error = fabs(max_err);
	if ( fabs(min_err) > abs_error ) abs_error = fabs(min_err);
	#ifdef _DBG_SENCSIGN_CHECK
		cout << "min_err= " << min_err << " max_err= " << max_err;
		cout << endl;
	#endif
	ret.abs_error = abs_error;
	cout << "abs_error= +/-" << abs_error << "[%]" << endl;
	
	//========================================================================================
	//
	//
	#ifdef _DBG_SENCSIGN_CHECK
		cout << endl;
		cout << " -- -- check index correct sequence n -n (n+1) -(n+1) -- -- " << endl;
		cout << " -- --      and find index direction (ixdir)          -- -- " << endl;
		cout << endl;
		{
		std::vector<int>::iterator iit0;
		//
		//nc[0] non contiene un valido indice, pertanto non va considerato: iit0=nc.begin()+1
		//
		for (iit0=nc.begin()+1; iit0!= nc.end(); ++iit0){
			cout << "*iit0= " << *iit0 << " ";
		}
		cout << endl;
		cout << "nc.size()= " << nc.size() << endl;
		}
		cout << endl;
	#endif

	//check index correct sequence n -n (n+1) -(n+1)
	// and find index direction (ixdir)
	//
	// supp n>0
	// previit iit 		
	//   n     -n   : ok, ixdir = + 
	//  -n    -n+1  : ok, ixdir = +
	//  -n	    n   : ok, ixdir = -
	//   n    -n+1  : ok, ixdir = -
	//
	std::vector<int>::iterator iit, previit;
	previit = nc.begin()+1;
	iit = previit + 1;
	int ixdir = 0;
	int _ixdir;

	if ( nc.size() > 4 ) {
//..............|
		for (; iit != nc.end()-1; ++iit){
			_ixdir = 0;
			#ifdef _DBG_SENCSIGN_CHECK
				cout << "*previit= " << *previit << " ";
				cout << "*iit= " << *iit << " ";
				cout << endl;
			#endif
			if ( abs(*previit) == abs(*iit) ){
				//skip 
				#ifdef _DBG_SENCSIGN_CHECK
					cout << "skip for abs()=="  << endl;
				#endif
			} else {
				if ( ((*previit)<0) && ((*iit)==(-(*previit)-1)) ) _ixdir = -1; else
				if ( ((*previit)>0) && ((*iit)==(-(*previit)+1)) ) _ixdir = -1; else
				if ( ((*previit)>0) && ((*iit)==(-(*previit)-1)) ) _ixdir =  1; else
				if ( ((*previit)<0) && ((*iit)==(-(*previit)+1)) ) _ixdir =  1; else _ixdir = 0;

				if (ixdir==0) ixdir = _ixdir;
				else if (ixdir != _ixdir) {
					cout << "ixdir error!" << endl;
					ret.err =  ENCSIGN_IXDIR_ERROR;
					return ret;
				}

				#ifdef _DBG_SENCSIGN_CHECK
					cout << "ixdir= " << ixdir << " _ixdir=" << _ixdir << endl;
				#endif
			}

			previit = iit;
		}
//..............|
	} else {
		#ifdef _DBG_SENCSIGN_CHECK
			cout << "Too few values for this check!" << endl;
		#endif
		ixdir = 0;
		ret.ix_dir = ixdir;
		ret.err =  ENCSIGN_FEWELEMENTS_ERROR;
		return ret;
	}

	cout << "ixdir= " << ixdir << endl;

	ret.ix_dir = ixdir;

	if ( (s==1 && ixdir==1) || (s==-1 && ixdir==-1) ){
		// current enc dir & ixdir OK, the same
		for (int k=1; k<(int)((ss.values).size())-1; k++){
			cout << "Enc= " << (ss.values)[k] << " ";
			cout << "diff= " << diff[k] << " ";
			cout << "nc= " << nc[k] << " ";
			if ( (diff[k]<0) && (nc[k]<0) ) cout << "abs= " << (-nc[k]*20000) << " ";
			if ( (diff[k]<0) && (nc[k]>0) ) cout << "abs= " << (nc[k]*(20000+SencPitch)+10000) << " ";
			if ( (diff[k]>0) && (nc[k]<0) ) cout << "abs= " << ((-nc[k]+1)*20000) << " ";
			if ( (diff[k]>0) && (nc[k]>0) ) cout << "abs= " << (nc[k]*(20000+SencPitch)+10000) << " ";
			cout << endl;
		}
		ret.err =  ENCSIGN_OK;
		return ret;
	} else {
		// current enc dir & ixdir opposie => NOT OK,
		cout << "Please reconfigure the encoder with the opposite direction!" << endl;
		ret.err =  ENCSIGN_ENC_IXDIR_ERROR;
		return ret;
	}


	ret.err =  ENCSIGN_GENERIC_ERROR;
	return ret;
}


/*===============================
*
*
*
*===============================*/
/*
typedef struct {
        EncoderLimit_t startPoint;
        double SencRes;
        double SencPitch;
} SencParams_t;
*/
SencParams_t Encoder::SencParams(){
	#ifdef _DBG_SENC_PARAMS
		cout << "SencParams()" << endl;
	#endif
	SencParams_t sp;
	sp.startPoint = _sencparams.startPoint;
	sp.SencRes    = _sencparams.SencRes;
	sp.SencPitch  = _sencparams.SencPitch;
	return sp;
}

/*===============================
*
*
*
*===============================*/
bool Encoder::SencParams(SencParams_t param){
	#ifdef _DBG_SENC_PARAMS
		cout << "SencParams(.)" << endl;
	#endif
	
	_sencsign.startPoint   = param.startPoint;;
	_sencparams.startPoint = param.startPoint;
	_sencparams.SencRes    = param.SencRes;
	_sencparams.SencPitch  = param.SencPitch;

	return true;
}


} /* namespace Encoder_ns */
