#ifdef _WITH_GALILDMC2XXXDRV_
#include <iostream>
#include "GalilDMC2xxxDrv.h"
#include "GalilTangoLink.h"

namespace Controller_ns {


GalilDMC2xxxDrv::GalilDMC2xxxDrv(string target, string extra): Drv(target, extra) {
	#ifdef _DBG_GALILDMC2XXXDRV
		_verbose = 1;
	#else
		_verbose = 0;
	#endif

	_name = target;

	std::stringstream ss("");

        ss << " _verbose = " << _verbose;

       	prtverbose(1, "GalilDMC2xxxDrv::GalilDMC2xxxDrv()", ss.str());


	__hw_write_enable = false;

        Link_ns::LinkType t;
        size_t found;
        found = _target.find_first_of("tango://");
        if (found != string::npos ) t = Link_ns::TANGO_CHAN;
        else {
                found = _target.find_first_of("tcp://");
                if (found != string::npos ) t = Link_ns::TCP_CHAN;
                else {
                        found = _target.find_first_of("serial://");
                        if (found != string::npos ) t = Link_ns::SERIAL_CHAN;
                        else t = Link_ns::NULL_CHAN;
                }
        }

	switch (t){
        	case Link_ns::TANGO_CHAN:
                	_link = new Link_ns::GalilTangoLink(_target);
                        break;
		default:
        	case Link_ns::NULL_CHAN:
        	case Link_ns::SERIAL_CHAN:
        	case Link_ns::TCP_CHAN:
                	//_link = new Link_ns::GalilTcpLink(_target);
                        break;
        }

	ss.str("");

	if (_link){
        	ss << " lnk_verbose = " << _link->verbose();
	} else {
		ss << " NO LINK";
	}

       	prtverbose(1, "GalilDMC2xxxDrv::GalilDMC2xxxDrv()", ss.str());
	
}

GalilDMC2xxxDrv::~GalilDMC2xxxDrv(){
        #ifdef _DBG_CLASSDESCTRUCTOR
                cout << "~GalilDMC2xxxDrv() : ";
        #endif
	if (_link) {
        	#ifdef _DBG_CLASSDESCTRUCTOR
                	cout << "going to delete _link ...  ";
        	#endif
		delete _link;
	}
        #ifdef _DBG_CLASSDESCTRUCTOR
                cout << endl;
        #endif
}




bool GalilDMC2xxxDrv::stop(string channel){
	string obj("GalilDMC2xxxDrv::stop("+channel+")");
        //prtverbose(1, obj, "");
	std::stringstream ss;

	if (!__hw_write_enable) return true;

	string n, ans;
	n.assign("ST");
        n.append(channel);
	
	RAW_IO(1, 2, obj, n, ans, false)

	return true;
}


bool GalilDMC2xxxDrv::speed(string channel, double v){
	string obj("GalilDMC2xxxDrv::speed("+channel+",.)");
	std::stringstream ss;
	ss << "(speed=" << v << ")";
	prtverbose(1, obj, ss.str());

	if (!__hw_write_enable) return true;

	string n, ans;
	n.assign("SP");
        n.append(channel);
        n.append("=");
	
	char buff[100];
	if ( fabs(v) >= MIN_SPx_VALUE ){
		snprintf(buff, 100, "%d", (int)fabs(v));
	} else {
		snprintf(buff, 100, "%d", (int)fabs(MIN_SPx_VALUE));
	};
        n.append(buff);
	
	RAW_IO(1, 2, obj, n, ans, false) 
	
	return true;
}

double GalilDMC2xxxDrv::speed(string channel){
	std::stringstream ss;
	string obj("GalilDMC2xxxDrv::speed("+channel+")");
        prtverbose(1, obj, "");

	string n, ans;
	n.assign("SP");
        n.append(channel);
        n.append("=?");

	RAW_IO(1, 2, obj, n, ans, nan("")) 

	char *endptr;
	double speed;
	speed = strtod(ans.c_str(), &endptr);
        if ( endptr == ans.c_str())  speed = nan("");
	return speed;
}

bool GalilDMC2xxxDrv::forward(string channel, double v){
	std::stringstream ss;
	string obj("GalilDMC2xxxDrv::forward("+channel+")");
        prtverbose(1, obj, "");

	if (!__hw_write_enable) return true;

	string n, ans;
	n.assign("JG");
        n.append(channel);
        n.append("=");
	
	char buff[100];
	snprintf(buff, 100, "%d", (int)fabs(v));
        n.append(buff);
        n.append(";BG");
        n.append(channel);
	
	RAW_IO(1, 2, obj, n, ans, false) 

        prtverbose(2, obj, "exit!");
	return true;
}

bool GalilDMC2xxxDrv::backward(string channel, double v){
	std::stringstream ss;
	string obj("GalilDMC2xxxDrv::backward("+channel+")");
        prtverbose(1, obj, "");

	if (!__hw_write_enable) return true;

	string n, ans;
	n.assign("JG");
        n.append(channel);
        n.append("=");
	
	char buff[100];
	snprintf(buff, 100, "-%d", (int)fabs(v));
        n.append(buff);
        n.append(";BG");
        n.append(channel);
	
	RAW_IO(1, 2, obj, n, ans, false) 

        prtverbose(2, obj, "exit!");
	return true;

}

Link_ns::rvalues_t GalilDMC2xxxDrv::refresh(string channel){
	std::stringstream ss;
	string obj("GalilDMC2xxxDrv::refresh("+channel+")");
        prtverbose(1, obj, "");

        timeval start, end;
        gettimeofday(&start, NULL);

        Link_ns::rvalues_t rv;

        //double speed;
        //double acceleration;
        //double deceleration;

        rv.speed        = nan("");
        rv.acceleration = nan("");
        rv.deceleration = nan("");


        try {
        	string n, ans;
                char *endptr;

                n.assign("SP");
                n.append(channel);
                n.append("=?");
                ans = raw_io(n);
                rv.speed = strtod(ans.c_str(), &endptr);
                if ( endptr == ans.c_str())  rv.speed = nan("");

                n.assign("AC");
                n.append(channel);
                n.append("=?");
                ans = raw_io(n);
                rv.acceleration = strtod(ans.c_str(), &endptr);
                if ( endptr == ans.c_str())  rv.acceleration = nan("");

                n.assign("DC");
                n.append(channel);
                n.append("=?");
                ans = raw_io(n);
                rv.deceleration = strtod(ans.c_str(), &endptr);
                if ( endptr == ans.c_str())  rv.deceleration = nan("");

	} catch (...) {
        }

        gettimeofday(&end, NULL);
        _last_refresh_ms = (double)(end.tv_sec - start.tv_sec)*1000 + ((double)(end.tv_usec - start.tv_usec))/1000;

        return rv;
}


string  GalilDMC2xxxDrv::stop_code2str(unsigned long stop_code){

	switch (stop_code){
		case   0: return "Motors are running, independent mode"; break;
		case   1: return "Motors decelerating or stopped at commanded independent position"; break;
		case   2: return "Decelerating or stopped by FWD limit switch or soft limit FL"; break;
		case   3: return "Decelerating or stopped by REV limit switch or soft limit BL"; break;
		case   4: return "Decelerating or stopped by Stop Command (ST)"; break;
		case   6: return "Stopped by Abort input"; break;
		case   7: return "Stopped by Abort Command (AB)"; break;
		case   8: return "Decelerating or stopped by On-off-Error (EOI)"; break;
		case   9: return "Stopped after Finding Edge (FE)"; break;
		case  10: return "Stopped after Homing/Find Index (HM/FI)"; break;
		case  11: return "Stopped by Selective Abort input"; break;
		case  50: return "Contour running"; break;
		case  51: return "Contour Stop"; break;
		case  99: return "MC Timeout"; break;
		case 100: return "Motors are running, vector sequence"; break;
		case 101: return "Motors stopped at commanded vector"; break;

		default: return "-"; break;
	}
	return "?";
}

bool    GalilDMC2xxxDrv::mpos(string channel, double p){       // mpos : motor position
	string obj("GalilDMC2xxxDrv::mpos("+channel+",.)");
	std::stringstream ss;
	ss << "(p=" << p << ")";
        prtverbose(1, obj, ss.str());

	if (!__hw_write_enable) return true;

	string n, ans;
	n.assign("PA");
        n.append(channel);
        n.append("=");
	
	char buff[100];
	snprintf(buff, 100, "%d", (int)p);
        n.append(buff);
	
        n.append(";BG");
        n.append(channel);
	
	RAW_IO(1, 2, obj, n, ans, false) 
	
	return true;
}

double  GalilDMC2xxxDrv::mpos(string channel){
	std::stringstream ss;
	string obj("GalilDMC2xxxDrv::mpos("+channel+")");
        prtverbose(1, obj, "");

	//if (!__hw_write_enable) return true;

	string n, ans;
	n.assign("TD");
        n.append(channel);

	RAW_IO(1, 2, obj, n, ans, nan("")) 

	char *endptr;
	double mpos;
	mpos = strtod(ans.c_str(), &endptr);
        if ( endptr == ans.c_str())  mpos = nan("");
	return mpos;
}

bool    GalilDMC2xxxDrv::rpos(string channel, double p){       // rpos : motor relative position
	string obj("GalilDMC2xxxDrv::rpos("+channel+",.)");
	std::stringstream ss;
	ss << "(p=" << p << ")";
        prtverbose(1, obj, ss.str());

	if (!__hw_write_enable) return true;

	string n, ans;
	n.assign("PR");
        n.append(channel);
        n.append("=");
	
	char buff[100];
	snprintf(buff, 100, "%d", (int)p);
        n.append(buff);
	
        n.append(";BG");
        n.append(channel);
	
	RAW_IO(1, 2, obj, n, ans, false) 
	
	return true;
}


int     GalilDMC2xxxDrv::cnt_io(string quest){
	string obj("GalilDMC2xxxDrv::cnt_io("+quest+")");
        prtverbose(1, obj, "");

	if (quest.size() == 0) return 1;  //almeno il crnl viene poi inviato.....
	return 1+std::count(quest.begin(), quest.end()-1, ';'); // un ; alla fine non conta
}


int     GalilDMC2xxxDrv::chk_io(string ans){
	string obj("GalilDMC2xxxDrv::chk_io()");
        prtverbose(1, obj, "");
        prtverbose(2, obj, "on "+ans);

	if (ans.size() == 0) return 0;
	size_t question = std::count(ans.begin(), ans.end(), '?');
	size_t colon = std::count(ans.begin(), ans.end(), ':');

	std::stringstream ss;
	ss << "ans=" << ans << " || question= " << question << " colon= " << colon;
        prtverbose(2, obj, ss.str());

	if (question !=0) return -question;
	else return colon;
}

string  GalilDMC2xxxDrv::raw_io(string quest)    {
	string obj("GalilDMC2xxxDrv::raw_io()");
        prtverbose(1, obj, "cmd="+quest);
	std::stringstream ss;

	int c, r;

	c = cnt_io(quest);

	ss.str("");
	ss << "#; = " << c;
        prtverbose(2, obj, ss.str());

	string ans = _link->raw_io(quest);
	r = chk_io(ans);

	ss.str("");
	ss << "#(:?) = " << r;
        prtverbose(2, obj, ss.str());

	// facendo cosi' in "due parti", ovvero se c'e' un ? allora faccio una altra
	// raw_io di lettura codice di errore. puo' essre che tra la prima e questa seconda
	// raw_io, il server axisg2drv riceva una raw_io da un altro e ce quindi il codice di 
	// errore non si riferisca piu' alla prima raw_io fatta in questa funzione.
	// sarebbe meglio fare una unica raw_io con ache TC...

	if ( r < 0 ) {
		string errmsg = _link->raw_io("TC 1", false);
		char buff[100];
		snprintf(buff, 100, "Sent %d cmds, rcved %d bad", c, -r);
		throw Controller_ns::DrvExcpt(Controller_ns::RAWIO, buff, ans, errmsg);
	};
	return ans;
}

bool    GalilDMC2xxxDrv::defmpos(string channel, double p){
	std::stringstream ss;
	string obj("GalilDMC2xxxDrv::defmpos("+channel+",.)");
	ss << "p= " << p;
        prtverbose(1, obj, ss.str());

	if (!__hw_write_enable) return true;

	string n, ans;
	n.assign("DP");
        n.append(channel);
        n.append("=");

	char buff[100];
	snprintf(buff, 100, "%d", (int)p);
        n.append(buff);

	RAW_IO(1, 2, obj, n, ans, false) 
	
	return true;
}


bool GalilDMC2xxxDrv::acceleration(string channel, double a){
	string obj("GalilDMC2xxxDrv::acceleration("+channel+",.)");
	std::stringstream ss;
	ss << "(acceleration=" << a << ")";
	prtverbose(1, obj, ss.str());

	if (!__hw_write_enable) return true;

	string n, ans;
	n.assign("AC");
        n.append(channel);
        n.append("=");
	
	char buff[100];
	snprintf(buff, 100, "%d", (int)fabs(a));
        n.append(buff);
	
	RAW_IO(1, 2, obj, n, ans, false) 
	
	return true;
}

double GalilDMC2xxxDrv::acceleration(string channel){
	std::stringstream ss;
	string obj("GalilDMC2xxxDrv::acceleration("+channel+")");
        prtverbose(1, obj, "");

	string n, ans;
	n.assign("AC");
        n.append(channel);
        n.append("=?");

	RAW_IO(1, 2, obj, n, ans, nan("")) 

	char *endptr;
	double acceleration;
	acceleration = strtod(ans.c_str(), &endptr);
        if ( endptr == ans.c_str())  acceleration = nan("");
	return acceleration;
}


bool GalilDMC2xxxDrv::deceleration(string channel, double a){
	string obj("GalilDMC2xxxDrv::deceleration("+channel+",.)");
	std::stringstream ss;
	ss << "(deceleration=" << a << ")";
	prtverbose(1, obj, ss.str());

	if (!__hw_write_enable) return true;

	string n, ans;
	n.assign("DC");
        n.append(channel);
        n.append("=");
	
	char buff[100];
	snprintf(buff, 100, "%d", (int)fabs(a));
        n.append(buff);
	
	RAW_IO(1, 2, obj, n, ans, false) 
	
	return true;
}

double GalilDMC2xxxDrv::deceleration(string channel){
	std::stringstream ss;
	string obj("GalilDMC2xxxDrv::deceleration("+channel+")");
        prtverbose(1, obj, "");

	string n, ans;
	n.assign("DC");
        n.append(channel);
        n.append("=?");

	RAW_IO(1, 2, obj, n, ans, nan("")) 

	char *endptr;
	double deceleration;
	deceleration = strtod(ans.c_str(), &endptr);
        if ( endptr == ans.c_str())  deceleration = nan("");
	return deceleration;
}


// define hw encoder position
bool    GalilDMC2xxxDrv::defepos(string channel, double p){
	std::stringstream ss;
	string obj("GalilDMC2xxxDrv::defepos("+channel+",.)");
	ss << "p= " << p;
        prtverbose(1, obj, ss.str());

	if (!__hw_write_enable) return true;

	string n, ans;
	n.assign("DE");
        n.append(channel);
        n.append("=");

	char buff[100];
	snprintf(buff, 100, "%d", (int)p);
        n.append(buff);

	RAW_IO(1, 2, obj, n, ans, false) 
	
	return true;
}


bool    GalilDMC2xxxDrv::program_running(){
	std::stringstream ss;
	string obj("GalilDMC2xxxDrv::program_running()");
        prtverbose(1, obj, "");


	string n, ans;
	n.assign("TB");

	RAW_IO(1, 2, obj, n, ans, false) 

	int v;
	if ( 1 != sscanf(ans.c_str(), "%d", &v) ) return false;
		
	if (v & 0x80) return true;

	return false;
}

bool    GalilDMC2xxxDrv::findedge(string channel, double v){
	std::stringstream ss;
	string obj("GalilDMC2xxxDrv::findedge("+channel+")");
        prtverbose(1, obj, "");

	if (!__hw_write_enable) return true;

	if ( !speed(channel, v) ) return false;

	string n, ans;
	n.assign("FE");
        n.append(channel);
        n.append(";BG");
        n.append(channel);
	
	RAW_IO(1, 2, obj, n, ans, false) 

        prtverbose(2, obj, "exit!");

	return true;
}

bool    GalilDMC2xxxDrv::findindex(string channel, double v){
	std::stringstream ss;
	string obj("GalilDMC2xxxDrv::findindex("+channel+")");
        prtverbose(1, obj, "");

	if (!__hw_write_enable) return true;

	if ( !speed(channel, v) ) return false;

	string n, ans;
	n.assign("JG");
        n.append(channel);
        n.append("=");
	
	char buff[100];
	if ( fabs(v) >= MIN_SPx_VALUE ){
		snprintf(buff, 100, "%d", (int)v);
	} else {
		return false;
	};
        n.append(buff);
        n.append(";");

	n.append("FI");
        n.append(channel);
        n.append(";BG");
        n.append(channel);
	
	RAW_IO(1, 2, obj, n, ans, false) 

        prtverbose(2, obj, "exit!");

	return true;
}

bool    GalilDMC2xxxDrv::motor_off(string channel){
	std::stringstream ss;
	string obj("GalilDMC2xxxDrv::motor_off("+channel+")");
        prtverbose(1, obj, "");

	if (!__hw_write_enable) return true;

	string n, ans;
	n.assign("MO");
        n.append(channel);
	
	RAW_IO(1, 2, obj, n, ans, false) 

	return true;
}

bool    GalilDMC2xxxDrv::motor_on(string channel){
	std::stringstream ss;
	string obj("GalilDMC2xxxDrv::motor_on("+channel+")");
        prtverbose(1, obj, "");

	if (!__hw_write_enable) return true;

	string n, ans;
	n.assign("SH");
        n.append(channel);
	
	RAW_IO(1, 2, obj, n, ans, false) 

	return true;
}


} /* namespace Controller_ns */
#endif /* _WITH_GALILDMC2XXXDRV_ */
