#include "GalilTangoLink.h"

#include <sys/time.h>
#include <stdlib.h>




namespace Link_ns {


GalilTangoLink::GalilTangoLink(string target):Link(Link_ns::TANGO_CHAN, target) {
	#ifdef _DBG_GALILTANGOLINK
		_verbose = 1;
	#else
		_verbose = 0;
	#endif

	_name = target;

	std::stringstream ss("");

	ss << " _verbose = " << _verbose;


        string obj("GalilTangoLink()");
        prtverbose(1, obj, ss.str());

	#ifdef _DBG_GALILTANGOLINK_PRTLOG
		ss.str("");
		if (_prtLog) ss << "================  _prtLog TRUE!";
		else         ss << "================  _prtLog FALSE!";
        	prtverbose(-1, obj, ss.str());
	#endif

	_flag = 0;
	dev = NULL;

	AN["A"] = '1'; AN["B"] = '2'; AN["C"] = '3'; AN["D"] = '4';
	AN["E"] = '5'; AN["F"] = '6'; AN["G"] = '7'; AN["H"] = '8';

	try {
		#ifdef _DBG_GALILTANGOLINK_2
			cout << "GalilTangoLink().dev old dev = " << dev << endl;
		#endif

		_linkstate = LINK_UNREACTIVE;
		dev = new Tango::DeviceProxy(target);

		#ifdef _DBG_GALILTANGOLINK_2
			cout << "GalilTangoLink().dev new dev = " << dev << endl;
		#endif

		ss.str("");
/*
		int ms;
		ms = dev->ping();
		ss << "ping " << ms;
*/

		Tango::DevState	ustate;
		try {
			ustate = dev->state();
			_linkstate = LINK_REACTIVE;
			ss.str("");
			switch (ustate){
				case Tango::ON:
				case Tango::OFF:
				case Tango::CLOSE:
				case Tango::OPEN:
				case Tango::INSERT:
				case Tango::EXTRACT:
				case Tango::MOVING:
				case Tango::STANDBY:
				case Tango::FAULT:
				case Tango::INIT:
				case Tango::RUNNING:
				case Tango::ALARM:
				case Tango::DISABLE:
				case Tango::UNKNOWN:
					ss << " :: " << Tango::DevStateName[ustate] << " :: ";
					break;
				default:
					ss << "++++++++++++++++++++" << ustate;
					break;
			};
        		if (_prtLog) prtverbose(0, obj, ss.str());

			switch (ustate){
				case Tango::CLOSE: _linkstate = LINK_UNREACTIVE; break;
				default: break;
			};

		} catch(Tango::DevFailed &e) {
			ss.str("");

			ss << e.errors.length() ;
			for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
       			if (_prtLog) prtverbose(0, obj, "Tango except!: "+ss.str());

			//2 [0] API_CorbaException [1] API_DeviceTimedOut
			//2 [0] API_CantConnectToDevice [1] API_CommandFailed

			for (size_t i = 0; i < e.errors.length(); i++) {
				if ( strcmp(e.errors[i].reason, API_CantConnectToDevice) == 0 ){ _linkstate = LINK_UNREACTIVE; break; } else
				if ( strcmp(e.errors[i].reason, API_DeviceTimedOut)      == 0 ){ _linkstate = LINK_TIMEOUTED;  break; } else
				{ _linkstate = LINK_OTHEREXCEPT; }
			}
		} catch (...) {
			_linkstate = LINK_GENERALEXCEPT;
       			if (_prtLog) prtverbose(0, obj, "General except!");
		}

		ss.str("");
		ss << linkstatus();
        	prtverbose(1, obj, ss.str());

	} catch(Tango::DevFailed &e) {
		ss.str("");

		ss << e.errors.length() ;
		for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
       		if (_prtLog) prtverbose(0, obj, "Tango except!: "+ss.str());

		#ifdef _DBG_GALILTANGOLINK_2
			cout << "GalilTangoLink().dev = " <<  ss.str() << endl;
		#endif

		if (dev) delete dev;
		dev = NULL;
		_linkstate = LINK_CLOSE;

		ss.str("");
		ss << linkstatus();
        	if (_prtLog) prtverbose(1, obj, ss.str());

	} catch (...) {
		if (dev) delete dev;
		dev = NULL;
		_linkstate = LINK_CLOSE;

		ss.str("");
		ss << linkstatus();
        	if (_prtLog) prtverbose(1, obj, ss.str());
	}

	if ( !refresh() ) _updateDelay = 1.5;

	#ifdef _DBG_GALILTANGOLINK_2
		cout << "GalilTangoLink().dev exiting with = " << dev << endl;
	#endif
}

GalilTangoLink::~GalilTangoLink(){
        #ifdef _DBG_CLASSDESCTRUCTOR
                cout << "~GalilTangoLink() : ";
        #endif
	if (dev) {
        	#ifdef _DBG_CLASSDESCTRUCTOR
                	cout << "going to delete dev ... ";
        	#endif
		delete dev;
	}
        #ifdef _DBG_CLASSDESCTRUCTOR
                cout << endl;
        #endif
}

bool GalilTangoLink::refresh(){
        string obj("GalilTangoLink::refresh()");
        prtverbose(1, obj, "");
	std::stringstream ss;

	int pool_period_ms;

	if (dev){
		try {
			pool_period_ms = dev->get_attribute_poll_period("DataRecordSampleNumber");
			if ( pool_period_ms <= 0 ) return false;
			_updateDelay = ((double) pool_period_ms) * 1.1 / 1000;
			_linkstate = LINK_REACTIVE;
			//bf->log();
			return true;

		} catch(Tango::DevFailed &e) {
			ss.str("");

			ss << e.errors.length() ;
			for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
        		//prtverbose(0, obj, "Tango except!: "+ss.str());
			if (_prtLog) bf->log(obj, "Tango except!: "+ss.str());

			//2 [0] API_CorbaException [1] API_DeviceTimedOut
			//2 [0] API_CantConnectToDevice [1] API_CommandFailed

			for (size_t i = 0; i < e.errors.length(); i++) {
				if ( strcmp(e.errors[i].reason, API_CantConnectToDevice) == 0 ){ _linkstate = LINK_UNREACTIVE; break; } else
				if ( strcmp(e.errors[i].reason, API_DeviceTimedOut)      == 0 ){ _linkstate = LINK_TIMEOUTED;  break; } else
				{ _linkstate = LINK_OTHEREXCEPT; }
			}

		} catch (...) {
			_linkstate = LINK_GENERALEXCEPT;
        		//prtverbose(0, obj, "except!");
			bf->log(obj, "Tango except!: "+ss.str());
			return false;
		}
	} else { /* if no dev */
		_linkstate = LINK_CLOSE;
        	prtverbose(1, obj, "no dev");
		return false;
	}

	return false;
}

string GalilTangoLink::raw_io(string quest, bool setstate){
        string obj("GalilTangoLink::raw_io()");
        prtverbose(1, obj, "quest="+quest);
	std::stringstream ss;

	string ans;
	if (dev && (dev->state() != Tango::CLOSE)){
	//if (dev){
		try {
			Tango::DeviceData din, dout;

			din << quest;
			dout = dev->command_inout("WriteRead", din);
			dout >> ans;

			if (setstate) _linkstate = LINK_REACTIVE;

        		prtverbose(2, obj, "ans="+ans);

			bf->log();
		} catch(Tango::DevFailed &e) {
			ss.str("");

			ss << e.errors.length() ;
			for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
        		//prtverbose(0, obj, "Tango except!: "+ss.str());
			bf->log(obj, "Tango except!: "+ss.str());

			//2 [0] API_CorbaException [1] API_DeviceTimedOut
			//2 [0] API_CantConnectToDevice [1] API_CommandFailed

			if (setstate){
				for (size_t i = 0; i < e.errors.length(); i++) {
					if ( strcmp(e.errors[i].reason, API_CantConnectToDevice) == 0 ){ _linkstate = LINK_UNREACTIVE; break; } else
					if ( strcmp(e.errors[i].reason, API_DeviceTimedOut)      == 0 ){ _linkstate = LINK_TIMEOUTED;  break; } else
					{ _linkstate = LINK_OTHEREXCEPT; }
				}
			}

			ans="?";

		} catch (...) {
			if (setstate) _linkstate = LINK_GENERALEXCEPT;
        		//prtverbose(0, obj, "except!");
			bf->log(obj, "Tango except!: "+ss.str());
			ans="?";
		}
	} else { /* if no dev */
		if (setstate) _linkstate = LINK_CLOSE;
        	prtverbose(1, obj, "no dev");
		ans="?";
	}
        prtverbose(1, obj, "ans="+ans);
	return ans;
}

uvalues_t GalilTangoLink::update(string channel){
	std::stringstream ss;
        string obj("GalilTangoLink::update("+channel+")");
        prtverbose(1, obj, "");

	timeval start, end;
	gettimeofday(&start, NULL);

	uvalues_t uv;
	// uvalues_t:
        uv.motor 	= nan("");
        uv.encoder	= nan("");
        uv.velocity	= nan("");
        uv.analog	= nan("");
        uv.fls		= true;
        uv.rls		= true;
        uv.home		= true;
        uv.stop_code	= 0;
        uv.CC_code	= 0;
        uv.status	= OFF;

	vector<Tango::DeviceAttribute> *devattr = NULL;

	Tango::DevState	ustate;
	try {
		ustate = dev->state();
		bf->log();
	} catch(Tango::DevFailed &e) {
		ss.str("");

		ss << " " << __FILE__ << " " << __LINE__ << " ";
		ss << e.errors.length() ;
		for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
       		//prtverbose(0, obj, "Tango except!: "+ss.str());
		if (_prtLog) bf->log(obj, "Tango except!: "+ss.str());

		//2 [0] API_CorbaException [1] API_DeviceTimedOut
		//2 [0] API_CantConnectToDevice [1] API_CommandFailed

		for (size_t i = 0; i < e.errors.length(); i++) {
			if ( strcmp(e.errors[i].reason, API_CantConnectToDevice) == 0 ){ _linkstate = LINK_UNREACTIVE; break; } else
			if ( strcmp(e.errors[i].reason, API_DeviceTimedOut)      == 0 ){ _linkstate = LINK_TIMEOUTED;  break; } else
			{ _linkstate = LINK_OTHEREXCEPT; }
		}
		goto end_label;

	} catch (...) {
		_linkstate = LINK_GENERALEXCEPT;
       		//prtverbose(0, obj, "General except!");
       		bf->log(obj, "General except!");
		goto end_label;
	}

	ss.str("");
	ss << "flag="<< _flag << " || ";
	switch (_flag) {
		case TANGO_GROUP_SPARSE_ATTRIBUTES:
			ss << "TANGO_GROUP_SPARSE_ATTRIBUTES";
			break;
		case TANGO_GROUP_ATTRIBUTE:
			ss << "TANGO_GROUP_ATTRIBUTE";
			break;
		case TANGO_GROUP_ATTRIBUTE_2:
			ss << "TANGO_GROUP_ATTRIBUTE_2";
			break;
		default:
			ss << "?";
			break;
	}
	ss << " : " << Tango::DevStateName[ustate];
        prtverbose(2, obj, ss.str());


	//if (dev){
	if (dev && (ustate != Tango::CLOSE)){ 
		switch (_flag) {
		  case 0:
		  case TANGO_GROUP_SPARSE_ATTRIBUTES:
        		prtverbose(3, obj, "case TANGO_GROUP_SPARSE_ATTRIBUTES");
			try {
				vector<string> attr_names;
				string n;

				n.assign("TD");
				n.append(channel);
				attr_names.push_back(n);

				n.assign("TP");
				n.append(channel);
				attr_names.push_back(n);

				n.assign("TV");
				n.append(channel);
				attr_names.push_back(n);

				n.assign("AN");
				n += (AN.find(channel))->second;
				attr_names.push_back(n);

				n.assign(channel);
				n.append("RLS");
				attr_names.push_back(n);

				n.assign(channel);
				n.append("FLS");
				attr_names.push_back(n);

				n.assign("TS");
				n.append(channel);
				attr_names.push_back(n);

				n.assign("SC");
				n.append(channel);
				attr_names.push_back(n);

				n.assign("CC");
				n.append(channel);
				attr_names.push_back(n);

				devattr = dev->read_attributes(attr_names);

				Tango::DevLong l;
				Tango::DevDouble f;
				Tango::DevBoolean b;
				Tango::DevUChar uc;
				Tango::DevULong ul;

				(*devattr)[0] >> l;  uv.motor = l;
				(*devattr)[1] >> l;  uv.encoder = l;
				(*devattr)[2] >> l;  uv.velocity = l;
				(*devattr)[3] >> f;  uv.analog = f;
				(*devattr)[4] >> b;  uv.rls = b;
				(*devattr)[5] >> b;  uv.fls = b;
				(*devattr)[6] >> uc; uv.home = ((unsigned char)uc & 0x2) >> 1;
						     if    ( ((((unsigned char)uc)>>5)&0x1) == 1 ) uv.status = OFF;
						     else {
							if ( ((((unsigned char)uc)>>7)&0x1) == 1 ) uv.status = MOVING;
						     	else uv.status = STANDSTILL;
						     };
				(*devattr)[7] >> uc; uv.stop_code = (unsigned long)uc;
				(*devattr)[8] >> ul; uv.CC_code = ul;

				_linkstate = LINK_REACTIVE;

				bf->log();

			} catch(Tango::DevFailed &e) {
				ss.str("");

				ss << " " << __FILE__ << " " << __LINE__ << " ";
				ss << e.errors.length() ;
				for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
        			//prtverbose(0, obj, "Tango except!: "+ss.str());
				bf->log(obj, "Tango except!: "+ss.str());

				//2 [0] API_CorbaException [1] API_DeviceTimedOut
				//2 [0] API_CantConnectToDevice [1] API_CommandFailed

				for (size_t i = 0; i < e.errors.length(); i++) {
					if ( strcmp(e.errors[i].reason, API_CantConnectToDevice) == 0 ){ _linkstate = LINK_UNREACTIVE; break; } else
					if ( strcmp(e.errors[i].reason, API_DeviceTimedOut)      == 0 ){ _linkstate = LINK_TIMEOUTED;  break; } else
					{ _linkstate = LINK_OTHEREXCEPT; }
				}

			} catch (...) {
				_linkstate = LINK_GENERALEXCEPT;
        			//prtverbose(0, obj, "General except!");
				bf->log(obj, "Tango except!: "+ss.str());
			}
			break;

		  case TANGO_GROUP_ATTRIBUTE:
		  case TANGO_GROUP_ATTRIBUTE_2:
        		prtverbose(3, obj, "case TANGO_GROUP_ATTRIBUTE");
			try {
				vector<string> attr_names;

				vector<Tango::DevLong> vl;
				Tango::DevULong ul;
				
				string n;
				n.assign(channel);
				n.append("BlockData");
				attr_names.push_back(n);

				n.assign("CC");
				n.append(channel);
				attr_names.push_back(n);

				if ( _flag == TANGO_GROUP_ATTRIBUTE_2 ){
					n.assign("AN");
					n += (AN.find(channel))->second;
					attr_names.push_back(n);
				}

				devattr = dev->read_attributes(attr_names);
				//cout << "ans.size() = " << (*devattr).size() << endl;
				(*devattr)[0] >> vl;

				uv.motor     = (double)vl[3]; // oppure 6
				uv.encoder   = (double)vl[4];
				uv.velocity  = (double)vl[7];
				uv.analog    = (double)vl[9] / 32767.0 * 10.0;
				uv.rls       = (vl[1] & 0x4)==0?1:0;
				uv.fls       = (vl[1] & 0x8)==0?1:0;
				uv.home	     = (vl[1] & 0x2)>>1;
				uv.stop_code = (unsigned long)vl[2];

				ss.str("");
				ss << hex << attr_names[0] << "[0]: 0x" << vl[0] << dec;
        			prtverbose(2, obj, ss.str());

				if ( (vl[0]&0x1) == 1 ) uv.status = OFF;
				else {
				   if ( (vl[0]&0x8000) == 0x8000 ) uv.status = MOVING;
			     	   else uv.status = STANDSTILL;
				}
	
				#ifdef __TANGO805__
				#warning Tango805 on GalilTangoLink::update()
					{
					  //cout << ".........." << endl;
					  unsigned long  _ul;
					  (*devattr)[1] >> _ul;
					  ul = _ul;
					}
				#else /* __TANGO805__ */
				//+++++  warning Tango on GalilTangoLink::update()
					(*devattr)[1] >> ul;
				#endif /* __TANGO805__ */

				uv.CC_code = ul;

				ss.str("");
				ss << hex << attr_names[1] << " 0x" << ul << dec;
        			prtverbose(2, obj, ss.str());

				if ( _flag == TANGO_GROUP_ATTRIBUTE_2 ){
					  (*devattr)[2] >> uv.analog;
				}

				_linkstate = LINK_REACTIVE;

				bf->log();

			} catch(Tango::DevFailed &e) {
				ss.str("");

				ss << " " << __FILE__ << " " << __LINE__ << " ";
				ss << e.errors.length() ;
				for (size_t i = 0; i < e.errors.length(); i++)  ss << " [" << i << "] " << e.errors[i].reason; 
        			//prtverbose(0, obj, "Tango except!: "+ss.str());
				bf->log(obj, "Tango except!: "+ss.str());

				//2 [0] API_CorbaException [1] API_DeviceTimedOut
				//2 [0] API_CantConnectToDevice [1] API_CommandFailed

				for (size_t i = 0; i < e.errors.length(); i++) {
					if ( strcmp(e.errors[i].reason, API_CantConnectToDevice) == 0 ){ _linkstate = LINK_UNREACTIVE; break; } else
					if ( strcmp(e.errors[i].reason, API_DeviceTimedOut)      == 0 ){ _linkstate = LINK_TIMEOUTED;  break; } else
					{ _linkstate = LINK_OTHEREXCEPT; }
				}

			} catch (...) {
				_linkstate = LINK_GENERALEXCEPT;
        			//prtverbose(1, obj, "General except!");
				bf->log(obj, "Tango except!: "+ss.str());
			}
			break;

		  default:
        		prtverbose(2, obj, "case default");
			break;
		} /* switch (_flag) */
	} else { /* if (dev) */
		_linkstate = LINK_CLOSE;
        	prtverbose(1, obj, "LINK_CLOSE");
	}

/*
	sopra:
	vector<Tango::DeviceAttribute> *devattr = NULL;
*/
end_label:
	if (devattr) delete devattr;

	gettimeofday(&end, NULL);
	_last_update_ms = (double)(end.tv_sec - start.tv_sec)*1000 + ((double)(end.tv_usec - start.tv_usec))/1000;

	ss.str("");
	ss << "update time: " << _last_update_ms << "ms";
       	prtverbose(2, obj, ss.str());

	_uv = uv;
	return uv;
}


} /* namespace Link_ns */
