#include "N5Drv.h"

namespace Controller_ns {

N5Drv::N5Drv(string target, string extra): Drv(target, extra) {
        #ifdef _DBG_N5DRV
                _verbose = 1;
        #else
                _verbose = 0;
        #endif

        _name = target;

        std::stringstream ss("");
        ss << " _verbose = " << _verbose;

        #ifdef _DBG_CSTOR
        	prtverbose(0, "N5Drv::N5Drv()", ss.str());
        #else
        	prtverbose(1, "N5Drv::N5Drv()", ss.str());
        #endif



        __hw_write_enable = false;

	_with_microcode = false;

	_hw_brake_conf = 0;

        Link_ns::LinkType t;
        size_t found;
        found = _target.find_first_of("http://");
        if (found != string::npos ) {
		t = Link_ns::WEBREST_CHAN;
        } else {
        	t = Link_ns::NULL_CHAN;
        }

        switch (t){
                case Link_ns::WEBREST_CHAN:
			ss.str("");
			ss << "new Link_ns::N5WebLink(" << _target << ")";
        		prtverbose(1, "N5Drv::N5Drv()", ss.str());
                        _n5link = new Link_ns::N5WebLink(_target);
			_link = (Link_ns::Link*)_n5link;
                        break;
                default:
                case Link_ns::NULL_CHAN:
			ss.str("");
			ss << "new Link_ns::NULL_CHAN";
        		prtverbose(1, "N5Drv::N5Drv()", ss.str());
			_link = NULL;
                        break;
        }

	if (_n5link) {
		_sw_vers = _n5link->sw_vers();
		//cout << "_sw_vers: " << _sw_vers << endl;
	}

	_isinit = get_cinit();

	_Cia402HomingParams = Cia402HomingParams();

	_analog1.type = get_analogtype(1);
	_analog2.type = get_analogtype(2);

	_nbrtr = -1;

	if (_n5link) {
		_brake_conf = brake_conf();
	}


	//=================================================
	//
	//
	if ( _sw_vers >= SW_VERS_2 ) {
		#ifdef _DBG_V2ALL
			cout << "----------------- V2 ---------------" << endl;
		#endif

		_n5v2_feedback_map = n5v2_feedback_map();

		#ifdef _DBG_V2ALL
			cout << "----" << endl;
		#endif
	}
	//
	//
	//=================================================

        #ifdef _DBG_CSTOR
		ss.str("");
		ss << "... exiting ...";
        	prtverbose(0, "N5Drv::N5Drv()", ss.str());
        #endif
}

N5Drv::~N5Drv()
		{ }

string  N5Drv::raw_io(string quest)
		{ return "-"; }

int     N5Drv::cnt_io(string quest)
		{ return 0; }

int     N5Drv::chk_io(string ans)
		{ return 0; }

//============================================================================


uint32_t N5Drv::pp_speed(){
        uint32_t x = 0;
        if (_n5link) x = _n5link->readU32(PP_VELOCITY);
        return x;
}

bool N5Drv::pp_speed(uint32_t v){
	if (_n5link) return _n5link->writeU32(PP_VELOCITY, v);
	return false;
}

uint32_t N5Drv::pp_acceleration(){
        uint32_t x = 0;
        if (_n5link) x = _n5link->readU32(PP_ACCELERATION);
        return x;
}

bool N5Drv::pp_acceleration(uint32_t v){
        if (_n5link) return _n5link->writeU32(PP_ACCELERATION, v);
	return false;
}

uint32_t N5Drv::pp_deceleration(){
        uint32_t x = 0;
        if (_n5link) x = _n5link->readU32(PP_DECELERATION);
        return x;
}

bool N5Drv::pp_deceleration(uint32_t v){
        if (_n5link) return _n5link->writeU32(PP_DECELERATION, v);
	return false;
}

uint16_t N5Drv::statusWord(){
        if (_n5link) _statusword = _n5link->readU16(STATUSWORD);
	else _statusword = 0;
        return _statusword;
}

bool N5Drv::SetPointAckd(){
        return (statusWord() >> 12)&0x1;
}

uint16_t N5Drv::controlWord(){
        if (_n5link) _controlword = _n5link->readU16(CONTROLWORD);
	else _controlword = 0;
	//std::cout << "controlword() = " << std::bitset<16>(_controlword) << std::endl;
        return _controlword;
}

bool N5Drv::controlWord(uint16_t cw){
	//std::cout << "controlword(" << std::bitset<16>(cw) << ")" << std::endl;
        if (_n5link) return _n5link->writeU16(CONTROLWORD, cw);
	return false;
}

void N5Drv::prt_statusExt(){
        uint16_t sw = statusWord();
	cout << "--------------------------------" << endl;
	cout << "b00:  RTSO=" << (sw&0x1) << endl;
	cout << "b01:    SO=" << (sw&0x2) << endl;
	cout << "b02:    OE=" << (sw&0x4) << endl;
	cout << "b03: FAULT=" << (sw&0x8) << endl;
	cout << "b04:    VE=" << (sw&0x10) << endl;
	cout << "b05:    QS=" << (sw&0x20) << endl;
	cout << "b06:   SOD=" << (sw&0x40) << endl;
	cout << "b07:  WARN=" << (sw&0x80) << endl;
	cout << "b08:  SYNC=" << (sw&0x100) << endl;
	cout << "b09:   REM=" << (sw&0x200) << endl;
	cout << "b10:  TARG=" << (sw&0x400) << endl;
	cout << "b11:   ILA=" << (sw&0x800) << endl;
	cout << "b12:   OMS=" << (sw&0x1000) << endl;
	cout << "b13:   OMS=" << (sw&0x2000) << endl;
	cout << "b14:      -" << endl;
	cout << "b15:   CLA=" << (sw&0x8000) << endl;
	cout << "--------------------------------" << endl;
}

state_t N5Drv::state(){
        uint16_t sw = statusWord();
        state_t  st = STATE_UNKNOWN;
        if ((sw & 0b0000000001001111) == 0b0000000000000000) st = NOT_READY_TO_SWITCHON;
        if ((sw & 0b0000000001001111) == 0b0000000001000000) st = SWITCHON_DISABLED;
        if ((sw & 0b0000000001101111) == 0b0000000000100001) st = READY_TO_SWITCHON;
        if ((sw & 0b0000000001101111) == 0b0000000000100011) st = SWITCHON;
        if ((sw & 0b0000000001101111) == 0b0000000000100111) st = OPERATION_ENABLED;
        if ((sw & 0b0000000001101111) == 0b0000000000000111) st = QUICK_STOP;
        if ((sw & 0b0000000001001111) == 0b0000000000001111) st = FAULT_REACTION;
        if ((sw & 0b0000000001001111) == 0b0000000000001000) st = FAULT;
        _state = st;
        return _state;
}

std::string N5Drv::status(){
        return statusNames[state()];
}

state_t N5Drv::CiA402StateCmd(stateCmd_t cmd){
        state_t  actualstate = state();
        uint16_t cw          = controlWord();

        switch (actualstate) {
                case SWITCHON_DISABLED:
                        if ( cmd == SHUTDOWN_CMD )         controlWord((cw & 0xFFF8) | 0x0006);
                        break;
                case READY_TO_SWITCHON:
                        if ( cmd == SWITCH_ON_CMD )        controlWord((cw & 0xFFF0) | 0x0007);
                        if ( cmd == QUICK_STOP_CMD )       controlWord((cw & 0xFFF6) | 0x0002);
                        break;
                case SWITCHON:
                        if ( cmd == ENABLE_OPERATION_CMD ) controlWord((cw & 0xFFF0) | 0x000F);
                        if ( cmd == SHUTDOWN_CMD )         controlWord((cw & 0xFFF8) | 0x0006);
                        if ( cmd == QUICK_STOP_CMD )       controlWord((cw & 0xFFF6) | 0x0002);
                        if ( cmd == DISABLE_VOLTAGE_CMD )  controlWord((cw & 0xFFF2) | 0x0000);
                        break;
                case OPERATION_ENABLED:
                        if ( cmd == SHUTDOWN_CMD )         controlWord((cw & 0xFFF8) | 0x0006);
                        if ( cmd == DISABLE_VOLTAGE_CMD )  controlWord((cw & 0xFFF2) | 0x0000);
                        if ( cmd == DISABLE_OPERATION_CMD )controlWord((cw & 0xFFF0) | 0x0007);
                        if ( cmd == QUICK_STOP_CMD )       controlWord((cw & 0xFFF6) | 0x0002);
                        break;
                case QUICK_STOP:
                        if ( cmd == ENABLE_OPERATION_CMD ) controlWord((cw & 0xFFF0) | 0x000F);
                        if ( cmd == DISABLE_VOLTAGE_CMD )  controlWord((cw & 0xFFF2) | 0x0000);
                        break;
                case FAULT:
                        if ( cmd == FAULT_RESET_CMD )      {
                                                                controlWord((cw & 0xFF7B) | 0x0004);
                                                                controlWord((cw & 0xFF7F) | 0x0080);
                                                                controlWord((cw & 0xFF7F) | 0x0000);
                                                           }
                        break;
                default:
                        break;
        }


	int max = 1 + 2 * ( _brake_conf.close_time + _brake_conf.shutdown_time + _brake_conf.open_time + _brake_conf.start_time ); // msec
	do {
        	usleep(1000); // 1 ms
		max -= 1; 	// cosi' max * 1000 .... sec
	} while ( (state() == actualstate) && (max > 0) );
        return state();
}

bool N5Drv::CiA402FaultResetOnLimitSwitch(){
	if ( get_fls() || get_rls() ) {
		CiA402StateCmd(FAULT_RESET_CMD);
		usleep(1000);
		CiA402StateCmd(SHUTDOWN_CMD);
		usleep(1000);
		CiA402StateCmd(SWITCH_ON_CMD);
		usleep(1000);
		return true;
	}
	return false;
}


bool N5Drv::get_rls(){
        bool rls = true;
	if (_n5link){
        	try {
                	uint32_t u32;
                	u32 = _n5link->readU32(DIGITAL_INPUT);
                	rls  =  (u32 & 0x1);
        	} catch (...) {
                	// errori di comunicazione
        	}
        }
        return rls;
}

bool N5Drv::get_fls(){
        bool fls = true;
	if (_n5link){
        	try {
                	uint32_t u32;
                	u32 = _n5link->readU32(DIGITAL_INPUT);
                	fls  = ((u32 & 0x2) >> 1);
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
        return fls;
}

bool N5Drv::get_home(){
        bool home = false;
	if (_n5link){
        	try {
                	uint32_t u32;
                	u32 = _n5link->readU32(DIGITAL_INPUT);
                	home  = ((u32 & 0x4) >> 1);
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
        return home;
}

bool N5Drv::have_brake(){
        bool brake = false;
	if (_n5link){
        	try {
                	int32_t i32;
                	i32 = _n5link->readI32(MOTORBRAKE);
			if ( i32 != 0 ) brake = true;
        	} catch (...) {
                	// errori di comunicazione
        	}
        }
        return brake;
}

/*
typedef struct {
        bool    have_brake;
        bool    automatic;
        int     close_time;             //[ms] Close Brake Idle Time
        int     shutdown_time;          //[ms] Shutdown Power Idle Time
        int     open_time;              //[ms] Open Brake Delay Time
        int     start_time;             //[ms] Start Operation Delay Time
        int     pwm_freq;               //[Hz] PWM Frequency
        int     pwm_dutycycle;          //[%]  PWM percent
} brake_conf_t;
*/
brake_conf_t N5Drv::brake_conf(){
	brake_conf_t bc;
	uint32_t u32;

	bc.have_brake = have_brake(); // dal microcodice
	if (_n5link){
        	try {
        		uint32_t mds = _n5link->readU32(MOTOR_DRIVE_SUBMODE);
        		if ((mds & 0x4) == 0x4) bc.automatic = true;
			else 		        bc.automatic = false;  
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	if (_n5link){
        	try {
        		u32 = _n5link->readU32(BRAKE_CLOSE_TIME);
			bc.close_time = (int)u32;

        		u32 = _n5link->readU32(BRAKE_SHUTDOWN_TIME);
			bc.shutdown_time = (int)u32;

        		u32 = _n5link->readU32(BRAKE_OPEN_TIME);
			bc.open_time = (int)u32;

        		u32 = _n5link->readU32(BRAKE_START_TIME);
			bc.start_time = (int)u32;

        		u32 = _n5link->readU32(BRAKE_PWM_FREQ);
			bc.pwm_freq = (int)u32;

        		u32 = _n5link->readU32(BRAKE_PWM_DUTYCYCLE);
			bc.pwm_dutycycle = (int)u32;

        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	
	return bc;
}

bool N5Drv::pp_polarity(){
	// bit = 1 = reverse
	// return true if positive
	bool p = true;
	if (_n5link){
        	try {
                	uint8_t u8;
                	u8 = _n5link->readU8(MOTOR_POLARITY);
                	p  = ((u8 & 0x80) >> 7);
			p &= 0x1;
			p ^= 0x1;
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return p;
}

bool N5Drv::pp_polarity(bool pos){
	// bit = 1 = reverse
	bool ret = false;
	if (_n5link){
        	try {
                	uint8_t u8;
                	u8 = (_n5link->readU8(MOTOR_POLARITY)) & 0x80;
			if (!pos) u8 |= 0x80;
			ret =  _n5link->writeU8(MOTOR_POLARITY, u8);
        	} catch (...) {
                	// errori di comunicazione
			cout << __func__ << "@" << __FILE__ << ":" << __LINE__ << ": Error on link.write() !!" << endl;
        	}
	}
	#ifdef _DBG_N5DRV
		cout << __func__ << "@" << __FILE__ << ":" << __LINE__ << ": link.write()=" << (ret?"OK":"FAIL") << endl;
	#endif
	return ret;
}

n5enc_conf_t N5Drv::enc_conf(){
	n5enc_conf_t ec = { N5_ENC_NOTYPE, N5_ENC_NOV };
	uint32_t u32;
	if (_n5link){
        	try {
        		u32 = _n5link->readU32(ENCODER_CONFIGURATION);
			switch (u32 & 0x1) {
				case 0: ec.supply = N5_ENC_5V; break;
				case 1: ec.supply = N5_ENC_24V; break;
				default: break;
			}
			switch (u32 & 0x2) {
				case 0: ec.type = N5_ENC_DIFFERENTIAL; break;
				case 2: ec.type = N5_ENC_SINGLE; break;
				default: break;
			}
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return ec;
}


bool N5Drv::is_bldc(){
	bool p = false;
	if (_n5link){
        	try {
        		uint32_t mds = _n5link->readU32(MOTOR_DRIVE_SUBMODE);
        		p = mds & (0x1 << 6);
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return p;
}

bool N5Drv::is_bldc(bool bldc){
	bool ret = false;
	if (_n5link){
        	try {
        		uint32_t mds = (_n5link->readU32(MOTOR_DRIVE_SUBMODE)) & 0xFFBF;
			if (bldc) mds |= (0x1 << 6);
			ret =  _n5link->writeU32(MOTOR_DRIVE_SUBMODE, mds);
        	} catch (...) {
                	// errori di comunicazione
			cout << __func__ << "@" << __FILE__ << ":" << __LINE__ << ": Error on link.write() !!" << endl;
        	}
	}
	#ifdef _DBG_N5DRV
		cout << __func__ << "@" << __FILE__ << ":" << __LINE__ << ": link.write()=" << (ret?"OK":"FAIL") << endl;
	#endif
	return ret;
}

double N5Drv::powerVoltage(){
	double v(nan(""));
	if (_n5link){
        	try {
        		int32_t x = _n5link->readI32(POWER_VOLTAGE);
        		v = double(x)/1000;
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return v;
}

double N5Drv::logicVoltage(){
	double v(nan(""));
	if (_n5link){
        	try {
        		int32_t x = _n5link->readI32(LOGIC_VOLTAGE);
        		v = double(x)/1000;
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return v;
}

double N5Drv::pcbTemperature(){
	double v(nan(""));
	if (_n5link){
        	try {
        		int32_t x = _n5link->readI32(PCB_TEMPERATURE);
        		v = double(x)/10;
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return v;
}

int32_t N5Drv::get_sw_limit_min(){
	int32_t x = 0;
	if (_n5link){
        	try {
        		x = _n5link->readI32(SW_LIMIT_MIN);
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return x;
}

int32_t N5Drv::get_sw_limit_max(){
	int32_t x = 0;
	if (_n5link){
        	try {
        		x = _n5link->readI32(SW_LIMIT_MAX);
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return x;
}

uint32_t N5Drv::get_ls_tolerance(){
	uint32_t x = 0;
	if (_n5link){
        	try {
        		x = _n5link->readU32(LS_TOLERANCE);
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return x;
}


#if 0
void N5Drv::describe(){
	std::cout << "Power Voltage   : " << powerVoltage() << std::endl;
	std::cout << "Logic Voltage   : " << logicVoltage() << std::endl;
	std::cout << "PCB Temperature : " << pcbTemperature() << std::endl;
	std::cout << std::endl;

	std::cout << SW_LIMIT_MIN << ": ";
	std::cout << "SW Limit min    : " << get_sw_limit_min() << std::endl;

	std::cout << SW_LIMIT_MAX << ": ";
	std::cout << "SW Limit max    : " << get_sw_limit_max() << std::endl;

	std::cout << LS_TOLERANCE << ": ";
	std::cout << "LS Tolerance    : " << get_ls_tolerance() << std::endl;
	std::cout << std::endl;

	std::cout << "OpMode : " << OpMode2str(get_opmode()) << std::endl;
	std::cout << std::endl;
}
#else
std::vector<std::string> N5Drv::describe(){
	std::stringstream ss("");
	std::vector<std::string> desc;


	ss.str("");
	ss << "Hardware Version: " << hardware_version();
	desc.push_back(ss.str());

	ss.str("");
	ss << "Software Version: " << software_version();
	desc.push_back(ss.str());

	ss.str("");
	ss << "Power Voltage   : " << powerVoltage();
	desc.push_back(ss.str());

	ss.str("");
	ss << "Logic Voltage   : " << logicVoltage();
	desc.push_back(ss.str());

	ss.str("");
	ss << "PCB Temperature : " << pcbTemperature();
	desc.push_back(ss.str());

	ss.str("");
	ss << SW_LIMIT_MIN << ": ";
	ss << "SW Limit min    : " << get_sw_limit_min();
	desc.push_back(ss.str());

	ss.str("");
	ss << SW_LIMIT_MAX << ": ";
	ss << "SW Limit max    : " << get_sw_limit_max();
	desc.push_back(ss.str());

	ss.str("");
	ss << LS_TOLERANCE << ": ";
	ss << "LS Tolerance    : " << get_ls_tolerance();
	desc.push_back(ss.str());

	ss.str("");
	ss << "OpMode : " << OpMode2str(get_opmode());
	desc.push_back(ss.str());

	ss.str("");
	ss << "Cia402 : " << status();
	desc.push_back(ss.str());

	if ( state() == FAULT ){
		ss.str("");
		ss << "Last error : " << getLastError();
		desc.push_back(ss.str());
	}

	ss.str("");
	ss << "Analog1 : " << analog1().value << " [";
	switch (_analog1.type){
		case AMP_020MA:
			ss << "mA";
			break;
		case VOLT_10_BIPOL:
			ss << "V";
			break;
		default:
			ss << "?";
			break;
	}
	ss << "]";
	desc.push_back(ss.str());

	ss.str("");
	ss << "Analog2 : " << analog2().value << " [";
	switch (_analog2.type){
		case AMP_020MA:
			ss << "mA";
			break;
		case VOLT_10_BIPOL:
			ss << "V";
			break;
		default:
			ss << "?";
			break;
	}
	ss << "]";
	desc.push_back(ss.str());

	
	ss.str("");
	ss << "have_brake : " << (_brake_conf.have_brake?"YES":"NO");
	desc.push_back(ss.str());

	ss.str("");
	ss << "automatic : " << (_brake_conf.automatic?"YES":"NO");
	desc.push_back(ss.str());

	ss.str("");
	ss << "close_time : " << _brake_conf.close_time << "[ms]";
	desc.push_back(ss.str());

	ss.str("");
	ss << "shutdown_time : " << _brake_conf.shutdown_time << "[ms]";
	desc.push_back(ss.str());

	ss.str("");
	ss << "open_time : " << _brake_conf.open_time << "[ms]";
	desc.push_back(ss.str());

	ss.str("");
	ss << "start_time : " << _brake_conf.start_time << "[ms]";
	desc.push_back(ss.str());

	ss.str("");
	ss << "pwm_freq : " << _brake_conf.pwm_freq << "[Hz]";
	desc.push_back(ss.str());

	ss.str("");
	ss << "pwm_dutycycle : " << _brake_conf.pwm_dutycycle << "[%]";
	desc.push_back(ss.str());
	
	return desc;
}
#endif

static std::string dumpInputCfg(uint32_t ire, uint32_t isfe, uint32_t ii, uint32_t mask){
	std::stringstream ss;
	ss.str("");

	if ( ire & mask ) {
		ss << "routed!";
	} else {
		if ( isfe & mask ) {
			switch (mask){
				case 0x1: ss << "-Limit switch "; break;
				case 0x2: ss << "+Limit switch "; break;
				case 0x4: ss << "Home switch "; break;
				default:  ss << "??"; break;
			}
		} else {
			ss << "general purpose ";
		}
		if ( ii & mask ){
			ss << "[INVERTED]";
		}
	}
	return ss.str();
}

std::vector<std::string> N5Drv::dumpConfig(){
	std::vector<std::string> dump;
	std::stringstream ss;
	uint32_t u32 = 0;
	int32_t i32 = 0;
	bool bldc = false;
	float f = 0;
	if (_n5link){
        	try {
			ss.str("");
			ss << "Motor type              : ";
			if ((bldc=is_bldc())) ss << "BLDC";
			else ss << "STEPPER";
			dump.push_back(ss.str());

			ss.str("");
        		u32 = _n5link->readU32(POLE_PAIR_COUNT);
			ss << "Motor pole pairs        : " << u32;
			dump.push_back(ss.str());

			ss.str("");
        		u32 = _n5link->readU32(MOTOR_CURRENT);
			if (bldc) {
				ss << "Motor peak current      : " << u32 << " mA";
			} else {
				ss << "Motor rated current     : " << u32 << " mA (bipolar)";
			}
			dump.push_back(ss.str());

			if (bldc) {
				ss.str("");
        			u32 = _n5link->readU32(MOTOR_NOMINAL_CURRENT);
				//------------------------:
				ss << "Motor rated current     : " << u32 << " mA";
				dump.push_back(ss.str());

				ss.str("");
        			u32 = _n5link->readU32(MOTOR_PEAK_CURRENT_TIME);
				//-----------------------------:
				ss << "Motor peak cur. max time: " << u32 << " ms";
				dump.push_back(ss.str());

			}

			ss.str("");
        		u32 = _n5link->readU32(MOTOR_RESISTANCE);
			memcpy((void*)(&f), (void*)(&u32), 4);
			ss << "Motor resistance        : " << f << " Ohm";
			dump.push_back(ss.str());
			f = nan("");

			ss.str("");
        		u32 = _n5link->readU32(MOTOR_INDUCTANCE);
			memcpy((void*)(&f), (void*)(&u32), 4);
			ss << "Motor inductance        : " << (f*1000) << " mH";
			dump.push_back(ss.str());
			f = nan("");

			ss.str("");
        		u32 = _n5link->readU32(MOTOR_FLUX);
			memcpy((void*)(&f), (void*)(&u32), 4);
			ss << "Motor flux              : " << f << " Vs/rad";
			dump.push_back(ss.str());
			f = nan("");

			n5enc_conf_t ec = enc_conf();
			ss.str("");
			ss << "Encoder                 : ";
			switch (ec.supply) {
				case N5_ENC_5V: ss << "5V"; break;
				case N5_ENC_24V: ss << "24V"; break;
				default: ss <<"unknown";
			}
			ss << "/";
			switch (ec.type) {
				case N5_ENC_SINGLE: ss << "single-ended"; break;
				case N5_ENC_DIFFERENTIAL: ss << "differential"; break;
				default: ss <<"unknown";
			}
			dump.push_back(ss.str());

			ss.str("");
			int32_t x = 0;
        		x = _n5link->readI32(ENCODER_RESOLUTION);
			ss << "Encoder resolution      : " << x << " ppr (=4x cpr)";
			dump.push_back(ss.str());

			ss.str("");
			//-----------------------------:
			ss << "Commutation sensor src  : ";
			if ( _sw_vers < SW_VERS_2 ) {
        			i32 = _n5link->readU32(CLOOP_SOURCE_SENSOR);
				switch(i32){
					case -1: ss << "sensorless"; break;
					case  1: ss << "encoder (+ index)"; break;
					case  2: ss << "hall"; break;
					case  3: ss << "hall + encoder"; break;
					default: ss << "unknown [" << i32 << "]"; break;
				}
				dump.push_back(ss.str());
			} else {
				ss << "V2: ";
				ss << n5v2encidNames[_n5v2_feedback_map.commutation_sensor];
				dump.push_back(ss.str());

				ss.str("");
				//-----------------------------:
				ss << "Speed sensor src        : ";
				ss << "V2: ";
				ss << n5v2encidNames[_n5v2_feedback_map.speed_sensor];
				dump.push_back(ss.str());

				ss.str("");
				//-----------------------------:
				ss << "Position sensor src     : ";
				ss << "V2: ";
				ss << n5v2encidNames[_n5v2_feedback_map.position_sensor];
				dump.push_back(ss.str());
			}

			ss.str("");
        		i32 = _n5link->readU32(ENCODER_ALIGNMENT);
			//-----------------------------:
			ss << "Encoder alignment       : " << i32;
			dump.push_back(ss.str());

			ss.str("");
        		uint32_t vei = _n5link->readU32(VIRT_ENC_INCREMENTS);
        		uint32_t vmr = _n5link->readU32(VIRT_MOT_REVOLUTIONS);
			if ( vmr != 0 ) f = (float)vei / ((float) vmr);
			else f = nan("");
			//-----------------------------:
			ss << "Virtual encoder         : " << f << " steps/rev";
			dump.push_back(ss.str());

			//===================================================================
			// non sembra funzionare, quindi per ora non considerato
			#if 0
			uint16_t sw = statusWord();
			ss.str("");
			//-----------------------------:
			//ss << "CLA (Close Loop Avail.)?: " << sw << " ";
			ss << "CLA (Close Loop Avail.)?: ";
			// la doc dice che CLA e' bit15 => .... 0x8000
			// ma a me sembra sia 0x1000
			if ( sw & 0x1000 ) {
				ss << "yes, Auto setup was successful and encoder index seen: closed loop mode possible";
			} else {
				ss << "no";
			}
			dump.push_back(ss.str());
			#endif
			//===================================================================

			ss.str("");
			bool closeloop = false;
        		uint32_t mds = _n5link->readU32(MOTOR_DRIVE_SUBMODE);
			//-----------------------------:
			ss << "Loop                    : ";
			switch (mds & 0x1){
				case 0: ss << "Open loop"; break;
				case 1: ss << "Close loop"; closeloop = true; break;
			}
			dump.push_back(ss.str());

			if (closeloop){
				if ( _sw_vers >= SW_VERS_2 ) {
					ss.str("");
					//-----------------------------:
					ss << "V1 style PI parameters  : ";
					dump.push_back(ss.str());
				}

				ss.str("");
        			u32 = _n5link->readU32(CLOOP_POSITION_P);
				f = (float)u32 / pow(2, 16);
				//-----------------------------:
				ss << "Position loop Kp        : " << f;
				dump.push_back(ss.str());

				ss.str("");
        			u32 = _n5link->readU32(CLOOP_POSITION_I);
				f = (float)u32 / pow(2, 16);
				ss << "              Ki        : " << f;
				dump.push_back(ss.str());

				ss.str("");
        			u32 = _n5link->readU32(CLOOP_VELOCITY_P);
				f = (float)u32 / pow(2, 16);
				//-----------------------------:
				//-------------
				ss << "Velocity loop Kp        : " << f;
				dump.push_back(ss.str());

				ss.str("");
        			u32 = _n5link->readU32(CLOOP_VELOCITY_I);
				f = (float)u32 / pow(2, 16);
				ss << "              Ki        : " << f;
				dump.push_back(ss.str());

				ss.str("");
        			u32 = _n5link->readU32(CLOOP_FLUX_CURRENT_P);
				f = (float)u32 / pow(2, 16);
				//-----------------------------:
				//-------------
				ss << "Flux     loop Kp        : " << f;
				dump.push_back(ss.str());

				ss.str("");
        			u32 = _n5link->readU32(CLOOP_FLUX_CURRENT_I);
				f = (float)u32 / pow(2, 16);
				ss << "              Ki        : " << f;
				dump.push_back(ss.str());

				ss.str("");
        			u32 = _n5link->readU32(CLOOP_TORQUE_CURRENT_P);
				f = (float)u32 / pow(2, 16);
				//-----------------------------:
				//-------------
				ss << "Torque   loop Kp        : " << f;
				dump.push_back(ss.str());

				ss.str("");
        			u32 = _n5link->readU32(CLOOP_TORQUE_CURRENT_I);
				f = (float)u32 / pow(2, 16);
				ss << "              Ki        : " << f;
				dump.push_back(ss.str());
			} else {
				if ( _sw_vers >= SW_VERS_2 ) {
					ss.str("");
					//-----------------------------:
					ss << "V1 style PI parameters  : ";
					dump.push_back(ss.str());
				}

				ss.str("");
        			u32 = _n5link->readU32(OLOOP_TORQUE_CURRENT_P);
				f = (float)u32 / pow(2, 16);
				//-----------------------------:
				//-------------
				ss << "Torque   loop Kp        : " << f;
				dump.push_back(ss.str());

				ss.str("");
        			u32 = _n5link->readU32(OLOOP_TORQUE_CURRENT_I);
				f = (float)u32 / pow(2, 16);
				ss << "              Ki        : " << f;
				dump.push_back(ss.str());
			}

			if (closeloop){
				ss.str("");
				if ( _sw_vers < SW_VERS_2 ) {
        				i32 = _n5link->readU32(CLOOP_DISPLAY_VELOCITY);
					switch(i32){
						case -1: ss << "0x6044/Vl Velocity      : internally calculated set value"; break;
						case  0: ss << "0x6044/Vl Velocity      : fixed 0"; break;
						case  1: ss << "0x6044/Vl Velocity      : encoder value"; break;
						default: ss << "??"; break;
					}
				} else {
					ss << "0x6044/Vl Velocity      : ";
					ss << "V2: ";
					ss << n5v2encidNames[_n5v2_feedback_map.speed_sensor];
				}
				dump.push_back(ss.str());

				ss.str("");
				if ( _sw_vers < SW_VERS_2 ) {
        				i32 = _n5link->readU32(CLOOP_DISPLAY_POSITION);
					switch(i32){
					         	//-----------------------------:
						case -1: ss << "0x6064/Position         : internally calculated set value"; break;
						case  0: ss << "0x6064/Position         : fixed 0"; break;
						case  1: ss << "0x6064/Position         : encoder value"; break;
						default: ss << "??"; break;
					}
				} else {
					ss << "0x6064/Position         : ";
					ss << "V2: ";
					ss << n5v2encidNames[_n5v2_feedback_map.position_sensor];
				}
				dump.push_back(ss.str());
			} else {
				ss.str("");
				if ( _sw_vers < SW_VERS_2 ) {
        				i32 = _n5link->readU32(OLOOP_DISPLAY_VELOCITY);
					switch(i32){
						case -1: ss << "0x6044/Vl Velocity      : internally calculated set value"; break;
						case  0: ss << "0x6044/Vl Velocity      : fixed 0"; break;
						case  1: ss << "0x6044/Vl Velocity      : encoder value"; break;
						default: ss << "??"; break;
					}
				} else {
					ss << "0x6044/Vl Velocity      : ";
					ss << "V2: ";
					ss << n5v2encidNames[_n5v2_feedback_map.speed_sensor];
				}
				dump.push_back(ss.str());

				ss.str("");
				if ( _sw_vers < SW_VERS_2 ) {
        				i32 = _n5link->readU32(OLOOP_DISPLAY_POSITION);
					switch(i32){
					         //-----------------------------:
						case -1: ss << "0x6064/Position         : internally calculated set value"; break;
						case  0: ss << "0x6064/Position         : fixed 0"; break;
						case  1: ss << "0x6064/Position         : encoder value"; break;
						default: ss << "??"; break;
					}
				} else {
					ss << "0x6064/Position         : ";
					ss << "V2: ";
					ss << n5v2encidNames[_n5v2_feedback_map.position_sensor];
				}
				dump.push_back(ss.str());
			}

        		uint32_t ire = _n5link->readU32(INPUT_ROUTING_ENABLE);
        		uint32_t isfe = _n5link->readU32(INPUT_SPECIAL_FUNC_ENABLE);
        		uint32_t ii = _n5link->readU32(INPUT_INVERT);
			ss.str("");
		        //-----------------------------:
			ss << "Input 1 {-LS}           : ";
			ss << dumpInputCfg(ire, isfe, ii, 0x1);
			dump.push_back(ss.str());
			ss.str("");
			ss << "Input 2 {+LS}           : ";
			ss << dumpInputCfg(ire, isfe, ii, 0x2);
			dump.push_back(ss.str());
			ss.str("");
			ss << "Input 3 {Home}          : ";
			ss << dumpInputCfg(ire, isfe, ii, 0x4);
			dump.push_back(ss.str());

			if ( _sw_vers < SW_VERS_2 ) {
				ss.str("");
				uint32_t velnum= _n5link->readU32(USER_VELOCITY_NUMERATOR);
				uint32_t velden= _n5link->readU32(USER_VELOCITY_DENOMINATOR);
				double n_speed;
				if ( velden != 0 ) n_speed = ((double)velnum) / ((double)velden);
				else n_speed = nan("");
			        //-----------------------------:
				ss << "n_speed                 : " << n_speed;
				ss << " i.e. speed in [" << n_speed << "rps]";
				ss << " or [" << (360.0*n_speed) << "degree/s]";
				dump.push_back(ss.str());
	
				ss.str("");
				uint32_t accelnum= _n5link->readU32(USER_ACCEL_NUMERATOR);
				uint32_t accelden= _n5link->readU32(USER_ACCEL_DENOMINATOR);
				double n_accel;
				if ( accelden != 0 ) n_accel = ((double)accelnum) / ((double)accelden);
				else n_accel = nan("");
			        //-----------------------------:
				ss << "n_accel                 : " << n_accel;
				ss << " i.e. accel in [" << n_accel << "r/s^2]";
				ss << " or [" << (360.0*n_accel) << "degree/s^2]";
				dump.push_back(ss.str());
			} else {
				ss.str("");
				uint32_t velnum= _n5link->readU32(V2_USER_VELOCITY_NUMERATOR);
				uint32_t velden= _n5link->readU32(V2_USER_VELOCITY_DENOMINATOR);
				double n_speed;
				if ( velden != 0 ) n_speed = ((double)velnum) / ((double)velden);
				else n_speed = nan("");
			        //-----------------------------:
				ss << "additional n_speed      : " << n_speed;
				dump.push_back(ss.str());

				ss.str("");
				uint32_t accelnum= _n5link->readU32(V2_USER_ACCEL_NUMERATOR);
				uint32_t accelden= _n5link->readU32(V2_USER_ACCEL_DENOMINATOR);
				double n_accel;
				if ( accelden != 0 ) n_accel = ((double)accelnum) / ((double)accelden);
				else n_accel = nan("");
			        //-----------------------------:
				ss << "additional n_accel      : " << n_accel;
				dump.push_back(ss.str());
			}


			ss.str("");
			int8_t opmode = get_opmode();
		        //-----------------------------:
		        ss << "OpMode                  : " << OpMode2str(opmode);
			dump.push_back(ss.str());



        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return dump;
}


int8_t N5Drv::get_opmode(){
	int8_t x = 0;
	if (_n5link){
        	try {
        		x = _n5link->readI8(OP_MODE);
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	_opmode = x;
	return x;
}

std::string N5Drv::OpMode2str(int8_t op){
	switch (op) {
		case -2 : return "Auto setup"; break;
		case -1 : return "Clock-direction mode"; break;
		case 0  : return "No mode change/no mode assigned"; break;
		case 1  : return "Profile Position Mode"; break;
		case 2  : return "Velocity Mode"; break;
		case 3  : return "Profile Velocity Mode"; break;
		case 4  : return "Profile Torque Mode"; break;
		case 5  : return "Reserved"; break;
		case 6  : return "Homing Mode"; break;
		case 7  : return "Interpolated Position Mode"; break;
		case 8  : return "Cyclic Synchronous Position Mode"; break;
		case 9  : return "Cyclic Synchronous Velocity Mode"; break;
		case 10 : return "Cyclic Synchronous Torque Mode"; break;
		default : return "Unknown mode!"; break;
	}
	return "Unknown mode!";
}

bool N5Drv::set_opmode(int8_t op){
        std::stringstream ss;
	ss << "[" << int(op) << "]:" << OpMode2str(op);

        string obj("N5Drv::set_opmode()");

	#ifdef _DBG_SET_OPMODE
        	state_t  actualstate = state();
		ss << " with CIA402State: " << statusNames[actualstate];
        	prtverbose(0, obj, ss.str());
	#else
        	prtverbose(1, obj, ss.str());
	#endif

        if (_n5link) return _n5link->writeI8(OP_MODE, op);
	return false;
}

/*
Bit 13 0 0 0 0 1 1
Bit 12 0 0 1 1 0 0
Bit 10 0 1 0 1 0 1
       | | | | | >Error during homing, motor at standstill
       | | | | >Error during homing, motor still turning
       | | | >Homing completed
       | | >Homing confirmed, but target not yet reached
       | >Homing is interrupted or not started
       >Homing is performed
*/
bool N5Drv::get_cinit(){
	bool ret = false;

	#ifdef __CIA402_INIT_STATE__
		int8_t op = get_opmode();
		if (set_opmode(HOMING)){
			uint16_t sw = statusWord();
			if ((sw & 0b0011010000000000) == 0b0001010000000000) ret = true;
			if (!set_opmode(op)) cout << "err on restore op!" << endl;
		}
	#else
	int32_t i32 = 0;
	if (_n5link){
        	try {
        		i32 = _n5link->readI32(AXISINIT_STATE);
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	if ( i32 == 1 ) ret = true;
	#endif

	_isinit = ret;
	return ret;
}


bool    N5Drv::homing_method(int8_t h){
	if (_n5link) return _n5link->homing_method(h);
	return false;
}

int8_t  N5Drv::homing_method(){
	if (_n5link) return _n5link->homing_method();
	return 0;
}



//============================================================================

int8_t N5Drv::opmode(string channel){
        std::stringstream ss;
        string obj("N5Drv::opmode("+channel+")");
        prtverbose(1, obj, "");

	return _opmode;
}
                                   
bool N5Drv::opmode(string channel, int8_t op){
        std::stringstream ss;
        string obj("N5Drv::opmode("+channel+",.)");
        ss << "(op=" << op << ")";
        prtverbose(1, obj, "");

	return set_opmode(op);
}

bool    N5Drv::cinit(string channel){         // true = INIT / false = NO INIT
	return _isinit;
}

bool    N5Drv::cinit(string channel, bool b){ // true = INIT / false = DEINIT
	#ifdef __CIA402_INIT_STATE__
	#else
		int32_t i32 = (b?1:0);
        	if (_n5link) b = _n5link->writeI32(AXISINIT_STATE, i32);
	#endif
	_isinit = b;
	return _isinit;
}


Link_ns::rvalues_t N5Drv::refresh(string channel) {
        std::stringstream ss;
        string obj("N5Drv::refresh("+channel+")");
        prtverbose(1, obj, "");

        timeval start, end;
        gettimeofday(&start, NULL);

        Link_ns::rvalues_t rv;

        //double speed;
        //double acceleration;
        //double deceleration;

        rv.speed        = (double)pp_speed();
        rv.acceleration = (double)pp_acceleration();
        rv.deceleration = (double)pp_deceleration();

	int8_t op = get_opmode();

        gettimeofday(&end, NULL);
        _last_refresh_ms = (double)(end.tv_sec - start.tv_sec)*1000 + ((double)(end.tv_usec - start.tv_usec))/1000;

        ss.str("");
        ss << "refresh time: " << _last_refresh_ms << "ms";
        prtverbose(2, obj, ss.str());

        return rv;
}

bool    N5Drv::stop(string channel){
        string obj("N5Drv::stop("+channel+")");

	#ifdef _DBG_STOPCMD
        	prtverbose(0, obj, "");
	#else
        	prtverbose(1, obj, "");
	#endif
        std::stringstream ss;
	bool ret1 = false;
	bool ret = false;

        if (!__hw_write_enable) return true;

	if (_with_microcode){
		ret1 = n5stop(channel);
	} else {
		ret1 = true;
	}

/*
	HALT
	Value = "1": Triggers a halt; valid in the following modes:
		Profile Position
		Velocity
		Profile Velocity
		Profile Torque
		Interpolated Position Mode
*/
	#ifdef _DBG_STOPCMD
		ss.str("");
		ss << OpMode2str(get_opmode());
        	prtverbose(0, obj, ss.str());
	#endif

	switch (get_opmode()){
        	case PROFILE_POSITION:
        	case VELOCITY:
        	case PROFILE_VELOCITY:
        	case PROFILE_TORQUE:
        	case INTERPOLATED_POSITION:
			#ifdef _DBG_STOPCMD
				ss.str("");
				ss << "PROFILE_POSITION, .....";
        			prtverbose(0, obj, ss.str());
			#endif
			break;

		default:
        	case AUTO_SETUP:
        	case CLOCK_DIRECTION:
        	case NO_MODE:
        	case RESERVED:
        	case HOMING:
        	case CYCLIC_SYNC_POSITION:
        	case CYCLIC_SYNC_VELOCITY:
        	case CYCLIC_SYNC_TORQUE:
			#ifdef _DBG_STOPCMD
				ss.str("");
				ss << "HOMING, .....";
        			prtverbose(0, obj, ss.str());
			#endif
			{
        		  state_t  newstate;
        		  state_t  actualstate = state();

			  #ifdef _DBG_STOPCMD
				  ss.str("");
				  ss << "actualstate: " << statusNames[actualstate];
        			  prtverbose(0, obj, ss.str());
			  #endif

        		  switch (actualstate) {
                		case OPERATION_ENABLED:
                        		newstate = CiA402StateCmd(DISABLE_OPERATION_CMD);
			  		#ifdef _DBG_STOPCMD
				  		ss.str("");
				  		ss << "newstate: " << statusNames[newstate];
        			  		prtverbose(0, obj, ss.str());
			  		#endif
                        		if ( newstate !=  SWITCHON ) break;
                        		ret = true;
                        		break;
                		default:
                        		break;
        		  }
			}
			break;
	}

	#ifdef _DBG_STOPCMD
		ss.str("");
		ss << "Going to set bit 8 of controlWord";
		prtverbose(0, obj, ss.str());
	#endif

        uint16_t cw = controlWord();
        cw |= 0x0100;   //set bit 8
        if ( !controlWord(cw) ) return false;
	return (ret && true && ret1);
}


bool    N5Drv::stop_no_brake(string channel)
		{ return N5Drv::stop(channel); }

bool    N5Drv::speed(string channel, double v){
        string obj("N5Drv::speed("+channel+",.)");
        std::stringstream ss;
        ss << "(speed=" << v << ")";
        prtverbose(1, obj, ss.str());

        if (!__hw_write_enable) return true; // forse anche qui false ma vedi axisg2

	if ( fabs(v) > (double)UINT32_MAX ) return false;

	return pp_speed((uint32_t)fabs(v));
}

double  N5Drv::speed(string channel){
        std::stringstream ss;
        string obj("N5Drv::speed("+channel+")");
        prtverbose(1, obj, "");

	return (double)pp_speed();
}

bool    N5Drv::acceleration(string channel, double a){
        string obj("N5Drv::acceleration("+channel+",.)");
        std::stringstream ss;
        ss << "(acceleration=" << a << ")";
        prtverbose(1, obj, ss.str());

        if (!__hw_write_enable) return true; // forse anche qui false ma vedi axisg2

	if ( fabs(a) > (double)UINT32_MAX ) return false;

	return pp_acceleration((uint32_t)fabs(a));
}

double  N5Drv::acceleration(string channel){
        std::stringstream ss;
        string obj("N5Drv::acceleration("+channel+")");
        prtverbose(1, obj, "");

	return (double)pp_acceleration();
}

bool    N5Drv::deceleration(string channel, double a){
        string obj("N5Drv::deceleration("+channel+",.)");
        std::stringstream ss;
        ss << "(deceleration=" << a << ")";
        prtverbose(1, obj, ss.str());

        if (!__hw_write_enable) return true; // forse anche qui false ma vedi axisg2

	if ( fabs(a) > (double)UINT32_MAX ) return false;

	return pp_deceleration((uint32_t)fabs(a));
}

double  N5Drv::deceleration(string channel){
        std::stringstream ss;
        string obj("N5Drv::deceleration("+channel+")");
        prtverbose(1, obj, "");

	return (double)pp_deceleration();
}

bool    N5Drv::forward(string channel, double v){
	//senza cambiare profilo (ad es. da pp a pv)
	// simulo il fwd con mpos(+infinito)

        std::stringstream ss;
        string obj("N5Drv::forward("+channel+",.)");
        ss << "(speed=" << v << ")";
        prtverbose(1, obj, ss.str());

        if (!__hw_write_enable) return true;

	if (!get_fls()) {
		if ( speed(channel, v) ) return mpos(channel, (double)INT32_MAX);
	};

	return false;
}

bool    N5Drv::backward(string channel, double v){
	//senza cambiare profilo (ad es. da pp a pv)
	// simulo il bck con mpos(-infinito)

        std::stringstream ss;
        string obj("N5Drv::forward("+channel+",.)");
        ss << "(speed=" << v << ")";
        prtverbose(1, obj, ss.str());

        if (!__hw_write_enable) return true;

	if (!get_rls()) {
		if ( speed(channel, v) ) return mpos(channel, (double)INT32_MIN);
	};

	return false;
}

bool    N5Drv::findedge(string channel, double v)
		{ return false; }

bool    N5Drv::findindex(string channel, double v)
		{ return false; }

double  N5Drv::mpos(string channel){
        std::stringstream ss;
        string obj("N5Drv::mpos("+channel+")");
        prtverbose(1, obj, "");

	if (_n5link) {
        	int32_t x = _n5link->readI32(POSITION_DEMANDED);
        	return double(x);
	}
	return nan("");
}

bool    N5Drv::mpos(string channel, double p){       // mpos : motor position
        string obj("N5Drv::mpos("+channel+",.)");
        std::stringstream ss;
        ss << "(p=" << p << ")";
        prtverbose(1, obj, ss.str());

        if (!__hw_write_enable) return true;

	if (!_n5link) return false;

        if ( isnan(p) ) return false;

	if ( p > (double)INT32_MAX ) return false;
	if ( p < (double)INT32_MIN ) return false;

	double cur_pos = mpos(channel);
	if ( isnan(cur_pos) ) return false;
	if ( ((p - cur_pos) > 0) && get_fls() ) return false;
	if ( ((p - cur_pos) < 0) && get_rls() ) return false;

        //prtverbose(1, obj, "Proceed 1");

	if ( state() == FAULT ){
		if ( CiA402FaultResetOnLimitSwitch() ) motor_on(channel);
	}

        //if ( !_n5link->writeI32(PP_TARGET_POSITION, (int32_t)p) ) return false;
        uint16_t cw = controlWord();
        cw &= 0xFEEF;   //clear bit 4 & bit 8
        cw &= 0xFFBF;   //bit 6 (100 000 = 0x40) absolute position
        cw |= 0x0020;   //set bit 5: Change Set-Point Immediately" if set, the controller operates without the buffer
        if ( !controlWord(cw) ) return false;

        if ( !_n5link->writeI32(PP_TARGET_POSITION, (int32_t)p) ) return false;

        cw |= 0x0010;   //set bit 4
        if ( !controlWord(cw) ) return false;
        int c = 10;
        bool sp = false;
        while (!(sp=SetPointAckd()) && c) { usleep(1000); c--; };
        if ( !c ) return false;
        return sp;
}

bool    N5Drv::xpos(VNOTU string channel, VNOTU double p){ // xpos : motor position in encoder units 

	if (_with_microcode) {
        	if (!__hw_write_enable) return true; /// anche qui forse false...

        	if (program_running()) return n5go(channel, (long) p, _nbrtr);
        	else                return false;
	} else {
		return false;
	}

}

bool    N5Drv::defmpos(string channel, double p){      // define motor position
        std::stringstream ss;
        string obj("N5Drv::defmpos("+channel+")");
	ss << "(" << p << ")";
        prtverbose(1, obj, ss.str());
	#ifdef _DBG_DEFPOSINIT
        	prtverbose(0, obj, ss.str());
	#endif

	if (!_n5link) return false;

        if ( !_n5link->writeI32(HOME_OFFSET, (int32_t)p) ) return false;
	return Cia402HomeMode35();
}

bool    N5Drv::defepos(string channel, double p){       // define hw encoder position
	if (_n5link) return _n5link->defepos(p); 
	return false;
}

bool    N5Drv::rpos(string channel, double p)       // rpos : motor relative position
		{ return false; }

bool    N5Drv::brake(VNOTU string channel, VNOTU bool b)
		{ return true;}

int     N5Drv::hw_brake_conf(VNOTU string channel)
		{ return have_brake(); }

bool    N5Drv::motor_off(string channel){
        std::stringstream ss;
        string obj("N5Drv::motor_off("+channel+")");
        prtverbose(1, obj, "");

        if (!__hw_write_enable) return true;

	ss.str("");
	ss << "CiA402: " << status();
        prtverbose(2, obj, ss.str());

        bool ret = false;
        state_t  newstate;
        state_t  actualstate = state();

        switch (actualstate) {
                case OPERATION_ENABLED:
                        newstate = CiA402StateCmd(DISABLE_OPERATION_CMD);
                        if ( newstate !=  SWITCHON ) break;
                        ret = true;
                        break;
                default:
                        break;
        }

        return ret;
}

bool    N5Drv::motor_on(string channel) {
        std::stringstream ss;
        string obj("N5Drv::motor_on("+channel+")");
        prtverbose(1, obj, "");

        if (!__hw_write_enable) return true;

	#ifdef _DBG_BRAKE
		cout << __func__ << "!!!!" << endl;
		_verbose = 2;
	#endif


	ss.str("");
	ss << "CiA402: " << status();
        prtverbose(2, obj, ss.str());

        bool ret = false;
        state_t  newstate;
        state_t  actualstate = state();

        switch (actualstate) {
                case SWITCHON_DISABLED:
                        newstate = CiA402StateCmd(SHUTDOWN_CMD);
                        if ( newstate !=  READY_TO_SWITCHON ) break;
                case READY_TO_SWITCHON:
                        newstate = CiA402StateCmd(SWITCH_ON_CMD);
                        if ( newstate !=  SWITCHON ) break;
                case SWITCHON:
                case QUICK_STOP:
                        newstate = CiA402StateCmd(ENABLE_OPERATION_CMD);
                        if ( newstate !=  OPERATION_ENABLED ) break;
                        ret = true;
                        break;
                case OPERATION_ENABLED:
                        ret = true;
                        break;
		case FAULT:
			// allora se siamo sui limitswitch posso tentare
			// di resettare la condizione di FAULT;
			if ( CiA402FaultResetOnLimitSwitch() ){
                        	newstate = CiA402StateCmd(ENABLE_OPERATION_CMD);
                        	if ( newstate !=  OPERATION_ENABLED ) break;
                        	ret = true;
			}
                        break;
                default:
                        break;
        }
	#ifdef _DBG_BRAKE
		cout << __func__ << "!!!!" << endl;
		_verbose = 0;
	#endif

        return ret;
}

string  N5Drv::stop_code2str(unsigned long stop_code)
		{ return "-"; }

bool    N5Drv::mconf(VNOTU string channel, VNOTU mconf_t mc){
	string obj("N5Drv::mconf("+channel+",.)");
        std::stringstream ss;
        ss << "mc.yc.mt =" << mc.yc.mt;

        prtverbose(1, obj, ss.str());

        if (!__hw_write_enable) return true;

	if (0) {
		ss.str("");
		ss << "XXX per ora nessuna conf verso N5! XXX";
        	prtverbose(1, obj, ss.str());
		return true;
	}

	bool b1(false), b2(false);

	if ( mc.yc.mt ==  1.0 ) {
		if (_n5link) {
			_n5link->with_microcode(false);
			_with_microcode = false;
		};
		b1 = is_bldc(true);
		b2 = pp_polarity(true); } else

	if ( mc.yc.mt == -1.0 ) {
		if (_n5link) {
			_n5link->with_microcode(false);
			_with_microcode = false;
		};
		b1 = is_bldc(true);
		b2 = pp_polarity(false); } else

	if ( mc.yc.mt == -2.0 ) {
		if (_n5link) {
			_n5link->with_microcode(true);
			_with_microcode = true;
		};
		b1 = is_bldc(false);
		b2 = pp_polarity(true); } else

	if ( mc.yc.mt == -2.5 ) {
		if (_n5link) {
			_n5link->with_microcode(true);
			_with_microcode = true;
		};
		b1 = is_bldc(false);
		b2 = pp_polarity(false); } else

	{
		b1 = false;
		b2 = false;
	}

	if (_with_microcode) {
		bool b3(false), b4(false);
	        b3 = n5nbrtr(channel, mc.yc.nbrtr);
	        b4 = n5conf(channel, mc.yc);

		return (b1 && b2 && b3 && b4);
	} else {
		return (b1 && b2);
	}
}

mconf_t N5Drv::mconf(VNOTU string channel){
        string obj("N5Drv::mconf("+channel+")");
        std::stringstream ss;
        prtverbose(1, obj, "");


	mconf_t mc;

	mc.yc.mt = nan("");
	if ( is_bldc() ){
		mc.yc.mt = 1; 
		if ( !pp_polarity() ) mc.yc.mt = -1; 
	} else {
		mc.yc.mt = -2; 
		if ( !pp_polarity() ) mc.yc.mt = -2.5; 
	}

	ss.str("");
	ss << "READ mc.yc.mt =" << mc.yc.mt;
        prtverbose(1, obj, ss.str());

	if (_with_microcode) {
		mc.yc.nbrtr     = _nbrtr;

		n5rvalues_t n5r     = n5refresh(channel);
	        mc.yc.dbw           = n5r.yc.dbw;
	        mc.yc.scale         = n5r.yc.scale;
	        mc.yc.pc            = n5r.yc.pc;
	        mc.yc.bck           = n5r.yc.bck;
	        mc.yc.dir           = n5r.yc.dir;
	  	mc.yc.hw_brake_conf = n5r.yc.hw_brake_conf;
	} else {
		if (have_brake()) mc.yc.hw_brake_conf = 1;
		else              mc.yc.hw_brake_conf = 0;
	}

	_hw_brake_conf = mc.yc.hw_brake_conf;

	//cout << __func__ << " mc.yc.hw_brake_conf = " << mc.yc.hw_brake_conf << endl;

	return mc;
}

bool    N5Drv::econf(VNOTU string channel, VNOTU econf_t ec) {
        string obj("N5Drv::econf("+channel+",.)");
        std::stringstream ss;
        ss << "ec.ce =" << ec.ce;
        prtverbose(1, obj, ss.str());

        switch (ec.which){
                case Controller_ns::ECONF_CE:
                        return ce_econf(channel, ec);
                        break;

                case Controller_ns::ECONF_MOFFON:
                        //return moffon_econf(channel, ec);
                        return false;
                        break;

                case Controller_ns::ECONF_ALL:
                        {
                          bool b1, b2;
                          b1 = ce_econf(channel, ec);
			  b2 = true;
                          //b2 = moffon_econf(channel, ec);
                          return (b1 && b2);
                        }
                        break;

                case Controller_ns::ECONF_NONE:
                        return true;
                        break;

                default:
                        return false;
                        break;
        }
        return false;
}

econf_t N5Drv::econf(string channel){
        string obj("N5Drv::econf("+channel+")");
        std::stringstream ss;
        prtverbose(1, obj, "");

        econf_t ec;

        econf_t ec1 = ce_econf(channel);
        //econf_t ec2 = moffon_econf(channel);
	econf_t ec2 = {ECONF_MOFFON, 0, {false, nan(""), nan("")}};

        ec.which          = Controller_ns::ECONF_ALL;
        ec.ce             = ec1.ce;
        ec.mOffOn.enabled = ec2.mOffOn.enabled;
        ec.mOffOn.m       = ec2.mOffOn.m;
        ec.mOffOn.q       = ec2.mOffOn.q;

        return ec;
}

econf_t N5Drv::ce_econf(VNOTU string channel) {
        string obj("N5Drv::ce_econf("+channel+")");
        std::stringstream ss;
        prtverbose(1, obj, "");

	econf_t ec;
	ec.ce = -1;

	int32_t x = 0;
	if (_n5link){
        	try {
        		x = _n5link->readI32(ENCODER_RESOLUTION);
			ec.ce = int(x);
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
 	return ec;
}

bool N5Drv::ce_econf(string channel, econf_t ec){
        string obj("N5Drv::ce_econf("+channel+",.)");
        std::stringstream ss;
        ss << "ec.ce =" << ec.ce;
        prtverbose(1, obj, ss.str());

        if (!__hw_write_enable) return true;

	if (_n5link){
        	try {
        		return _n5link->writeI32(ENCODER_RESOLUTION, (int32_t)ec.ce);
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return false;
}

bool    N5Drv::program_running(){
        string obj("N5Drv::program_running()");
        std::stringstream ss;
        prtverbose(2, obj, "");

	if (_n5link){
        	try {
        		bool run_state = (_n5link->readU32(NANOJ_STATUS)) & 0x1;
			ss.str("");
			ss << (run_state?"RUNNING":"STOPPED");
        		prtverbose(2, obj, ss.str());
			return run_state;
        	} catch (...) {
                	// errori di comunicazione
        	}
	}

	return false;
}


/*******************************************************
*
*
*
*
*******************************************************/
bool N5Drv::Cia402_opEnable(){

        bool ret = false;
        state_t  newstate;
        state_t  actualstate = state();

        switch (actualstate) {
                case SWITCHON_DISABLED:
                        newstate = CiA402StateCmd(SHUTDOWN_CMD);
                        if ( newstate !=  READY_TO_SWITCHON ) break;
                case READY_TO_SWITCHON:
                        newstate = CiA402StateCmd(SWITCH_ON_CMD);
                        if ( newstate !=  SWITCHON ) break;
                case SWITCHON:
                case QUICK_STOP:
                        newstate = CiA402StateCmd(ENABLE_OPERATION_CMD);
                        if ( newstate !=  OPERATION_ENABLED ) break;
                        ret = true;
                        break;
                case OPERATION_ENABLED:
                        ret = true;
                        break;
		case FAULT:
			// allora se siamo sui limitswitch posso tentare
			// di resettare la condizione di FAULT;
			if ( CiA402FaultResetOnLimitSwitch() ){
                        	newstate = CiA402StateCmd(ENABLE_OPERATION_CMD);
                        	if ( newstate !=  OPERATION_ENABLED ) break;
                        	ret = true;
			}
                        break;
                default:
                        break;
        }
	#if defined(_DBG_AXISINIT) || defined(_DBG_CIA402INIT)
		if (ret) {
		} else {
			uint16_t sw = statusWord();
			cout << __func__ << "@" << __LINE__ << " : " << sw << endl;
			prt_statusExt();
		}
	#endif

        return ret;
}

bool N5Drv::Cia402_opDisable(){
        std::stringstream ss;
        string obj("N5Drv::Cia402_opDisable()");

        bool ret = false;
        state_t  newstate;
        state_t  actualstate = state();

        ss.str("");
	#if defined(_DBG_AXISINIT) || defined(_DBG_CIA402INIT)
		ss << "ACTUAL STATE: " << statusNames[actualstate];
        	prtverbose(0, obj, ss.str());
	#else
        	prtverbose(1, obj, ss.str());
	#endif

        switch (actualstate) {
                case OPERATION_ENABLED:
                        newstate = CiA402StateCmd(DISABLE_OPERATION_CMD);
			#if defined(_DBG_AXISINIT) || defined(_DBG_CIA402INIT)
        			ss.str("");
				ss << "From OPERATION_ENABLED to ";
				ss << statusNames[newstate];
        			prtverbose(0, obj, ss.str());
			#endif
                        if ( newstate !=  OPERATION_ENABLED ) ret = true;
                        break;
                default:
			ret = true;
                        break;
        }
	ss.str("ret: ");
	ss << (ret?"TRUE":"FALSE");
	#if defined(_DBG_AXISINIT) || defined(_DBG_CIA402INIT)
        	prtverbose(0, obj, ss.str());
	#else
        	prtverbose(1, obj, ss.str());
	#endif
        return ret;
}


/*******************************************************
*
*
*
*
*******************************************************/
bool N5Drv::Cia402HomeMode35(){

        if (homing_method(35)){
		#ifdef _DBG_DEFPOSINIT
			cout << "homing_method() = " << int(homing_method()) << endl;
		#endif

        	int8_t op = get_opmode();
                if (set_opmode(HOMING)){
                	uint16_t cw = controlWord();
                        cw &= !((uint16_t)0x10);
                        if (controlWord(cw)) {
                        	cw |= 0x10;
                                if (controlWord(cw)) {
                                	// restore op mode
                                        if (!set_opmode(op)) cout << "err on restore op!" << endl;
                                        else return true;
                                }
                        }
                }
        }
        return false;
}

bool N5Drv::Cia402HomingStart(int mode){
        std::stringstream ss;
        string obj("N5Drv::Cia402HomingStart()");

        ss.str("");
	ss << "{Cia402 mode: " << mode << "}";
       	prtverbose(0, obj, ss.str());

	#if defined(_DBG_AXISINIT) || defined(_DBG_CIA402INIT)
		cout << __func__ << "@" << __LINE__ << " : " << status() << " | " << "OpMode : " << OpMode2str(get_opmode()) << endl;
	#endif

	if ( !Cia402_opDisable() ) return false;
	#if defined(_DBG_AXISINIT) || defined(_DBG_CIA402INIT)
		cout << __func__ << "@" << __LINE__ << " : " << status() << " | " << "OpMode : " << OpMode2str(get_opmode()) << endl;
	#endif

        if (homing_method(mode)){
		#if defined(_DBG_AXISINIT) || defined(_DBG_CIA402INIT)
			cout << __func__ << "@" << __LINE__ << " : " << "homing_method() = " << int(homing_method()) << endl;
			Cia402HomingState();
		#endif

                if (set_opmode(HOMING)){
			bool cia402ret = Cia402_opEnable();
			#if defined(_DBG_AXISINIT) || defined(_DBG_CIA402INIT)
				cout << __func__ << "@" << __LINE__ << " : cia402ret=" << (cia402ret?"true":"false");
				cout << " : " << status() << " | " << "OpMode : " << OpMode2str(get_opmode()) << endl;
			#endif
			Cia402HomingState();
			#if defined(_DBG_AXISINIT) || defined(_DBG_CIA402INIT)
				cout << __func__ << "@" << __LINE__ << " : cia402ret=" << (cia402ret?"true":"false");
				cout << " : " << status() << " | " << "OpMode : " << OpMode2str(get_opmode()) << endl;
			#endif

                	uint16_t cw = controlWord();
                        cw &= 0xFFEF;
                        if (controlWord(cw)) {
				usleep(10000);
				Cia402HomingState();
				#if defined(_DBG_AXISINIT) || defined(_DBG_CIA402INIT)
					cout << __func__ << "@" << __LINE__ << " : " << status() << " | " << "OpMode : " << OpMode2str(get_opmode()) << endl;
				#endif
                        	cw |= 0x10;
                                if (controlWord(cw)) {
					usleep(10000);
					Cia402HomingState();
					#if defined(_DBG_AXISINIT) || defined(_DBG_CIA402INIT)
						cout << __func__ << "@" << __LINE__ << " : " << status() << " | " << "OpMode : " << OpMode2str(get_opmode()) << endl;
					#endif
                                	return true;
                                }
                        }
                }
        }
	#if defined(_DBG_AXISINIT) || defined(_DBG_CIA402INIT)
		ss.str("");
		ss << "EXIT(FALSE)";
        	prtverbose(0, obj, ss.str());
	#endif
	return false;
}

bool N5Drv::Cia402HomingEndRestore(int mode){
       	int8_t op = get_opmode();

	#if defined(_DBG_AXISINIT) || defined(_DBG_CIA402INIT)
		cout << __func__ << " =========================== " << status() << endl;
	#endif
	if ( !Cia402_opDisable() ) return false;
	usleep(1500);
	#if defined(_DBG_AXISINIT) || defined(_DBG_CIA402INIT)
		cout << __func__ << " =========================== " << __LINE__ << status() << " | " << "OpMode : " << OpMode2str(get_opmode()) << endl;
	#endif

	switch(op){
		case HOMING:
                	if (!set_opmode(mode)) {
				cout << "err on restore op!" << endl;
				return false;
			} else {
				#if defined(_DBG_AXISINIT) || defined(_DBG_CIA402INIT)
					cout << __func__ << " ============== OKOK ============= " << __LINE__ << status() << " | " << "OpMode : " << OpMode2str(get_opmode()) << endl;
				#endif
				return true;
			}
			break;
		default:
			break;
	}
	return false;
}


/*
Bit 13 0 0 0 0 1 1
Bit 12 0 0 1 1 0 0
Bit 10 0 1 0 1 0 1
       | | | | | >Error during homing, motor at standstill	CIA402_HOME_ERROR_STANDSTILL
       | | | | >Error during homing, motor still turning	CIA402_HOME_ERROR_MOVING
       | | | >Homing completed					CIA402_HOME_COMPLETED
       | | >Homing confirmed, but target not yet reached	CIA402_HOME_PROGRESS
       | >Homing is interrupted or not started			CIA402_HOME_INTERRUPTED
       >Homing is performed					CIA402_HOME_PERFORMED
*/
Cia402HomingState_t N5Drv::Cia402HomingState(){
        std::stringstream ss;
        string obj("N5Drv::Cia402HomingState()");

        ss.str("");
        prtverbose(1, obj, ss.str());

	uint16_t hmst = 0;
	Cia402HomingState_t hst = Controller_ns::CIA402_HOME_PERFORMED;

	switch (get_opmode()){
		case HOMING:
			hmst = statusWord() & 0b0011010000000000;
			switch (hmst){
				default:
				case 0b0000000000000000: hst = Controller_ns::CIA402_HOME_PERFORMED; break;
				case 0b0000010000000000: hst = Controller_ns::CIA402_HOME_INTERRUPTED; break;
				case 0b0001000000000000: hst = Controller_ns::CIA402_HOME_PROGRESS; break;
				case 0b0001010000000000: hst = Controller_ns::CIA402_HOME_COMPLETED; break;
				case 0b0010000000000000: hst = Controller_ns::CIA402_HOME_ERROR_MOVING; break;
				case 0b0010010000000000: hst = Controller_ns::CIA402_HOME_ERROR_STANDSTILL; break;
			}
			break;
		default:
			break;
	}

        #ifdef _DBG_AXISINIT
		ss.str("");
		switch (hst){
			default:
			case Controller_ns::CIA402_HOME_PERFORMED:
				ss << "CIA402_HOME_PERFORMED";
				break;
			case Controller_ns::CIA402_HOME_INTERRUPTED:
				ss << "CIA402_HOME_INTERRUPTED";
				break;
			case Controller_ns::CIA402_HOME_PROGRESS:
				ss << "CIA402_HOME_PROGRESS";
				break;
			case Controller_ns::CIA402_HOME_COMPLETED:
				ss << "CIA402_HOME_COMPLETED";
				break;
			case Controller_ns::CIA402_HOME_ERROR_MOVING:
				ss << "CIA402_HOME_ERROR_MOVING";
				break;
			case Controller_ns::CIA402_HOME_ERROR_STANDSTILL:
				ss << "CIA402_HOME_ERROR_STANDSTILL";
				break;
		}
                prtverbose(0, obj, ss.str());
        #endif
	return hst;
}

/*
typedef struct {
        double home_offset;
        double home_speed_switch;
        double home_speed_reference;
        double home_acceleration;
        double home_switch_tolerance_band;
        double home_min_current_block_detection;
        double home_period_of_blocking;
} Cia402HomingParams_t;
*/
Cia402HomingParams_t N5Drv::Cia402HomingParams(){
	Cia402HomingParams_t p = {nan(""),nan(""),nan(""),nan(""),nan(""),nan(""),nan("")};

	if (!_n5link) return p;
	double v;
	
	v = nan("");
	v = double(_n5link->readI32(HOME_OFFSET));
	p.home_offset = v;
	
	v = nan("");
	v = double(_n5link->readU32(HOME_SPEED_SWITCH));
	p.home_speed_switch = v;
	
	v = nan("");
	v = double(_n5link->readU32(HOME_SPEED_REFERENCE));
	p.home_speed_reference = v;
	
	v = nan("");
	v = double(_n5link->readU32(HOME_ACCELERATION));
	p.home_acceleration = v;
	
	v = nan("");
	if ( _sw_vers < SW_VERS_2 ) v = double(_n5link->readU32(LS_TOLERANCE));
	p.home_switch_tolerance_band = v;
	
	v = nan("");
	v = double(_n5link->readI32(HOME_BLOCK_MIN_CURR));
	p.home_min_current_block_detection = v;
	
	v = nan("");
	v = double(_n5link->readI32(HOME_BLOCK_DECT_TIME));
	p.home_period_of_blocking = v;

	#ifdef _DBG_AXISINIT
		cout << "Cia402HomingParams: " << endl;
		cout << "      p.home_offset                      : " << p.home_offset << endl;
		cout << "      p.home_speed_switch                : " << p.home_speed_switch << endl;
		cout << "      p.home_speed_reference             : " << p.home_speed_reference << endl;
		cout << "      p.home_acceleration                : " << p.home_acceleration << endl;
		cout << "      p.home_switch_tolerance_band       : " << p.home_switch_tolerance_band << endl;
		cout << "      p.home_min_current_block_detection : " << p.home_min_current_block_detection << endl;
		cout << "      p.home_period_of_blocking          : " << p.home_period_of_blocking << endl;
	#endif

	return p;
}

bool N5Drv::Cia402HomingParams(Cia402HomingParams_t p){
	if (!_n5link) return false;

	if ( !_n5link->writeI32(HOME_OFFSET, (int32_t)p.home_offset) ) return false;
	if ( !_n5link->writeU32(HOME_SPEED_SWITCH, (uint32_t)p.home_speed_switch) ) return false;
	if ( !_n5link->writeU32(HOME_SPEED_REFERENCE, (uint32_t)p.home_speed_reference) ) return false;
	if ( !_n5link->writeU32(HOME_ACCELERATION, (uint32_t)p.home_acceleration) ) return false;
	if ( !_n5link->writeU32(LS_TOLERANCE, (uint32_t)p.home_switch_tolerance_band) ) return false;
	if ( !_n5link->writeI32(HOME_BLOCK_MIN_CURR, (int32_t)p.home_min_current_block_detection) ) return false;
	if ( !_n5link->writeI32(HOME_BLOCK_DECT_TIME, (int32_t)p.home_period_of_blocking) ) return false;

	#ifdef _DBG_AXISINIT
		cout << "Cia402HomingParams READBACK: " << endl;
		Cia402HomingParams();
	#endif

	return true;
}


//=================================================

AnalogType_t N5Drv::get_analogtype(int ch){
	AnalogType_t at = UNCONFIGURED;
	int32_t x = 0;
	if (_n5link){
        	try {
        		x = _n5link->readI32(ANALOG_IN_CONTROL);
			switch (ch){
				case 1:
					if ( x & 0x1 ) at = AMP_020MA;
					else           at = VOLT_10_BIPOL;
					break;
				case 2:
					if ( x & 0x2 ) at = AMP_020MA;
					else           at = VOLT_10_BIPOL;
					break;
				default:
					break;
			}
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return at;
}

Analog_t N5Drv::analog1(){
	return analog(1);
}

Analog_t N5Drv::analog2(){
	return analog(2);
}

Analog_t N5Drv::analog(int ch){
	Analog_t a = {UNCONFIGURED, nan("")};
	int16_t x = 0;
	if (_n5link){
        	try {
			switch (ch){
				case 1:
					a.type = _analog1.type;
        				x = _n5link->readI16(ANALOG1_INPUT);
					switch (_analog1.type){
						case AMP_020MA:
							a.value = double(x) * 20.0 / 1024.0;
							break;
						case VOLT_10_BIPOL:
							a.value = double(x - 512) * 20.0 / 1024.0;
							break;
						default:
							break;
					}
					break;
				case 2:
					a.type = _analog2.type;
        				x = _n5link->readI16(ANALOG2_INPUT);
					switch (_analog2.type){
						case AMP_020MA:
							a.value = double(x) * 20.0 / 1024.0;
							break;
						case VOLT_10_BIPOL:
							a.value = double(x - 512) * 20.0 / 1024.0;
							break;
						default:
							break;
					}
					break;
				default:
					break;
			}
        	} catch (...) {
                	// errori di comunicazione
        	}
	}

	return a;
}

//-----------------------------------------------------------------------
////ifdef _WITH_N5MICROCODE_
#if 1

/***************
// values for refresh
typedef struct {
        long target;            // Target position provided by the client in encoder units.
        long nbrtr;             // Number of retries before giving up/ending with movement
        double err;             // Positioning error. Encoder units
        double olderr;          // Previous positioning error. Encoder units
        unsigned long CC_code;
        yconf_t yc;
} n5rvalues_t;
****************/

n5rvalues_t N5Drv::n5refresh(string channel){
        std::stringstream ss;
        string obj("N5Drv::n5refresh("+channel+")");
        prtverbose(1, obj, "");

        n5rvalues_t yr;
	bzero((void*)(&yr), sizeof(n5rvalues_t));

	/*
        n.append("MT");         n.append(channel); n.append("=?;");
        yr.yc.mt        = (double) a0;
        yr.yc.home_polarity = (int) a12;
	*/

	int32_t x = 0;
	float f = 0;
	if (_n5link){
        	try {
        		x = _n5link->readI32(MicroCode_Pos);
        		yr.target = (long)x;

        		x = _n5link->readI32(MicroCode_Dbw);
        		yr.yc.dbw = (long)x;

        		x = _n5link->readI32(MicroCode_Scale);
			memcpy((void*)(&f), (void*)(&x), 4);
        		yr.yc.scale = (double)f;

        		x = _n5link->readI32(MicroCode_Pc);
			memcpy((void*)(&f), (void*)(&x), 4);
        		yr.yc.pc = (double)f;

        		x = _n5link->readI32(MicroCode_Bck);
        		yr.yc.bck = (double)x;

        		x = _n5link->readI32(MicroCode_Dir);
        		yr.yc.dir = (long)x;

        		x = _n5link->readI32(MicroCode_NbRtr);
        		yr.nbrtr = (long)x;

        		x = _n5link->readI32(MicroCode_Err);
        		yr.err = (double)x;

        		x = _n5link->readI32(MicroCode_prevErr);
        		yr.olderr = (double)x;

        		x = _n5link->readI32(MicroCode_CC);
        		yr.CC_code = (unsigned long)x;

        		x = _n5link->readI32(MicroCode_Cp);
        		yr.yc.cp = (unsigned long)x;

                	x = _n5link->readI32(MOTORBRAKE);
			yr.yc.hw_brake_conf = (int)x;

        	} catch (...) {
                	// errori di comunicazione
        	}
	}


	return yr;
}

long N5Drv::n5target(VNOTU string channel){                // depends on Cp, Scale....            i.e. in encoder units!!!!

	int32_t x = 0;
	if (_n5link){
        	try {
        		x = _n5link->readI32(MicroCode_Pos);
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return (long)x;
}

bool N5Drv::n5target(VNOTU string channel, long p){          // p: position

	bool ret = false;
	if (_n5link){
        	try {
			if ( !_n5link->writeI32(MicroCode_Pos, (int32_t)p) ) ret = false;
			else ret = true;
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return ret;
}

bool N5Drv::n5nbrtr(VNOTU string channel, int nbrtr){	       // nbrtr: number of retries

	bool ret = false;
	if (_n5link){
        	try {
			if ( !_n5link->writeI32(MicroCode_NbRtr, (int32_t)nbrtr) ) ret = false;
			else ret = true;
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	if (ret ) _nbrtr = nbrtr;
	return ret;
}

unsigned long N5Drv::n5CC_code(VNOTU string channel){
        return n5CC_code(channel, true);
}

unsigned long N5Drv::n5CC_code(VNOTU string channel, bool do_update){
        std::stringstream ss;
        string obj("N5Drv::n5CC_code()");
        prtverbose(1, obj, "");

        if (do_update) update(channel);
        unsigned long a = uv().CC_code;

        return (unsigned long)a;

}

bool N5Drv::n5CC_code(VNOTU string channel, unsigned long CC_code){
        std::stringstream ss;
        ss << " new code: 0x" << hex << CC_code << dec << "[" << CC_code << "]";
        string obj("N5Drv::n5CC_code(,)");
        prtverbose(1, obj, ss.str());

        if (!__hw_write_enable) return true;

	bool ret = false;
	if (_n5link){
        	try {
			if ( !_n5link->writeI32(MicroCode_CC, (int32_t)CC_code) ) ret = false;
			else ret = true;
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return ret;

}



bool N5Drv::n5CC_cmd(VNOTU string channel, n5cmd_t cmd){
        std::stringstream ss;
        ss << " new cmd: 0x" << hex << ((int)cmd) << dec << "[" << ((int)cmd) << "]";
        string obj("N5Drv::n5CC_cmd()");
        prtverbose(1, obj, ss.str());

        if (!__hw_write_enable) return true;

        return n5CC_code(channel, (n5CC_code(channel) & 0xFFF00) | (unsigned long)cmd );
}



bool N5Drv::n5go(VNOTU string channel, long p, int nbrtr){ // p: position, nbrtr: number of retries
        std::stringstream ss;
        ss << "p= " << p ;
        string obj("N5Drv::ygo()");
        prtverbose(1, obj, ss.str());

        if (!__hw_write_enable) return true;

        bool b1, b2, b3;
        b1 = n5target(channel, p);
        b2 = n5nbrtr(channel, nbrtr);

        ss.str("");
        ss << "n5cmd_go= " << n5cmd_go << " -:" << ((int)n5cmd_go) << ":-";
        prtverbose(2, obj, ss.str());

        b3 = n5CC_cmd(channel, n5cmd_go);

        return b1 && b2 && b3;


	return false;
}

bool N5Drv::n5conf(VNOTU string channel, yconf_t yc){
        std::stringstream ss;
        string obj("N5Drv::n5conf("+channel+",.)");
        prtverbose(1, obj, "yc[]");

        if (!__hw_write_enable) return true;

	bool ret = false;

	int32_t pc = 0;
	float fpc = fabs(yc.pc);
	memcpy((void*)(&pc), (void*)(&fpc), 4);

	int32_t sc = 0;
	// float fsc = fabs(yc.scale); PERCHE' FABS()?
	float fsc = yc.scale;
	memcpy((void*)(&sc), (void*)(&fsc), 4);

	if (_n5link){
        	try {
			if ( !_n5link->writeI32(MicroCode_Dbw, (int32_t)yc.dbw) ) ret = false;
			else
			if ( !_n5link->writeI32(MicroCode_Scale, sc) ) ret = false;
			else
			if ( !_n5link->writeI32(MicroCode_Pc, pc) ) ret = false;
			else
			if ( !_n5link->writeI32(MicroCode_Bck, (int32_t)yc.bck) ) ret = false;
			else
			if ( !_n5link->writeI32(MicroCode_Dir, (int32_t)yc.dir) ) ret = false;
			else
			if ( !_n5link->writeI32(MicroCode_Cp, (int32_t)yc.cp) ) ret = false;
			else
			ret = true;
        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return ret;
}

bool N5Drv::n5stop(VNOTU string channel){
        std::stringstream ss;
        string obj("N5Drv::n5stop("+channel+",.)");
        prtverbose(1, obj, "");

        return n5CC_cmd(channel, n5cmd_stop);
}



#endif /* _WITH_N5MICROCODE_ */
//-----------------------------------------------------------------------
//
//      N5: V E R S I O N       2


n5v2_feedback_map_t N5Drv::n5v2_feedback_map(){
	char buf[128];
	uint8_t x;
	uint8_t e[10];
	n5v2_feedback_map_t fmap;

	if (_n5link) {
        	try {
			memset(buf, 0, 128);
			snprintf(buf, 128, "%s/%02X", FEEDBACK_SELECTION, 0);
        		x = _n5link->readU8(buf);
				
			#ifdef _DBG_V2ALL
				cout << "Obj:" << buf << "= " << int(x) << endl;
			#endif

			for (int i = 1; i <= x; i++){
				memset(buf, 0, 128);
				snprintf(buf, 128, "%s/%02X", FEEDBACK_SELECTION, i);
        			e[i] = _n5link->readU8(buf);

				#ifdef _DBG_V2ALL
					cout << "Obj:" << buf << "= " << int(e[i]) << endl;
				#endif
			}

			for (int i = x; i > 0; i--){
				if ( (e[i] & 0x4) == 0x4 ) {
					fmap.commutation_sensor = (n5v2enc_id_t)i;
					break;
				}
			}
			for (int i = x; i > 0; i--){
				if ( (e[i] & 0x1) == 0x1 ) {
					fmap.position_sensor = (n5v2enc_id_t)i;
					break;
				}
			}
			for (int i = x; i > 0; i--){
				if ( (e[i] & 0x2) == 0x2 ) {
					fmap.speed_sensor = (n5v2enc_id_t)i;
					break;
				}
			}
				
			#ifdef _DBG_V2ALL
				cout << "Feedback map: commutation_sensor: " << n5v2encidNames[fmap.commutation_sensor] << endl;
				cout << "Feedback map: position_sensor   : " << n5v2encidNames[fmap.position_sensor] << endl;
				cout << "Feedback map: speed_sensor      : " << n5v2encidNames[fmap.speed_sensor] << endl;
			#endif

        	} catch (...) {
                	// errori di comunicazione
        	}
	}
	return fmap;
}

vector<string> N5Drv::getErrors(){
	if ( _sw_vers < SW_VERS_2 ) {
	} else {
	}
}

string N5Drv::getLastError(){
	string e_str("Unknown");

       	uint16_t e = _n5link->readU16(LAST_ERROR_CODE);
	switch (e) {
		case 0x0000:    e_str = "No err"; break; // mia codifica
		case 0x1000:	e_str = "General error"; break;
		case 0x2300:	e_str = "Current at the controller output too large"; break;
		case 0x3100:	e_str = "Overvoltage/undervoltage at controller input"; break;
		case 0x4200:	e_str = "Temperature error within the controller"; break;
		case 0x6010:	e_str = "Software reset (watchdog)"; break;
		case 0x6100:	e_str = "Internal software error, generic"; break;
		case 0x7121:	e_str = "Motor blocked"; break;
		case 0x7600:	e_str = "Nonvolatile memory full or corrupt; restart the controller for cleanup work"; break;
		case 0x8130:	e_str = "CANopen only: 'Life Guard' error or 'Heartbeat' error"; break;
		case 0x8200:	e_str = "CANopen only: Slave took too long to send PDO messages."; break;
		case 0x8210:	e_str = "CANopen only: PDO was not processed due to a length error"; break;
		case 0x8220:	e_str = "CANopen only: PDO length exceeded"; break;
		case 0x8611:	e_str = "Position monitoring error: Following error too large"; break;
		case 0x8612:	e_str = "Position monitoring error: Limit switch and tolerance zone exceeded"; break;
	}
	if ( _sw_vers < SW_VERS_2 ) {
		switch (e) {
			case 0x6320:	e_str = "Rated current must be set (203Bh:01h)"; break;
			case 0x7305:	e_str = "Incremental encoder or Hall sensor faulty"; break;
			case 0x8000:	e_str = "Error during fieldbus monitoring"; break;
			case 0x9000:	e_str = "EtherCAT: Motor running while EtherCAT changes from OP -> SafeOp, PreOP, etc."; break;
		}
	} else {
		switch (e) {
			case 0x5540:	e_str = "Interlock error: Bit 3 in 60FDh is set to '0', the motor may not start (see the section Interlock function in the chapter Digital inputs)"; break;
			case 0x6320:	e_str = "Rated current must be set (203Bh:01h/6075h)"; break;
			case 0x7113:	e_str = "Warning: Ballast resistor thermally overloaded"; break;
			case 0x7200:	e_str = "Internal error: Correction factor for reference voltage missing in the OTP"; break;
			case 0x7305:	e_str = "Sensor 1 (see 3204h) faulty"; break;
			case 0x7306:	e_str = "Sensor 2 (see 3204h) faulty"; break;
			case 0x7307:	e_str = "Sensor n (see 3204h), where n is greater than 2"; break;
			case 0x8100:	e_str = "Error during fieldbus monitoring"; break;
			case 0x8240:	e_str = "CANopen only: unexpected sync length"; break;
			case 0x8400:	e_str = "Error in speed monitoring: slippage error too large"; break;
		}
	}
	return e_str;
}



//-----------------------------------------------------------------------
} // namespace Controller_ns
