#include "N5WebLink.h"

namespace Link_ns {

N5WebLink::N5WebLink(string target):Link(Link_ns::WEBREST_CHAN, target){
        #ifdef _DBG_N5WEBLINK
                _verbose = 1;
        #else
                _verbose = 0;
        #endif

        _name = target;
	_sw_vers = 0;

        std::stringstream ss("");
        ss << " _verbose = " << _verbose;

        string obj("N5WebLink()");
        prtverbose(1, obj, ss.str());

        _baseurl = target + "/od/";
        curl = curl_easy_init();
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, cb);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &readBuffer);


	_linkstate = LINK_REACTIVE;

	_epos_sw_offset = double(get_enc_offset_on_init());

	_with_microcode = false;

	 _software_version_str = software_version();

        //FIR-v1650-B527540
        //cout << _software_version_str.substr(5,4) << endl;

        _sw_vers = std::stoul(_software_version_str.substr(5,4), nullptr, 10);

        //cout << "_sw_vers: " << _sw_vers << endl;


}

N5WebLink::~N5WebLink(){
        curl_easy_cleanup(curl);

}

size_t N5WebLink::cb(void *contents, size_t size, size_t nmemb, void *userp) {
    ((std::string*)userp)->append((char*)contents, size * nmemb);
    return size * nmemb;
}

objans_t N5WebLink::readobj(std::string obj){

        objans_t ans;
        CURLcode  res;
        readBuffer.clear();

        ans.str = "NONE!";
        ans.sec = -1;
        if(curl) {
                curl_easy_setopt(curl, CURLOPT_URL, (_baseurl+obj).c_str());
                res = curl_easy_perform(curl);

                /* Check for errors */
                if(res != CURLE_OK) {
                        ans.str = std::string("ERROR: curl_easy_perform() failed: ") + std::string(curl_easy_strerror(res));
                } else {
                        res = curl_easy_getinfo(curl, CURLINFO_TOTAL_TIME, &(ans.sec));
                        readBuffer.erase(std::remove(readBuffer.begin(), readBuffer.end(), '"'), readBuffer.end());
                        ans.str = readBuffer;
                }
        }

        return ans;
}


string N5WebLink::raw_io(string quest, bool setstate){
	return "";
}

bool N5WebLink::refresh(){
	return true;
}

bool    N5WebLink::defepos(double p){       // define hw encoder position
        int32_t  i32 = 0;

        try {
                //acquisisci
	        if ( _sw_vers <  SW_VERS_2 ) {
                	i32 = readI32(ENCODER_ACT_INTERNAL);
		} else {
                	i32 = readI32(V2_ENCODER_ACT);
		}
               _linkstate = LINK_REACTIVE;
                _epos_sw_offset = p - double(i32);
                return writeI32(ENC_OFFSET_ON_INIT, (int32_t)_epos_sw_offset);
        } catch (...) {
                // errori di comunicazione
                _linkstate = LINK_OTHEREXCEPT;
        }
        /***********************************************
        } catch (...) {
                // altri errori
                _linkstate = LINK_GENERALEXCEPT;
                prtverbose(0, obj, "General except!");
        }
        ***********************************************/
        return false;
}



string N5WebLink::readSTR(std::string obj, double *t){
        objans_t ans = readobj(obj);
        string x = ans.str;

        if (t) *t = ans.sec;
        return x;
}

string N5WebLink::readSTR(std::string obj){
        return readSTR(obj, NULL);
}

int32_t N5WebLink::readI32(std::string obj, double *t){
        objans_t ans = readobj(obj);
        unsigned int x = std::stoul(ans.str, nullptr, 16);

        if (t) *t = ans.sec;
        return (int32_t)x;
}

int32_t N5WebLink::readI32(std::string obj){
        return readI32(obj, NULL);
}

uint16_t N5WebLink::readU16(std::string obj, double *t){
        objans_t ans = readobj(obj);
        unsigned int x = std::stoul(ans.str, nullptr, 16);

        if (t) *t = ans.sec;
        return (uint16_t)x;
}

uint16_t N5WebLink::readU16(std::string obj){
        return readU16(obj, NULL);
}

int16_t N5WebLink::readI16(std::string obj, double *t){
        objans_t ans = readobj(obj);
        unsigned int x = std::stoul(ans.str, nullptr, 16);

        if (t) *t = ans.sec;
        return (int16_t)x;
}

int16_t N5WebLink::readI16(std::string obj){
        return readI16(obj, NULL);
}

int8_t N5WebLink::readI8(std::string obj, double *t){
        objans_t ans = readobj(obj);
        unsigned int x = std::stoul(ans.str, nullptr, 16);

        if (t) *t = ans.sec;
        return (int8_t)x;
}

int8_t N5WebLink::readI8(std::string obj){
        return readI8(obj, NULL);
}

uint8_t N5WebLink::readU8(std::string obj, double *t){
        objans_t ans = readobj(obj);
        unsigned int x = std::stoul(ans.str, nullptr, 16);

        if (t) *t = ans.sec;
        return (uint8_t)x;
}

uint8_t N5WebLink::readU8(std::string obj){
        return readU8(obj, NULL);
}

uint32_t N5WebLink::readU32(std::string obj, double *t){
        objans_t ans = readobj(obj);
        unsigned int x = std::stoul(ans.str, nullptr, 16);

        if (t) *t = ans.sec;
        return (uint32_t)x;
}

uint32_t N5WebLink::readU32(std::string obj){
        return readU32(obj, NULL);
}

bool N5WebLink::writeobj(std::string obj, std::string value){
        CURLcode  res;
        std::string v("\"");
        v.append(value);
        v.append("\"");
        bool ret = false;
        readBuffer.clear();

        //std::cout << "|" << v.c_str() << "|" << v.size()<< std::endl;
	//std::cout << __FILE__ << ": "<< obj << " : " << value << std::endl;

        if(curl) {
                curl_easy_setopt(curl, CURLOPT_URL, (_baseurl+obj).c_str());
                curl_easy_setopt(curl, CURLOPT_POSTFIELDS, v.c_str());
                curl_easy_setopt(curl, CURLOPT_POSTFIELDSIZE, (long)v.size());
                res = curl_easy_perform(curl);
                /* Check for errors */
                if(res != CURLE_OK) {
                        ret = false;
                        std::cerr << "curl_easy_perform() failed: " <<  curl_easy_strerror(res) << std::endl;
                } else {
                        ret = true;
                }
                curl_easy_setopt(curl, CURLOPT_POST, 0L);
        }
	//std::cout << __FILE__ << ": "<< "ret = " << ret << std::endl;
        return ret;
}

bool N5WebLink::writeU8(std::string obj, uint8_t value){
        char buff[128];

        snprintf(buff, 128, "%02X", value);
        return writeobj(obj, std::string(buff));
}

bool N5WebLink::writeI8(std::string obj, int8_t value){
        char buff[128];

        snprintf(buff, 128, "%02X", value);
        return writeobj(obj, std::string(buff));
}

bool N5WebLink::writeI32(std::string obj, int32_t value){
        char buff[128];

        snprintf(buff, 128, "%08X", value);
        return writeobj(obj, std::string(buff));
}

bool N5WebLink::writeU16(std::string obj, uint16_t value){
        char buff[128];

        snprintf(buff, 128, "%04X", value);
        return writeobj(obj, std::string(buff));
}

bool N5WebLink::writeU32(std::string obj, uint32_t value){
        char buff[128];

        snprintf(buff, 128, "%08X", value);
        return writeobj(obj, std::string(buff));
}



uvalues_t N5WebLink::update(string channel){
        std::stringstream ss;
        string obj("N5WebLink::update("+channel+")");
        prtverbose(1, obj, "");

        timeval start, end;
        gettimeofday(&start, NULL);

        uvalues_t uv;
        // uvalues_t:
        uv.motor        = nan("");
        uv.encoder      = nan("");
        uv.velocity     = nan("");
        uv.analog       = nan("");
        uv.fls          = true;
        uv.rls          = true;
        uv.home         = true;
        uv.stop_code    = 0;
        uv.CC_code      = 0;
        uv.status       = OFF;	// OFF/MOVING/STANDSTILL

	try {
		//acquisisci
		int32_t  i32;
		int16_t  i16;
		uint32_t u32;

		i32 = readI32(POSITION_DEMANDED);
		uv.motor = double(i32);

	        if ( _sw_vers <  SW_VERS_2 ) {
	        	i32 = readI32(ENCODER_ACT_INTERNAL);
			uv.encoder = double(i32) + _epos_sw_offset;
		} else {
	        	i32 = readI32(V2_ENCODER_ACT);
			uv.encoder = double(i32) + _epos_sw_offset;
		}

		#if 0 
			// velocita' interna calcolata
	        	if ( _sw_vers <  SW_VERS_2 ) {
		       		i16 = readI16(VL_VELOCITY_ACTUAL);
				uv.velocity = double(i16);
			} else {
	        		i32 = readI32(V2_VELOCITY_ACT);
				uv.velocity = double(i32);
			}
		#else
			// velocita' da enc calcolata in prog on controller
	        	i32 = readI32(NANOJ_OUT_01);
			uv.velocity = double(i32);
		#endif


	        i32 = readI32(ANALOG_INPUT_1);
		uv.analog = double(i32);


		u32 = readU32(DIGITAL_INPUT);
		uv.home = ((u32 & 0x4) >> 2);
		uv.fls  = ((u32 & 0x2) >> 1);
		uv.rls  =  (u32 & 0x1);

        	uint16_t sw = readU16(STATUSWORD);
        	if ((sw & 0b0000000001101111) == 0b0000000000100111) { // se non in Operation enabled
                	//uint32_t motion = (readU32(MOTION_DEBUG_ENTRIES) & 0x40000) >> 18;
                	uint32_t motion = readU32(MOTION_DEBUG_ENTRIES);
			//motion = (motion & 0x00040001) || (motion & 0x00002006);
			#if 0
		        motion = (((motion & 0x0000000F)==0x1) && (motion & 0x00040000)) ||
				 (((motion & 0x0000000F)==0x6) && (motion & 0x00002000));
			#endif
			// per cosiderare anche la ricerca idx
		        motion = (motion & 0x00040000) ||
				 (((motion & 0x0000000F)==0x6) && (motion & 0x00006000));


                        if (motion) 	uv.status = MOVING;
                        else 		uv.status = STANDSTILL;
        	} else {
                			uv.status = OFF;
        	}

		if (_with_microcode) {
	        	i32 = readI32(MicroCode_CC);
			uv.CC_code = i32;
		} else {
			if ( uv.status == MOVING ) uv.CC_code |= 0x100;
			else uv.CC_code &= !((unsigned long)0x100);
		}

		//
		// elaborazione per stop_code
		//
		//Galil:
		//_stop_code  == 0 : Motors are running, independent mode
		//_stop_code  == 1 : Motors decelerating or stopped at commanded independent position
		//_stop_code  == 2 : Decelerating or stopped by FWD limit switch or soft limit FL
		//_stop_code  == 3 : Decelerating or stopped by REV limit switch or soft limit BL
		//
		switch (uv.status){
			case MOVING: 			uv.stop_code = 0; break;
			case STANDSTILL:
			case OFF:
				if      (uv.fls) 	uv.stop_code = 2;
				else if (uv.rls)	uv.stop_code = 3;
				else 			uv.stop_code = 1;
				break;
			default:
				break;
		}


		_linkstate = LINK_REACTIVE;
	} catch (...) {
		// errori di comunicazione
		_linkstate = LINK_OTHEREXCEPT;
	}
	/***********************************************
	} catch (...) {
		// altri errori
		_linkstate = LINK_GENERALEXCEPT;
                prtverbose(0, obj, "General except!");
	}
	***********************************************/

	bf->log();
        gettimeofday(&end, NULL);
        _last_update_ms = (double)(end.tv_sec - start.tv_sec)*1000 + ((double)(end.tv_usec - start.tv_usec))/1000;

        ss.str("");
        ss << "update time: " << _last_update_ms << "ms";
        prtverbose(2, obj, ss.str());

        _uv = uv;
        return uv;


}

bool N5WebLink::homing_method(int8_t h){
        std::stringstream ss;
        string obj("N5WebLink::homing_method(.)");
        prtverbose(1, obj, "");

	try {
		bool ret = writeI8(HOMING_METHOD, h);
		_linkstate = LINK_REACTIVE;
		return ret;
	} catch (...) {
		// errori di comunicazione
		_linkstate = LINK_OTHEREXCEPT;
	}
	/***********************************************
	} catch (...) {
		// altri errori
		_linkstate = LINK_GENERALEXCEPT;
                prtverbose(0, obj, "General except!");
	}
	***********************************************/
	return false;
}

int8_t N5WebLink::homing_method(){
        std::stringstream ss;
        string obj("N5WebLink::homing_method()");
        prtverbose(1, obj, "");

	int8_t i8 = 0;
	try {
		//acquisisci
		i8 = readI8(HOMING_METHOD);
		_linkstate = LINK_REACTIVE;
		return i8;
	} catch (...) {
		// errori di comunicazione
		_linkstate = LINK_OTHEREXCEPT;
	}
	/***********************************************
	} catch (...) {
		// altri errori
		_linkstate = LINK_GENERALEXCEPT;
                prtverbose(0, obj, "General except!");
	}
	***********************************************/
	return 0;
}

int32_t N5WebLink::get_enc_offset_on_init(){
        std::stringstream ss;
        string obj("N5WebLink::get_enc_offset_on_init()");
        prtverbose(1, obj, "");

	int32_t i32 = 0;
	try {
		//acquisisci
		i32 = readI32(ENC_OFFSET_ON_INIT);
		_linkstate = LINK_REACTIVE;
	} catch (...) {
		// errori di comunicazione
		_linkstate = LINK_OTHEREXCEPT;
	}
	/***********************************************
	} catch (...) {
		// altri errori
		_linkstate = LINK_GENERALEXCEPT;
                prtverbose(0, obj, "General except!");
	}
	***********************************************/

	return i32;
}

string  N5WebLink::hardware_version(){
        string x("");
        try {
        	x = readSTR(HARDWARE_VERSION);
        } catch (...) {
                // errori di comunicazione
        }
        return x;
}

string  N5WebLink::software_version(){
        string x("");
        try {
        	x = readSTR(SOFTWARE_VERSION);
        } catch (...) {
        	// errori di comunicazione
        }
        return x;
}



} // namespace
