#ifdef _WITH_SIMULDRV_
#include <iostream>
#include "SimulDrv.h"

#include <stdlib.h>
#include <string>
#include <algorithm>

#include <unistd.h>


namespace Controller_ns {


SimulDrv::SimulDrv(string target, string extra): Drv (target, extra){
	#ifdef _DBG_SIMULDRV
		_verbose = 1;
	#else
		_verbose = 0;
	#endif

	_name = target;

	std::stringstream ss("");

        ss << " _verbose = " << _verbose;

        #ifdef _DBG_CSTOR
                prtverbose(0, "SimulDrv::SimulDrv()", ss.str());
        #else
                prtverbose(1, "SimulDrv::SimulDrv()", ss.str());
        #endif

	_link = NULL;

       	Link_ns::LinkType t = Link_ns::SIMUL_CHAN;

        switch (t){
                case Link_ns::SIMUL_CHAN:
                        ss.str("");
                        ss << "new Link_ns::SimulLink(" << _target << ")";
                        prtverbose(1, "SimulDrv::SimulDrv()", ss.str());
                        _simullink = new Link_ns::SimulLink(_target);
                        _link = (Link_ns::Link*)_simullink;
                        break;
                default:
                case Link_ns::NULL_CHAN:
                        ss.str("");
                        ss << "new Link_ns::NULL_CHAN";
                        prtverbose(1, "SimulDrv::SimulDrv()", ss.str());
                        _link = NULL;
                        break;
        }


        simulthreadp = new SimulThread_ns::SimulThread();

	if (_simullink && simulthreadp) _simullink->set_simulthread(simulthreadp);

}

SimulDrv::~SimulDrv(){

	if (simulthreadp) delete simulthreadp;
}

double  SimulDrv::mpos(string channel){
        std::stringstream ss;
	ss << "SimulDrv::mpos(" << channel << ")";
	string obj(ss.str());
        prtverbose(1, obj, "");
	ss.str("");
	
	return (double)simulthreadp->mpos();
}

bool    SimulDrv::mpos(string channel, double p){                // mpos : motor position
        std::stringstream ss;
	ss << "SimulDrv::mpos(" << channel << "," << p << ")";
	string obj(ss.str());
        prtverbose(1, obj, "");
	ss.str("");

	return simulthreadp->mpos((int64_t) p);
}

bool    SimulDrv::xpos(string channel, double p){                // xpos : motor position in encoder units

	return false;
}

bool    SimulDrv::rpos(string channel, double p){                // rpos : motor relative position
        std::stringstream ss;
	ss << "SimulDrv::rpos(" << channel << "," << p << ")";
	string obj(ss.str());
        prtverbose(1, obj, "");
	ss.str("");

	double _p = (double)simulthreadp->mpos();
	_p += p;
	return simulthreadp->mpos((int64_t) _p);
}
                    
bool    SimulDrv::defmpos(string channel, double p){             // define motor position
        std::stringstream ss;
	ss << "SimulDrv::defmpos(" << channel << "," << p <<")";
	string obj(ss.str());
        prtverbose(1, obj, "");
	ss.str("");
	
	return simulthreadp->defmpos((int64_t) p);
}

bool    SimulDrv::defepos(string channel, double p){             // define encoder position

	return false;
}

bool    SimulDrv::stop(string channel){

	return simulthreadp->stop();
}

bool    SimulDrv::forward(string channel, double v){              // v = speed

	return false;
}

bool    SimulDrv::backward(string channel, double v){             // v = speed

	return false;
}

bool    SimulDrv::motor_off(string channel){
        std::stringstream ss;
	ss << "SimulDrv::motor_off(" << channel << ")";
	string obj(ss.str());
        prtverbose(1, obj, "");
	ss.str("");

	return simulthreadp->motor_off();
}

bool    SimulDrv::motor_on(string channel){
        std::stringstream ss;
	ss << "SimulDrv::motor_off(" << channel << ")";
	string obj(ss.str());
        prtverbose(1, obj, "");
	ss.str("");

	return simulthreadp->motor_on();
}

bool    SimulDrv::mconf(string channel, mconf_t mc){
        std::stringstream ss;
	ss << "SimulDrv::mconf(" << channel << ", mc)";
	string obj(ss.str());
        prtverbose(0, obj, "");
	ss.str("");

	_mc = mc;
	return true;
}

mconf_t SimulDrv::mconf(string channel){
        std::stringstream ss;
	ss << "SimulDrv::mconf(" << channel << ")";
	string obj(ss.str());
        prtverbose(1, obj, "");
	ss.str("");

	return _mc;
}

bool    SimulDrv::program_running(){

	if (simulthreadp) return true;
	return false;
}

bool    SimulDrv::sinit(string channel){         // true = INIT / false = NO INIT

	if (simulthreadp) return simulthreadp->isinit();
        return false;
}

bool    SimulDrv::sinit(string channel, bool b){ // true = INIT / false = DEINIT

	if (simulthreadp) {
		simulthreadp->isinit(b);
		return simulthreadp->isinit();
	}
	return false;
}

Link_ns::rvalues_t SimulDrv::refresh(string channel) {
        std::stringstream ss;
	ss << "SimulDrv::refresh(" << channel << ")";
	string obj(ss.str());
        prtverbose(1, obj, "");
	ss.str("");

	Link_ns::rvalues_t rt;

	if (simulthreadp) {
		rt.speed	= (double)simulthreadp->mspeed();
        	rt.acceleration	= (double)simulthreadp->maccel();
        	rt.deceleration	= (double)simulthreadp->mdecel();
	}
	return rt;
};


} /* namespace Controller_ns */
#endif /* _WITH_SIMULDRV_ */
