#include "SimulLink.h"

#include <sys/time.h>
#include <stdlib.h>

namespace Link_ns {


SimulLink::SimulLink(string target):Link(Link_ns::TANGO_CHAN, target) {
	#ifdef _DBG_SIMULLINK
		_verbose = 1;
	#else
		_verbose = 0;
	#endif

	_name = target;

	std::stringstream ss("");

	ss << " _verbose = " << _verbose;


        string obj("SimulLink()");
        prtverbose(1, obj, ss.str());

	#ifdef _DBG_SIMULLINK_PRTLOG
		ss.str("");
		if (_prtLog) ss << "================  _prtLog TRUE!";
		else         ss << "================  _prtLog FALSE!";
        	prtverbose(-1, obj, ss.str());
	#endif

	_flag = 0;

	try {
		_linkstate = LINK_UNREACTIVE;
		_linkstate = LINK_REACTIVE;
		ss.str("");

	} catch (...) {
		_linkstate = LINK_CLOSE;

		ss.str("");
		ss << linkstatus();
        	if (_prtLog) prtverbose(1, obj, ss.str());
	}

	if ( !refresh() ) _updateDelay = 1.5;

	#ifdef _DBG_SIMULLINK_2
		cout << "SimulLink().dev exiting with = " << dev << endl;
	#endif
}

SimulLink::~SimulLink(){
        #ifdef _DBG_CLASSDESCTRUCTOR
                cout << "~SimulLink() : ";
        #endif
        #ifdef _DBG_CLASSDESCTRUCTOR
                cout << endl;
        #endif
}

void SimulLink::set_simulthread(SimulThread_ns::SimulThread * t) {
        string obj("SimulLink::set_simulthread()");
	std::stringstream ss;
	ss.str("");
	ss << t;
        prtverbose(0, obj, ss.str());
	_simulthread = t;
};

bool SimulLink::refresh(){
        string obj("SimulLink::refresh()");
        prtverbose(1, obj, "");
	std::stringstream ss;

	int pool_period_ms;

	_linkstate = LINK_REACTIVE;

	return true;
}

string SimulLink::raw_io(string quest, bool setstate){
        string obj("SimulLink::raw_io()");
        prtverbose(1, obj, "quest="+quest);
	std::stringstream ss;

	string ans("");
	return ans;
}

uvalues_t SimulLink::update(string channel){
	std::stringstream ss;
        string obj("SimulLink::update("+channel+")");
        prtverbose(1, obj, "");

	timeval start, end;
	gettimeofday(&start, NULL);

	uvalues_t uv;
	// uvalues_t:
        uv.motor 	= nan("");
        uv.encoder	= nan("");
        uv.velocity	= nan("");
        uv.analog	= nan("");
        uv.fls		= true;
        uv.rls		= true;
        uv.home		= true;
        uv.stop_code	= 0;
        uv.CC_code	= 0;
        uv.status	= OFF;

	/*
	ss.str("");
	ss << _simulthread;
       	prtverbose(2, obj, ss.str());
	*/

	if(_simulthread){
		uv.motor = _simulthread->mpos();
        	uv.fls	 = false;
        	uv.rls	 = false;
        	uv.home	 = false;
		if (_simulthread->isinit()) uv.CC_code |= 0x10000;
		else			    uv.CC_code &= 0xEFFFF;

		if (_simulthread->isrunning()) 	uv.status = MOVING;
		else {
			if (_simulthread->ismotor_on() ) uv.status = STANDSTILL;
			else				 uv.status = OFF;
		}

		if ( uv.status == MOVING ) uv.CC_code |= 0x100;
                else uv.CC_code &= !((unsigned long)0x100);

		ss.str("");
		ss << "mpos:" << uv.motor;
        	prtverbose(2, obj, ss.str());
	}

	bf->log();
end_label:

	gettimeofday(&end, NULL);
	_last_update_ms = (double)(end.tv_sec - start.tv_sec)*1000 + ((double)(end.tv_usec - start.tv_usec))/1000;

	ss.str("");
	ss << "update time: " << _last_update_ms << "ms";
       	prtverbose(2, obj, ss.str());

	_uv = uv;
	return uv;
}


} /* namespace Link_ns */
