#ifdef _WITH_SIMULDRV_
#include <iostream>
#include "SimulDrv.h"

#include <stdlib.h>
#include <string>
#include <algorithm>

#include <unistd.h>

namespace SimulThread_ns {

void SimulThread::loop(){

	#ifdef _DBG_SIMULTHREAD
		cout << "GO!" << endl;
	#endif
	while (_go){
		#ifdef _DBG_SIMULTHREAD
			cout << "GOing!" << endl;
			cout << "_mtarget:" << _mtarget << " | ";
			cout << "_mpos:" << _mpos;
			cout << endl;
		#endif

		if ( (_mpos != _mtarget) && (!_stop) && (_ismotor_on) ){
			_running = true;

			double dir = _mtarget>_mpos?1:-1;
			double missing  = (double)abs(_mtarget-_mpos);


			double space_stop = fabs( 0.5 * pow(_innermspeed, 2) / (double)_mdecel );

			if ( missing < space_stop){
				_innermspeed -= (double)_mdecel * ((double)_cycle_time) / 1000.0;
			} else 
			if ( fabs(_innermspeed) < fabs((double)_mspeed) ){
				_innermspeed += (double)_maccel * ((double)_cycle_time) / 1000.0;
			}

			#ifdef _DBG_SIMULTHREAD
				cout << "_innermspeed/_mspeed " << _innermspeed << "/" << _mspeed;
				cout << " | ";
				cout << "_mtarget:" << _mtarget << " | ";
				cout << "_mpos from:" << _mpos;
			#endif
			
			double dstp = dir * _innermspeed * ((double)_cycle_time) / 1000.0;

			if ( fabs(dstp) > missing ) dstp = dir * missing;

			double next = _innermpos + dstp;
			
			_innermpos =  next;
			_mpos = (int64_t) _innermpos;

			#ifdef _DBG_SIMULTHREAD
				cout << " to:" << _mpos;
				cout << endl;
			#endif
		} else {
			_innermspeed = 0;
			_running = false;
		}

		usleep((int)(_cycle_time * 1000));
	}

}

void* SimulThread::loopWrapper (void* thisPtr){
	((SimulThread*) thisPtr)->loop();
	return NULL;
}

SimulThread::~SimulThread() {
	kill();
    pthread_cancel (thread_id);
}

SimulThread::SimulThread(int cycle_time) : _cycle_time(cycle_time) {

	#ifdef _DBG_SIMULTHREAD
		cout << "_cycle_time=" << _cycle_time << endl;
	#endif

	_mspeed = 0;
	_mtarget = 0;
	_mpos = 0;
	_innermpos = 0;
	_stop = true;
	_running = false;

	_innermspeed = 0;

	_maccel = 1;
	_mdecel = 1;

	_ismotor_on = false;

	/// get default value of arrts
	pthread_attr_init(&attr);

	/// start thread
	pthread_create(&thread_id, &attr, &SimulThread::loopWrapper, this);

}

bool SimulThread::mpos(int64_t p){

	_innermpos = (double)_mpos;
	_innermspeed = 0;
	_mtarget = p;
	_stop = false;

	return true;
}


} /* namespace SimulThread_ns */


#endif /* _WITH_SIMULDRV_ */
