#include "Axis.h"

#include <stdio.h>     /* for printf */
#include <stdlib.h>    /* for exit */
#include <getopt.h>

using namespace Axis_ns;

int main (int argc, char *argv[]){

bool ctrl_flg = false;
char ctrl[256];

snprintf(ctrl, 256, "http://localhost:8080");
//==========================================================================================
//------- getopt
	bool preset = false;
	bool rocco = false;
    {
          int option_index = 0;
          int r;
          int c_c;

          static struct option long_options[] = {
                                {"nopreset", no_argument, 0, 'n'},
                                {"preset", no_argument, 0, 'p'},
                                {"controller", required_argument, 0, 'c'},
                                {"rocco", no_argument, 0, 'r'},
                                {0, 0, 0, 0}
                                };
          c_c=0;
          while ( (c_c = getopt_long(argc, argv, "nc:pr", long_options, &option_index)) != -1 ){
                switch (c_c) {
                                case 'n':
                                        preset = false;
                                        break;
                                case 'p':
                                        preset = true;
                                        break;
                                case 'r':
                                        rocco = true;
                                        break;
				case 'c':
					snprintf(ctrl, 256, "%s", optarg);
                                case '?':
                                        break;

                                default:
                                        printf("?? getopt returned character code 0%o ??\n", c_c);
                }
          }
    }
//==========================================================================================

	string name("N5Axis");
	string channel("");

        Controller_ns::Controller 	*controller	= new Controller_ns::Controller(Controller_ns::N5, ctrl, "");
	Controller_ns::N5Drv		*n5d		= (Controller_ns::N5Drv*)controller->drv();

	cout << __func__ << " :: Controller : " << controller << endl;
	Axis_ns::Axis 			*axis 		= new Axis_ns::Axis(name, channel, controller);

	cout << endl;
	cout << "Controller type      : " << controller->controller_typeName() << endl;
	cout << "Axis Controller type : " << axis->controller_typeName() << endl;
	cout << endl;
	std::vector<std::string> desc = n5d->describe();
	std::vector<std::string>::iterator it;
	for (it = desc.begin(); it != desc.end(); ++it) cout << *it << endl;
	
	cout << endl;

	//================================================================================
	//
	controller->max_tmo(0.5); // 0.5 sec
	//
	//================================================================================

        Controller_ns::Cia402HomingParams_t  hmparam = n5d->Cia402HomingParams();
	hmparam.home_switch_tolerance_band = 100000;
	if (preset) n5d->Cia402HomingParams(hmparam);


	//================================================================================
	//
	axis->drv_verbose(0);
	axis->verbose(0);
	//
	//================================================================================
	//
	if (rocco) {
	        axis->etype(Encoder_ns::ENC_DIGITAL_INCREMENTAL, 1);
		axis->config_InitType(Axis_ns::INIT_ON_DEFPOS);
		cout << "INIT_ON_DEFPOS[" << int(Axis_ns::INIT_ON_DEFPOS) << "]: ";
					cout << "inittype   : " << axis->config_InitType2str() << endl;
	}
	//
	//================================================================================
	//
	if (preset) {
	        //axis->etype(Encoder_ns::ENC_NO, 1);
	        axis->etype(Encoder_ns::ENC_DIGITAL_INCREMENTAL, 1);
	
		axis->per(0.001);
		axis->emr(0.1);
		
		cout << "---------------------------------------" << endl;
		axis->config_InitType(Axis_ns::INIT_ON_DEFPOS);
		cout << "INIT_ON_DEFPOS[" << int(Axis_ns::INIT_ON_DEFPOS) << "]: ";
					cout << "inittype   : " << axis->config_InitType2str() << endl;
	
		axis->config_InitType(Axis_ns::INIT_ON_RLS);
		cout << "INIT_ON_RLS[" << int(Axis_ns::INIT_ON_RLS) << "]: ";
					cout << "inittype   : " << axis->config_InitType2str() << endl;
	
	#if 0
		axis->config_InitType(Axis_ns::INIT_ON_SENCPOS);
		cout << "INIT_ON_SENCPOS[" << int(Axis_ns::INIT_ON_SENCPOS) << "]: ";
					cout << "inittype   : " << axis->config_InitType2str() << endl;
	
		axis->config_InitType(Axis_ns::INIT_ON_ROTIDXPOS);
		cout << "INIT_ON_ROTIDXPOS[" << int(Axis_ns::INIT_ON_ROTIDXPOS) << "]: ";
					cout << "inittype   : " << axis->config_InitType2str() << endl;
	
		axis->config_InitType(Axis_ns::INIT_ON_ROTIDXNEG);
		cout << "INIT_ON_ROTIDXNEG[" << int(Axis_ns::INIT_ON_ROTIDXNEG) << "]: ";
					cout << "inittype   : " << axis->config_InitType2str() << endl;
	#endif
	
		axis->config_InitType(Axis_ns::CIA402_INIT_ON_RLS);
		cout << "CIA402_INIT_ON_RLS[" << int(Axis_ns::CIA402_INIT_ON_RLS) << "]: ";
					cout << "inittype   : " << axis->config_InitType2str() << endl;
	
	#if 1
		axis->config_InitType(Axis_ns::CIA402_INIT_ON_FLS);
		cout << "CIA402_INIT_ON_FLS[" << int(Axis_ns::CIA402_INIT_ON_FLS) << "]: ";
					cout << "inittype   : " << axis->config_InitType2str() << endl;
	#endif
		axis->config_InitType(Axis_ns::CIA402_INIT_ON_RLSIDX);
		cout << "CIA402_INIT_ON_RLSIDX[" << int(Axis_ns::CIA402_INIT_ON_RLSIDX) << "]: ";
					cout << "inittype   : " << axis->config_InitType2str() << endl;
	#if 0
		axis->config_InitType(Axis_ns::CIA402_INIT_ON_FLSIDX);
		cout << "CIA402_INIT_ON_FLSIDX[" << int(Axis_ns::CIA402_INIT_ON_FLSIDX) << "]: ";
					cout << "inittype   : " << axis->config_InitType2str() << endl;
	#endif
		cout << "---------------------------------------" << endl;
	}

	axis->hw_write_enable(true);

	if (preset) {
		axis->speed(0.5);
		axis->acceleration(.1);
		axis->deceleration(1);
	}
	//
	//================================================================================
	//
        vector<double> sp = axis->find_speeds();
        cout << "find_speeds: " << sp[0] << " " << sp[1]   << endl;
	if (preset) {
        	sp[0] = 5000.0;
        	sp[1] = 1000.0;
        	cout << "set find_speeds ret: " << axis->find_speeds(sp) << endl;
		sp[0] = sp[1] = nan("");
        	sp = axis->find_speeds();
        	cout << "find_speeds: " << sp[0] << " " << sp[1]   << endl;
	}
	//
	//================================================================================

        std::string cmd;
        size_t i;
        Axis_ns::AxisFSMTransitions_t FSMop = Axis_ns::FSMT_NOP;
        Axis_ns::AxisFSM_t FSMst;

        do {
                std::cout  << "===========================================================================================\n";
                std::cout << "PRE UPD:: ";
                cout << "fsm=" << axis->fsm_status() << " ";
                cout << "ifsm=" << axis->ifsm_status() << " ";
                cout << "  -- link status =" << controller->linkstatus() << " ";
                cout << "  -- ctrl status =" << controller->status() << " ";
                std::cout  << "\n";

                if (axis->update()) {
                        std::cout << "UPDATE():: " ;
                        FSMst = axis->fsm(Axis_ns::FSMT_NOP);
                        std::cout << " OK! " << "\n";
                } else std::cout << "NO UPDATE():: " << "\n";

                std::cout << "POST UPD:: ";
                std::cout << Controller_ns::ControllerTypeName[controller->controller()] << " ";
                std::cout << "ch:" << axis->channel() << " :: ";
                cout << "POSITION = " << axis->position() << " ";
                cout << "mpos=" << axis->mpos() << " ";
                cout << "epos=" << axis->epos() << " ";
                cout << "f/r/h=" << axis->fls() << "/" << axis->rls() << "/" << axis->home() << " ";
                cout << "fsm=" << axis->fsm_status() << " ";
                cout << "ifsm=" << axis->ifsm_status() << " ";
                cout << "mot=" << axis->mstatus() << " ";
                cout << endl;
                cout << "  -- link status =" << controller->linkstatus() << " ";
                cout << "  -- ctrl status =" << controller->status() << " ";
                cout << "  -- CiA402 status =" << n5d->CiA402() << " ";
		cout << endl;
		cout << endl;
                std::cout << ">";
                std::getline (std::cin, cmd);
                //std::cout <<  cmd;
                //std::cout  << "\n";

                if ( (i=cmd.find("help"))  != string::npos )    {
                		cout << "channel" << endl;
                                cout << "forward speed" << endl;
                                cout << "backward speed" << endl;
                                cout << "stop" << endl;
                                cout << "mspeed" << endl;
                                cout << "mpos" << endl;
                                cout << "mforward speed" << endl;
                                cout << "mbackward speed" << endl;
                                cout << "motor_on/off" << endl;
                                cout << "cia402" << endl;
                                cout << ".......: shutdown | switchon | enableop | disableop" << endl;
                                cout << ".......: quickstop | removequickstop" << endl;
                                cout << ".......: enablevoltage | disablevoltage" << endl;
                                cout << ".......: faultreset | halt | clearhalt" << endl;
                                cout << "drv_verbose" << endl;
                                cout << "verbose" << endl;
                                cout << "clrls" << endl;
                                cout << "macceleration" << endl;
                                cout << "mdeceleration" << endl;
                                cout << "axis" << endl;
                                cout << "emr" << endl;
                                cout << "emo" << endl;
                                cout << "per" << endl;
                                cout << "peo" << endl;
                                cout << "position" << endl;
                                cout << "config_InitType" << endl;
                                cout << "init" << endl;
                                cout << "findspeeds" << endl;
                                cout << "findfls" << endl;
                                cout << "findrls" << endl;
                                cout << "describe" << endl;
                                cout << "speed" << endl;
                                cout << "acceleration" << endl;
                                cout << "opmode" << endl;
                                cout << "defposition p" << endl;
                                cout << "pos_on_init" << endl;
                                cout << "econf" << endl;
                                cout << "etype type" << endl;
                } else
               	if ( (i=cmd.find("motor_on"))     != string::npos ) {

                		cout << "mOn ret: " << axis->motor_on() << endl;
                } else
               	if ( (i=cmd.find("motor_off"))    != string::npos ) {

                		cout << "mOff ret: " << axis->motor_off() << endl;
                } else
               	if ( (i=cmd.find("mpos"))  != string::npos ) {

                		char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+4).c_str());
                                double v = strtod(str, &endptr);
                                if (endptr == str) cout << "mpos: " << axis->mpos() << endl;
                                else cout << "mpos ret:" << axis->mpos(v) << endl;
                } else
               	if ( (i=cmd.find("stop"))    != string::npos ) {

                		cout << "stop ret: " << axis->stop() << endl;
                } else
                if ( (i=cmd.find("mspeed"))    != string::npos ) {

				axis->refresh();
                		char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+6).c_str());
                                double v = strtod(str, &endptr);
                                if (endptr == str) cout << "mspeed:" << axis->mspeed() << endl;
                                else cout << "mspeed ret:" << axis->mspeed(v) << endl;
				axis->refresh();
                } else
                if ( (i=cmd.find("mforward"))  != string::npos ) {

                                double v = atof(cmd.substr(i+8).c_str());
                                cout << "forward ret: " << ((Motor_ns::Motor*)axis)->forward(v) << endl;
                } else
                if ( (i=cmd.find("mbackward")) != string::npos ) {
                                double v = atof(cmd.substr(i+9).c_str());
                                cout << "backward ret: " << ((Motor_ns::Motor*)axis)->backward(v) << endl;
                } else
                if ( (i=cmd.find("cia402"))    != string::npos ) {

				bool cmd_f = false;
				Controller_ns::stateCmd_t sc;
				string s = cmd.substr(6);
				if ( s.find("shutdown") != string::npos ) { cmd_f = true; sc = Controller_ns::SHUTDOWN_CMD; } else
				if ( s.find("switchon") != string::npos ) { cmd_f = true; sc = Controller_ns::SWITCH_ON_CMD; } else
				if ( s.find("enableop") != string::npos ) { cmd_f = true; sc = Controller_ns::ENABLE_OPERATION_CMD; } else
				if ( s.find("disableop") != string::npos ) { cmd_f = true; sc = Controller_ns::DISABLE_OPERATION_CMD; } else
				if ( s.find("quickstop") != string::npos ) { cmd_f = true; sc = Controller_ns::QUICK_STOP_CMD; } else
				if ( s.find("removequickstop") != string::npos ) { cmd_f = true; sc = Controller_ns::REMOVE_QUICK_STOP_CMD; } else
				if ( s.find("enablevoltage") != string::npos ) { cmd_f = true; sc = Controller_ns::ENABLE_VOLTAGE_CMD; } else
				if ( s.find("disablevoltage") != string::npos ) { cmd_f = true; sc = Controller_ns::DISABLE_VOLTAGE_CMD; } else
				if ( s.find("faultreset") != string::npos ) { cmd_f = true; sc = Controller_ns::FAULT_RESET_CMD; } else
				if ( s.find("halt") != string::npos ) { cmd_f = true; sc = Controller_ns::HALT_CMD; } else
				if ( s.find("clearhalt") != string::npos ) { cmd_f = true; sc = Controller_ns::CLEAR_HALT_CMD; } else
				{};

                                if (cmd_f) cout << "cia402 ret:" << n5d->CiA402(sc) << endl;
				else cout << "cia402:" << n5d->CiA402() << endl;
                } else
                if ( (i=cmd.find("forward"))  != string::npos ) {

                                double v = atof(cmd.substr(i+7).c_str());
                                cout << "forward ret: " << axis->forward(v) << endl;
                } else
                if ( (i=cmd.find("backward")) != string::npos ) {
                                double v = atof(cmd.substr(i+8).c_str());
                                cout << "backward ret: " << axis->backward(v) << endl;
                } else
                if ( (i=cmd.find("drv_verbose"))!= string::npos ) {
                                char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+11).c_str());
                                double v = strtod(str, &endptr);
                                if (endptr == str) cout << "drv_verbose:" << axis->drv_verbose() << endl;
                                else {cout << "drv_verbose" << endl; axis->drv_verbose((int)v);}
                } else
                if ( (i=cmd.find("verbose"))  != string::npos ) {
                                char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+7).c_str());
                                double v = strtod(str, &endptr);
                                if (endptr == str) cout << "verbose:" << axis->verbose() << endl;
                                else {cout << "verbose" << endl; axis->verbose((int)v);}
                } else
                if ( (i=cmd.find("clrls"))  != string::npos ) {
				Controller_ns::stateCmd_t sc;

				sc = Controller_ns::FAULT_RESET_CMD;
                                cout << "cia402(faultreset) ret:" << n5d->CiA402(sc) << endl;
				usleep(2*1000);
				
				sc = Controller_ns::SHUTDOWN_CMD;
                                cout << "cia402(shutdown) ret:" << n5d->CiA402(sc) << endl;
				usleep(2*1000);

				sc = Controller_ns::SWITCH_ON_CMD;
                                cout << "cia402(switchon) ret:" << n5d->CiA402(sc) << endl;
				
                } else
                if ( (i=cmd.find("macceleration"))    != string::npos ) {

				axis->refresh();
                		char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+13).c_str());
                                double v = strtod(str, &endptr);
                                if (endptr == str) cout << "macceleration:" << axis->macceleration() << endl;
                                else cout << "macceleration ret:" << axis->macceleration(v) << endl;
				axis->refresh();
                } else
                if ( (i=cmd.find("mdeceleration"))    != string::npos ) {

				axis->refresh();
                		char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+13).c_str());
                                double v = strtod(str, &endptr);
                                if (endptr == str) cout << "mdeceleration:" << axis->mdeceleration() << endl;
                                else cout << "mdeceleration ret:" << axis->mdeceleration(v) << endl;
				axis->refresh();
                } else
               	if ( (i=cmd.find("axis"))     != string::npos ) {

				int lv = axis->verbose();
				axis->verbose(0);
				axis->refresh();
                		cout << "axis per   :" << axis->per() << endl;
                		cout << "axis peo   :" << axis->peo() << endl;
                		cout << "axis pmr   :" << axis->pmr() << endl;
                		cout << "axis emr   :" << axis->emr() << endl;
                		cout << "axis emo   :" << axis->emo() << endl;
                		cout << "axis speed :" << axis->speed() << endl;
                		cout << "axis accel :" << axis->acceleration() << endl;
                		cout << "axis decel :" << axis->deceleration() << endl;
                		cout << "axis maccel:" << axis->macceleration() << endl;
                		cout << "axis mdecel:" << axis->mdeceleration() << endl;

				cout << endl;
				cout << "have_rls   : " << (axis->have_rls()?"yes":"no") << endl;
				cout << "have_fls   : " << (axis->have_fls()?"yes":"no") << endl;
				cout << "have_home  : " << (axis->have_home()?"yes":"no") << endl;
				cout << "have_index : " << (axis->have_index()?"yes":"no") << endl;

				cout << endl;
				cout << "opmode     : " << axis->opmode() << " : " << axis->opmode2str(axis->opmode()) << endl;
				cout <<	"isinit     : " << (axis->isinit(true)?"yes":"no") << endl;
				cout << "inittype   : " << axis->config_InitType2str() << endl;
                                cout << "pos_on_init: " << axis->pos_on_init() << endl;

				cout << endl;
        			hmparam = n5d->Cia402HomingParams();
				cout << "hmparam.home_offset                      : "<< hmparam.home_offset << endl;
				cout << "hmparam.home_speed_switch                : "<< hmparam.home_speed_switch << endl;
				cout << "hmparam.home_speed_reference             : "<< hmparam.home_speed_reference << endl;
				cout << "hmparam.home_acceleration                : "<< hmparam.home_acceleration << endl;
				cout << "hmparam.home_switch_tolerance_band       : "<< hmparam.home_switch_tolerance_band << endl;
				cout << "hmparam.home_min_current_block_detection : "<< hmparam.home_min_current_block_detection << endl;
				cout << "hmparam.home_period_of_blocking          : "<< hmparam.home_period_of_blocking << endl;


				cout << endl;
				axis->verbose(lv);
                } else
                if ( (i=cmd.find("emr")) != string::npos ) {

                		char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+3).c_str());
                                double v = strtod(str, &endptr);
                                if (endptr == str) cout << "emr:" << axis->emr() << endl;
                                else cout << "emr ret:" << axis->emr(v) << endl;
                } else
                if ( (i=cmd.find("emo")) != string::npos ) {

                		char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+3).c_str());
                                double v = strtod(str, &endptr);
                                if (endptr == str) cout << "emo:" << axis->emo() << endl;
                                else cout << "emo ret:" << axis->emo(v) << endl;
                } else
                if ( (i=cmd.find("per")) != string::npos ) {

                		char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+3).c_str());
                                double v = strtod(str, &endptr);
                                if (endptr == str) cout << "per:" << axis->per() << endl;
                                else cout << "per ret:" << axis->per(v) << endl;
                } else
                if ( (i=cmd.find("peo")) != string::npos ) {

                		char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+3).c_str());
                                double v = strtod(str, &endptr);
                                if (endptr == str) cout << "peo:" << axis->peo() << endl;
                                else cout << "peo ret:" << axis->peo(v) << endl;
                } else
                if ( (i=cmd.find("defposition"))    != string::npos ) {

				axis->refresh();
                		char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+11).c_str());
                                double v = strtod(str, &endptr);
                                if (endptr != str) cout << "defposition ret:" << axis->defposition(v) << endl;
				axis->refresh();
                } else
                if ( (i=cmd.find("position")) != string::npos ) {

                		char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+8).c_str());
                                double v = strtod(str, &endptr);
                                if (endptr == str) cout << "position:" << axis->position() << endl;
                                else cout << "position ret:" << axis->position(v) << " : " << axis->axis_error_str() << endl;
                } else
		if ( (i=cmd.find("config_InitType"))    != string::npos ) {
                                char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+15).c_str());
				if ( string(str).find("h") != std::string::npos ) {

					cout << "INIT_ON_DEFPOS        :" << INIT_ON_DEFPOS << " : " << AxisInitTypeName[ INIT_ON_DEFPOS ] << endl;
					cout << "INIT_ON_HOME          :" << INIT_ON_HOME << " : " << AxisInitTypeName[ INIT_ON_HOME ] << endl;
					cout << "INIT_ON_RLS           :" << INIT_ON_RLS << " : " << AxisInitTypeName[ INIT_ON_RLS ] << endl;
					cout << "INIT_NO_3             :" << INIT_NO_3 << " : " << AxisInitTypeName[ INIT_NO_3 ] << endl;
					cout << "INIT_ON_FLS           :" << INIT_ON_FLS << " : " << AxisInitTypeName[ INIT_ON_FLS ] << endl;
					cout << "INIT_ON_CUSTOM        :" << INIT_ON_CUSTOM << " : " << AxisInitTypeName[ INIT_ON_CUSTOM ] << endl;
					cout << "INIT_NO_6             :" << INIT_NO_6 << " : " << AxisInitTypeName[ INIT_NO_6 ] << endl;
					cout << "INIT_ON_RLSIDX        :" << INIT_ON_RLSIDX << " : " << AxisInitTypeName[ INIT_ON_RLSIDX ] << endl;
					cout << "INIT_ON_FLSIDX        :" << INIT_ON_FLSIDX << " : " << AxisInitTypeName[ INIT_ON_FLSIDX ] << endl;
					cout << "INIT_ON_EDGEPOSDEF    :" << INIT_ON_EDGEPOSDEF << " : " << AxisInitTypeName[ INIT_ON_EDGEPOSDEF ] << endl;
					cout << "INIT_ON_EDGENEGDEF    :" << INIT_ON_EDGENEGDEF << " : " << AxisInitTypeName[ INIT_ON_EDGENEGDEF ] << endl;
					cout << "INIT_ON_ABSOLUTE      :" << INIT_ON_ABSOLUTE << " : " << AxisInitTypeName[ INIT_ON_ABSOLUTE ] << endl;
					cout << "INIT_ON_SENCPOS       :" << INIT_ON_SENCPOS << " : " << AxisInitTypeName[ INIT_ON_SENCPOS ] << endl;
					cout << "INIT_ON_SENCNEG       :" << INIT_ON_SENCNEG << " : " << AxisInitTypeName[ INIT_ON_SENCNEG ] << endl;
					cout << "INIT_ON_ROTIDXPOS     :" << INIT_ON_ROTIDXPOS << " : " << AxisInitTypeName[ INIT_ON_ROTIDXPOS ] << endl;
					cout << "INIT_ON_ROTIDXNEG     :" << INIT_ON_ROTIDXNEG << " : " << AxisInitTypeName[ INIT_ON_ROTIDXNEG ] << endl;
					cout << "CIA402_INIT_ON_RLS    :" << CIA402_INIT_ON_RLS << " : " << AxisInitTypeName[ CIA402_INIT_ON_RLS ] << endl;
					cout << "CIA402_INIT_ON_RLS    :" << CIA402_INIT_ON_FLS << " : " << AxisInitTypeName[ CIA402_INIT_ON_FLS ] << endl;
					cout << "CIA402_INIT_ON_RLSIDX :" << CIA402_INIT_ON_RLSIDX << " : " << AxisInitTypeName[ CIA402_INIT_ON_RLSIDX ] << endl;
					cout << "CIA402_INIT_ON_FLSIDX :" << CIA402_INIT_ON_FLSIDX << " : " << AxisInitTypeName[ CIA402_INIT_ON_FLSIDX ] << endl;

					cout << endl;
					cout << "config_InitType:" << axis->config_InitType() << endl;
					cout << endl;
				} else {
                                	int v = strtol(str, &endptr, 10);
                                	Axis_ns::AxisInitType_t w = (Axis_ns::AxisInitType_t) v;
                                	if (endptr == str) cout << "config_InitType:" << axis->config_InitType() << ":" << AxisInitTypeName[axis->config_InitType()]  << endl;
                                	else {cout << "config_InitType ret:" << endl; axis->config_InitType(w); }
				}
                } else
                if ( (i=cmd.find("findspeeds"))!= string::npos ){
                                char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+10).c_str());
                                float s1, s2;
                                vector<double> sp = axis->find_speeds();
                                int r =  sscanf(str, "%f %f", &s1, &s2);
                                if ( r != 2 ) cout << "find_speeds: " << sp[0] << " " << sp[1]   << endl;
                                else  {
                                	sp[0] = s1;
                                        sp[1] = s2;
                                        cout << "find_speeds ret: " << axis->find_speeds(sp) << endl;
                                }
                } else
                if ( (i=cmd.find("findfls"))     != string::npos ) {
                                axis->fsm(Axis_ns::FSMT_FINDFLS);
                } else
                if ( (i=cmd.find("findrls"))     != string::npos ) {
                                axis->fsm(Axis_ns::FSMT_FINDRLS);
                } else
                if ( (i=cmd.find("init"))     != string::npos ) {
                                axis->fsm(Axis_ns::FSMT_INIT);
                } else
               	if ( (i=cmd.find("describe"))     != string::npos ) {

				cout << endl;
				std::vector<std::string> desc = n5d->describe();
				std::vector<std::string>::iterator it;
				for (it = desc.begin(); it != desc.end(); ++it) cout << *it << endl;
				cout << endl;
                } else
                if ( (i=cmd.find("speed"))    != string::npos ) {

				axis->refresh();
                		char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+5).c_str());
                                double v = strtod(str, &endptr);
                                if (endptr == str) cout << "speed:" << axis->speed() << endl;
                                else cout << "speed ret:" << axis->speed(v) << endl;
				axis->refresh();
                } else
                if ( (i=cmd.find("acceleration"))    != string::npos ) {

				axis->refresh();
                		char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+12).c_str());
                                double v = strtod(str, &endptr);
                                if (endptr == str) cout << "acceleration:" << axis->acceleration() << endl;
                                else cout << "acceleration ret:" << axis->acceleration(v) << endl;
				axis->refresh();
                } else
               	if ( (i=cmd.find("opmode"))  != string::npos ) {

				axis->refresh();
                		char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+6).c_str());
                                int v = (int)strtol(str, &endptr, 10);
                                if (endptr == str) cout << "opmode: " << axis->opmode() << " : " << axis->opmode2str(axis->opmode()) << endl;
                                else cout << "opmode ret:" << axis->opmode((Controller_ns::opmode_t)v) << endl;
				axis->refresh();
                } else
                if ( (i=cmd.find("pos_on_init")) != string::npos ) {

                		char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+11).c_str());
                                double v = strtod(str, &endptr);
                                if (endptr == str) cout << "pos_on_init:" << axis->pos_on_init() << endl;
                                else cout << "pos_on_init ret:" << axis->pos_on_init(v) << endl;
                } else
                if ( (i=cmd.find("econf")) != string::npos ) {

                		char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+5).c_str());
                                int v = strtod(str, &endptr);
                                if (endptr == str) {
					Controller_ns::econf_t ec = axis->get_econf();
					cout << "econf.which          :" << ec.which << endl;
					cout << "econf.ce             :" << ec.ce << endl;
					cout << "econf.mOffOn.enabled :" << ec.mOffOn.enabled << endl;
					cout << "econf.mOffOn.m       :" << ec.mOffOn.m << endl;
					cout << "econf.mOffOn.q       :" << ec.mOffOn.q << endl;
                                } else {
					//cout << "pos_on_init ret:" << axis->pos_on_init(v) << endl;
				}
                } else
               	if ( (i=cmd.find("etype"))  != string::npos ) {

                		char *endptr, str[500];
                                snprintf(str, 500, "%s", cmd.substr(i+5).c_str());
                                long int v = strtol(str, &endptr, 10);
                                if (endptr == str) cout << "etype: " << axis->etype() << ":" << Encoder_ns::EncoderTypeName[axis->etype()]<< endl;
                                else cout << "etype ret:" << axis->etype((Encoder_ns::EncoderType_t)v, 1) << endl;
                } else


                if ( (i=cmd.find("q"))    != string::npos ) cmd="quit";
        } while ( cmd.find("quit") == string::npos );

	cout << endl;
       	cout << "ON EXIT: mOff ret: " << axis->motor_off() << endl;
	cout << endl;

        return 0;

}
