#include <AxisG2.h>
#include <AxisG2Class.h>


namespace AxisG2_ns
{
bool AxisG2::AxisCreate(bool withtmo){
	std::string obj("AxisG2::AxisCreate()");
	std::stringstream ss("");

	#ifdef _DBG_AXISCREATE
		prtlog(obj, ss.str());
	#endif

	if (driver.size() == 0){
                __prevstate = Tango::ALARM;
                set_state(__prevstate);
                _ALARMMsg = "No driver specified";
		ss.str("");
                ss << "No driver specified";
		prtlog(obj, ss.str());
		return false;
	}

	if (withtmo){
       		struct timeval t;
        	gettimeofday(&t, NULL);
        	double t_s = (double)(t.tv_sec-_last_create.tv_sec)+((double)(t.tv_usec-_last_create.tv_usec))/1000000;
        	if ( t_s < _recreate_tmo ) return false;
        	_last_create.tv_sec  = t.tv_sec;
        	_last_create.tv_usec = t.tv_usec;
	}
	
	axis = NULL;
	
//---------------- CONTROLLER  -----------
	if (controller) delete controller;
	ss.str("");
	ss << "new controller";

	#ifdef _DBG_AXISCREATE_2
		prtlog(obj, ss.str());
	#else
        if (prtTrue){
		//cout << "prtTrue = " << prtTrue << endl;
		prtlog(obj, ss.str());
	};
	#endif

       	controller = new Controller_ns::Controller(controller_type, driver, "", prtTrue);
	if ( !controller ){
                __prevstate = Tango::ALARM;
                set_state(__prevstate);
                _ALARMMsg = "No controller created";
		ss.str("");
                ss << "No controller created";
		prtlog(obj, ss.str());
		return false;
	}

	#ifdef _DBG_HARDWAREWRITE
		cout << "HWWriteEnable = " << (hWWriteEnable?"TRUE":"FALSE") << endl;
	#endif /* _DBG_HARDWAREWRITE */
	controller->hw_write_enable(hWWriteEnable);

	ct = controller->state();
        if ( ct == Controller_ns::CTRL_CLOSE ){
		#ifdef _DBG_WAITFORCTRL
			ss.str("");
                	ss << "***** WAIT for controller to come up";
			prtlog(obj, ss.str());
		#endif /* _DBG_WAITFORCTRL */
                __prevstate = Tango::ALARM;
                set_state(__prevstate);
                _ALARMMsg = "No controller at start-up";
        } else {
		#ifdef _DBG_WAITFORCTRL
			ss.str("");
                	ss << "***** PROCEED: controller->state()  != CTRL_CLOSE";
			prtlog(obj, ss.str());
		#endif /* _DBG_WAITFORCTRL */

		*attr_IsMicrocodeOK_read = controller->drv()->program_running();

//N5N5
        	switch (controller->controller()){
                	case Controller_ns::N5:
	        		n5d  = (Controller_ns::N5Drv*)controller->drv();
				break;
			default:
				break;
		};


//---------------- AXIS -----------
		if (channel.size() == 0){
                	__prevstate = Tango::ALARM;
                	set_state(__prevstate);
                	_ALARMMsg = "No channel specified";
			ss.str("");
                	ss << "No channel specified";
			prtlog(obj, ss.str());
			return false;
		}

		try {
			#ifdef _DBG_AXISCREATE_3
				ss.str("");
                        	ss << "---------------- Going to create Axis_th(" << device_name << ",";
				ss << channel << "," << controller << ")";
                        	prtlog(obj, ss.str());
			#endif
                	axis      = new Axis_ns::Axis_th(device_name, channel, controller, prtTrue==0?0:1);
		} catch (...) {
			axis_init_at_first_fsm = false;
                	__prevstate = Tango::FAULT;
                	set_state(__prevstate);
                	_ALARMMsg = "Error on Axis() creation";
		}
               	if (axis) _conf_flag = AxisConf();
		if (_conf_flag ){
               		__prevstate = Tango::ON;
               		set_state(__prevstate);
               		_ALARMMsg = "";
		} else {
                	__prevstate = Tango::ALARM;
                	set_state(__prevstate);
                	_ALARMMsg = "Axis configuration failed!";
			ss.str("");
			ss << "[" << __LINE__ << "] ";
                	ss << "Axis configuration failed!";
        		if (prtTrue){
				prtlog(obj, ss.str());
			}
			if(axis) delete axis;
			axis = NULL;
		}
        }

	//cout << "prtTrue = " << prtTrue << endl;
        if (prtTrue){
       		if (!controller){
			ss.str("");
			ss << "NOT CONTROLLER!";
			prtlog(obj, ss.str());
		} else {
			ss.str("");
        		if ( ct == Controller_ns::CTRL_UP ) ss << "GOT CONTROLLER!";
			else ss << "CONTROLLER in down!";
			prtlog(obj, ss.str());
		}
       		if (!axis){
			ss.str("");
			ss << "NOT AXIS!";
			prtlog(obj, ss.str());
		} else {
			ss.str("");
			ss << "GOT AXIS!";
			prtlog(obj, ss.str());
		}
                if (prtTrue == 1) {
			ss.str("");
			ss << "~~~~~~~~~~~~~ LAST LOG! ~~~~~~~~~~~~~";
			prtlog(obj, ss.str());
		};
                if (prtTrue >= 1) prtTrue --;
        } else {
		if ( controller && axis ) {
		        prtTrue = _PRTTRUE_;
			ss.str("");
			ss << "GOT CONTROLLER & AXIS!";
			prtlog(obj, ss.str());
		}
	}


	if (axis) return true;
	return false;
}

bool AxisG2::AxisConf(){
	std::string obj("AxisG2::AxisConf()");
	std::stringstream ss("");
	

#ifdef _DBG_CONF
	axis->verbose(1);
	axis->drv_verbose(2);
	axis->lnk_verbose(1);
#endif
	#ifdef _DBG_CONF
		cout << "---- AXIS CONFIGURATION Entering !----@" << __LINE__ << " State: " << Tango::DevStateName[get_state()] << endl;
	#endif /* _DBG_CONF */
	//sleep(1); ??

	if (initType == -1){
		ss.str("");
		ss << "InitType UNCONFIGURED";
		prtlog(obj, ss.str());
		return false;
	} else {
		axis->config_InitType((Axis_ns::AxisInitType_t)initType);
	}

	axis->pos_on_init(initPosition); // dB default 0

	if (findSpeeds.size() < 2){
		ss.str("");
		ss << "FindSpeeds MIS/UN-CONFIGURED";
		prtlog(obj, ss.str());
		return false;
	} else {
		axis->find_speeds(findSpeeds);
		#if defined(_DBG_AXISINIT) || defined(_DBG_CONF)
		vector<double> sp = axis->find_speeds();
        	cout  << __func__ << "@" << __LINE__ << " : " << "find_speeds: " << sp[0] << " " << sp[1]   << endl;

		#endif
	}

	if (positionEncoderRatio==(double)0){
		__prevstate = Tango::FAULT;
		set_state(__prevstate);
		ss.str("");
		ss << "PositionEncoderRatio MIS/UN-CONFIGURED";
		prtlog(obj, ss.str());
		return false;
	}

	if (encoderMotorRatio==(double)0){
		__prevstate = Tango::FAULT;
		set_state(__prevstate);
		ss.str("");
		ss << "EncoderMotorRatio MIS/UN-CONFIGURED";
		prtlog(obj, ss.str());
		return false;
	}

	#ifdef _DBG_CONF
		cout << "---- AXIS CONFIGURATION !----@" << __LINE__ << " State: " << Tango::DevStateName[get_state()] << endl;
	#endif /* _DBG_CONF */
	//
	// ENCODER CONFIGURATION
	//
	axis->mOffOn(false);

	if ( encoderPrecision <= 0 ){
		__prevstate = Tango::FAULT;
		set_state(__prevstate);
		ss.str("");
		ss << "EncoderPrecision MIS/UN-CONFIGURED";
		prtlog(obj, ss.str());
		return false;
	}
	if ( encoderType < 0 ){
		__prevstate = Tango::FAULT;
		set_state(__prevstate);
		ss.str("");
		ss << "EncoderType UNCONFIGURED";
		prtlog(obj, ss.str());
		return false;
	}
	ss.str("");
	axis->etype((Encoder_ns::EncoderType_t)encoderType, encoderPrecision);
	ss << "axis->etype() = " << axis->etype();
	prtlog(obj, ss.str());
	switch (axis->etype()){
		case Encoder_ns::ENC_ANALOG_ABSOLUTE:
			break;
	        case Encoder_ns::ENC_NO:
        	case Encoder_ns::ENC_DIGITAL_INCREMENTAL:
        	case Encoder_ns::ENC_DIGITAL_ABSOLUTE:
        	case Encoder_ns::ENC_DIGITAL_SEMIABSOLUTE:
		default:
			if(!encoderDirection_defined) {
				ss.str("");
				ss << "axis missing EncoderDirection";
				prtlog(obj, ss.str());
         			__prevstate = Tango::FAULT;
               			set_state(__prevstate);
               			_ALARMMsg = "Error on Axis() creation";
				return false;
			}
			break;
	}

	ss.str("");
	ss << "encoderExtras.size() = " << encoderExtras.size();
	prtlog(obj, ss.str());

	if ( encoderExtras.size()>=1){
		ss.str("");
		ss << "encoderExtras[0].size()= " << encoderExtras[0].size();
		prtlog(obj, ss.str());
	}

	if ( (axis->etype()==Encoder_ns::ENC_DIGITAL_SEMIABSOLUTE) && (encoderExtras.size()!=0) ){
		if ( (encoderExtras.size()>=1) && (encoderExtras[0].size()!=0) ){
			if (!configSencEnc(encoderExtras)){
				ss.str("");
				ss << "error on configSencEnc()!";
				prtlog(obj, ss.str());

         			__prevstate = Tango::FAULT;
               			set_state(__prevstate);
               			_ALARMMsg = "Error on Axis() creation, extras";
				return false;
			}
			/*
			typedef struct {
        			EncoderLimit_t startPoint;
        			double SencRes;
        			double SencPitch;
			} SencParams_t;
			*/
			Encoder_ns::SencParams_t sp = axis->SencParams();

			ss.str("");
			ss << "startPoint= " << sp.startPoint;
			ss << " SencRes=    " << sp.SencRes;
			ss << " SencPitch=  " << sp.SencPitch;
			prtlog(obj, ss.str());

		}
	}

	Controller_ns::econf_t ec;
	bool econf_ok=false;
        switch (controller->controller()){
                case Controller_ns::Yams:
			if (axis->etype() != Encoder_ns::ENC_ANALOG_ABSOLUTE){
				ec.which = Controller_ns::ECONF_CE;
				ec.ce = encoderDirection;
				econf_ok = axis->econf(ec); 
			} else {
				ec = axis->get_econf();
				//cout << "axis->econf() = " << ec.mOffOn.enabled << " " << ec.mOffOn.m << " " << ec.mOffOn.q << endl;
				econf_ok = true;
				if ( isnan(ec.mOffOn.m) || isnan(ec.mOffOn.q) ){
					axis->mOffOn(false);
				} else {
					if (ec.mOffOn.m!=0.0) {
						axis->mOffOn(ec.mOffOn.m, ec.mOffOn.q);
						axis->mOffOn(true);
					} else {
						axis->mOffOn(false);
					}
				}
			}
			break;
                case Controller_ns::DMC2xxxPiezo:
			switch (axis->etype()){
				case Encoder_ns::ENC_NO:
				case Encoder_ns::ENC_ANALOG_ABSOLUTE:
					// ok
					econf_ok = axis->econf(ec); 
					break;
				default:
					ss.str("");
					ss << __FILE__ << " @line:" << __LINE__ << " :: ";
					ss << "for " << controller->controller_typeName() << " encoder must be ANALOG_ABSOLUTE";
					prtlog(obj, ss.str());
					econf_ok = false;
					break;
			}
			break;
//N5N5
                case Controller_ns::N5:
			switch (axis->etype()) {
				case Encoder_ns::ENC_DIGITAL_INCREMENTAL:
					ec.which = Controller_ns::ECONF_CE;
					ec.ce = encoderDirection;
					econf_ok = axis->econf(ec); 
					#ifdef _DBG_ENC_CONF
						cout << "ec.ce    = " << encoderDirection << endl;
						cout << "econf_ok = " << econf_ok << endl;
					#endif
					break;
				case Encoder_ns::ENC_NO:
					#ifdef _DBG_CONF
						ss.str("");
						ss << __FILE__ << " @line:" << __LINE__ << " :: N5 !! WITHOUT ENCODER !!";
						prtlog(obj, ss.str());
					#endif
					econf_ok = true;
					break;
				default:
					ss.str("");
					ss << __FILE__ << " @line:" << __LINE__ << " :: not valid encoder for N5";
					prtlog(obj, ss.str());
					econf_ok = false;
					break;
			}
			break;
//SIMUL
                case Controller_ns::SIMUL:
			switch (axis->etype()) {
				case Encoder_ns::ENC_NO:
					#ifdef _DBG_CONF
						ss.str("");
						ss << __FILE__ << " @line:" << __LINE__ << " :: SIMUL !! WITHOUT ENCODER !!";
						prtlog(obj, ss.str());
					#endif
					econf_ok = true;
					break;
				default:
					ss.str("");
					ss << __FILE__ << " @line:" << __LINE__ << " :: not valid encoder for SIMUL";
					prtlog(obj, ss.str());
					econf_ok = false;
					break;
			}
			break;
//BCS
                case Controller_ns::BCS:
			switch (axis->etype()) {
				case Encoder_ns::ENC_NO:
					econf_ok = true;
					break;
				case Encoder_ns::ENC_DIGITAL_ABSOLUTE:
					econf_ok = true;
					break;
				default:
					ss.str("");
					ss << __FILE__ << " @line:" << __LINE__ << " :: not valid encoder for SIMUL";
					prtlog(obj, ss.str());
					econf_ok = false;
					break;
			}
			break;
//
		default:
			ss.str("");
			ss << __FILE__ << " @line:" << __LINE__ << " :: ";
			ss << controller->controller_typeName() << " :: ";
			ss << "Not known controller for encoder econf(.)";
			prtlog(obj, ss.str());
			break;
	};
	
	if (!econf_ok){
		ss.str("");
		ss << "[" << __LINE__ << "] ";
		ss << "axis->econf false";
		prtlog(obj, ss.str());

         	__prevstate = Tango::FAULT;
               	set_state(__prevstate);
               	_ALARMMsg = "Error on Axis() creation";
	} else {
		ss.str("");
		ss << "axis->econf true";
		prtlog(obj, ss.str());
	}

	#ifdef _DBG_CONF
		cout << "encoderAnalogRatio= " << encoderAnalogRatio << endl;
		cout << "encoderAnalogOffset= " << encoderAnalogOffset << endl;
	#endif /* _DBG_CONF */

	axis->ear(encoderAnalogRatio);
	axis->eao(encoderAnalogOffset);

	axis->per(positionEncoderRatio);
	axis->peo(positionEncoderOffset);

	#ifdef _DBG_CONF
		cout << "encoderMotorRatio = " << encoderMotorRatio << endl;
	#endif /* _DBG_CONF */

	axis->emr(encoderMotorRatio);
	axis->emo(encoderMotorOffset);

	#ifdef _DBG_CONF
		cout << "EMR = " <<  axis->emr() << endl;
		cout << "EMO = " <<  axis->emo() << endl;
		cout << "PER = " <<  axis->per() << endl;
		cout << "PEO = " <<  axis->peo() << endl;
		cout << "etype = " << axis->etype() << endl;
		cout << "e precision = " << axis->precision() << endl;
	#endif /* _DBG_CONF */


	if ( encoderRippleOnStay > 0 ) axis->ripple_on_stay(fabs(encoderRippleOnStay));
	else axis->ripple_on_stay(fabs(2));

	if ( encoderRippleOnRun > 0 ) axis->ripple_on_run(fabs(encoderRippleOnRun));
	else axis->ripple_on_run(fabs(2));

	#ifdef _DBG_CONF
		cout << "EncoderRippleOnStay = " << encoderRippleOnStay << " / ";
		cout << "axis->ripple_on_stay = " << axis->ripple_on_stay() << endl;

		cout << "EncoderRippleOnRun = " << encoderRippleOnRun << " / ";
		cout << "axis->ripple_on_run = " << axis->ripple_on_run() << endl;
	#endif /* _DBG_CONF */


	double pt = 0;
	pt = controller->updateDelay();

	#ifdef _DBG_CONF
		cout << "pt = " << pt << endl;
	#endif /* _DBG_CONF */

	pt = pt > 0 ? pt : 1;

	if ( encoderCheckDelay > 0 ) axis->delay_on_check(fabs(encoderCheckDelay), pt);
	else axis->delay_on_check(1.5, pt);
	
	#ifdef _DBG_CONF
		cout << "---- AXIS CONFIGURATION !----@" << __LINE__ << " State: " << Tango::DevStateName[get_state()] << endl;
	#endif /* _DBG_CONF */

//--------------------------------   CONF --------------------------------------START

	long   dbw   = 2;
	double pc    = 1;
	double bck   = 0;
	long   dir   = 0;
	long   nbrtr = 10;

//===================================================================
// retrive memorized attribute values __values
	// long   dbw ------>> MotionAccuracy	 double
	// double pc  ------>> MotionPercent	 double
	// double bck ------>> Backlash		 double
	// long   dir ------>> BacklashDirection double
	// long   nbrtr ---->> MotionRetry	 double

        Tango::DbDatum dbdatum;
	double v;

	try {
                if (read_memorized_attribute("MotionAccuracy", dbdatum)) {
			dbdatum >> v;
			dbw = (long) (axis->position2enc(0 + v) - axis->position2enc(0));
			if ( dbw <= 0 ) dbw = 2;
		};
        } catch(Tango::DevFailed &ex) {
		Tango::Except::throw_exception(
					(const char *)"AxisG2",
					(const char *)"Error on read memorized attribute MotionAccuracy",
					(const char *)"AxisG2::init_device()");
        }
	try {
                if (read_memorized_attribute("MotionPercent", dbdatum)) {
			dbdatum >> v;
			if (( v <= 0 ) || ( v > 1 )) v = 1;
			pc = v;
		};
        } catch(Tango::DevFailed &ex) {
		Tango::Except::throw_exception(
					(const char *)"AxisG2",
					(const char *)"Error on read memorized attribute MotionPercent",
					(const char *)"AxisG2::init_device()");
        }
	try {
                if (read_memorized_attribute("Backlash", dbdatum)) {
			dbdatum >> v;
			bck = (axis->position2enc(0 + v) - axis->position2enc(0));
		};
        } catch(Tango::DevFailed &ex) {
		Tango::Except::throw_exception(
					(const char *)"AxisG2",
					(const char *)"Error on read memorized attribute Backlash",
					(const char *)"AxisG2::init_device()");
        }
	try {
                if (read_memorized_attribute("BacklashDirection", dbdatum)) {
			dbdatum >> v;
			long l = (long)v;
			switch (l){
				case -1:
				case  0:
				case  1:
					dir= l;
					break;
				default:
					break;
			}
		};
        } catch(Tango::DevFailed &ex) {
		Tango::Except::throw_exception(
					(const char *)"AxisG2",
					(const char *)"Error on read memorized attribute BacklashDirection",
					(const char *)"AxisG2::init_device()");
        }
	try {
                if (read_memorized_attribute("MotionRetry", dbdatum)) {
			dbdatum >> v;
			if ( v > 0 ) nbrtr = (long)v;
		};
        } catch(Tango::DevFailed &ex) {
		Tango::Except::throw_exception(
					(const char *)"AxisG2",
					(const char *)"Error on read memorized attribute MotionRetry",
					(const char *)"AxisG2::init_device()");
        }

//===================================================================
/*
namespace Controller_ns {

typedef struct {
        double mt;              // Motor type
        long dbw;               // Dead Band Width = GalilMotionAccuracy positioning accuracy. Written by the client in encoder units
        double scale;           // EncoderMotorRatio = encoder impulse / motor step. Written by the client
        double pc;              // Percent of the movement to be performed from 0.0 to 1.0
        double bck;             // Backlash. Encoder units
        long dir;               // Direction of the last positioning movement in backlash compensation
        long cp;                // Type/presence of encoder: 0 = no encoder
        long nbrtr;             // Number of retries before giving up/ending with movement
} yconf_t;

typedef union {
        yconf_t yc;
} mconf_t;
*/
	
	#ifdef _DBG_CONF
		cout << "---- AXIS CONFIGURATION !----@" << __LINE__ << " State: " << Tango::DevStateName[get_state()] << endl;
	#endif /* _DBG_CONF */


	Controller_ns::mconf_t mc, hw_mc;
        switch (controller->controller()){
                case Controller_ns::Yams:
			mc.yc.mt    = (double) motorDirection;
			mc.yc.scale = (double) axis->emr();
			mc.yc.cp    = (long)  axis->etype();
			mc.yc.dbw   = (long)  dbw; 	//==================> memorized attribute
			mc.yc.pc    = (double) pc;	//==================> memorized attribute
			mc.yc.bck   = (double) bck;	//==================> memorized attribute
			mc.yc.dir   = (long) dir;	//==================> memorized attribute
			mc.yc.nbrtr = (long) nbrtr;	// ==================> memorized attribute

			if (motorBrakeExtraTMO >= 0) mc.yc.brake_ONOFF_extra_TMO = motorBrakeExtraTMO * 1000000; // da sec a microsec
			else mc.yc.brake_ONOFF_extra_TMO = 1000000;

			{
			  int pool_period_ms = get_attribute_poll_period("Position");
			  if (motorRawBrakeONTMO >= 0) mc.yc.raw_brakeON_TMO = round(motorRawBrakeONTMO * 1000 / (double) pool_period_ms); // times
			  else mc.yc.raw_brakeON_TMO = 10;
			}

			break;
                case Controller_ns::DMC2xxxPiezo:
			mc.pc.mt    = (double) motorDirection;
			break;
                case Controller_ns::N5:
			mc.yc.mt    = (double) motorDirection;
			mc.yc.nbrtr = (long) nbrtr;	// ==================> memorized attribute
			mc.yc.scale = (double) axis->emr();
			mc.yc.dbw   = (long)  dbw; 	//==================> memorized attribute
			mc.yc.pc    = (double) pc;	//==================> memorized attribute
			mc.yc.bck   = (double) bck;	//==================> memorized attribute
			mc.yc.dir   = (long) dir;	//==================> memorized attribute
			mc.yc.cp    = (long)  axis->etype();
			break;
//SIMUL
                case Controller_ns::SIMUL:
			cout << __func__ << "@" << __LINE__ << endl;
			mc.yc.mt    = (double) motorDirection;
			break;
//BCS
                case Controller_ns::BCS:
			cout << __func__ << "@" << __LINE__ << endl;
			mc.yc.scale = (double) axis->emr();
			break;
		default:
			break;
	};
	if (!axis->mconf(mc)){
		ss.str("");
		ss << "axis->mconf false";
		prtlog(obj, ss.str());

               	__prevstate = Tango::FAULT;
               	set_state(__prevstate);
               	_ALARMMsg = "Error on Axis() creation";
	} else {
		ss.str("");
		ss << "axis->mconf true";
		prtlog(obj, ss.str());
	}

	#ifdef _DBG_CONF
		cout << "---- AXIS CONFIGURATION !----@" << __LINE__ << " State: " << Tango::DevStateName[get_state()] << endl;
	#endif /* _DBG_CONF */

//===================================================================

//............   READ BACK

	hw_mc = axis->mconf();
        switch (controller->controller()){
                case Controller_ns::Yams:
			if( (hw_mc.yc.mt    == mc.yc.mt)    &&
			    (hw_mc.yc.scale == mc.yc.scale) &&
			    (hw_mc.yc.cp    == mc.yc.cp)    &&
			    (hw_mc.yc.dbw   == mc.yc.dbw)   &&
			    (hw_mc.yc.pc    == mc.yc.pc)    &&
			    (hw_mc.yc.bck   == mc.yc.bck)   &&
			    (hw_mc.yc.dir   == mc.yc.dir)   &&
			    (hw_mc.yc.nbrtr == mc.yc.nbrtr)
			  ){
				*attr_MotionAccuracy_read 	= axis->enc2position(0+hw_mc.yc.dbw) - axis->enc2position(0);
				*attr_MotionPercent_read        = (double) hw_mc.yc.pc;
				*attr_Backlash_read       	= axis->enc2position(0+hw_mc.yc.bck) - axis->enc2position(0);
				*attr_BacklashDirection_read    = (double) hw_mc.yc.dir;
				*attr_MotionRetry_read          = (double) hw_mc.yc.nbrtr;
			} else {
				ss.str("");
				ss << " READ BACK error ";
				prtlog(obj, ss.str());

				__prevstate = Tango::FAULT;
				set_state(__prevstate);
			}
			break;
                case Controller_ns::DMC2xxxPiezo:
			if( (hw_mc.pc.mt    == mc.pc.mt)   ){
			} else {
				ss.str("");
				ss << " READ BACK error ";
				prtlog(obj, ss.str());

				__prevstate = Tango::FAULT;
				set_state(__prevstate);
			}
			break;
                case Controller_ns::N5:
			if( (hw_mc.yc.mt    == mc.yc.mt)    &&
			    (hw_mc.yc.scale == mc.yc.scale) &&
			    (hw_mc.yc.dbw   == mc.yc.dbw)   &&
			    (hw_mc.yc.pc    == mc.yc.pc)    &&
			    (hw_mc.yc.bck   == mc.yc.bck)   &&
			    (hw_mc.yc.dir   == mc.yc.dir)   &&
			    (hw_mc.yc.nbrtr == mc.yc.nbrtr)
			  ){
				*attr_MotionAccuracy_read 	= axis->enc2position(0+hw_mc.yc.dbw) - axis->enc2position(0);
				*attr_MotionPercent_read        = (double) hw_mc.yc.pc;
				*attr_Backlash_read       	= axis->enc2position(0+hw_mc.yc.bck) - axis->enc2position(0);
				*attr_BacklashDirection_read    = (double) hw_mc.yc.dir;
				*attr_MotionRetry_read          = (double) hw_mc.yc.nbrtr;
			} else {
				ss.str("");
				ss << " READ BACK error ";
				prtlog(obj, ss.str());

				__prevstate = Tango::FAULT;
				set_state(__prevstate);
			}
			break;
		default:
			break;
	};


	#ifdef _DBG_CONF
		cout << "---- AXIS CONFIGURATION !----@" << __LINE__ << " State: " << Tango::DevStateName[get_state()] << endl;
	#endif /* _DBG_CONF */


//--------------------------------   CONF --------------------------------------END


	#ifdef _DBG_MOTOR_CONF
		cout << endl;
		prtlog("config_InitType", axis->config_InitType2str());

		vector<double> sp = axis->find_speeds();
		ss << sp[0] << " " << sp[1];
		prtlog("find_speeds", ss.str());

		cout << endl;
	#endif /* _DBG_MOTOR_CONF */


// automatic axisinit on ABSOLUTE
// qui pero' la macchina a stati puo' non essere ancora in funzione

	switch ( axis->config_InitType() ){
		case Axis_ns::INIT_ON_ABSOLUTE:
			axis_init_at_first_fsm = true;
#if 0
			axis_init();
#endif
			break;
		default:
			break;
	}




	if ( get_state() == Tango::FAULT ) {
		#ifdef _DBG_CONF
			cout << "---- AXIS CONFIGURATION ERROR!----@" << __LINE__ << " : " << Tango::DevStateName[get_state()] << endl;
		#endif /* _DBG_CONF */
		return false;
	}
	__prevstate = Tango::UNKNOWN;
	set_state(__prevstate);
	
	#ifdef _DBG_CONF
		cout << "---- AXIS CONFIGURATION end----" << endl;
	#endif /* _DBG_CONF */

#ifdef _DBG_CONF
	axis->verbose(0);
	axis->drv_verbose(0);
	axis->lnk_verbose(0);
#endif
	
	return true;
}

} //	namespace
