#include <AxisG2.h>
#include <devicestate.h>

#include <time.h>


namespace AxisG2_ns
{



/*********************************************
*
*
*
*********************************************/
Tango::DevString AxisG2::_service(Tango::DevString argin){
  Tango::DevString argout;

  typedef enum {
	NOCMD,
	HELP2,
	HELP,
	VERBOSE,
	DRV_VERBOSE,
	LNK_VERBOSE,
	FORCE_INIT_LAST,
	UPDATE_MICROCODE_FLAG,
	FINDRLS,
	FINDFLS,
	FINDHOME,
	CHECK_ENCODER_ENABLE,
	CLR_STATEPOS_STAT,
	CLR_CHK_E_STAT,
	ENCSIGNSTORE,
	ENCSIGNCHECK,
	ENCSIGN
  } scmd_t;
  int n_cmd = 17;

  const char * const scmdName[] = {
	"NoCmd",
	"Help2",
	"Help",
	"verbose",
	"drv_verbose",
	"lnk_verbose",
	"force_init_last",
	"update_microcode_flag",
	"findrls",
	"findfls",
	"findhome",
	"check_encoder_enable",
	"clr_statepos_stat",
	"clr_chk_e_stat",
	"encsignstore",
	"encsigncheck",
	"encsign"
  };

  vector<string> scmdHelp2;
  scmdHelp2.push_back("Help2");
  scmdHelp2.push_back("verbose nn");
  scmdHelp2.push_back("drv_verbose nn");
  scmdHelp2.push_back("lnk_verbose nn");
  scmdHelp2.push_back("findrls");
  scmdHelp2.push_back("findfls");
  scmdHelp2.push_back("findhome");
  scmdHelp2.push_back("check_encoder_enable 0/1");
  scmdHelp2.push_back("clr_statepos_stat");
  scmdHelp2.push_back("clr_chk_e_stat");
  scmdHelp2.push_back("encsignstore");
  scmdHelp2.push_back("encsigncheck");
  scmdHelp2.push_back("encsign");

  vector<string> scmdHelp;
  scmdHelp.push_back("Help");
  scmdHelp.push_back("force_init_last");
  scmdHelp.push_back("update_microcode_flag");

  string w;
  vector<string> v;

	if ((*attr_DeviceState_read) != SERVICE)
        	Tango::Except::throw_exception ((const char *)"AxisG2",
                                        	(const char *)"Not Allowed in this DeviceState",
                                        	(const char *)"AxisG2::_service()");

	#ifdef _DBG_SERVICE
        	cout <<"::" << argin <<"::" << endl;
	#endif /* _DBG_SERVICE */

	string in(argin);

	scmd_t cmd = NOCMD;
	int i;
	for (i=0; i<n_cmd; i++){
		if ( 0==in.compare(0,strlen(scmdName[i]), scmdName[i]) ){
			#ifdef _DBG_SERVICE
				cout << "FOUND : [" << i << "]" << scmdName[i] << endl;
			#endif /* _DBG_SERVICE */
			break;
		}
	}
	cmd = (scmd_t)i;

	w.clear();
	switch (cmd){
		case HELP2:
			for(size_t i=0; i<scmdHelp2.size(); i++){
				w.append(scmdHelp2[i]);
				w.append("\n");
			}
		case HELP:
			for(size_t i=0; i<scmdHelp.size(); i++){
				w.append(scmdHelp[i]);
				w.append("\n");
			}
        		argout = CORBA::string_dup(w.c_str());
			break;

		case VERBOSE:
			{
			  int v;
			  std::stringstream ss;
			  ss.str("");
			  int n = sscanf(argin, "verbose  %d", &v);
			  if (axis) {
			  	if (n==1){
					ss << "."; axis->verbose((int)v);
			  	} else {
                          		ss << axis->verbose();
			  	}
			  } else {
				ss << "NO AXIS";
			  }
			  argout = CORBA::string_dup(ss.str().c_str());
			}
			break;

		case DRV_VERBOSE:
			{
			  int v;
			  std::stringstream ss;
			  ss.str("");
			  int n = sscanf(argin, "drv_verbose  %d", &v);
			  if (axis) {
			  	if (n==1){
                          		ss << "."; axis->drv_verbose((int)v);
			  	} else {
                          		ss << axis->drv_verbose();
			  	}
			  } else {
				ss << "NO AXIS";
			  }
			  argout = CORBA::string_dup(ss.str().c_str());
			}
			break;

		case LNK_VERBOSE:
			#ifdef _DBG_SERVICE 
				cout << "LNK_VERBOSE" << endl;
			#endif /* _DBG_SERVICE */
			{
			  int v;
			  std::stringstream ss;
			  ss.str("");
			  int n = sscanf(argin, "lnk_verbose  %d", &v);
			  if (axis) {
			  	if (n==1){
                          		ss << "."; axis->lnk_verbose((int)v);
			  	} else {
                          		ss << axis->lnk_verbose();
			  	}
			  } else {
				ss << "NO AXIS";
			  }
			  argout = CORBA::string_dup(ss.str().c_str());
			}
			break;

		case FORCE_INIT_LAST:
			#ifdef _DBG_SERVICE 
				cout << "FORCE_INIT_LAST" << endl;
			#endif /* _DBG_SERVICE */
			{ // reconf
			  if (axis) _conf_flag = AxisConf();
			  else 
     		       		Tango::Except::throw_exception(
                       	                 (const char *)"AxisG2",
               	                         (const char *)"Axis not created!",
                       		         (const char *)"AxisG2::service()");

			  // read_memorized_attribute
			  Tango::DbDatum dbdatum;
        		  double v;

		          try {
		                if (read_memorized_attribute("AxisCurrentPosition", dbdatum)) {
     		                	dbdatum >> v;
       		         	};
 		       	  } catch(Tango::DevFailed &ex) {
     		           	Tango::Except::throw_exception(
                       		                 (const char *)"AxisG2",
               		                         (const char *)"Error on read memorized attribute AxisCurrentPosition",
                               		         (const char *)"AxisG2::service()");
       		 	  }
			  if(axis){ axis->force_init(v); }
			  else {
     		           	Tango::Except::throw_exception(
                       		                 (const char *)"AxisG2",
               		                         (const char *)"Axis not created!",
                               		         (const char *)"AxisG2::service()");
			  }
			}
			argout = CORBA::string_dup("FORCE_INIT_LAST");
			break;

		case UPDATE_MICROCODE_FLAG:
			#ifdef _DBG_SERVICE 
				cout << "UPDATE_MICROCODE_FLAG" << endl;
			#endif /* _DBG_SERVICE */
			*attr_IsMicrocodeOK_read = controller->drv()->program_running();
			if (*attr_IsMicrocodeOK_read){
			  	if (axis) _conf_flag = AxisConf();
			  	else 
     		       			Tango::Except::throw_exception(
                       	                 	(const char *)"AxisG2",
               	                         	(const char *)"Axis not created!",
                       		         	(const char *)"AxisG2::service()");
				argout = CORBA::string_dup("Micro code is running");
			} else argout = CORBA::string_dup("Micro code is NOT running");
			break;

		case FINDRLS:
			#ifdef _DBG_SERVICE 
				cout << "FINDRLS" << endl;
			#endif /* _DBG_SERVICE */
			if (axis) axis->fsm(Axis_ns::FSMT_FINDRLS);
			argout = CORBA::string_dup("FINDRLS");
			break;

		case FINDFLS:
			#ifdef _DBG_SERVICE 
				cout << "FINDFLS" << endl;
			#endif /* _DBG_SERVICE */
			if (axis) axis->fsm(Axis_ns::FSMT_FINDFLS);
			argout = CORBA::string_dup("FINDFLS");
			break;

		case FINDHOME:
			#ifdef _DBG_SERVICE 
				cout << "FINDHOME" << endl;
			#endif /* _DBG_SERVICE */
			if (axis) axis->fsm(Axis_ns::FSMT_FINDHOME);
			argout = CORBA::string_dup("FINDHOME");
			break;

		case CHECK_ENCODER_ENABLE:
			#ifdef _DBG_SERVICE 
				cout << "CHECK_ENCODER_ENABLE" << endl;
			#endif /* _DBG_SERVICE */
			{
			  int v;
			  int n = sscanf(argin, "check_encoder_enable  %d", &v);
			  if (n==1){
				if ( v == 0 ) *attr_CheckEncoder_read = false;
				else 	      *attr_CheckEncoder_read = true;
				argout = CORBA::string_dup("CHECK_ENCODER_ENABLE");
			  } else {
				argout = CORBA::string_dup("CHECK_ENCODER_ENABLE error in value");
			  }
			}
			break;

		case CLR_STATEPOS_STAT:
			#ifdef _DBG_SERVICE 
				cout << "CLR_STATEPOS_STAT" << endl;
			#endif /* _DBG_SERVICE */
			statepos_stat.mean = 0;
        		statepos_stat.min  = MAXFLOAT;
        		statepos_stat.max  = MINFLOAT;
			argout = CORBA::string_dup("CLR_STATEPOS_STAT");
			break;

		case CLR_CHK_E_STAT:
			#ifdef _DBG_SERVICE 
				cout << "CLR_CHK_E_STAT" << endl;
			#endif /* _DBG_SERVICE */
			if (axis)
			{omni_mutex_lock guard(lock1);
				axis->clr_chk_diff_on_run();
				chk_diff_on_run_stat.mean = 0;
        			chk_diff_on_run_stat.min  = MAXFLOAT;
        			chk_diff_on_run_stat.max  = -MAXFLOAT;
			} 
			argout = CORBA::string_dup("CLR_CHK_E_STAT");
			break;

		case ENCSIGN:
			#ifdef _DBG_SERVICE 
				cout << "ENCSIGN" << endl;
			#endif /* _DBG_SERVICE */
			if (axis) axis->fsm(Axis_ns::FSMT_FINDENCSIGN);
			argout = CORBA::string_dup("ENCSIGN");
			break;

		case ENCSIGNSTORE:
			#ifdef _DBG_SERVICE 
				cout << "ENCSIGNSTORE" << endl;
			#endif /* _DBG_SERVICE */
			{
			  Encoder_ns::SencSignature_t _ss = axis->SencSignature();

        		  Tango::DbDatum _attr_prop_LS("__sigLS");
			  switch(_ss.startPoint){
                                        case Encoder_ns::LIMIT_FLS: _attr_prop_LS << "LIMIT_FLS";
                                        break;
                                        case Encoder_ns::LIMIT_RLS: _attr_prop_LS << "LIMIT_RLS";
                                        break;
                                }
        		  if ( !put_memorized_attribute("EncoderPosition", _attr_prop_LS) ){
				// signature not memorized
				cout << "put_memorized_attribute() ERROR!" << endl;
			  } else {
				// signature memorized!
				cout << "put_memorized_attribute() ok!" << endl;
			  }

			  Tango::DbDatum _attr_prop_V0("__sigV0");
			  _attr_prop_V0 << _ss.v0;
				cout << "_ss.v0 : " << _ss.v0 << endl;
        		  if ( !put_memorized_attribute("EncoderPosition", _attr_prop_V0) ){
				// signature not memorized
				cout << "put_memorized_attribute() ERROR!" << endl;
			  } else {
				// signature memorized!
				cout << "put_memorized_attribute() ok!" << endl;
			  }

			  Tango::DbDatum _attr_prop_V("__sigValues");
			  _attr_prop_V << _ss.values;
        		  if ( !put_memorized_attribute("EncoderPosition", _attr_prop_V) ){
				// signature not memorized
				cout << "put_memorized_attribute() ERROR!" << endl;
			  } else {
				// signature memorized!
				cout << "put_memorized_attribute() ok!" << endl;
			  }

			}
			argout = CORBA::string_dup("ENCSIGNSTORE");
			break;

		case ENCSIGNCHECK:
			#ifdef _DBG_SERVICE 
				cout << "ENCSIGNCHECK" << endl;
			#endif /* _DBG_SERVICE */
			if (axis)
			{
			  Encoder_ns::SencSignature_t ss;
			  Encoder_ns::CheckSignature_t ret;
			  Tango::DbDatum dbdatum;
			  if (!read_memorized_attribute("EncoderPosition", "__sigValues", dbdatum)){
				cout << "read_memorized_attribute() ERROR!" << endl;
			  } else {
				cout << "read_memorized_attribute() ok!" << endl;
				dbdatum >> ss.values;
			  }
			  cout << "SencSignature.size() = " << ss.values.size() << endl;
			  ret = axis->checkSencSignature(ss);
			  std::stringstream msg;
			  msg.str("");
			  msg << "ENCSIGNCHECK:" << "\n";
			  msg << axis->printEncSignatureError(ret.err) << "\n";;
			/*
			double abs_error;
        		int enc_dir;
        		int ix_dir;
			*/
			  msg << "abs_error= " << ret.abs_error << "[%]" << "\n";
			  msg << "enc_dir=   " << ret.enc_dir << "\n";
			  msg << "ix_dir=    " << ret.ix_dir << "\n";

			  argout = CORBA::string_dup((msg.str()).c_str());
			}
			break;

		case NOCMD:
		default:
        		argout = CORBA::string_dup(">");
			break;
	}

	return argout;
}



} //    namespace
