#include <AxisG2.h>


namespace AxisG2_ns {

//--------------------------------------------------------
/*
	AxisStatePosition()

	Puo' generare exception da axis->update()
*/
//--------------------------------------------------------
void AxisG2::AxisStatePosition() {
	DEBUG_STREAM << "===================================AxisG2::AxisStatePosition() entering... controller= " << controller << " axis= " << axis << endl;

	if ( !controller || !axis ) return;

        struct timeval _t0, _t1;
	gettimeofday(&_t0, NULL);

	bool flag_positioning = false;

	DEBUG_STREAM << "AxisG2::AxisStatePosition() entering... " << endl;

#ifdef _DBG_PROF_STATEPOS
	{ std::stringstream ss;
	ss << " BEGIN : ";
	ss << __FUNCTION__ << " : " << __LINE__ << " : going to controller->state() ... ";
	prtdbg(ss.str());
	}
#endif /* _DBG_PROF_STATEPOS */

	if (ct==Controller_ns::CTRL_DOWN) {
		omni_mutex_lock guard(lock1);
		__state  = Tango::ALARM;
		__status = "ALARM:: Controller comunication is down";
		std::stringstream es;
		es << "AxisG2::AxisStatePosition(): 1" << __status;
		INFOLOG(false,es.str());
		return; 
	}
	if (ct==Controller_ns::CTRL_CLOSE) {
		omni_mutex_lock guard(lock1);
		__state  = Tango::ALARM;
		__status = "ALARM:: Controller/Hardware comunication is closed";
		std::stringstream es;
		es << "AxisG2::AxisStatePosition(): 2" << __status;
		//INFOLOG(false,es.str());
		return; 
	}

	if (!axis){
		omni_mutex_lock guard(lock1);
		__state  = Tango::ALARM;
		__status = "ALARM:: Waiting first controller comunication";
		std::stringstream es;
		es << "AxisG2::AxisStatePosition(): 3" << __status;
		INFOLOG(false,es.str());
		return;
	}
#ifdef _DBG_PROF_STATEPOS
	{ std::stringstream ss;
	ss << __LINE__;
	prtdbg(ss.str());
	}
#endif /* _DBG_PROF_STATEPOS */

	string __micro_code_status = "";
	string __motor_status      = "";
	string __fsm_status        = "";
	string __init_status       = "";

	Tango::DevState __micro_code_state = Tango::UNKNOWN;
	Tango::DevState __motor_state      = Tango::UNKNOWN;
	Tango::DevState __fsm_state        = Tango::UNKNOWN;
	Tango::DevState __init_state       = Tango::UNKNOWN;


	// Spostati in AxisG2.h in modo che rimanga tra chiamata e chiamata di AxisStatePosition()
	// Tango::DevState __enc_err_state
	// string __enc_err_status

#ifdef _DBG_PROF_STATEPOS
	{ std::stringstream ss;
	ss << __FUNCTION__ << " : " << __LINE__ << " : going to axis->update() ... ";
	prtdbg(ss.str());
	}
#endif /* _DBG_PROF_STATEPOS */

	// FSM
        if (axis->update()) {
		// cout << "UPDATE!" << endl;
 		if ( !*attr_IsMicrocodeOK_read ){
			__micro_code_status = "__!MICROCODE STOPPED!__";
			__micro_code_state  = Tango::ALARM;
			_conf_flag = false;
			std::stringstream es;
			es << "AxisG2::AxisStatePosition(): 4" << __micro_code_status;
			INFOLOG(false,es.str());
		} else {
			__micro_code_status = "";
			__micro_code_state  = Tango::ON;
		}

#ifdef _DBG_PROF_STATEPOS
	{ std::stringstream ss;
	ss << __LINE__;
	prtdbg(ss.str());
	}
#endif /* _DBG_PROF_STATEPOS */

        	FSMst = axis->fsm(Axis_ns::FSMT_NOP);

#ifdef _DBG_PROF_STATEPOS
	{ std::stringstream ss;
	ss << __LINE__;
	prtdbg(ss.str());
	}
#endif /* _DBG_PROF_STATEPOS */

		if (ct==Controller_ns::CTRL_UP) {
                	double t = controller->drv()->update_etime();
			if ( t < update_stat.min ) update_stat.min = t;
			if ( t > update_stat.max ) update_stat.max = t;
		}


		// CHECKENCODER
		/* typedef enum { CHK_ENC_UNKNOWN, CHK_ENC_OK, CHK_ENC_ERROR, CHK_ENC_ERROR_ONSTAY, CHK_ENC_ERROR_ONRUN } chk_enc_t; */

		Axis_ns::chk_enc_t ce;
		ce = axis->check_encoder();
		{omni_mutex_lock guard(lock1);
			_chk_enc = ce;
		}
#ifdef _DBG_PROF_STATEPOS
	{ std::stringstream ss;
	ss << __LINE__;
	prtdbg(ss.str());
	}
#endif /* _DBG_PROF_STATEPOS */

		if ( *attr_CheckEncoder_read ) {
			if ( *attr_CheckEncoderHoldOn_read  && (__enc_err_state == _chk_enc_onErr) ) {
				// hold on
			} else {
				switch (_chk_enc){
					case Axis_ns::CHK_ENC_ERROR:
					case Axis_ns::CHK_ENC_ERROR_ONSTAY:
					case Axis_ns::CHK_ENC_ERROR_ONRUN:
						if ( encoderCheckOnErrorStop ) axis->stop();
						{ omni_mutex_lock guard(lock1); __enc_err_state = _chk_enc_onErr;}
						__enc_err_status = Axis_ns::CheckEncMsg[_chk_enc];

						{
							std::stringstream ss;
							ss << fixed;
							ss << "AxisG2::AxisStatePosition(): 5";
							ss << " " << Axis_ns::CheckEncName[_chk_enc] << " " << axis->mstatus();
							ss << " Motor::chk_mtn_prgr=" << axis->Motor::check_motion_progress();
							ss << " e=" << (axis->epos());
							ss << " last_e =" << (axis->chk_enc_last_value());
							ss << " m=" << (axis->motor2enc(axis->mpos()));
							ss << " last_m=" << (axis->chk_mot_last_value());
							ss << " e/m in enc units";
							INFOLOG(false,ss.str());
						}

						if ( *attr_CheckEncoderHoldOn_read ){
				  			time_t rawtime;
  							struct tm * timeinfo;
  							char buffer [80];

  							time (&rawtime);
  							timeinfo = localtime (&rawtime);
  							strftime (buffer,80," [at %F %T]",timeinfo);

  							std::string r(buffer);
							__enc_err_status += r;
						}
						break;
					case Axis_ns::CHK_ENC_UNKNOWN:
						__enc_err_status = "";
						{ omni_mutex_lock guard(lock1); __enc_err_state = Tango::UNKNOWN;}
						{
							std::stringstream ss;
							ss << fixed;
							ss << "AxisG2::AxisStatePosition(): 6";
							ss << " " << Axis_ns::CheckEncName[_chk_enc] << " " << axis->mstatus();
							ss << " Motor::chk_mtn_prgr=" << axis->Motor::check_motion_progress();
							ss << " e=" << (axis->epos());
							ss << " last_e =" << (axis->chk_enc_last_value());
							ss << " m=" << (axis->motor2enc(axis->mpos()));
							ss << " last_m=" << (axis->chk_mot_last_value());
							ss << " e/m in enc units";
							INFOLOG(false,ss.str());
						}
						break;
					case Axis_ns::CHK_ENC_OK:
						__enc_err_status = "";
						{ omni_mutex_lock guard(lock1); __enc_err_state = Tango::ON;}
						{
							std::stringstream ss;
							ss << fixed;
							ss << "AxisG2::AxisStatePosition(): 7";
							ss << " " << Axis_ns::CheckEncName[_chk_enc] << " " << axis->mstatus();
							ss << " Motor::chk_mtn_prgr=" << axis->Motor::check_motion_progress();
							ss << " e=" << (axis->epos());
							ss << " last_e =" << (axis->chk_enc_last_value());
							ss << " m=" << (axis->motor2enc(axis->mpos()));
							ss << " last_m=" << (axis->chk_mot_last_value());
							ss << " e/m in enc units";
							INFOLOG(true,ss.str());
						}
						break;
					default:
						break;
				}
			}
		} else {
			omni_mutex_lock guard(lock1);
			__enc_err_state = Tango::ON;
			__enc_err_status = "";
		}
		//cout << "__enc_err_status = " << __enc_err_status << endl;


		// INITTIALIZED or NOT
		if ( !axis->isinit() ) {omni_mutex_lock guard(lock1); __init_state  = Tango::ALARM; __init_status = "Uninitialized axis!";}
		else { __init_state = Tango::ON; __init_status = ""; }
#ifdef _DBG_PROF_STATEPOS
	{ std::stringstream ss;
	ss << __LINE__;
	prtdbg(ss.str());
	}
#endif /* _DBG_PROF_STATEPOS */

		// MOTOR
		switch (axis->mstate()){
			case Motor_ns::MOVING:
       			case Motor_ns::ACCELERATING:
       			case Motor_ns::DECELERATING:
				{ omni_mutex_lock guard(lock1);
				  __motor_status = "motor moving";
				  __motor_state  = Tango::MOVING;
				    	 // ??axis->fsm_error_clear();
				}
				break;
	
			case Motor_ns::DEACTIVATED:
       			case Motor_ns::OFFLINE:
       			case Motor_ns::STANDSTILL:
       			case Motor_ns::BRAKED:
       			case Motor_ns::SERVO:
       			case Motor_ns::SWCTL:
				{ omni_mutex_lock guard(lock1);
				  __motor_status = "";
				  __motor_state  = Tango::STANDBY;
				}
				break;

				case Motor_ns::UNKNOWN:
					{ omni_mutex_lock guard(lock1);
					  __motor_status = "motor unknown!";
				  	  __motor_state  = Tango::UNKNOWN;
					}
					break;

       				case Motor_ns::ERROR:
					{ omni_mutex_lock guard(lock1);
				  	  __state = Tango::FAULT;
				  	  __motor_status = "motor in ERROR!";
					}
					break;

				default:
					break;
			}
#ifdef _DBG_PROF_STATEPOS
	{ std::stringstream ss;
	ss << __LINE__;
	prtdbg(ss.str());
	}
#endif /* _DBG_PROF_STATEPOS */


		// FSM
		switch (axis->fsm_state()){
			case Axis_ns::FSM_INITIALIZING:
				{ omni_mutex_lock guard(lock1);
				  __fsm_status = "initializing...";
				  __fsm_state = Tango::ON;
				}
				break;

			case Axis_ns::FSM_START_POSITIONING:
			case Axis_ns::FSM_POSITIONING:
			case Axis_ns::FSM_STOP_POSITIONING:
			case Axis_ns::FSM_POSITIONING_END:
				{ omni_mutex_lock guard(lock1);
				  __fsm_status = "positioning...";
				}
				flag_positioning = true;
				break;


			case Axis_ns::FSM_ERROR:
			case Axis_ns::FSM_WAIT:
				 __fsm_state = Tango::ON;
				switch (axis->fsm_error_state()){
					case Axis_ns::FSM_E000:
						 { omni_mutex_lock guard(lock1); __fsm_status =  ""; }	
						break;
					case Axis_ns::FSM_E003: {
						  omni_mutex_lock guard(lock1);
                                               	__fsm_status = "No motion due to MICROCODE STOPPED!";
                                               	*attr_IsMicrocodeOK_read = false;
						_conf_flag = false;
						}
						break;
					case Axis_ns::FSM_E001:
					case Axis_ns::FSM_E005: 
						if ( (axis->fls() || axis->rls()) ){
							omni_mutex_lock guard(lock1);
                                               		__fsm_status = "No motion due to limit switch";
						}
						break;
					case Axis_ns::FSM_E002: {
							omni_mutex_lock guard(lock1);
							__fsm_status =  "No target reached";
						}	
						break;
					case Axis_ns::FSM_E006: {
							omni_mutex_lock guard(lock1);
							__fsm_status =  "Controller/Database initialization parameter mismatch";
						}	
						break;
					case Axis_ns::FSM_E007: {
							omni_mutex_lock guard(lock1);
							__fsm_status =  "No target reached due to limit switch";
						}	
						break;
					case Axis_ns::FSM_E008: {
							omni_mutex_lock guard(lock1);
							__fsm_status =  "Error on init!";
						}	
						break;
					default: {
							omni_mutex_lock guard(lock1);
							__fsm_status =  "Error";
						}	
						break;
				}
				break;

			default:
				//if (axis->fsm_error_state() == Axis_ns::FSM_E000) {};
				{omni_mutex_lock guard(lock1); __fsm_status =  ""; }
				break;
		}
		if ( axis->fsm_state() == Axis_ns::FSM_ERROR ){
			omni_mutex_lock guard(lock1);
			axis->fsm(Axis_ns::FSMT_ABORT);
		}

		// NEW STATE
		/* __micro_code_state __motor_state __enc_err_state __fsm_state __init_state */

		if (
			(__micro_code_state == Tango::FAULT) ||
			(__motor_state == Tango::FAULT) ||
			(__enc_err_state == Tango::FAULT) ||
			(__fsm_state == Tango::FAULT) ||
			(__init_state == Tango::FAULT)
		   ){
			omni_mutex_lock guard(lock1);
			__state = Tango::FAULT;
		} else 
		if (
			(__micro_code_state == Tango::ALARM) ||
			(__motor_state == Tango::ALARM) ||
			(__enc_err_state == Tango::ALARM) ||
			(__fsm_state == Tango::ALARM) ||
			(__init_state == Tango::ALARM)
		   ){
			omni_mutex_lock guard(lock1);
			__state = Tango::ALARM;
		} else if (flag_positioning){
			omni_mutex_lock guard(lock1);
			__state = Tango::MOVING;
		} else {
			omni_mutex_lock guard(lock1);
			__state = __motor_state;
		}

#ifdef _DBG_PROF_STATEPOS
	{ std::stringstream ss;
	ss << __LINE__;
	prtdbg(ss.str());
	}
#endif /* _DBG_PROF_STATEPOS */

		// NEW STATUS
		string tangoState(Tango::DevStateName[__state]);
		{
	  	omni_mutex_lock guard(lock1);
	  	__status =  tangoState;
		if (__micro_code_status.size() > 0 ) __status = __status + " " + __micro_code_status;
		if (__init_status.size() > 0 )       __status = __status + " " + __init_status;
		if (__motor_status.size() > 0 )      __status = __status + " " + __motor_status;
		if (__enc_err_status.size() > 0 )    __status = __status + " " + __enc_err_status;
		if (__fsm_status.size() > 0 )        __status = __status + " :: " + __fsm_status;
		}

        } else { /* (axis->update()) */
		std::stringstream ss;
		ss << "AxisG2::AxisStatePosition(): 8 NO UPDATE!";
		INFOLOG(false,ss.str());
	} /* (axis->update()) */

#ifdef _DBG_PROF_STATEPOS
	{ std::stringstream ss;
	ss << __LINE__;
	prtdbg(ss.str());
	}
#endif /* _DBG_PROF_STATEPOS */


	{omni_mutex_lock guard(lock1);
		chk_diff_on_run_stat.current = axis->chk_diff_on_run();
		if ( isnan(chk_diff_on_run_stat.min) ) chk_diff_on_run_stat.min = chk_diff_on_run_stat.current;
		else if (chk_diff_on_run_stat.current < chk_diff_on_run_stat.min) chk_diff_on_run_stat.min = chk_diff_on_run_stat.current;
		if ( isnan(chk_diff_on_run_stat.max) ) chk_diff_on_run_stat.max = chk_diff_on_run_stat.current;
		else if (chk_diff_on_run_stat.current > chk_diff_on_run_stat.max) chk_diff_on_run_stat.max = chk_diff_on_run_stat.current;
	}

/*

        typedef struct _stat {
                float  mean;
                float  min;
                float  max;
        } stat_t;

        stat_t  statepos_stat;
*/
	float statepos_etime; //millisec
	gettimeofday(&_t1, NULL);
	statepos_etime = (_t1.tv_sec - _t0.tv_sec)*1000 + ((float)(_t1.tv_usec - _t0.tv_usec))/1000;

	{omni_mutex_lock guard(lock1);
		statepos_stat.current = statepos_etime;
		if ( statepos_etime > statepos_stat.max ) statepos_stat.max = statepos_etime;
		if ( statepos_etime < statepos_stat.min ) statepos_stat.min = statepos_etime;
	}

	if (axis_init_at_first_fsm) {
		axis_init();
		axis_init_at_first_fsm = false;
	}

#ifdef _DBG_PROF_STATEPOS
	{ std::stringstream ss;
	ss << __LINE__;
	ss << " END";
	prtdbg(ss.str());
	}
#endif /* _DBG_PROF_STATEPOS */

}

} //	namespace
