#ifndef _AXIS_TH_H_
#define _AXIS_TH_H_

#include "Axis.h"

namespace Axis_ns {


class Axis_th : public Axis {

private:
        //- A mutex to protect the device against race conditions
        omni_mutex lock;


public:
	Axis_th(string name, string channel, Controller_ns::Controller *controller, bool prtLog) : Axis(name, channel, controller, prtLog) {};
	~Axis_th() {};

        void            prtLog(bool v) 		{ omni_mutex_lock guard(lock); return Axis::prtLog(v);        };
        bool            prtLog() 		{ omni_mutex_lock guard(lock); return Axis::prtLog();         };
        int            	verbose()              	{ omni_mutex_lock guard(lock); return Axis::verbose();        };
        void           	verbose(int b)         	{ omni_mutex_lock guard(lock); return Axis::verbose(b);       };
	string		name()			{ omni_mutex_lock guard(lock); return Axis::name();           };
	void		name(string n)		{ omni_mutex_lock guard(lock); return Axis::name(n);          };
        string          channel()               { omni_mutex_lock guard(lock); return Axis::channel();        };
        void            channel(string v)       { omni_mutex_lock guard(lock); return Axis::channel(v);       };
	// MOTION
        bool            forward(double v)       { omni_mutex_lock guard(lock); return Axis::forward(v);       };
        bool            backward(double v)      { omni_mutex_lock guard(lock); return Axis::backward(v);      };

        double          position()              { omni_mutex_lock guard(lock); return Axis::position();       };
	bool		position(double p)	{ omni_mutex_lock guard(lock); return Axis::position(p);      };
	bool		rawposition(double p, bool rel)	{ omni_mutex_lock guard(lock); return Axis::rawposition(p, rel);   };
	double		speed()			{ omni_mutex_lock guard(lock); return Axis::speed();          };
	bool		speed(double v)		{ omni_mutex_lock guard(lock); return Axis::speed(v);         };
	double		acceleration()		{ omni_mutex_lock guard(lock); return Axis::acceleration();   };
	bool		acceleration(double a)	{ omni_mutex_lock guard(lock); return Axis::acceleration(a);  };
	double		deceleration()		{ omni_mutex_lock guard(lock); return Axis::deceleration();   };
	bool		deceleration(double a)	{ omni_mutex_lock guard(lock); return Axis::deceleration(a);  };
	double          velocity()              { omni_mutex_lock guard(lock); return Axis::velocity();       };
	// CONFIGURATION
	double		per()			{ omni_mutex_lock guard(lock); return Axis::per();            };
	bool 		per(double v)		{ omni_mutex_lock guard(lock); return Axis::per(v);           };
	double		peo()			{ omni_mutex_lock guard(lock); return Axis::peo();            };
	bool 		peo(double v)		{ omni_mutex_lock guard(lock); return Axis::peo(v);           };
	double		emr()			{ omni_mutex_lock guard(lock); return Axis::emr();            };
	bool 		emr(double v)		{ omni_mutex_lock guard(lock); return Axis::emr(v);           };
	double		emo()			{ omni_mutex_lock guard(lock); return Axis::emo();            };
	bool 		emo(double v)		{ omni_mutex_lock guard(lock); return Axis::emo(v);           };
        double          pmr()                   { omni_mutex_lock guard(lock); return Axis::pmr();	      }; 
        double          ear()			{ omni_mutex_lock guard(lock); return Axis::ear();	      };
        bool            ear(double v)		{ omni_mutex_lock guard(lock); return Axis::ear(v);	      };
        double          eao()      		{ omni_mutex_lock guard(lock); return Axis::eao();	      };
        bool            eao(double v)		{ omni_mutex_lock guard(lock); return Axis::eao(v);	      };

        Controller_ns::mOffOn_t mOffOn()       	{  omni_mutex_lock guard(lock); return Axis::mOffOn();	      };
        bool            mOffOn(double m, double q){omni_mutex_lock guard(lock); return Axis::mOffOn(m,q);     };
        bool            mOffOn(bool b)          {  omni_mutex_lock guard(lock); return Axis::mOffOn(b);       };
        bool            isEnabled_mOffOn()      {  omni_mutex_lock guard(lock); return Axis::isEnabled_mOffOn();};


	double		position2motor(double p){ omni_mutex_lock guard(lock); return Axis::position2motor(p);};
	double		motor2position(double m){ omni_mutex_lock guard(lock); return Axis::motor2position(m);};
	double		position2enc(double p)	{ omni_mutex_lock guard(lock); return Axis::position2enc(p);  };
	double		enc2position(double e)	{ omni_mutex_lock guard(lock); return Axis::enc2position(e); };
        double          motor2enc(double m)	{ omni_mutex_lock guard(lock); return Axis::motor2enc(m);    };
        double          enc2motor(double e)	{ omni_mutex_lock guard(lock); return Axis::enc2motor(e);    };
	double		speed2motor(double v)	{ omni_mutex_lock guard(lock); return Axis::speed2motor(v);   };
	double		motor2speed(double vm)	{ omni_mutex_lock guard(lock); return Axis::motor2speed(vm);  };
	double		speed2enc(double v)	{ omni_mutex_lock guard(lock); return Axis::speed2enc(v);     };
	double		enc2speed(double ve)	{ omni_mutex_lock guard(lock); return Axis::enc2speed(ve);    };
	double		acceleration2motor(double a) { omni_mutex_lock guard(lock); return Axis::acceleration2motor(a); };
	double		motor2acceleration(double am){ omni_mutex_lock guard(lock); return Axis::motor2acceleration(am);};
	double 		enc2acceleration(double ae)  { omni_mutex_lock guard(lock); return Axis::enc2acceleration(ae);  };
	double 		acceleration2enc(double a)   { omni_mutex_lock guard(lock); return Axis::acceleration2enc(a);   };
	double		deceleration2motor(double a) { omni_mutex_lock guard(lock); return Axis::deceleration2motor(a); };
	double		motor2deceleration(double am){ omni_mutex_lock guard(lock); return Axis::motor2deceleration(am);};
	double 		enc2deceleration(double ae)  { omni_mutex_lock guard(lock); return Axis::enc2deceleration(ae);  };
	double 		deceleration2enc(double a)   { omni_mutex_lock guard(lock); return Axis::deceleration2enc(a);   };
        bool           	mconf(Controller_ns::mconf_t mc) { omni_mutex_lock guard(lock); return Axis::mconf(mc);  };
        Controller_ns::mconf_t	mconf()			 { omni_mutex_lock guard(lock); return Axis::mconf();    };
	Controller_ns::mconf_t  get_mconf()		 { omni_mutex_lock guard(lock); return Axis::get_mconf();};
        bool            econf(Controller_ns::econf_t ec) { omni_mutex_lock guard(lock); return Axis::econf(ec);};
        Controller_ns::econf_t  econf() 		 { omni_mutex_lock guard(lock); return Axis::econf();    };       
        Controller_ns::econf_t  get_econf()  		 { omni_mutex_lock guard(lock); return Axis::get_econf();};
	//
	bool 		fls()			{ omni_mutex_lock guard(lock); return Axis::fls();               };
	bool 		rls()			{ omni_mutex_lock guard(lock); return Axis::rls();               };
	bool 		home()			{ omni_mutex_lock guard(lock); return Axis::home();              };
	bool 		hwfls()			{ omni_mutex_lock guard(lock); return Axis::hwfls();             };
	bool 		hwrls()			{ omni_mutex_lock guard(lock); return Axis::hwrls();             };
	bool 		hwhome()		{ omni_mutex_lock guard(lock); return Axis::hwhome();            };
	AxisLimit_t	axis2motorLimit(AxisLimit_t l){ omni_mutex_lock guard(lock); return Axis::axis2motorLimit(l);};
	double 		target() 		{ omni_mutex_lock guard(lock); return Axis::target();            };
	bool 		mInit()			{ omni_mutex_lock guard(lock); return Axis::mInit();             };
	bool 		rflsInit(AxisLimit_t l)	{ omni_mutex_lock guard(lock); return Axis::rflsInit(l);         };
	bool            homeInit(AxisFindHomeVersus_t v) { omni_mutex_lock guard(lock); return Axis::homeInit(v);};
	bool            indexInit(AxisInitType_t t){ omni_mutex_lock guard(lock); return Axis::indexInit(t);	 };

        bool            rotIdxInit(AxisInitType_t t) { omni_mutex_lock guard(lock); return Axis::rotIdxInit(t);  };


	AxisFindHomeVersus_t findedge_versus()	{ omni_mutex_lock guard(lock); return Axis::findedge_versus();	 };

	vector<double>	find_speeds()		{ omni_mutex_lock guard(lock); return Axis::find_speeds();       };
	bool 		find_speeds(vector<double> v){ omni_mutex_lock guard(lock); return Axis::find_speeds(v); };
	AxisFindLimit_t	findlimit(AxisLimit_t l, AxisFindLimit_t oldstate)
						{ omni_mutex_lock guard(lock); return Axis::findlimit(l, oldstate); };
	bool		findhome() 		{ omni_mutex_lock guard(lock); return Axis::findhome();          };
	bool		findfls() 		{ omni_mutex_lock guard(lock); return Axis::findfls();           };
	bool		findrls() 		{ omni_mutex_lock guard(lock); return Axis::findrls();           };
	//
	bool		update()		{ omni_mutex_lock guard(lock); return Axis::update();            };
	bool		update(double tmo)	{ omni_mutex_lock guard(lock); return Axis::update(tmo);         };
	bool		refresh()		{ omni_mutex_lock guard(lock); return Axis::refresh();           };
	bool		refresh(double tmo)	{ omni_mutex_lock guard(lock); return Axis::refresh(tmo);	 };
	AxisFSM_t  	fsm(AxisFSMTransitions_t tr){ omni_mutex_lock guard(lock); return Axis::fsm(tr);         };
	AxisFSM_t	fsm_state() 		{ omni_mutex_lock guard(lock); return Axis::fsm_state();         };
	string		fsm_status() 		{ omni_mutex_lock guard(lock); return Axis::fsm_status();        };
	AxisFSMError_t	fsm_error_state() 	{ omni_mutex_lock guard(lock); return Axis::fsm_error_state();   };
	string		fsm_error_status() 	{ omni_mutex_lock guard(lock); return Axis::fsm_error_status();  };
	bool            fsm_error_clear() 	{ omni_mutex_lock guard(lock); return Axis::fsm_error_clear();   };
	AxisInitFSM_t	ifsm_state() 		{ omni_mutex_lock guard(lock); return Axis::ifsm_state();        };
	string		ifsm_status()		{ omni_mutex_lock guard(lock); return Axis::ifsm_status();       };

	void		config_InitType(AxisInitType_t t) 	{ Axis::config_InitType(t); 			 };
	AxisInitType_t	config_InitType() 			{ return Axis::config_InitType(); 		 };
	string		config_InitType2str() 			{ return Axis::config_InitType2str(); 		 };
	void		config_rls(AxisLimitConf_t v)  { omni_mutex_lock guard(lock); return Axis::config_rls(v);};
	void		config_fls(AxisLimitConf_t v)  { omni_mutex_lock guard(lock); return Axis::config_fls(v);};
	void		config_home(AxisLimitConf_t v) { omni_mutex_lock guard(lock); return Axis::config_home(v);};
	void		config_index(AxisLimitConf_t v){ omni_mutex_lock guard(lock); return Axis::config_index(v);};
	AxisLimitConf_t	config_rls()		{ omni_mutex_lock guard(lock); return Axis::config_rls();        };
	AxisLimitConf_t	config_fls()		{ omni_mutex_lock guard(lock); return Axis::config_fls();        };
	AxisLimitConf_t	config_home()		{ omni_mutex_lock guard(lock); return Axis::config_home();       };
	AxisLimitConf_t	config_index()		{ omni_mutex_lock guard(lock); return Axis::config_index();      };
        AxisHomeType_t  home_type()     	{ omni_mutex_lock guard(lock); return Axis::home_type();         };
        //bool            home_type(AxisHomeType_t t) { omni_mutex_lock guard(lock); return Axis::home_type(t);	 };
        AxisHomePolarity_t   home_polarity()    { omni_mutex_lock guard(lock); return Axis::home_polarity();	 };
        AxisFindHomeVersus_t home_versus()      { omni_mutex_lock guard(lock); return Axis::home_versus();	 };
        bool            home_versus(AxisFindHomeVersus_t v) { omni_mutex_lock guard(lock); return Axis::home_versus(v);};

	bool  		check_mconf_mt()	{ omni_mutex_lock guard(lock); return Axis::check_mconf_mt();	 };

	bool 		sencInit(AxisFindPolarity_t p)	{ omni_mutex_lock guard(lock); return Axis::sencInit(p); };
	bool 		get_SencSignature()	{ omni_mutex_lock guard(lock); return Axis::get_SencSignature(); };


	string 		units()           	{ omni_mutex_lock guard(lock); return Axis::units();             };
	bool   		units(string v)   	{ omni_mutex_lock guard(lock); return Axis::units(v);            };
	bool		isinit()		{ omni_mutex_lock guard(lock); return Axis::isinit();            };
	bool 		stop()			{ omni_mutex_lock guard(lock); return Axis::stop();              };
	bool		mpos_on_init(double p)	{ omni_mutex_lock guard(lock); return Axis::mpos_on_init(p);     };
	double		mpos_on_init()		{ omni_mutex_lock guard(lock); return Axis::mpos_on_init();      };
        bool            pos_on_init(double p)	{ omni_mutex_lock guard(lock); return Axis::pos_on_init(p);      };
        double          pos_on_init()   	{ omni_mutex_lock guard(lock); return Axis::pos_on_init();       };
        bool            defposition(double p)	{ omni_mutex_lock guard(lock); return Axis::defposition(p);      };
        vector<double>  saved_mpos()     	{ omni_mutex_lock guard(lock); return Axis::saved_mpos();        };
        vector<double>  saved_epos()     	{ omni_mutex_lock guard(lock); return Axis::saved_epos();        };
	double          relative_mpos_preinit() { omni_mutex_lock guard(lock); return Axis::relative_mpos_preinit(); };
	double          relative_epos_preinit() { omni_mutex_lock guard(lock); return Axis::relative_epos_preinit(); };

	chk_enc_t       check_encoder()		{ omni_mutex_lock guard(lock); return Axis::check_encoder();     };
	bool            check_encoder_restart()	{ omni_mutex_lock guard(lock); return Axis::check_encoder_restart();};
        bool            force_init(double p)	{ omni_mutex_lock guard(lock); return Axis::force_init(p);       };
        double          chk_enc_last_value()    { omni_mutex_lock guard(lock); return Axis::chk_enc_last_value();};
        double          chk_mot_last_value()    { omni_mutex_lock guard(lock); return Axis::chk_mot_last_value();};
	double          chk_diff_on_run()       { omni_mutex_lock guard(lock); return Axis::chk_diff_on_run();	 };
	void            clr_chk_diff_on_run() 	{ omni_mutex_lock guard(lock); return Axis::clr_chk_diff_on_run();};

	void            chk_enc_enable(VNOTU bool e)  { }; // <<<<<======
	bool 		chk_enc_enable()	{ omni_mutex_lock guard(lock); return Axis::chk_enc_enable(); 	 };

	string		brake_conf_str()	{ omni_mutex_lock guard(lock); return Axis::brake_conf_str();	 };
	bool 		motor_on()		{ omni_mutex_lock guard(lock); return Axis::motor_on();		 };
	bool 		motor_off()		{ omni_mutex_lock guard(lock); return Axis::motor_off();		 };


	string          axis_error_str()	{ omni_mutex_lock guard(lock); return Axis::axis_error_str();	 };
	string          check_enc_name_str(chk_enc_t ce){ omni_mutex_lock guard(lock); return Axis::check_enc_name_str(ce);};
	restore_last_position_t get_restore_last_position(){ omni_mutex_lock guard(lock); return Axis::get_restore_last_position();};
	bool    	restore_last_position() { omni_mutex_lock guard(lock); return Axis::restore_last_position();};



private:
#if 0
	bool 		set_init(bool b){ omni_mutex_lock guard(lock); return Axis::;
	bool 		set_init(double tmo, bool b){ omni_mutex_lock guard(lock); return Axis::;


	bool		__position(double p){ omni_mutex_lock guard(lock); return Axis::;

	bool 		check_motion_progress(double tmo){ omni_mutex_lock guard(lock); return Axis::;
#endif

}; /* class Axis */

} /* namespace Axis_ns */

#endif /* _AXIS_TH_H_ */
