

typedef enum {
         STATE_UNKNOWN
        ,NOT_READY_TO_SWITCHON
        ,SWITCHON_DISABLED
        ,READY_TO_SWITCHON
        ,SWITCHON
        ,OPERATION_ENABLED
        ,QUICK_STOP
        ,FAULT_REACTION
        ,FAULT
} state_t;


typedef enum {
         SHUTDOWN_CMD
        ,SWITCH_ON_CMD
        ,ENABLE_OPERATION_CMD
        ,DISABLE_OPERATION_CMD
        ,QUICK_STOP_CMD
        ,REMOVE_QUICK_STOP_CMD
        ,ENABLE_VOLTAGE_CMD
        ,DISABLE_VOLTAGE_CMD
        ,FAULT_RESET_CMD
        ,HALT_CMD
        ,CLEAR_HALT_CMD
} stateCmd_t;




state_t gstate(){
        U16 sw = (U16) od_read(0x6041, 00);
        state_t  st = STATE_UNKNOWN;
        if ((sw & 0b0000000001001111) == 0b0000000000000000) st = NOT_READY_TO_SWITCHON;
        if ((sw & 0b0000000001001111) == 0b0000000001000000) st = SWITCHON_DISABLED;
        if ((sw & 0b0000000001101111) == 0b0000000000100001) st = READY_TO_SWITCHON;
        if ((sw & 0b0000000001101111) == 0b0000000000100011) st = SWITCHON;
        if ((sw & 0b0000000001101111) == 0b0000000000100111) st = OPERATION_ENABLED;
        if ((sw & 0b0000000001101111) == 0b0000000000000111) st = QUICK_STOP;
        if ((sw & 0b0000000001001111) == 0b0000000000001111) st = FAULT_REACTION;
        if ((sw & 0b0000000001001111) == 0b0000000000001000) st = FAULT;
        return st;
}



state_t CiA402StateCmd(stateCmd_t cmd){
        state_t  actualstate = gstate();
        U16 	 cw = (U16) InOut.ControlWord;
        
        //actualstate = OPERATION_ENABLED;

        switch (actualstate) {
                case SWITCHON_DISABLED:
                        if ( cmd == SHUTDOWN_CMD )         InOut.ControlWord = ((cw & 0xFFF8) | 0x0006) ;
                        break; 
                case READY_TO_SWITCHON:
                        if ( cmd == SWITCH_ON_CMD )        InOut.ControlWord = ((cw & 0xFFF0) | 0x0007);
                        if ( cmd == QUICK_STOP_CMD )       InOut.ControlWord = ((cw & 0xFFF6) | 0x0002);
                        break;
                case SWITCHON:
                        if ( cmd == ENABLE_OPERATION_CMD ) InOut.ControlWord = ((cw & 0xFFF0) | 0x000F);
                        if ( cmd == SHUTDOWN_CMD )         InOut.ControlWord = ((cw & 0xFFF8) | 0x0006);
                        if ( cmd == QUICK_STOP_CMD )       InOut.ControlWord = ((cw & 0xFFF6) | 0x0002);
                        if ( cmd == DISABLE_VOLTAGE_CMD )  InOut.ControlWord = ((cw & 0xFFF2) | 0x0000);
                        break;
                case OPERATION_ENABLED:
                        if ( cmd == SHUTDOWN_CMD )         InOut.ControlWord = ((cw & 0xFFF8) | 0x0006);
                        if ( cmd == DISABLE_VOLTAGE_CMD )  InOut.ControlWord = ((cw & 0xFFF2) | 0x0000);
                        if ( cmd == DISABLE_OPERATION_CMD )InOut.ControlWord = ((cw & 0xFFF0) | 0x0007);
                        if ( cmd == QUICK_STOP_CMD )       InOut.ControlWord = ((cw & 0xFFF6) | 0x0002);
                        break;
                case QUICK_STOP:
                        if ( cmd == ENABLE_OPERATION_CMD ) InOut.ControlWord = ((cw & 0xFFF0) | 0x000F);
                        if ( cmd == DISABLE_VOLTAGE_CMD )  InOut.ControlWord = ((cw & 0xFFF2) | 0x0000);
                        break;
                case FAULT:
                        if ( cmd == FAULT_RESET_CMD )      {
                                                                InOut.ControlWord = ((cw & 0xFF7B) | 0x0004); yield();
                                                                InOut.ControlWord = ((cw & 0xFF7F) | 0x0080); yield();
                                                                InOut.ControlWord = ((cw & 0xFF7F) | 0x0000); yield();
                                                           }
                        break;
                default:
                        break;
                        
        }
        yield();
	/*
	 _brake_conf.close_time 	od_read(0x2038, 01)
	 _brake_conf.shutdown_time 			02
	 _brake_conf.open_time				03
	 _brake_conf.start_time				04
	*/
	
        int max = 1 + 2 * ( od_read(0x2038, 01) + od_read(0x2038, 02) + od_read(0x2038, 03) + od_read(0x2038, 04) ); // msec
        //max = 500;
        do {
                sleep(1); // 1 ms
                max -= 1;       // cosi' max * 1000 .... sec
                yield();
        } while ( (gstate() == actualstate) && (max > 0) );
        return gstate();
}

bool    motor_on() {
        bool ret = false;
        state_t  newstate;
        state_t  actualstate = gstate();

        switch (actualstate) {
                case SWITCHON_DISABLED:
                        newstate = CiA402StateCmd(SHUTDOWN_CMD);
                        if ( newstate !=  READY_TO_SWITCHON ) break;
                case READY_TO_SWITCHON:
                        newstate = CiA402StateCmd(SWITCH_ON_CMD);
                        if ( newstate !=  SWITCHON ) break;
                case SWITCHON:
                case QUICK_STOP:
                        newstate = CiA402StateCmd(ENABLE_OPERATION_CMD);
                        if ( newstate !=  OPERATION_ENABLED ) break;
                        ret = true;
                        break;
                case OPERATION_ENABLED:
                        ret = true;
                        break;
                case FAULT:
                        // allora se siamo sui limitswitch posso tentare
                        // di resettare la condizione di FAULT;
                        /*
                        if ( CiA402FaultResetOnLimitSwitch() ){
                                newstate = CiA402StateCmd(ENABLE_OPERATION_CMD);
                                if ( newstate !=  OPERATION_ENABLED ) break;
                                ret = true;
                        }
                        */
                        break;
                default:
                        break;
        }

        return ret;
}

bool    motor_off(){
        bool ret = false;
        
        state_t  newstate;
        state_t  actualstate = gstate();

        switch (actualstate) {
                case OPERATION_ENABLED:
                        newstate = CiA402StateCmd(DISABLE_OPERATION_CMD);
                        if ( newstate !=  SWITCHON ) break;
                        ret = true;
                        break;
                default:
                        break;
        }

        return ret;
}



