map U16 StatusWord  	as input  0x6041:00
map S32 EncoderObj  	as input  0x6063:00
map S32 EncoderOffset 	as input  0x2400:01
map U32 DI          	as input  0x60FD:00 
map U16 ControlWord 	as inout  0x6040:00
map S32 MotorObj		as input  0x6062:00
map S32 HomeOffset		as input  0x607C:00


map S32 CC 		as inout  0x2500:05
map S32 Pos		as inout  0x2500:06
map S32 Dir		as input  0x2500:07
map S32 Pc		as input  0x2500:08
map S32 Bck		as input  0x2500:09
//map S32 NbRtr_i		as input  0x2500:10
map S32 Dbw		as input  0x2500:11
map S32 Scale		as input  0x2500:12
//
map S32 Cp		as input 0x2500:13
/*
0 = no encoder
1 = incremental  ...... ok
2 = absolute
3 = semi absolute
4 = analogic absolute
*/

map S32 Err 		as output 0x2400:05
map S32 prevErr 	as output 0x2400:06

//map S32 NbRtr 		as output 0x2400:07
map S32 nbrtr 		as inout 0x2500:10

map S32 have_brake	as inout 0x2410:02

#include "wrapper.h"

#include "cia402.hpp"

typedef struct {
	int v;
	int div;
	bool use;
} fixp_t;

U32 tick; // 1 ms increment counter

bool flg = 1; 

long int old_encoder;
long int encoder;
long int speed;
int k;	
int fix;
	
bool orls;
bool ofls;
bool nrls, nfls;
U32 otick;

long int xpos;
long int e;
long int pe;
int dir;

U16 cw;

int db;
fixp_t emr;
fixp_t pc;

int cmd;
int state;
	
void leave(){
	yield();
	tick += 1;
}

bool moving(){
	//U16 sw = In.StatusWord;
	U16 sw = (U16) od_read(0x6041, 00);
	U32 m = (U32)od_read(0xC010, 01);
	if ( (sw & 0x6f) == 0x27 ) { // se in op enabled
		m = (m & 0x40000) || ( ((m & 0xf)==0x6) && (m & 0x6000) );
		if (m) return true;
		return false;
	} 
	return false;
};

bool fls(){
	return ((In.DI & 02) >> 1);
};

bool rls(){
	return (In.DI & 01);
};

void stop() {
	InOut.ControlWord = InOut.ControlWord | 0x100;
	leave();
	if (InOut.have_brake != 0 )	motor_off();
};
//=================================
int getenc(){
	switch (In.Cp){
		case 0:
			return In.MotorObj;
			break;
		case 1:
			return In.EncoderObj + In.EncoderOffset;
			break;
		default:
			return 0;
			break;
	}
#if 0
 return In.EncoderObj + In.EncoderOffset;
#endif
}


void notok(){
    stop();
    InOut.CC &= 0xFFFFF0FF;
    InOut.CC |= 0x400;
};

void ok(){
    stop();
    InOut.CC &= 0xFFFFF0FF;
    InOut.CC |= 0x200;
};


void posi(){        // # POSI_
	cw = InOut.ControlWord;
	if ( emr.v != 0 ) { 
		if (emr.use) xpos = e * emr.div / emr.v;
		else xpos = e * emr.v;
     	InOut.ControlWord = (cw & 0xFEEF) | 0x0040; // clear bit 4 e 8, set 6 = relative 2 = immediate
		leave();
		// set xpos
		od_write(0x607a, 0, xpos);
		leave();
		// set bit 4
		InOut.ControlWord = (cw & 0xFEEF) | 0x0040 | 0x10;
		leave();
	}
};


unsigned int abs(int v){
	if (v < 0 ) return -v;
	return v;
};

//=================================
void user()
{
	S08 mode = (S08) od_read(0x6060, 00);
	
	nrls = rls();
	nfls = fls();
	
	if ( (!orls) && nrls && (mode != 6) ) stop(); // stop 
	if ( (!ofls) && nfls && (mode != 6)) stop(); // stop
	orls = nrls; ofls = nfls;
	
	U32 ltick = tick;
	if ( ltick >= otick ) { // beware of tick overflow: U32 !!!
		U32 dtick = (ltick - otick);
		if ( dtick >= 100 ){
			encoder = (long int) In.EncoderObj;
			speed = (encoder - old_encoder) * 1000 / (int)dtick;
			od_write(0x2500,0x01, speed);
			old_encoder = encoder;
			otick = ltick;
		};
	} else leave();
	
		fix = InOut.Pos;
		
		db = In.Dbw;
		
		/* da float a int */
		/* -100.0000 < Scale < 100.0000 */
		S32 vi = In.Scale;
		float vf = *((float*)&vi);
		if ((int(vf) == 1) || (int(vf) == -1)) {
			emr.use = false;
			emr.div = 1;
			emr.v = int(vf);
		} else {
			emr.use = true;
			emr.div = 10000;
			emr.v = int(vf * 10000.0);
		}

		/* 0 < Pc < 1 */
		vi = In.Pc;
		vf = *((float*)&vi);
		pc.div = 100;
		pc.v = int(vf * 100.0);
		
       	cmd = InOut.CC & 0x7;
       	      	
       	if ( InOut.CC & 0x8 ) {
       		InOut.CC &= 0xFFFF0000;	
       		e = fix;
			posi();
       		goto loop;
       	};   
       	
       	if ( cmd == 0x6 ) { //STOP
             stop();
             InOut.CC &= 0xFFFF0000;
             goto loop;
        };  

        // #FISTP_
       	state = InOut.CC & 0xF00;
        if ( (state == 0x100) && !moving()) { 
        	// #VER_
        	
			pe = e;
    		e = InOut.Pos - getenc();		
     		db = In.Dbw;
     		
			if ( abs(e) >= abs(db) ) {
  		  		if ( abs(e) >= abs(pe) ) {
					InOut.nbrtr --;
       	 			if ( InOut.nbrtr < 1 ) {
    					InOut.CC &= 0xFFFFF0FF;
    					InOut.CC |= 0x400;
       	 			} else posi(); 
    			} else posi();
 			} else ok();
 			
            goto loop; 	
        };
         
        if ( cmd == 0x5 ) { 
        		// azzera nuovi dati (Ndr) e cmd
        		InOut.CC &= 0xFFFF0FA;       
        		if ( InOut.nbrtr < 1 ) goto loop;
    
    			e = InOut.Pos - getenc();
 		
    			if ( (In.Dir == 0) || (In.Dir * e > 0) ){   // se NO compensazione o compensazione giusta...
    				e = e * pc.div / pc.v;
    			} else {        // compensazione
    				if ( e < 0 )  e = e - In.Bck;
        			else          e = e + In.Bck;
    			}
				pe = e;   
         		InOut.CC |= 0x100;            	// imposta attivita; in corso
			    if (InOut.have_brake != 0 )	motor_on();
         		posi();
		};
	
loop:
		Out.Err = e;
		Out.prevErr = pe;
       tick += 1; // then leave 1ms slot time
}
